/* Processed by ecpg (4.7.0) */
/* These include files are added by the preprocessor */
#include <ecpglib.h>
#include <ecpgerrno.h>
#include <sqlca.h>
/* End of automatic include section */

#line 1 "appdb.ec"
/* 
 * Application with database connection.
 * Copyright (C) 2004-2008 Petr Kubanek <petr@kubanek.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "rts2db/appdb.h"
#include "configuration.h"

#include <ecpgtype.h>

#include <iostream>
#include <iomanip>
#include <string>

#define OPT_DEBUGDB    OPT_LOCAL + 201

using namespace rts2db;


#line 1 "/usr/include/sql3types.h"
#ifndef _ECPG_SQL3TYPES_H
#define _ECPG_SQL3TYPES_H

/* SQL3 dynamic type codes */

/* chapter 13.1 table 2: Codes used for SQL data types in Dynamic SQL */

enum
{
	SQL3_CHARACTER = 1,
	SQL3_NUMERIC,
	SQL3_DECIMAL,
	SQL3_INTEGER,
	SQL3_SMALLINT,
	SQL3_FLOAT,
	SQL3_REAL,
	SQL3_DOUBLE_PRECISION,
	SQL3_DATE_TIME_TIMESTAMP,
	SQL3_INTERVAL,				/* 10 */
	SQL3_CHARACTER_VARYING = 12,
	SQL3_ENUMERATED,
	SQL3_BIT,
	SQL3_BIT_VARYING,
	SQL3_BOOLEAN,
	SQL3_abstract
	/* the rest is xLOB stuff */
};

/* chapter 13.1 table 3: Codes associated with datetime data types in Dynamic SQL */

enum
{
	SQL3_DDT_DATE = 1,
	SQL3_DDT_TIME,
	SQL3_DDT_TIMESTAMP,
	SQL3_DDT_TIME_WITH_TIME_ZONE,
	SQL3_DDT_TIMESTAMP_WITH_TIME_ZONE,

	SQL3_DDT_ILLEGAL			/* not a datetime data type (not part of
								 * standard) */
};

#endif   /* !_ECPG_SQL3TYPES_H */

#line 33 "appdb.ec"


SqlQuery::SqlQuery (const char *in_from)
{
	sql = NULL;
	if (in_from)
	{
		from = new char[strlen (in_from) + 1];
		strcpy (from, in_from);
	}
	else
	{
		from = NULL;
	}
	where = NULL;
}

SqlQuery::~SqlQuery (void)
{
	std::list <SqlColumn *>::iterator col_iter1, col_iter2;
	if (sql)
		delete sql;
	if (from)
		delete from;
	for (col_iter1 = columns.begin (); col_iter1 != columns.end (); )
	{
		col_iter2 = col_iter1;
		col_iter1++;
		delete *col_iter2;
	}
	columns.clear ();
	if (where)
		delete[] where;
}

void SqlQuery::addColumn (SqlColumn *add_column)
{
	columns.push_back (add_column);
	if (sql)
		delete sql;
	sql = NULL;
}

void SqlQuery::addColumn (const char *in_sql, const char *in_name, int in_order)
{
	SqlQuery::addColumn (new SqlColumn (in_sql, in_name, in_order));
}

void SqlQuery::addWhere (const char *in_where)
{
	if (!where)
	{
		where = new char[strlen (in_where) + 1];
		strcpy (where, in_where);
	}
	else
	{
		char *new_where = new char[strlen (where) + strlen (in_where) + 6];
		strcpy (new_where, where);
		delete[] where;
		strcat (new_where, " and ");
		strcat (new_where, in_where);
		where = new_where;
	}
}

char * SqlQuery::genSql ()
{
	std::string query;
	std::string order_by;
	std::list <SqlColumn *>::iterator col_iter1;
	std::list <SqlColumn *> col_copy;
	if (sql)
		return sql;
	query = std::string ();
	query += "select ";
	for (col_iter1 = columns.begin (); col_iter1 != columns.end (); col_iter1++)
	{
		char *col_sql = (*col_iter1)->genSql ();
		if (col_iter1 != columns.begin ())
			query += ", ";
		query += col_sql;
	}
	// add from
	if (from)
	{
		query += " from ";
		query += from;
	}
	if (where)
	{
		query += " where ";
		query += where;
	}
	order_by = std::string ();
	// order by part..
	col_copy = std::list <SqlColumn *> (columns);
	col_copy.sort ();
	for (col_iter1 = col_copy.begin (); col_iter1 != col_copy.end (); col_iter1++)
	{
		SqlColumn *col;
		col = (*col_iter1);
		if (col->getOrderBy () != 0)
		{
			if (order_by.length () == 0)
				order_by += " order by ";
			else
				order_by += ", ";

			order_by += col->genSql ();
			if (col->getOrderBy () > 0)
				order_by += " asc";
			else
				order_by += " desc";
		}
	}
	query += order_by;

	sql = new char[query.length() + 1];
	strcpy (sql, query.c_str ());
	return sql;
}

void SqlQuery::displayMinusPlusLine ()
{
	std::list <SqlColumn *>::iterator col_iter;
	for (col_iter = columns.begin (); col_iter != columns.end (); col_iter++)
	{
		if (col_iter != columns.begin ())
			std::cout << "-+-";
		for (int i = 0; i < (*col_iter)->getFieldSize (); i++)
			std::cout << '-';
	}
	std::cout << std::endl;
}

void SqlQuery::display ()
{
	/* exec sql begin declare section */
		 
		 
		 
		 
		   
		 

		 
		 
		 
		 
		 

		 
		 

	
#line 172 "appdb.ec"
 char * stmp ;
 
#line 173 "appdb.ec"
 int row ;
 
#line 174 "appdb.ec"
 int cols ;
 
#line 175 "appdb.ec"
 int cur_col ;
 
#line 176 "appdb.ec"
 int col_car , scale , precision ;
 
#line 177 "appdb.ec"
 int type ;
 
#line 179 "appdb.ec"
 bool d_bool ;
 
#line 180 "appdb.ec"
 int d_int ;
 
#line 181 "appdb.ec"
 float d_float ;
 
#line 182 "appdb.ec"
 double d_double ;
 
#line 183 "appdb.ec"
 char d_string [ 1024 ] ;
 
#line 185 "appdb.ec"
 int len ;
 
#line 186 "appdb.ec"
 int is_null ;
/* exec sql end declare section */
#line 188 "appdb.ec"


	std::list <SqlColumn *>::iterator col_iter;

	ECPGallocate_desc(__LINE__, "disp_desc");
#line 192 "appdb.ec"


	std::cout << genSql () << std::endl;
	// try to execute query
	stmp = genSql ();

	{ ECPGprepare(__LINE__, NULL, 0, "disp_stmp", stmp);}
#line 198 "appdb.ec"


	/* declare disp_cur cursor for $1 */
#line 200 "appdb.ec"

	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "declare disp_cur cursor for $1", 
	ECPGt_char_variable,(ECPGprepared_statement(NULL, "disp_stmp", __LINE__)),(long)1,(long)1,(1)*sizeof(char), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, ECPGt_EORT);}
#line 201 "appdb.ec"


	row = 0;

	while (1)
	{
#ifdef HAVE_ECPG_9
		{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "fetch next from disp_cur", ECPGt_EOIT, 
	ECPGt_sqlda, &"disp_desc", 0L, 0L, 0L, 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EORT);}
#line 208 "appdb.ec"

#else
		{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "fetch next from disp_cur", ECPGt_EOIT, 
	ECPGt_sqlda, &disp_desc, 0L, 0L, 0L, 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EORT);}
#line 210 "appdb.ec"

#endif		

		if (sqlca.sqlcode)
		{
			if (sqlca.sqlcode != ECPG_NOT_FOUND)
				std::cerr << "Sql error " << sqlca.sqlerrm.sqlerrmc << std::endl;
			break;
		}

		{ ECPGget_desc_header(__LINE__, "disp_desc", &(cols));
}
#line 220 "appdb.ec"


		if (row == 0)
		{
			cur_col = 1;
			for (col_iter = columns.begin (); col_iter != columns.end (); col_iter++)
			{
				{ ECPGget_desc(__LINE__, "disp_desc", cur_col,ECPGd_ret_octet,
	ECPGt_int,&(len),(long)1,(long)1,sizeof(int), ECPGd_type,
	ECPGt_int,&(type),(long)1,(long)1,sizeof(int), ECPGd_EODT);
}
#line 229 "appdb.ec"

				switch (type)
				{
					case SQL3_BOOLEAN:
						len = 5;
						break;
					case SQL3_NUMERIC:
					case SQL3_DECIMAL:
						len = 8;
						break;
					case SQL3_INTEGER:
						len = 10;
						break;
					case SQL3_SMALLINT:
						len = 6;
						break;
					case SQL3_FLOAT:
					case SQL3_REAL:
						len = 8;
						break;
					case SQL3_DOUBLE_PRECISION:
						len = 10;
						break;
					case SQL3_DATE_TIME_TIMESTAMP:
					case SQL3_INTERVAL:
						len = 28;
						break;
					case SQL3_CHARACTER:
					case SQL3_CHARACTER_VARYING:
					default:
						// leave empty..
						break;
				}
				if (cur_col > 1)
				{
					std::cout << SEP;
				}
				std::cout << std::setw (len) << (*col_iter)->getHeader (len);
				cur_col++;
			}
			std::cout << std::endl;
			// display -+ line
			displayMinusPlusLine ();
		}

		row++;

		for (cur_col = 1, col_iter = columns.begin (); cur_col <= cols && col_iter != columns.end (); cur_col++, col_iter++)
		{
			if (cur_col > 1)
				std::cout << SEP;
			{ ECPGget_desc(__LINE__, "disp_desc", cur_col,ECPGd_cardinality,
	ECPGt_int,&(col_car),(long)1,(long)1,sizeof(int), ECPGd_indicator,
	ECPGt_int,&(is_null),(long)1,(long)1,sizeof(int), ECPGd_precision,
	ECPGt_int,&(precision),(long)1,(long)1,sizeof(int), ECPGd_scale,
	ECPGt_int,&(scale),(long)1,(long)1,sizeof(int), ECPGd_type,
	ECPGt_int,&(type),(long)1,(long)1,sizeof(int), ECPGd_EODT);
}
#line 285 "appdb.ec"

			// unsigned short..
			len = (*col_iter)->getFieldSize ();
			switch (type)
			{
				case SQL3_BOOLEAN:
					{ ECPGget_desc(__LINE__, "disp_desc", cur_col,ECPGd_data,
	ECPGt_bool,&(d_bool),(long)1,(long)1,sizeof(bool), ECPGd_EODT);
}
#line 291 "appdb.ec"

					std::cout << std::setw(len) << (d_bool ? "true" : "false");
					(*col_iter)->processValue (&d_bool, type, is_null);
					break;
				case SQL3_NUMERIC:
				case SQL3_DECIMAL:
					if (scale == 0)
					{
						{ ECPGget_desc(__LINE__, "disp_desc", cur_col,ECPGd_data,
	ECPGt_int,&(d_int),(long)1,(long)1,sizeof(int), ECPGd_EODT);
}
#line 299 "appdb.ec"

						std::cout << std::setw(len) << d_int;
						(*col_iter)->processValue (&d_int, type, is_null);
					}
					else
					{
						{ ECPGget_desc(__LINE__, "disp_desc", cur_col,ECPGd_data,
	ECPGt_double,&(d_double),(long)1,(long)1,sizeof(double), ECPGd_EODT);
}
#line 305 "appdb.ec"

						std::cout << std::setw(len) << d_double;
						(*col_iter)->processValue (&d_double, type, is_null);
					}
					break;
				case SQL3_INTEGER:
				case SQL3_SMALLINT:
					{ ECPGget_desc(__LINE__, "disp_desc", cur_col,ECPGd_data,
	ECPGt_int,&(d_int),(long)1,(long)1,sizeof(int), ECPGd_EODT);
}
#line 312 "appdb.ec"

					std::cout << std::setw(len) << d_int;
					(*col_iter)->processValue (&d_int, type, is_null);
					break;
				case SQL3_FLOAT:
				case SQL3_REAL:
					{ ECPGget_desc(__LINE__, "disp_desc", cur_col,ECPGd_data,
	ECPGt_float,&(d_float),(long)1,(long)1,sizeof(float), ECPGd_EODT);
}
#line 318 "appdb.ec"

					std::cout << std::setw(len) << d_float;
					(*col_iter)->processValue (&d_float, type, is_null);
					break;
				case SQL3_DOUBLE_PRECISION:
					{ ECPGget_desc(__LINE__, "disp_desc", cur_col,ECPGd_data,
	ECPGt_double,&(d_double),(long)1,(long)1,sizeof(double), ECPGd_EODT);
}
#line 323 "appdb.ec"

					std::cout << std::setw(len) << d_double;
					(*col_iter)->processValue (&d_double, type, is_null);
					break;
				case SQL3_DATE_TIME_TIMESTAMP:
				case SQL3_INTERVAL:
				case SQL3_CHARACTER:
				case SQL3_CHARACTER_VARYING:
				default:
					{ ECPGget_desc(__LINE__, "disp_desc", cur_col,ECPGd_data,
	ECPGt_char,(d_string),(long)1024,(long)1,(1024)*sizeof(char), ECPGd_EODT);
}
#line 332 "appdb.ec"

					std::cout << std::setw(len) << std::setprecision(len) << d_string;
					(*col_iter)->processValue (&d_string, type, is_null);
					break;
			}
		}
		std::cout << std::endl;
	}

	displayMinusPlusLine ();

	std::cout << std::endl << row << " rows were displayed" << std::endl << std::endl;

	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "close disp_cur", ECPGt_EOIT, ECPGt_EORT);}
#line 345 "appdb.ec"

	ECPGdeallocate_desc(__LINE__, "disp_desc");
#line 346 "appdb.ec"

}

AppDb::AppDb (int in_argc, char **in_argv) : rts2core::CliApp (in_argc, in_argv)
{
	connectString = NULL;
	configFile = NULL;

	addOption (OPT_DATABASE, "database", 1, "connect string to PSQL database (default to stars)");
	addOption (OPT_CONFIG, "config", 1, "configuration file");
	addOption (OPT_DEBUGDB, "debugdb", 0, "print database debugging messages");
}

AppDb::~AppDb ()
{
	if (connectString)
		delete connectString;
}

int AppDb::processOption (int in_opt)
{
	switch (in_opt)
	{
		case OPT_DATABASE:
			connectString = new char[strlen (optarg) + 1];
			strcpy (connectString, optarg);
			break;
		case OPT_CONFIG:
			configFile = optarg;
			break;
		case OPT_DEBUGDB:
			ECPGdebug (1, stderr);
			break;
		default:
			return rts2core::CliApp::processOption (in_opt);
	}
	return 0;
}

bool AppDb::doInitDB ()
{
	return true;
}

void AppDb::afterProcessing ()
{
	if (doInitDB ())
	{
		{ ECPGdisconnect(__LINE__, "CURRENT");}
#line 394 "appdb.ec"

	}
}

int AppDb::initDB ()
{
	std::string cs;
	/* exec sql begin declare section */
	  
	  
	  
	
#line 402 "appdb.ec"
 const char * c_db ;
 
#line 403 "appdb.ec"
 const char * c_username ;
 
#line 404 "appdb.ec"
 const char * c_password ;
/* exec sql end declare section */
#line 405 "appdb.ec"

	// try to connect to DB

	rts2core::Configuration *config = rts2core::Configuration::instance ();

	std::string db_username;
	std::string db_password;

	if (connectString)
	{
		c_db = connectString;
	}
	else
	{
		config->getString ("database", "name", cs);
		c_db = cs.c_str ();
	}

	if (config->getString ("database", "username", db_username, "") == 0)
	{
		c_username = db_username.c_str ();
		if (config->getString ("database", "password", db_password) == 0)
		{
			c_password = db_password.c_str ();
			{ ECPGconnect(__LINE__, 0, c_db , c_username , c_password , NULL, 0); }
#line 429 "appdb.ec"

			if (sqlca.sqlcode != 0)
			{
				logStream (MESSAGE_ERROR) << "AppDb::init Cannot connect to DB '" << c_db 
					<< "' with user '" << c_username
					<< "' and password xxxx (see rts2.ini) :"
					<< sqlca.sqlerrm.sqlerrmc << sendLog;
				return -1;
			}
		}
		else
		{
			{ ECPGconnect(__LINE__, 0, c_db , c_username , NULL , NULL, 0); }
#line 441 "appdb.ec"

			if (sqlca.sqlcode != 0)
			{
				logStream (MESSAGE_ERROR) << "AppDb::init Cannot connect to DB '" << c_db 
					<< "' with user '" << c_username
					<< "': " << sqlca.sqlerrm.sqlerrmc << sendLog;
				return -1;
			}
		}
	}
	else
	{
		{ ECPGconnect(__LINE__, 0, c_db , NULL, NULL , NULL, 0); }
#line 453 "appdb.ec"

		if (sqlca.sqlcode != 0)
		{
			logStream (MESSAGE_ERROR) << "AppDb::init Cannot connect to DB '" << c_db << "' : " << sqlca.sqlerrm.sqlerrmc << " (" << sqlca.sqlcode << ")" << sendLog;
			return -1;
		}
	}

	return 0;
}

int AppDb::init ()
{
	rts2core::Configuration *config;
	int ret;

	ret = rts2core::CliApp::init ();
	if (ret)
		return ret;

	// load config..

	config = rts2core::Configuration::instance ();
	ret = config->loadFile (configFile);
	if (ret)
		return ret;

	if (doInitDB ())
		// load config.
		ret = initDB ();
	return ret;
}
