/* Processed by ecpg (4.7.0) */
/* These include files are added by the preprocessor */
#include <ecpglib.h>
#include <ecpgerrno.h>
#include <sqlca.h>
/* End of automatic include section */

#line 1 "devicedb.ec"
/* 
 * Device with database connection.
 * Copyright (C) 2004-2008 Petr Kubanek <petr@kubanek.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "rts2db/devicedb.h"
#include "configuration.h"

#define OPT_DEBUGDB    OPT_LOCAL + 201

using namespace rts2db;


#line 1 "/usr/include/sqlca.h"
#ifndef POSTGRES_SQLCA_H
#define POSTGRES_SQLCA_H

#ifndef PGDLLIMPORT
#if  defined(WIN32) || defined(__CYGWIN__)
#define PGDLLIMPORT __declspec (dllimport)
#else
#define PGDLLIMPORT
#endif   /* __CYGWIN__ */
#endif   /* PGDLLIMPORT */

#define SQLERRMC_LEN	150

#ifdef __cplusplus
extern		"C"
{
#endif

struct sqlca_t
{
	char		sqlcaid[8];
	long		sqlabc;
	long		sqlcode;
	struct
	{
		int			sqlerrml;
		char		sqlerrmc[SQLERRMC_LEN];
	}			sqlerrm;
	char		sqlerrp[8];
	long		sqlerrd[6];
	/* Element 0: empty						*/
	/* 1: OID of processed tuple if applicable			*/
	/* 2: number of rows processed				*/
	/* after an INSERT, UPDATE or				*/
	/* DELETE statement					*/
	/* 3: empty						*/
	/* 4: empty						*/
	/* 5: empty						*/
	char		sqlwarn[8];
	/* Element 0: set to 'W' if at least one other is 'W'	*/
	/* 1: if 'W' at least one character string		*/
	/* value was truncated when it was			*/
	/* stored into a host variable.				*/

	/*
	 * 2: if 'W' a (hopefully) non-fatal notice occurred
	 */	/* 3: empty */
	/* 4: empty						*/
	/* 5: empty						*/
	/* 6: empty						*/
	/* 7: empty						*/

	char		sqlstate[5];
};

struct sqlca_t *ECPGget_sqlca(void);

#ifndef POSTGRES_ECPG_INTERNAL
#define sqlca (*ECPGget_sqlca())
#endif

#ifdef __cplusplus
}
#endif

#endif

#line 27 "devicedb.ec"


int DeviceDb::willConnect (rts2core::NetworkAddress * in_addr)
{
	if (in_addr->getType () < getDeviceType () || (in_addr->getType () == getDeviceType () && strcmp (in_addr->getName (), getDeviceName ()) < 0))
		return 1;
	return 0;
}

DeviceDb::DeviceDb (int argc, char **argv, int in_device_type, const char *default_name):rts2core::Device (argc, argv, in_device_type, default_name)
{
	connectString = NULL;		 // defualt DB
	configFile = NULL;

	addOption (OPT_DATABASE, "database", 1, "connect string to PSQL database (default to stars)");
	addOption (OPT_CONFIG, "config", 1, "configuration file");
	addOption (OPT_DEBUGDB, "debugdb", 0, "print database debugging messages");
}

DeviceDb::~DeviceDb (void)
{
	{ ECPGdisconnect(__LINE__, "CURRENT");}
#line 48 "devicedb.ec"

	if (connectString)
		delete connectString;
}

void DeviceDb::postEvent (rts2core::Event *event)
{
	switch (event->getType ())
	{
		case EVENT_DB_LOST_CONN:
			{ ECPGdisconnect(__LINE__, "CURRENT");}
#line 58 "devicedb.ec"

			if (initDB ())
			{
				addTimer (20, event);
				return;
			}
			break;
	}
	rts2core::Device::postEvent (event);
}

int DeviceDb::processOption (int in_opt)
{
	switch (in_opt)
	{
		case OPT_DATABASE:
			connectString = new char[strlen (optarg) + 1];
			strcpy (connectString, optarg);
			break;
		case OPT_CONFIG:
			configFile = optarg;
			break;
		case OPT_DEBUGDB:
			ECPGdebug (1, stderr);
			break;
		default:
			return rts2core::Device::processOption (in_opt);
	}
	return 0;
}

int DeviceDb::reloadConfig ()
{
	rts2core::Configuration *config;

	// load config..

	config = rts2core::Configuration::instance ();
	return config->loadFile (configFile);
}

int DeviceDb::initDB ()
{
	int ret;
	std::string cs;
	/* exec sql begin declare section */
	  
	  
	  
	
#line 104 "devicedb.ec"
 const char * c_db ;
 
#line 105 "devicedb.ec"
 const char * c_username ;
 
#line 106 "devicedb.ec"
 const char * c_password ;
/* exec sql end declare section */
#line 107 "devicedb.ec"

	// try to connect to DB

	rts2core::Configuration *config;

	ret = reloadConfig();

	config = rts2core::Configuration::instance ();

	if (ret)
		return ret;

	if (connectString)
	{
		c_db = connectString;
	}
	else
	{
		config->getString ("database", "name", cs);
		c_db = cs.c_str ();
	}

	std::string db_username;
	std::string db_password;

	if (config->getString ("database", "username", db_username, "") == 0)
	{
		c_username = db_username.c_str ();
		if (config->getString ("database", "password", db_password) == 0)
		{
			c_password = db_password.c_str ();
			{ ECPGconnect(__LINE__, 0, c_db , c_username , c_password , NULL, 0); }
#line 138 "devicedb.ec"

			if (sqlca.sqlcode != 0)
			{
				logStream (MESSAGE_ERROR) << "cannot connect to DB '" << c_db 
					<< "' with user '" << c_username
					<< "' and password xxxx (see rts2.ini) :"
					<< sqlca.sqlerrm.sqlerrmc << sendLog;
				return -1;
			}
		}
		else
		{
			{ ECPGconnect(__LINE__, 0, c_db , c_username , NULL , NULL, 0); }
#line 150 "devicedb.ec"

			if (sqlca.sqlcode != 0)
			{
				logStream (MESSAGE_ERROR) << "cannot connect to DB '" << c_db 
					<< "' with user '" << c_username
					<< "': " << sqlca.sqlerrm.sqlerrmc << sendLog;
				return -1;
			}
		}
	}
	else
	{
		{ ECPGconnect(__LINE__, 0, c_db , NULL, NULL , NULL, 0); }
#line 162 "devicedb.ec"

		if (sqlca.sqlcode != 0)
		{
			logStream (MESSAGE_ERROR) << "cannot connect to DB '" << c_db << "'. Please check if the database server is running (on specified port, or on port 5432, which is the default one; please be aware that RTS2 does not parse PostgreSQL configuration, so if the database is running on the non-default port, it will not be accessible unless you specify the port) : " << sqlca.sqlerrm.sqlerrmc << sendLog;
			return -1;
		}
	}

	cameras.load ();

	return 0;
}

int DeviceDb::init ()
{
	int ret;

	ret = rts2core::Device::init ();
	if (ret)
		return ret;

	// load config.
	return initDB ();
}

void DeviceDb::forkedInstance ()
{
	// dosn't work??
	//  EXEC SQL DISCONNECT;
	rts2core::Device::forkedInstance ();
}

void DeviceDb::signaledHUP ()
{
	reloadConfig();
}
