/* Processed by ecpg (4.7.0) */
/* These include files are added by the preprocessor */
#include <ecpglib.h>
#include <ecpgerrno.h>
#include <sqlca.h>
/* End of automatic include section */

#line 1 "observation.ec"
/* 
 * Observation entry.
 * Copyright (C) 2005-2010 Petr Kubanek <petr@kubanek.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "rts2db/observation.h"
#include "rts2db/sqlerror.h"
#include "rts2db/target.h"
#include "rts2db/taruser.h"

#include "libnova_cpp.h"
#include "timestamp.h"
#include "configuration.h"

#include <iostream>
#include <iomanip>
#include <sstream>


#line 1 "/usr/include/sqlca.h"
#ifndef POSTGRES_SQLCA_H
#define POSTGRES_SQLCA_H

#ifndef PGDLLIMPORT
#if  defined(WIN32) || defined(__CYGWIN__)
#define PGDLLIMPORT __declspec (dllimport)
#else
#define PGDLLIMPORT
#endif   /* __CYGWIN__ */
#endif   /* PGDLLIMPORT */

#define SQLERRMC_LEN	150

#ifdef __cplusplus
extern		"C"
{
#endif

struct sqlca_t
{
	char		sqlcaid[8];
	long		sqlabc;
	long		sqlcode;
	struct
	{
		int			sqlerrml;
		char		sqlerrmc[SQLERRMC_LEN];
	}			sqlerrm;
	char		sqlerrp[8];
	long		sqlerrd[6];
	/* Element 0: empty						*/
	/* 1: OID of processed tuple if applicable			*/
	/* 2: number of rows processed				*/
	/* after an INSERT, UPDATE or				*/
	/* DELETE statement					*/
	/* 3: empty						*/
	/* 4: empty						*/
	/* 5: empty						*/
	char		sqlwarn[8];
	/* Element 0: set to 'W' if at least one other is 'W'	*/
	/* 1: if 'W' at least one character string		*/
	/* value was truncated when it was			*/
	/* stored into a host variable.				*/

	/*
	 * 2: if 'W' a (hopefully) non-fatal notice occurred
	 */	/* 3: empty */
	/* 4: empty						*/
	/* 5: empty						*/
	/* 6: empty						*/
	/* 7: empty						*/

	char		sqlstate[5];
};

struct sqlca_t *ECPGget_sqlca(void);

#ifndef POSTGRES_ECPG_INTERNAL
#define sqlca (*ECPGget_sqlca())
#endif

#ifdef __cplusplus
}
#endif

#endif

#line 33 "observation.ec"


using namespace rts2db;

Observation::Observation (int in_obs_id)
{
	obs_id = in_obs_id;
	tar_id = -1;
	tar_name = std::string ("unset");
	tar_type = 0;
	imgset = NULL;
	displayImages = 0;
	displayCounts = 0;
	printHeader = true;
}

Observation::Observation (int in_tar_id, const char *in_tar_name, char in_tar_type, int in_obs_id, double in_obs_ra, double in_obs_dec, double in_obs_alt, double in_obs_az, double in_obs_slew, double in_obs_start, int in_obs_state, double in_obs_end)
{
	tar_id = in_tar_id;
	tar_name = std::string (in_tar_name);
	tar_type = in_tar_type;
	obs_id = in_obs_id;
	obs_ra = in_obs_ra;
	obs_dec = in_obs_dec;
	obs_alt = in_obs_alt;
	obs_az = in_obs_az;
	obs_slew = in_obs_slew;
	obs_start = in_obs_start;
	obs_state = in_obs_state;
	obs_end = in_obs_end;
	imgset = NULL;
	displayImages = 0;
	displayCounts = 0;
	printHeader = true;
}

Observation::~Observation (void)
{
	std::vector <rts2image::Image *>::iterator img_iter;
	if (imgset)
	{
		for (img_iter = imgset->begin (); img_iter != imgset->end (); img_iter++)
			delete (*img_iter);
		imgset->clear ();
		delete imgset;
	}
}

int Observation::load ()
{
	/* exec sql begin declare section */
		// cannot use TARGET_NAME_LEN, as it does not work with some ecpg veriosn
		 
		 
		 
		   
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
	
#line 85 "observation.ec"
  struct varchar_1  { int len; char arr[ 150 ]; }  db_tar_name ;
 
#line 86 "observation.ec"
 int db_tar_id ;
 
#line 87 "observation.ec"
 char db_tar_type ;
 
#line 88 "observation.ec"
 int db_obs_id = obs_id ;
 
#line 89 "observation.ec"
 double db_obs_ra ;
 
#line 90 "observation.ec"
 int ind_obs_ra ;
 
#line 91 "observation.ec"
 double db_obs_dec ;
 
#line 92 "observation.ec"
 int ind_obs_dec ;
 
#line 93 "observation.ec"
 double db_obs_alt ;
 
#line 94 "observation.ec"
 int ind_obs_alt ;
 
#line 95 "observation.ec"
 double db_obs_az ;
 
#line 96 "observation.ec"
 int ind_obs_az ;
 
#line 97 "observation.ec"
 double db_obs_slew ;
 
#line 98 "observation.ec"
 int ind_obs_slew ;
 
#line 99 "observation.ec"
 double db_obs_start ;
 
#line 100 "observation.ec"
 int ind_obs_start ;
 
#line 101 "observation.ec"
 int db_obs_state ;
 
#line 102 "observation.ec"
 double db_obs_end ;
 
#line 103 "observation.ec"
 int ind_obs_end ;
/* exec sql end declare section */
#line 104 "observation.ec"


	// already loaded
	if (tar_id > 0)
		return 0;

	obs_ra = rts2_nan ("f");
	obs_dec = rts2_nan ("f");
	obs_alt = rts2_nan ("f");
	obs_az = rts2_nan ("f");
	obs_slew = rts2_nan ("f");
	obs_start = rts2_nan ("f");
	obs_state = 0;
	obs_end = rts2_nan ("f");

	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "select tar_name , observations . tar_id , type_id , obs_ra , obs_dec , obs_alt , obs_az , extract ( EPOCH from obs_slew ) , extract ( EPOCH from obs_start ) , obs_state , extract ( EPOCH from obs_end ) from observations , targets where obs_id = $1  and observations . tar_id = targets . tar_id", 
	ECPGt_int,&(db_obs_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, 
	ECPGt_varchar,&(db_tar_name),(long)150,(long)1,sizeof(struct varchar_1), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(db_tar_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_char,&(db_tar_type),(long)1,(long)1,(1)*sizeof(char), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_obs_ra),(long)1,(long)1,sizeof(double), 
	ECPGt_int,&(ind_obs_ra),(long)1,(long)1,sizeof(int), 
	ECPGt_double,&(db_obs_dec),(long)1,(long)1,sizeof(double), 
	ECPGt_int,&(ind_obs_dec),(long)1,(long)1,sizeof(int), 
	ECPGt_double,&(db_obs_alt),(long)1,(long)1,sizeof(double), 
	ECPGt_int,&(ind_obs_alt),(long)1,(long)1,sizeof(int), 
	ECPGt_double,&(db_obs_az),(long)1,(long)1,sizeof(double), 
	ECPGt_int,&(ind_obs_az),(long)1,(long)1,sizeof(int), 
	ECPGt_double,&(db_obs_slew),(long)1,(long)1,sizeof(double), 
	ECPGt_int,&(ind_obs_slew),(long)1,(long)1,sizeof(int), 
	ECPGt_double,&(db_obs_start),(long)1,(long)1,sizeof(double), 
	ECPGt_int,&(ind_obs_start),(long)1,(long)1,sizeof(int), 
	ECPGt_int,&(db_obs_state),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_obs_end),(long)1,(long)1,sizeof(double), 
	ECPGt_int,&(ind_obs_end),(long)1,(long)1,sizeof(int), ECPGt_EORT);}
#line 149 "observation.ec"

	if (sqlca.sqlcode)
	{
		logStream (MESSAGE_ERROR) << "Observation::load DB error: " << sqlca.sqlerrm.sqlerrmc << " (" << sqlca.sqlcode << ")" << sendLog;
		{ ECPGtrans(__LINE__, NULL, "rollback");}
#line 153 "observation.ec"

		return -1;
	}
	{ ECPGtrans(__LINE__, NULL, "commit");}
#line 156 "observation.ec"

	tar_id = db_tar_id;
	tar_name = std::string (db_tar_name.arr);
	tar_type = db_tar_type;
	if (ind_obs_ra >= 0)
		obs_ra = db_obs_ra;
	if (ind_obs_dec >= 0)
		obs_dec = db_obs_dec;
	if (ind_obs_alt >= 0)
		obs_alt = db_obs_alt;
	if (ind_obs_az >= 0)
		obs_az = db_obs_az;
	if (ind_obs_slew >= 0)
		obs_slew = db_obs_slew;
	if (ind_obs_start >= 0)
		obs_start = db_obs_start;
	obs_state = db_obs_state;
	if (ind_obs_end >= 0)
		obs_end = db_obs_end;
	return 0;
}

int Observation::loadImages ()
{
	int ret;
	if (imgset)
		// already loaded..
		return 0;
	ret = load ();
	if (ret)
		return ret;

	imgset = new ImageSetObs (this);
	return imgset->load ();
}

int Observation::loadCounts ()
{
	/* exec sql begin declare section */
		   
		 
		 
		 
		 
		 
	
#line 195 "observation.ec"
 int db_obs_id = obs_id ;
 
#line 196 "observation.ec"
 long db_count_date ;
 
#line 197 "observation.ec"
 int db_count_usec ;
 
#line 198 "observation.ec"
 int db_count_value ;
 
#line 199 "observation.ec"
 float db_count_exposure ;
 
#line 200 "observation.ec"
 char db_count_filter ;
/* exec sql end declare section */
#line 201 "observation.ec"


	ECPGset_var( 0, &( db_obs_id ), __LINE__);\
 /* declare cur_counts cursor for select extract ( EPOCH from count_date ) , count_usec , count_value , count_exposure , count_filter from counts where obs_id = $1  */
#line 213 "observation.ec"


	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "declare cur_counts cursor for select extract ( EPOCH from count_date ) , count_usec , count_value , count_exposure , count_filter from counts where obs_id = $1 ", 
	ECPGt_int,&(db_obs_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, ECPGt_EORT);}
#line 215 "observation.ec"

	while (1)
	{
		{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "fetch next from cur_counts", ECPGt_EOIT, 
	ECPGt_long,&(db_count_date),(long)1,(long)1,sizeof(long), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(db_count_usec),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(db_count_value),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_float,&(db_count_exposure),(long)1,(long)1,sizeof(float), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_char,&(db_count_filter),(long)1,(long)1,(1)*sizeof(char), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EORT);}
#line 223 "observation.ec"

		if (sqlca.sqlcode)
			break;
		counts.push_back (Rts2Count (obs_id, db_count_date, db_count_usec, db_count_value, db_count_exposure,
			db_count_filter));
	}
	if (sqlca.sqlcode != ECPG_NOT_FOUND)
	{
		logStream (MESSAGE_DEBUG) << "Observation::loadCounts DB error: " << sqlca.sqlerrm.sqlerrmc << " (" << sqlca.sqlcode << sendLog;
		{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "close cur_counts", ECPGt_EOIT, ECPGt_EORT);}
#line 232 "observation.ec"

		{ ECPGtrans(__LINE__, NULL, "rollback");}
#line 233 "observation.ec"

		return -1;
	}
	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "close cur_counts", ECPGt_EOIT, ECPGt_EORT);}
#line 236 "observation.ec"

	{ ECPGtrans(__LINE__, NULL, "commit");}
#line 237 "observation.ec"

	return 0;
}

void Observation::startSlew (struct ln_equ_posn * coords, struct ln_hrz_posn * hrz)
{
	/* exec sql begin declare section */
	    
	   
	   
	 
	 
	
#line 244 "observation.ec"
 int d_obs_id = getObsId () ;
 
#line 245 "observation.ec"
 double d_obs_ra = coords -> ra ;
 
#line 246 "observation.ec"
 double d_obs_dec = coords -> dec ;
 
#line 247 "observation.ec"
 float d_obs_alt ;
 
#line 248 "observation.ec"
 float d_obs_az ;
/* exec sql end declare section */
#line 249 "observation.ec"


	d_obs_alt = hrz->alt;
	d_obs_az = hrz->az;

	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "update observations set obs_slew = now ( ) , obs_ra = $1  , obs_dec = $2  , obs_alt = $3  , obs_az = $4  where obs_id = $5 ", 
	ECPGt_double,&(d_obs_ra),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(d_obs_dec),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_float,&(d_obs_alt),(long)1,(long)1,sizeof(float), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_float,&(d_obs_az),(long)1,(long)1,sizeof(float), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(d_obs_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, ECPGt_EORT);}
#line 263 "observation.ec"

	
	if (sqlca.sqlcode != 0)
	{
		throw SqlError ("cannot update observation slew positions");
	}
	{ ECPGtrans(__LINE__, NULL, "commit");}
#line 269 "observation.ec"

}

void Observation::startObservation ()
{
	/* exec sql begin declare section */
	    
	
#line 275 "observation.ec"
 int d_obs_id = getObsId () ;
/* exec sql end declare section */
#line 276 "observation.ec"

	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "update observations set obs_start = now ( ) where obs_id = $1 ", 
	ECPGt_int,&(d_obs_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, ECPGt_EORT);}
#line 283 "observation.ec"

	if (sqlca.sqlcode != 0)
	{
		throw SqlError ();
	}
	{ ECPGtrans(__LINE__, NULL, "commit");}
#line 288 "observation.ec"

}

void Observation::endObservation (int _obs_state)
{
	/* exec sql begin declare section */
	    
	   
	
#line 294 "observation.ec"
 int d_obs_id = getObsId () ;
 
#line 295 "observation.ec"
 int d_obs_state = obs_state ;
/* exec sql end declare section */
#line 296 "observation.ec"


	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "update observations set obs_end = now ( ) , obs_state = $1  where obs_id = $2 ", 
	ECPGt_int,&(d_obs_state),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(d_obs_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, ECPGt_EORT);}
#line 305 "observation.ec"

	if (sqlca.sqlcode != 0)
	{
		{ ECPGtrans(__LINE__, NULL, "rollback");}
#line 308 "observation.ec"

		obs_id = -1;
		throw SqlError ();
	}
	{ ECPGtrans(__LINE__, NULL, "commit");}
#line 312 "observation.ec"

}

void Observation::printObsHeader (std::ostream & _os)
{
	if (!printHeader)
		return;
	std::ios_base::fmtflags old_settings = _os.flags ();
	int old_precision = _os.precision ();

	_os.setf (std::ios_base::fixed, std::ios_base::floatfield);
	_os.precision (2);
	_os << std::setw (5) << obs_id << SEP
		<< std::setw(6) << tar_id << SEP
		<< std::left << std::setw (20) << tar_name << std::right << SEP
		<< Timestamp (obs_slew) << SEP
		<< LibnovaRa (obs_ra) << SEP
		<< LibnovaDec (obs_dec) << SEP
		<< LibnovaDeg90 (obs_alt) << SEP
		<< LibnovaDeg (obs_az) << SEP
		<< std::setfill (' ');

	if (obs_start > 0)
		_os << std::setw (12) << TimeDiff (obs_slew, obs_start) << SEP;
	else
		_os << std::setw (12) << "not" << SEP;

	if (!isnan (obs_end))
		_os << std::setw (12) << TimeDiff (obs_start, obs_end) << SEP;
	else
		_os << std::setw (12) << "not" << SEP;

	_os << SEP
		<< std::setw (7) << getSlewSpeed() << SEP
		<< ObservationState (obs_state)
		<< std::endl;

	_os.flags (old_settings);
	_os.precision (old_precision);
}

void Observation::printCountsShort (std::ostream &_os)
{
	std::vector <Rts2Count>::iterator count_iter;
	if (counts.empty ())
	{
		return;
	}
	for (count_iter = counts.begin (); count_iter != counts.end (); count_iter++)
	{
		_os
			<< tar_id
			<< SEP << obs_id
			<< SEP << (*count_iter);
	}
}

void Observation::printCounts (std::ostream &_os)
{
	std::vector <Rts2Count>::iterator count_iter;
	if (counts.empty ())
	{
		_os << "      " << "--- no counts ---" << std::endl;
		return;
	}
	for (count_iter = counts.begin (); count_iter != counts.end (); count_iter++)
	{
		_os << "      " << (*count_iter);
	}
}

void Observation::printCountsSummary (std::ostream &_os)
{
	_os << "       Number of counts:" << counts.size () << std::endl;
}

double Observation::altitudeMerit (double _start, double _end)
{
	double minA, maxA;
	struct ln_hrz_posn hrz;
	getTarget ()->getMinMaxAlt (_start, _end, minA, maxA);

	getTarget ()->getAltAz (&hrz, getObsJDMid ());

	if (maxA == minA)
		return 1;

	if ((hrz.alt - minA) / (maxA - minA) > 1)
	{
		std::cout << "hrz.alt: " << hrz.alt
			<< " minA: " << minA
			<< " maxA: " << maxA
			<< " from " << LibnovaDate (_start)
			<< " to " << LibnovaDate (_end)
			<< " obs from " << LibnovaDate (getObsJDStart ())
			<< " to " << LibnovaDate (getObsJDEnd ())
			<< std::endl;
	}

	if (isnan (hrz.alt) || isnan (minA) || isnan (maxA))
	{
		std::cout << "nan hrz.alt: " << hrz.alt
			<< " minA: " << minA
			<< " maxA: " << maxA
			<< " from " << LibnovaDate (_start)
			<< " to " << LibnovaDate (_end)
			<< " obs from " << LibnovaDate (getObsJDStart ())
			<< " to " << LibnovaDate (getObsJDEnd ())
			<< std::endl;
	}

	return (hrz.alt - minA) / (maxA - minA);
}

int Observation::getUnprocessedCount ()
{
	/* exec sql begin declare section */
		    
		   
	
#line 429 "observation.ec"
 int db_obs_id = getObsId () ;
 
#line 430 "observation.ec"
 int db_count = 0 ;
/* exec sql end declare section */
#line 431 "observation.ec"


	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "select count ( * ) from images where obs_id = $1  and ( ( process_bitfield & 1 ) = 0 )", 
	ECPGt_int,&(db_obs_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, 
	ECPGt_int,&(db_count),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EORT);}
#line 442 "observation.ec"

	{ ECPGtrans(__LINE__, NULL, "rollback");}
#line 443 "observation.ec"


	return db_count;
}

int Observation::checkUnprocessedImages (rts2core::Block *master)
{
	int ret;
	load ();
	if (isnan (obs_end))
		return -1;
	// obs_end is not null - observation ends sucessfully
	// get unprocessed counts..
	ret = getUnprocessedCount ();
/*	if (ret == 0)
	{
		int count;
		Rts2TarUser tar_user = Rts2TarUser (getTargetId (), getTargetType ());
		std::string mails = tar_user.getUsers (SEND_ASTRO_OK, count);
		if (count == 0)
			return ret;
		std::ostringstream subject;
		subject << "TARGET #" << getTargetId ()
			<< ", OBSERVATION " << getObsId ()
			<< " ALL IMAGES PROCESSED";
		std::ostringstream os;
		os << *this;
		master->sendMailTo (subject.str().c_str(), os.str().c_str(), mails.c_str());
	} */
	return ret;
}

int Observation::getNumberOfImages ()
{
	loadImages ();
	if (imgset)
		return imgset->size ();
	return 0;
}

int Observation::getNumberOfGoodImages ()
{
	loadImages ();
	if (!imgset)
		return 0;
	std::vector <rts2image::Image *>::iterator img_iter;
	int ret = 0;
	for (img_iter = imgset->begin (); img_iter != imgset->end (); img_iter++)
	{
		if ((*img_iter)->haveOKAstrometry ())
			ret++;
	}
	return ret;
}

double Observation::getTimeOnSky ()
{
	loadImages ();
	if (!imgset)
		return 0;
	std::vector <rts2image::Image *>::iterator img_iter;
	double ret = 0;
	for (img_iter = imgset->begin (); img_iter != imgset->end (); img_iter++)
		ret += (*img_iter)->getExposureLength ();
	return ret;
}

int Observation::getFirstErrors (double &eRa, double &eDec, double &eRad)
{
	loadImages ();
	if (!imgset)
		return -1;

	std::vector <rts2image::Image *>::iterator img_iter;
	for (img_iter = imgset->begin (); img_iter != imgset->end (); img_iter++)
	{
		if (((*img_iter)->getError (eRa, eDec, eRad)) == 0)
		{
			return 0;
		}
	}
	return -1;
}

int Observation::getAverageErrors (double &eRa, double &eDec, double &eRad)
{
	loadImages ();
	if (!imgset)
		return -1;
	return imgset->getAverageErrors (eRa, eDec, eRad);
}

int Observation::getPrevPosition (struct ln_equ_posn &prevEqu, struct ln_hrz_posn &prevHrz)
{
	int ret;
	Observation prevObs = Observation (getObsId () - 1);
	ret = prevObs.load();
	if (ret)
		return ret;
	prevObs.getEqu (prevEqu);
	prevObs.getHrz (prevHrz);
	return 0;
}

double Observation::getPrevSeparation ()
{
	struct ln_equ_posn prevEqu, currEqu;
	struct ln_hrz_posn prevHrz;
	int ret;
	ret = getPrevPosition (prevEqu, prevHrz);
	if (ret)
		return rts2_nan ("f");

	getEqu (currEqu);

	return ln_get_angular_separation (&prevEqu, &currEqu);
}

double Observation::getSlewSpeed ()
{
	double prevSep = getPrevSeparation ();
	if (isnan (prevSep)
		|| isnan(obs_slew)
		|| isnan (obs_start)
		|| (obs_start - obs_slew) <= 0)
		return rts2_nan ("f");
	return prevSep / (obs_start - obs_slew);
}

double Observation::getSlewTime ()
{
	if (isnan (obs_slew) || isnan (obs_start))
		return rts2_nan ("f");
	return obs_start - obs_slew;
}

double Observation::getObsTime ()
{
	if (isnan (obs_start) || isnan (obs_end))
		return rts2_nan ("f");
	return obs_end - obs_slew;
}

void Observation::maskState (int newBits)
{
	/* exec sql begin declare section */
		   
		   
	
#line 589 "observation.ec"
 int db_obs_id = obs_id ;
 
#line 590 "observation.ec"
 int db_newBits = newBits ;
/* exec sql end declare section */
#line 591 "observation.ec"


	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "update observations set obs_state = obs_state | $1  where obs_id = $2 ", 
	ECPGt_int,&(db_newBits),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(db_obs_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, ECPGt_EORT);}
#line 598 "observation.ec"

	if (sqlca.sqlcode)
	{
		logStream (MESSAGE_ERROR) << "Observation::maskState: " << sqlca.sqlerrm.sqlerrmc << " (" << sqlca.sqlcode << ")" << sendLog;
		{ ECPGtrans(__LINE__, NULL, "rollback");}
#line 602 "observation.ec"

		return;
	}
	{ ECPGtrans(__LINE__, NULL, "commit");}
#line 605 "observation.ec"

	obs_state |= newBits;
}

void Observation::unmaskState (int newBits)
{
	/* exec sql begin declare section */
		   
		   
	
#line 612 "observation.ec"
 int db_obs_id = obs_id ;
 
#line 613 "observation.ec"
 int db_newBits = ~ newBits ;
/* exec sql end declare section */
#line 614 "observation.ec"


	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "update observations set obs_state = obs_state & $1  where obs_id = $2 ", 
	ECPGt_int,&(db_newBits),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(db_obs_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, ECPGt_EORT);}
#line 621 "observation.ec"

	if (sqlca.sqlcode)
	{
		logStream (MESSAGE_ERROR) << "Observation::unmaskState: " << sqlca.sqlerrm.sqlerrmc << " (" << sqlca.sqlcode << ")" << sendLog;
		{ ECPGtrans(__LINE__, NULL, "rollback");}
#line 625 "observation.ec"

		return;
	}
	{ ECPGtrans(__LINE__, NULL, "commit");}
#line 628 "observation.ec"

	obs_state |= newBits;
}
