/* Processed by ecpg (4.7.0) */
/* These include files are added by the preprocessor */
#include <ecpglib.h>
#include <ecpgerrno.h>
#include <sqlca.h>
/* End of automatic include section */

#line 1 "observationset.ec"
/* 
 * Observation class set.
 * Copyright (C) 2005-2010 Petr Kubanek <petr@kubanek.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "imgdisplay.h"

#include "rts2db/observationset.h"
#include "rts2db/sqlerror.h"
#include "rts2db/target.h"

#include <algorithm>
#include <sstream>

#include "libnova_cpp.h"
#include "configuration.h"

using namespace rts2db;

ObservationSet::ObservationSet (void)
{
	images = 0;
	counts = 0;
	collocated = false;
	successNum = 0;
	failedNum = 0;

	allNum = 0;
	goodNum = 0;

	firstNum = 0;

	errFirstRa = 0;
	errFirstDec = 0;
	errFirstRad = 0;

	errAvgRa = 0;
	errAvgDec = 0;
	errAvgRad = 0;
}

ObservationSet::~ObservationSet (void)
{
	clear ();
}

void ObservationSet::loadTarget (int _tar_id, const time_t * start_t, const time_t * end_t)
{
	std::ostringstream os;
	os << "observations.tar_id = " << _tar_id;
	if (start_t)
	{
		os << " AND observations.obs_slew >= to_timestamp (" << *start_t<< ")";
	}
	if (end_t)
	{
	 	os << " AND ((observations.obs_slew <= to_timestamp ("
		<< *end_t
		<< ") AND (observations.obs_end is NULL OR observations.obs_end < to_timestamp ("
		<< *end_t
		<< "))";
	}
	load (os.str());
}

void ObservationSet::loadTime (const time_t * start_t, const time_t * end_t)
{
	std::ostringstream os;
	if (start_t)
	{
		os << "observations.obs_slew >= to_timestamp (" << *start_t << ")";
	}
	if (end_t)
	{
		if (start_t)
			os << " AND ";
		os << "((observations.obs_slew <= to_timestamp ("
			<< *end_t
			<< ") AND observations.obs_end is NULL) OR observations.obs_end < to_timestamp ("
			<< *end_t
			<< "))";
	}
	load (os.str());
}

void ObservationSet::loadType (char type_id, int state_mask, bool inv)
{
	std::ostringstream os;
	os << "(targets.type_id = "
		<< type_id
		<< ") and ((observations.obs_state & "
		<< state_mask
		<< ") ";
	if (inv)
		os << "!";
	os << "= "
		<< state_mask
		<< ")";
	load (os.str());
}

void ObservationSet::loadRadius (struct ln_equ_posn * position, double radius)
{
	std::ostringstream os;
	os << "ln_angular_separation (observations.obs_ra, observations.obs_dec, "
		<< position->ra << ", "
		<< position->dec << ") < "
		<< radius;
	load (os.str());
}

void ObservationSet::load (std::string in_where)
{
	/* exec sql begin declare section */
	 

	// cannot use TARGET_NAME_LEN, as it does not work with some ecpg veriosn
	 
	 
	 
	 
	 
	 
	 
	 
	 
	 
	 

	 
	 
	 
	 
	 
	 
	 
	 
	 
	 
	
#line 129 "observationset.ec"
 char * stmp_c ;
 
#line 132 "observationset.ec"
  struct varchar_1  { int len; char arr[ 150 ]; }  db_tar_name ;
 
#line 133 "observationset.ec"
 int db_tar_id ;
 
#line 134 "observationset.ec"
 int db_obs_id ;
 
#line 135 "observationset.ec"
 double db_obs_ra ;
 
#line 136 "observationset.ec"
 double db_obs_dec ;
 
#line 137 "observationset.ec"
 double db_obs_alt ;
 
#line 138 "observationset.ec"
 double db_obs_az ;
 
#line 139 "observationset.ec"
 double db_obs_slew ;
 
#line 140 "observationset.ec"
 double db_obs_start ;
 
#line 141 "observationset.ec"
 int db_obs_state ;
 
#line 142 "observationset.ec"
 double db_obs_end ;
 
#line 144 "observationset.ec"
 int db_tar_ind ;
 
#line 145 "observationset.ec"
 char db_tar_type ;
 
#line 146 "observationset.ec"
 int db_obs_ra_ind ;
 
#line 147 "observationset.ec"
 int db_obs_dec_ind ;
 
#line 148 "observationset.ec"
 int db_obs_alt_ind ;
 
#line 149 "observationset.ec"
 int db_obs_az_ind ;
 
#line 150 "observationset.ec"
 int db_obs_slew_ind ;
 
#line 151 "observationset.ec"
 int db_obs_start_ind ;
 
#line 152 "observationset.ec"
 int db_obs_state_ind ;
 
#line 153 "observationset.ec"
 int db_obs_end_ind ;
/* exec sql end declare section */
#line 154 "observationset.ec"


	std::ostringstream _os;
	_os << 
		"SELECT "
		"tar_name,"
		"observations.tar_id,"
		"type_id,"
		"obs_id,"
		"obs_ra,"
		"obs_dec,"
		"obs_alt,"
		"obs_az,"
		"EXTRACT (EPOCH FROM obs_slew),"
		"EXTRACT (EPOCH FROM obs_start),"
		"obs_state,"
		"EXTRACT (EPOCH FROM obs_end)"
		" FROM "
		"observations, targets"
		" WHERE "
		"observations.tar_id = targets.tar_id "
		"AND " << in_where << 
		" ORDER BY obs_id ASC;";
	stmp_c  = new char[_os.str ().length () + 1];
	strcpy (stmp_c, _os.str ().c_str ());

	{ ECPGprepare(__LINE__, NULL, 0, "obs_stmp", stmp_c);}
#line 180 "observationset.ec"


	delete[] stmp_c;

	/* declare obs_cur_timestamps cursor for $1 */
#line 184 "observationset.ec"


	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "declare obs_cur_timestamps cursor for $1", 
	ECPGt_char_variable,(ECPGprepared_statement(NULL, "obs_stmp", __LINE__)),(long)1,(long)1,(1)*sizeof(char), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, ECPGt_EORT);}
#line 186 "observationset.ec"

	while (1)
	{
		{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "fetch next from obs_cur_timestamps", ECPGt_EOIT, 
	ECPGt_varchar,&(db_tar_name),(long)150,(long)1,sizeof(struct varchar_1), 
	ECPGt_int,&(db_tar_ind),(long)1,(long)1,sizeof(int), 
	ECPGt_int,&(db_tar_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_char,&(db_tar_type),(long)1,(long)1,(1)*sizeof(char), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(db_obs_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_obs_ra),(long)1,(long)1,sizeof(double), 
	ECPGt_int,&(db_obs_ra_ind),(long)1,(long)1,sizeof(int), 
	ECPGt_double,&(db_obs_dec),(long)1,(long)1,sizeof(double), 
	ECPGt_int,&(db_obs_dec_ind),(long)1,(long)1,sizeof(int), 
	ECPGt_double,&(db_obs_alt),(long)1,(long)1,sizeof(double), 
	ECPGt_int,&(db_obs_alt_ind),(long)1,(long)1,sizeof(int), 
	ECPGt_double,&(db_obs_az),(long)1,(long)1,sizeof(double), 
	ECPGt_int,&(db_obs_az_ind),(long)1,(long)1,sizeof(int), 
	ECPGt_double,&(db_obs_slew),(long)1,(long)1,sizeof(double), 
	ECPGt_int,&(db_obs_slew_ind),(long)1,(long)1,sizeof(int), 
	ECPGt_double,&(db_obs_start),(long)1,(long)1,sizeof(double), 
	ECPGt_int,&(db_obs_start_ind),(long)1,(long)1,sizeof(int), 
	ECPGt_int,&(db_obs_state),(long)1,(long)1,sizeof(int), 
	ECPGt_int,&(db_obs_state_ind),(long)1,(long)1,sizeof(int), 
	ECPGt_double,&(db_obs_end),(long)1,(long)1,sizeof(double), 
	ECPGt_int,&(db_obs_end_ind),(long)1,(long)1,sizeof(int), ECPGt_EORT);}
#line 201 "observationset.ec"

		if (sqlca.sqlcode)
			break;
		if (db_tar_ind < 0)
			db_tar_name.arr[0] = '\0';
		if (db_obs_ra_ind < 0)
			db_obs_ra = rts2_nan("f");
		if (db_obs_dec_ind < 0)
			db_obs_dec = rts2_nan("f");
		if (db_obs_alt_ind < 0)
			db_obs_alt = rts2_nan("f");
		if (db_obs_az_ind < 0)
			db_obs_az = rts2_nan("f");
		if (db_obs_slew_ind < 0)
			db_obs_slew = rts2_nan("f");
		if (db_obs_start_ind < 0)
			db_obs_start = rts2_nan("f");
		if (db_obs_state_ind < 0)
			db_obs_state = 0;
		if (db_obs_end_ind < 0)
			db_obs_end = rts2_nan("f");

		// add new observations to vector
		Observation obs = Observation (db_tar_id, db_tar_name.arr, db_tar_type, db_obs_id, db_obs_ra, db_obs_dec, db_obs_alt,
			db_obs_az, db_obs_slew, db_obs_start, db_obs_state, db_obs_end);
		push_back (obs);
		if (db_obs_state & OBS_BIT_STARTED)
		{
			if (db_obs_state & OBS_BIT_ACQUSITION_FAI)
				failedNum++;
			else
				successNum++;
		}
	}
	if (sqlca.sqlcode != ECPG_NOT_FOUND)
	{
		{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "close obs_cur_timestamps", ECPGt_EOIT, ECPGt_EORT);}
#line 237 "observationset.ec"

		{ ECPGtrans(__LINE__, NULL, "rollback");}
#line 238 "observationset.ec"

		throw SqlError ();
	}
	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "close obs_cur_timestamps", ECPGt_EOIT, ECPGt_EORT);}
#line 241 "observationset.ec"

	{ ECPGtrans(__LINE__, NULL, "rollback");}
#line 242 "observationset.ec"

}

int ObservationSet::computeStatistics ()
{
	int anum = 0;

	std::vector <Observation>::iterator obs_iter;
	allNum = 0;
	goodNum = 0;
	firstNum = 0;

	errFirstRa = 0;
	errFirstDec = 0;
	errFirstRad = 0;

	errAvgRa = 0;
	errAvgDec = 0;
	errAvgRad = 0;

	for (obs_iter = begin (); obs_iter != end (); obs_iter++)
	{
		double efRa, efDec, efRad;
		double eaRa, eaDec, eaRad;
		int ret;
		allNum += (*obs_iter).getNumberOfImages ();
		goodNum += (*obs_iter).getNumberOfGoodImages ();
		ret = (*obs_iter).getFirstErrors (efRa, efDec, efRad);
		if (!ret)
		{
			errFirstRa += efRa;
			errFirstDec += efDec;
			errFirstRad += efRad;
			firstNum++;
		}
		ret = (*obs_iter).getAverageErrors (eaRa, eaDec, eaRad);
		if (ret > 0)
		{
			errAvgRa += eaRa;
			errAvgDec += eaDec;
			errAvgRad += eaRad;
			anum ++;
		}
	}

	errFirstRa /= firstNum;
	errFirstDec /= firstNum;
	errFirstRad /= firstNum;

	errAvgRa /= anum;
	errAvgDec /= anum;
	errAvgRad /= anum;

	return 0;
}

void ObservationSet::printStatistics (std::ostream & _os)
{
	computeStatistics ();
	int success = getSuccess ();
	int failed = getFailed ();
	int prec = _os.precision (2);
	_os << "Number of observations: " << size () << std::endl
		<< "Succesfully ended:" << success << std::endl
		<< "Failed:" << failed << std::endl
		<< "Number of images:" << allNum
		<< " with astrometry:" << goodNum
		<< " without astrometry:" << (allNum - goodNum);
	if (allNum > 0)
	{
		_os << " (" << (((double)(goodNum * 100)) / allNum) << "%)";
	}
	_os << std::endl;
	if (goodNum > 0)
	{
		_os
			<< "First images : " << firstNum << " from " << size ()
			<< "(" << (size () > 0 ? 100 * firstNum / size () : rts2_nan ("f")) << "%)" << std::endl
			<< "First images errors: ra " << LibnovaDegArcMin (errFirstRa)
			<< " dec: " << LibnovaDegArcMin (errFirstDec)
			<< " radius: " << LibnovaDegArcMin (errFirstRad) << std::endl
			<< "Average error ra: " << LibnovaDegArcMin (errAvgRa)
			<< " dec: " << LibnovaDegArcMin (errAvgDec)
			<< " radius: " << LibnovaDegArcMin (errAvgRad) << std::endl;
	}
	_os.precision (prec);
}

std::map <int, int> ObservationSet::getTargetObservations ()
{
	std::map <int, int> ret;
	for (ObservationSet::iterator iter = begin (); iter != end (); iter++)
	{
		ret[(*iter).getTargetId ()]++;
	}
	return ret;
}

double ObservationSet::altitudeMerit ()
{
	double altMerit = 0;
	int s = 0;
	for (ObservationSet::iterator iter = begin (); iter != end (); iter++)
	{
		double am = (*iter).altitudeMerit (getJDStart (), getJDEnd ());
		if (isnan (am))
			continue;
		altMerit += am;
		s++;
	}
	altMerit /= s;
	return altMerit;
}

double ObservationSet::distanceMerit ()
{
	double distMerit = 0;

	unsigned int s = 0;

	ObservationSet::iterator iter1 = begin ();
	ObservationSet::iterator iter2 = begin () + 1;

	for (; iter2 != end (); iter1++, iter2++)
	{
		struct ln_equ_posn pos1, pos2;
		if ((*iter1).getEndPosition (pos1) == 0 && (*iter2).getStartPosition (pos2) == 0)
		{
			distMerit += ln_get_angular_separation (&pos1, &pos2);
			s++;
		}
	}
	if (distMerit == 0)
	{
		distMerit = 1;
	}
	else
	{
		distMerit = distMerit / s;
	}

	return distMerit;
}

std::ostream & ObservationSet::print (std::ostream &_os)
{
	// list of target ids we already printed
	std::vector <int> processedTargets;

	std::vector <Observation>::iterator obs_iter;

	if (!isCollocated ())
		_os << "Observations list follow:" << std::endl;

	for (obs_iter = begin (); obs_iter != end (); obs_iter++)
	{
		if (isCollocated ())
		{
			int tar_id = (*obs_iter).getTargetId ();
			if (find (processedTargets.begin (), processedTargets.end (), tar_id) == processedTargets.end ())
			{
				processedTargets.push_back (tar_id);
				int totalImages = 0;
				int goodImages = 0;
				double totalObsTime = 0;
				double totalSlewTime = 0;
				int obsNum = 0;
				// compute target image statitics..
				for (std::vector <Observation>::iterator obs_iter2 = obs_iter; obs_iter2 != end (); obs_iter2 ++)
				{
					if ((*obs_iter2).getTargetId () == tar_id)
					{
						totalImages += (*obs_iter2).getNumberOfImages ();
						goodImages += (*obs_iter2).getNumberOfGoodImages ();

						totalSlewTime += (*obs_iter2).getSlewTime ();
						totalObsTime += (*obs_iter2).getObsTime ();

						obsNum++;
					}
				}

				_os << "  " << std::setw (6) << (*obs_iter).getTargetId () 
					<< SEP << std::setw (20) << (*obs_iter).getTargetName ()
					<< SEP << std::setw (3) << obsNum
					<< SEP << TimeDiff (totalObsTime + totalSlewTime) 
					<< SEP << TimeDiff (totalObsTime)
					<< SEP << TimeDiff (totalSlewTime)
					<< SEP << std::setw (5) << totalImages
					<< SEP << std::setw (5) << goodImages
					<< std::endl;

				// print record that we found the observation
				// find all observations of target
				for (std::vector <Observation>::iterator obs_iter2 = obs_iter; obs_iter2 != end (); obs_iter2 ++)
				{
					if ((*obs_iter2).getTargetId () == tar_id)
					{
						if (getPrintImages () == 0 && getPrintCounts () == 0)
						{
							// print time of observation
							_os << Timestamp ((*obs_iter2).getObsStart ()) << SEP;
						}
						else
						{
							(*obs_iter2).setPrintImages (getPrintImages ());
							(*obs_iter2).setPrintCounts (getPrintCounts ());
							(*obs_iter2).setPrintHeader (false);
							_os << (*obs_iter2);
						}
					}
				}
				if (getPrintImages () == 0 && getPrintCounts () == 0)
				{
					_os << std::endl;
				}
			}
		}
		else
		{
			obs_iter->setPrintImages (getPrintImages ());
			obs_iter->setPrintCounts (getPrintCounts ());
			_os << (*obs_iter);
		}
	}
	return _os;
}

double ObservationSet::getNextCtime ()
{
	if (timeLogIter != end ())
		return timeLogIter->getObsSlew ();
	return rts2_nan ("f");
}

void ObservationSet::printUntil (double time, std::ostream &os)
{
	while (timeLogIter != end () && timeLogIter->getObsSlew () <= time)
	{
		timeLogIter->setPrintImages (getPrintImages ());
		timeLogIter->setPrintCounts (getPrintCounts ());
		os << (*timeLogIter);
	
		timeLogIter++;
	}
}

void ObservationSetDate::load (int year, int month, int day, int hour, int minutes)
{
	/* exec sql begin declare section */
	 
	 
	 
	 
	 
	 
	 
	 
	
#line 492 "observationset.ec"
 int d_value1 ;
 
#line 493 "observationset.ec"
 int d_value2 ;
 
#line 494 "observationset.ec"
 int d_value2g ;
 
#line 495 "observationset.ec"
 int d_c ;
 
#line 496 "observationset.ec"
 int d_i ;
 
#line 497 "observationset.ec"
 int d_gi ;
 
#line 498 "observationset.ec"
 double d_tt ;
 
#line 499 "observationset.ec"
 char * stmp_c ;
/* exec sql end declare section */
#line 500 "observationset.ec"


	const char *group_by;
	std::ostringstream _where;

	double lng = rts2core::Configuration::instance ()->getObserver ()->lng;

	if (year == -1)
	{
		group_by = "year";
	}
	else
	{
		_where << "EXTRACT (year FROM to_night (obs_slew, " << lng << ")) = " << year;
		if (month == -1)
		{
			group_by = "month";
		}
		else
		{
			_where << " AND EXTRACT (month FROM to_night (obs_slew, " << lng << ")) = " << month;
			if (day == -1)
			{
				group_by = "day";
			}
			else
			{
				_where << " AND EXTRACT (day FROM to_night (obs_slew, " << lng << ")) = " << day;
				if (hour == -1)
				{
				 	group_by = "hour";
				}
				else
				{
					_where << " AND EXTRACT (hour FROM obs_slew) = " << hour;
					if (minutes == -1)
					{
					 	group_by = "minute";
					}
					else
					{
						_where << " AND EXTRACT (minutes FROM obs_slew) = " << minutes;
						group_by = "second";
					}
				}
			}
		}
	}

	std::ostringstream _os;
	_os << "SELECT EXTRACT (" << group_by << " FROM to_night (obs_slew, " << lng << ")) as value, count (observations.*) as c FROM observations";
	
	if (_where.str ().length () > 0)
		_os << " WHERE " << _where.str ();

	_os << " GROUP BY value ORDER BY value;";

	stmp_c = new char[_os.str ().length () + 1];
	memcpy (stmp_c, _os.str().c_str(), _os.str ().length () + 1);
	{ ECPGprepare(__LINE__, NULL, 0, "obsdate_stmp", stmp_c);}
#line 559 "observationset.ec"


	std::cout << _os.str() << std::endl;

	delete[] stmp_c;

	/* declare obsdate_cur cursor for $1 */
#line 565 "observationset.ec"


	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "declare obsdate_cur cursor for $1", 
	ECPGt_char_variable,(ECPGprepared_statement(NULL, "obsdate_stmp", __LINE__)),(long)1,(long)1,(1)*sizeof(char), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, ECPGt_EORT);}
#line 567 "observationset.ec"


	std::ostringstream _osi;
	std::ostringstream _osig;
	_osi << "SELECT EXTRACT (" << group_by << " FROM to_night (obs_slew, " << lng << ")) as value, count (im_all.*) as i, sum(im_all.img_exposure) as tt FROM observations, images im_all WHERE observations.obs_id = im_all.obs_id";
	
	if (_where.str ().length () > 0)
		_osi << " and " << _where.str ();

	_osig << _osi.str ();
	_osig << " AND im_all.process_bitfield & 8 = 8 GROUP BY value ORDER BY value;";

	_osi << " GROUP BY value ORDER BY value;";

	stmp_c = new char[_osi.str ().length () + 1];
	memcpy (stmp_c, _osi.str().c_str(), _osi.str ().length () + 1);
	{ ECPGprepare(__LINE__, NULL, 0, "obsdateimg_stmp", stmp_c);}
#line 583 "observationset.ec"


	delete[] stmp_c;

	/* declare obsdateimg_cur cursor for $1 */
#line 587 "observationset.ec"


	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "declare obsdateimg_cur cursor for $1", 
	ECPGt_char_variable,(ECPGprepared_statement(NULL, "obsdateimg_stmp", __LINE__)),(long)1,(long)1,(1)*sizeof(char), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, ECPGt_EORT);}
#line 589 "observationset.ec"


	stmp_c = new char[_osig.str ().length () + 1];
	memcpy (stmp_c, _osig.str().c_str(), _osig.str ().length () + 1);
	{ ECPGprepare(__LINE__, NULL, 0, "obsdateimggood_stmp", stmp_c);}
#line 593 "observationset.ec"


	delete[] stmp_c;

	/* declare obsdateimggood_cur cursor for $1 */
#line 597 "observationset.ec"


	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "declare obsdateimggood_cur cursor for $1", 
	ECPGt_char_variable,(ECPGprepared_statement(NULL, "obsdateimggood_stmp", __LINE__)),(long)1,(long)1,(1)*sizeof(char), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, ECPGt_EORT);}
#line 599 "observationset.ec"


	d_value2 = -1;
	d_value2g = -1;

	while (1)
	{
		{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "fetch next from obsdate_cur", ECPGt_EOIT, 
	ECPGt_int,&(d_value1),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(d_c),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EORT);}
#line 608 "observationset.ec"

		if (sqlca.sqlcode)
			break;
		if (d_value2 < 0)
		{
			{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "fetch next from obsdateimg_cur", ECPGt_EOIT, 
	ECPGt_int,&(d_value2),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(d_i),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(d_tt),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EORT);}
#line 616 "observationset.ec"

			if (sqlca.sqlcode)
			{
				if (sqlca.sqlcode == ECPG_NOT_FOUND)
				{
					(*this)[d_value1] = DateStatistics (d_c);
				}
				else
				{
				  	break;
				}
			}
		}
		else
		{
			d_i = 0;
			d_tt = NAN;
		}

		if (d_value2g < 0)
		{
			{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "fetch next from obsdateimggood_cur", ECPGt_EOIT, 
	ECPGt_int,&(d_value2g),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(d_gi),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EORT);}
#line 639 "observationset.ec"

			if (sqlca.sqlcode)
			{
				if (sqlca.sqlcode == ECPG_NOT_FOUND)
				{
					d_gi = 0;
				}
				else
				{
					break;
				}
			}
		}
		else
		{
			d_gi = 0;
			d_tt = NAN;
		}

		if (d_value1 == d_value2)
		{
			(*this)[d_value1] = DateStatistics (d_c, d_i, d_gi, d_tt);
			d_value2 = -1;
			if (d_value2 == d_value2g)
				d_value2g = -1;
		}
		else
		{
			(*this)[d_value1] = DateStatistics (d_c);
		}
	}
	if (sqlca.sqlcode != ECPG_NOT_FOUND)
	{
		{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "close obsdate_cur", ECPGt_EOIT, ECPGt_EORT);}
#line 672 "observationset.ec"

		{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "close obsdateimg_cur", ECPGt_EOIT, ECPGt_EORT);}
#line 673 "observationset.ec"

		{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "close obsdateimggood_cur", ECPGt_EOIT, ECPGt_EORT);}
#line 674 "observationset.ec"

		{ ECPGtrans(__LINE__, NULL, "rollback");}
#line 675 "observationset.ec"


		throw SqlError ();
	}

	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "close obsdate_cur", ECPGt_EOIT, ECPGt_EORT);}
#line 680 "observationset.ec"

	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "close obsdateimg_cur", ECPGt_EOIT, ECPGt_EORT);}
#line 681 "observationset.ec"

	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "close obsdateimggood_cur", ECPGt_EOIT, ECPGt_EORT);}
#line 682 "observationset.ec"

	{ ECPGtrans(__LINE__, NULL, "rollback");}
#line 683 "observationset.ec"

}
