/* Processed by ecpg (4.7.0) */
/* These include files are added by the preprocessor */
#include <ecpglib.h>
#include <ecpgerrno.h>
#include <sqlca.h>
/* End of automatic include section */

#line 1 "target.ec"
/* 
 * Target class.
 * Copyright (C) 2003-2010 Petr Kubanek <petr@kubanek.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "rts2db/target_auger.h"
#include "rts2db/targetell.h"
#include "rts2targetplanet.h"
#include "targetgrb.h"

#include "rts2db/constraints.h"
#include "rts2db/target.h"
#include "rts2db/observation.h"
#include "rts2db/observationset.h"
#include "rts2db/targetset.h"
#include "rts2db/sqlerror.h"
#include "rts2db/simbadtarget.h"

#include "infoval.h"
#include "app.h"
#include "configuration.h"
#include "libnova_cpp.h"
#include "timestamp.h"

#include <sstream>
#include <iomanip>

#include <sys/types.h>
#include <sys/stat.h>


#line 1 "/usr/include/sqlca.h"
#ifndef POSTGRES_SQLCA_H
#define POSTGRES_SQLCA_H

#ifndef PGDLLIMPORT
#if  defined(WIN32) || defined(__CYGWIN__)
#define PGDLLIMPORT __declspec (dllimport)
#else
#define PGDLLIMPORT
#endif   /* __CYGWIN__ */
#endif   /* PGDLLIMPORT */

#define SQLERRMC_LEN	150

#ifdef __cplusplus
extern		"C"
{
#endif

struct sqlca_t
{
	char		sqlcaid[8];
	long		sqlabc;
	long		sqlcode;
	struct
	{
		int			sqlerrml;
		char		sqlerrmc[SQLERRMC_LEN];
	}			sqlerrm;
	char		sqlerrp[8];
	long		sqlerrd[6];
	/* Element 0: empty						*/
	/* 1: OID of processed tuple if applicable			*/
	/* 2: number of rows processed				*/
	/* after an INSERT, UPDATE or				*/
	/* DELETE statement					*/
	/* 3: empty						*/
	/* 4: empty						*/
	/* 5: empty						*/
	char		sqlwarn[8];
	/* Element 0: set to 'W' if at least one other is 'W'	*/
	/* 1: if 'W' at least one character string		*/
	/* value was truncated when it was			*/
	/* stored into a host variable.				*/

	/*
	 * 2: if 'W' a (hopefully) non-fatal notice occurred
	 */	/* 3: empty */
	/* 4: empty						*/
	/* 5: empty						*/
	/* 6: empty						*/
	/* 7: empty						*/

	char		sqlstate[5];
};

struct sqlca_t *ECPGget_sqlca(void);

#ifndef POSTGRES_ECPG_INTERNAL
#define sqlca (*ECPGget_sqlca())
#endif

#ifdef __cplusplus
}
#endif

#endif

#line 45 "target.ec"


using namespace rts2db;

void Target::logMsgDb (const char *message, messageType_t msgType)
{
	logStream (msgType) << "SQL error: " << sqlca.sqlcode << " " <<
		sqlca.sqlerrm.sqlerrmc << " (at " << message << ")" << sendLog;
}

void Target::getTargetSubject (std::string &subj)
{
	std::ostringstream _os;
	_os
		<< getTargetName ()
		<< " #" << getObsTargetID () << " (" << getTargetID () << ")"
		<< " " << getTargetType ();
	subj = _os.str();
}

void Target::addWatch (const char *filename)
{
	if (watchConn == NULL)
		return;
	int ret = watchConn->addWatch (filename);
	if (ret < 0)
		logStream (MESSAGE_ERROR) << "cannot add " << filename << " to watched files: " << strerror (errno) << sendLog;
	watchIDs.push_back (ret);
}

void Target::writeAirmass (std::ostream & _os, double jd)
{
	double am = getAirmass (jd);
	if (am > 9)
		_os << "nan ";
	else
		_os << std::setw (3) << getAirmass (jd);
}

void Target::printAltTable (std::ostream & _os, double jd_start, double h_start, double h_end, double h_step, bool header)
{
	double i;
	struct ln_hrz_posn hrz;
	struct ln_equ_posn pos;
	double jd;

	int old_precison = 0;
	std::ios_base::fmtflags old_settings = std::ios_base::skipws;

	jd_start += h_start / 24.0;

	if (header)
	{
		old_precison = _os.precision (0);
		old_settings = _os.flags ();
		_os.setf (std::ios_base::fixed, std::ios_base::floatfield);
		
		_os << "H   ";

		char old_fill = _os.fill ('0');
		for (i = h_start; i <= h_end; i+=h_step)
		{
			_os << "  " << std::setw (2) << i;
		}
		_os.fill (old_fill);
		_os << std::endl;
	}

	// print alt + az + airmass or bonus
	std::ostringstream _os2;
	std::ostringstream _os3;

	if (header)
	{
		_os << "ALT ";
		_os2 << "AZ  ";
		_os2.precision (0);
		_os2.setf (std::ios_base::fixed, std::ios_base::floatfield);
		_os3 << "AIR ";
		_os3.precision (1);
		_os3.setf (std::ios_base::fixed, std::ios_base::floatfield);
	}

	jd = jd_start;
	for (i = h_start; i <= h_end; i+=h_step, jd += h_step/24.0)
	{
		getAltAz (&hrz, jd);
		_os << " " << std::setw (3) << hrz.alt;
		_os2 << " " << std::setw (3) << hrz.az;
		if (header)
		{
			_os3 << " ";
			writeAirmass (_os3, jd);
		}
		else
		{
			_os3 << " " << getBonus (jd);
		}
	}
	if (header)
	{
		_os
			<< std::endl
			<< _os2.str ()
			<< std::endl
			<< _os3.str ()
			<< std::endl;
	}
	else
	{
		_os << _os2.str () << _os3.str ();
		return;
	}

	// print lunar distances and airmass
	if (header)
	{
		_os << "LD  ";
	}
	jd = jd_start;
	for (i = h_start; i <= h_end; i+=h_step, jd += h_step/24.0)
	{
		_os << " " << std::setw(3) << getLunarDistance (jd);
	}
	if (header)
	{
		_os << std::endl;
	}

	// print solar distance
	if (header)
	{
		_os << "SD  ";
	}
	jd = jd_start;
	for (i = h_start; i <= h_end; i+=h_step, jd += h_step/24.0)
	{
		_os << " " << std::setw(3) << getSolarDistance (jd);
	}
	if (header)
	{
		_os << std::endl;
	}

	// print sun position
	std::ostringstream _os4, os_lal, os_laz;
	
	// lunar position

	if (header)
	{
		_os << "SAL ";
		_os4 << "SAZ ";
		_os4.precision (0);
		_os4.setf (std::ios_base::fixed, std::ios_base::floatfield);

		os_lal << "LAL ";
		os_laz << "LAZ ";
		os_lal.precision (0);
		os_lal.setf (std::ios_base::fixed, std::ios_base::floatfield);
		os_laz.precision (0);
		os_laz.setf (std::ios_base::fixed, std::ios_base::floatfield);

	}

	jd = jd_start;
	for (i = h_start; i <= h_end; i+=h_step, jd += h_step/24.0)
	{
		ln_get_solar_equ_coords (jd, &pos);
		ln_get_hrz_from_equ (&pos, getObserver (), jd, &hrz);
		_os << " " << std::setw (3) << hrz.alt;
		_os4 << " " << std::setw (3) << hrz.az;

		ln_get_lunar_equ_coords (jd, &pos);
		ln_get_hrz_from_equ (&pos, getObserver (), jd, &hrz);
		os_lal << " " << std::setw (3) << hrz.alt;
		os_laz << " " << std::setw (3) << hrz.az;
	}

	if (header)
	{
		_os << std::endl << _os4.str () << std::endl << os_lal.str () << std::endl << os_laz.str () << std::endl;
	}
	else
	{
		_os << _os4.str () << os_lal.str () << os_laz.str ();
	}

	if (header)
	{
		_os.setf (old_settings);
		_os.precision (old_precison);
	}
}

void Target::printAltTable (std::ostream & _os, double JD)
{
	double jd_start = ((int) JD) - 0.5;
	_os
		<< std::endl
		<< "** HOUR, ALTITUDE, AZIMUTH, MOON DISTANCE, SUN POSITION table from "
		<< TimeJD (jd_start)
		<< " **"
		<< std::endl
		<< std::endl;

	printAltTable (_os, jd_start, -1, 11);
	_os << std::endl;
	printAltTable (_os, jd_start, 12, 24);
}

void Target::printAltTableSingleCol (std::ostream & _os, double jd_start, double i, double step)
{
	printAltTable (_os, jd_start, i, i+step, step * 2.0, false);
}

Target::Target (int in_tar_id, struct ln_lnlat_posn *in_obs):Rts2Target ()
{
	rts2core::Configuration *config = rts2core::Configuration::instance ();

	watchConn = NULL;

	observer = in_obs;

	config->getDouble ("observatory", "min_alt", minObsAlt, 0);

	observation = NULL;

	target_id = in_tar_id;
	obs_target_id = -1;
	target_type = TYPE_UNKNOW;
	target_name = NULL;
	target_comment = NULL;

	startCalledNum = 0;

	airmassScale = 750.0;

	constraintsLoaded = CONSTRAINTS_NONE;
	
	constraintFile = NULL;
	constraints = NULL;

	groupConstraintFile = NULL;

	observationStart = -1;

	satisfiedFrom = NAN;
	satisfiedTo = NAN;
	satisfiedProbedUntil = NAN;
}

Target::Target ()
{
	rts2core::Configuration *config;
	config = rts2core::Configuration::instance ();

	watchConn = NULL;

	observer = config->getObserver ();

	config->getDouble ("observatory", "min_alt", minObsAlt, 0);

	observation = NULL;

	target_id = -1;
	obs_target_id = -1;
	target_type = TYPE_UNKNOW;
	target_name = NULL;
	target_comment = NULL;

	startCalledNum = 0;

	airmassScale = 750.0;

	constraintsLoaded = CONSTRAINTS_NONE;

	constraintFile = NULL;
	constraints = NULL;

	groupConstraintFile = NULL;

	observationStart = -1;

	satisfiedFrom = NAN;
	satisfiedTo = NAN;
	satisfiedProbedUntil = NAN;

	tar_priority = 0;
	tar_bonus = rts2_nan ("f");
	tar_bonus_time = 0;
	tar_next_observable = 0;

	config->getFloat ("newtarget", "priority", tar_priority, 0);
	setTargetEnabled (config->getBoolean ("newtarget", "enabled", false), false);
}

Target::~Target (void)
{
	endObservation (-1);
	delete[] target_name;
	delete[] target_comment;
	delete observation;
	delete constraints;
	delete[] constraintFile;
}

void Target::load ()
{
	loadTarget (getObsTargetID ());
}

void Target::loadTarget (int in_tar_id)
{
	/* exec sql begin declare section */
	// cannot use TARGET_NAME_LEN, as some versions of ecpg complains about it
	 
	 
	 
	 
	 
	 
	 
	 
	 
	 
	 
	 
	   
	
#line 361 "target.ec"
  struct varchar_1  { int len; char arr[ 150 ]; }  d_tar_name ;
 
#line 362 "target.ec"
  struct varchar_2  { int len; char arr[ 2000 ]; }  d_tar_info ;
 
#line 363 "target.ec"
 int d_tar_info_ind ;
 
#line 364 "target.ec"
 float d_tar_priority ;
 
#line 365 "target.ec"
 int d_tar_priority_ind ;
 
#line 366 "target.ec"
 float d_tar_bonus ;
 
#line 367 "target.ec"
 int d_tar_bonus_ind ;
 
#line 368 "target.ec"
 long d_tar_bonus_time ;
 
#line 369 "target.ec"
 int d_tar_bonus_time_ind ;
 
#line 370 "target.ec"
 long d_tar_next_observable ;
 
#line 371 "target.ec"
 int d_tar_next_observable_ind ;
 
#line 372 "target.ec"
 bool d_tar_enabled ;
 
#line 373 "target.ec"
 int db_tar_id = in_tar_id ;
/* exec sql end declare section */
#line 374 "target.ec"


	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "select tar_name , tar_info , tar_priority , tar_bonus , extract ( EPOCH from tar_bonus_time ) , extract ( EPOCH from tar_next_observable ) , tar_enabled from targets where tar_id = $1 ", 
	ECPGt_int,&(db_tar_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, 
	ECPGt_varchar,&(d_tar_name),(long)150,(long)1,sizeof(struct varchar_1), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_varchar,&(d_tar_info),(long)2000,(long)1,sizeof(struct varchar_2), 
	ECPGt_int,&(d_tar_info_ind),(long)1,(long)1,sizeof(int), 
	ECPGt_float,&(d_tar_priority),(long)1,(long)1,sizeof(float), 
	ECPGt_int,&(d_tar_priority_ind),(long)1,(long)1,sizeof(int), 
	ECPGt_float,&(d_tar_bonus),(long)1,(long)1,sizeof(float), 
	ECPGt_int,&(d_tar_bonus_ind),(long)1,(long)1,sizeof(int), 
	ECPGt_long,&(d_tar_bonus_time),(long)1,(long)1,sizeof(long), 
	ECPGt_int,&(d_tar_bonus_time_ind),(long)1,(long)1,sizeof(int), 
	ECPGt_long,&(d_tar_next_observable),(long)1,(long)1,sizeof(long), 
	ECPGt_int,&(d_tar_next_observable_ind),(long)1,(long)1,sizeof(int), 
	ECPGt_bool,&(d_tar_enabled),(long)1,(long)1,sizeof(bool), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EORT);}
#line 396 "target.ec"

	if (sqlca.sqlcode)
	{
	  	std::ostringstream err;
		err << "cannot load target with ID " << in_tar_id;
	  	throw SqlError (err.str ().c_str ());
	}

	delete[] target_name;

	target_name = new char[d_tar_name.len + 1];
	strncpy (target_name, d_tar_name.arr, d_tar_name.len);
	target_name[d_tar_name.len] = '\0';

	if (d_tar_info_ind >= 0)
	{
		char t_tar_info[d_tar_info.len + 1];
		strncpy (t_tar_info, d_tar_info.arr, d_tar_info.len);
		t_tar_info[d_tar_info.len] = '\0';
		tar_info = std::string (t_tar_info);
	}
	else
	{
		tar_info = std::string ("");
	}

	if (d_tar_priority_ind >= 0)
		tar_priority = d_tar_priority;
	else
		tar_priority = 0;

	if (d_tar_bonus_ind >= 0)
		tar_bonus = d_tar_bonus;
	else
		tar_bonus = -1;

	if (d_tar_bonus_time_ind >= 0)
		tar_bonus_time = d_tar_bonus_time;
	else
		tar_bonus_time = 0;

	if (d_tar_next_observable_ind >= 0)
		tar_next_observable = d_tar_next_observable;
	else
		tar_next_observable = 0;

	setTargetEnabled (d_tar_enabled, false);
}

int Target::save (bool overwrite)
{
	/* exec sql begin declare section */
	    
	
#line 448 "target.ec"
 int db_new_id = getTargetID () ;
/* exec sql end declare section */
#line 449 "target.ec"


	// generate new id, if we don't have any
	if (db_new_id <= 0)
	{
		{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "select nextval ( 'tar_id' )", ECPGt_EOIT, 
	ECPGt_int,&(db_new_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EORT);}
#line 458 "target.ec"

		if (sqlca.sqlcode)
		{
			logMsgDb ("Target::save cannot get new tar_id", MESSAGE_ERROR);
			return -1;
		}
	}

	return saveWithID (overwrite, db_new_id);
}

void Target::deleteTarget ()
{
	/* exec sql begin declare section */
	    
	 
	
#line 472 "target.ec"
 int d_tar_id = getTargetID () ;
 
#line 473 "target.ec"
 int d_ret ;
/* exec sql end declare section */
#line 474 "target.ec"


	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "select delete_target ( $1  )", 
	ECPGt_int,&(d_tar_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, 
	ECPGt_int,&(d_ret),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EORT);}
#line 476 "target.ec"

	if (sqlca.sqlcode)
	{
		throw SqlError ();
	}
	{ ECPGtrans(__LINE__, NULL, "commit");}
#line 481 "target.ec"

}

int Target::saveWithID (bool overwrite, int tar_id)
{
	// first, try an update..
	/* exec sql begin declare section */
		   
		    
		 
		   
		 
		   
		    
		    
		 
		    
		 
		    
		 
		 
	
#line 488 "target.ec"
 int db_tar_id = tar_id ;
 
#line 489 "target.ec"
 char db_type_id = getTargetType () ;
 
#line 490 "target.ec"
  struct varchar_3  { int len; char arr[ 150 ]; }  db_tar_name ;
 
#line 491 "target.ec"
 int db_tar_name_ind = 0 ;
 
#line 492 "target.ec"
  struct varchar_4  { int len; char arr[ 2000 ]; }  db_tar_comment ;
 
#line 493 "target.ec"
 int db_tar_comment_ind = 0 ;
 
#line 494 "target.ec"
 float db_tar_priority = getTargetPriority () ;
 
#line 495 "target.ec"
 float db_tar_bonus = getTargetBonus () ;
 
#line 496 "target.ec"
 int db_tar_bonus_ind ;
 
#line 497 "target.ec"
 long db_tar_bonus_time = * ( getTargetBonusTime () ) ;
 
#line 498 "target.ec"
 int db_tar_bonus_time_ind ;
 
#line 499 "target.ec"
 bool db_tar_enabled = getTargetEnabled () ;
 
#line 500 "target.ec"
  struct varchar_5  { int len; char arr[ 2000 ]; }  db_tar_info ;
 
#line 501 "target.ec"
 int db_tar_info_ind ;
/* exec sql end declare section */
#line 502 "target.ec"

	// fill in name and comment..
	if (getTargetName ())
	{
		db_tar_name.len = strlen (getTargetName ());
		strcpy (db_tar_name.arr, getTargetName ());
	}
	else
	{
		db_tar_name.len = 0;
		db_tar_name_ind = -1;
	}

	if (getTargetComment ())
	{
		db_tar_comment.len = strlen (getTargetComment ());
		strcpy (db_tar_comment.arr, getTargetComment ());
	}
	else
	{
		db_tar_comment.len = 0;
		db_tar_comment_ind = -1;
	}

	if (isnan (db_tar_bonus))
	{
		db_tar_bonus = 0;
		db_tar_bonus_ind =-1;
		db_tar_bonus_time = 0;
		db_tar_bonus_time_ind = -1;
	}
	else
	{
		db_tar_bonus_ind = 0;
		if (db_tar_bonus_time > 0)
		{
			db_tar_bonus_time_ind = 0;
		}
		else
		{
			db_tar_bonus_time_ind = -1;
		}
	}
	if (getTargetInfo ())
	{
		db_tar_info.len = strlen (getTargetInfo ());
		if (db_tar_info.len > 2000)
			db_tar_info.len = 2000;
		strncpy (db_tar_info.arr, getTargetInfo (), db_tar_info.len);
		db_tar_info_ind = 0;
	}
	else
	{
		db_tar_info.len = 0;
		db_tar_info_ind = -1;
	}

	// try inserting it..
	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "insert into targets ( tar_id , type_id , tar_name , tar_comment , tar_priority , tar_bonus , tar_bonus_time , tar_next_observable , tar_enabled , tar_info ) values ( $1  , $2  , $3  , $4  , $5  , $6  , to_timestamp ( $7  ) , null , $8  , $9  )", 
	ECPGt_int,&(db_tar_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_char,&(db_type_id),(long)1,(long)1,(1)*sizeof(char), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_varchar,&(db_tar_name),(long)150,(long)1,sizeof(struct varchar_3), 
	ECPGt_int,&(db_tar_name_ind),(long)1,(long)1,sizeof(int), 
	ECPGt_varchar,&(db_tar_comment),(long)2000,(long)1,sizeof(struct varchar_4), 
	ECPGt_int,&(db_tar_comment_ind),(long)1,(long)1,sizeof(int), 
	ECPGt_float,&(db_tar_priority),(long)1,(long)1,sizeof(float), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_float,&(db_tar_bonus),(long)1,(long)1,sizeof(float), 
	ECPGt_int,&(db_tar_bonus_ind),(long)1,(long)1,sizeof(int), 
	ECPGt_long,&(db_tar_bonus_time),(long)1,(long)1,sizeof(long), 
	ECPGt_int,&(db_tar_bonus_time_ind),(long)1,(long)1,sizeof(int), 
	ECPGt_bool,&(db_tar_enabled),(long)1,(long)1,sizeof(bool), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_varchar,&(db_tar_info),(long)2000,(long)1,sizeof(struct varchar_5), 
	ECPGt_int,&(db_tar_info_ind),(long)1,(long)1,sizeof(int), ECPGt_EOIT, ECPGt_EORT);}
#line 586 "target.ec"

	// insert failed - try update
	if (sqlca.sqlcode)
	{
		{ ECPGtrans(__LINE__, NULL, "rollback");}
#line 590 "target.ec"


		if (!overwrite)
		{
			return -1;
		}

		{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "update targets set type_id = $1  , tar_name = $2  , tar_comment = $3  , tar_priority = $4  , tar_bonus = $5  , tar_bonus_time = to_timestamp ( $6  ) , tar_enabled = $7  , tar_info = $8  where tar_id = $9 ", 
	ECPGt_char,&(db_type_id),(long)1,(long)1,(1)*sizeof(char), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_varchar,&(db_tar_name),(long)150,(long)1,sizeof(struct varchar_3), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_varchar,&(db_tar_comment),(long)2000,(long)1,sizeof(struct varchar_4), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_float,&(db_tar_priority),(long)1,(long)1,sizeof(float), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_float,&(db_tar_bonus),(long)1,(long)1,sizeof(float), 
	ECPGt_int,&(db_tar_bonus_ind),(long)1,(long)1,sizeof(int), 
	ECPGt_long,&(db_tar_bonus_time),(long)1,(long)1,sizeof(long), 
	ECPGt_int,&(db_tar_bonus_time_ind),(long)1,(long)1,sizeof(int), 
	ECPGt_bool,&(db_tar_enabled),(long)1,(long)1,sizeof(bool), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_varchar,&(db_tar_info),(long)2000,(long)1,sizeof(struct varchar_5), 
	ECPGt_int,&(db_tar_info_ind),(long)1,(long)1,sizeof(int), 
	ECPGt_int,&(db_tar_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, ECPGt_EORT);}
#line 610 "target.ec"


		if (sqlca.sqlcode)
		{
			logMsgDb ("Target::saveWithID", MESSAGE_ERROR);
			{ ECPGtrans(__LINE__, NULL, "rollback");}
#line 615 "target.ec"

			return -1;
		}
	}
	target_id = db_tar_id;

	{ ECPGtrans(__LINE__, NULL, "commit");}
#line 621 "target.ec"

	return 0;
}

int Target::compareWithTarget (Target *in_target, double in_sep_limit)
{
	struct ln_equ_posn other_position;
	in_target->getPosition (&other_position);

	return (getDistance (&other_position) < in_sep_limit);
}

moveType Target::startSlew (struct ln_equ_posn *position, bool update_position)
{
	/* exec sql begin declare section */
	    
	 
	 
	 
	 
	 
	 
	
#line 636 "target.ec"
 int d_tar_id = getObsTargetID () ;
 
#line 637 "target.ec"
 int d_obs_id ;
 
#line 638 "target.ec"
 double d_obs_ra ;
 
#line 639 "target.ec"
 double d_obs_dec ;
 
#line 640 "target.ec"
 float d_obs_alt ;
 
#line 641 "target.ec"
 float d_obs_az ;
 
#line 642 "target.ec"
 int d_pos_ind ;
/* exec sql end declare section */
#line 643 "target.ec"


	struct ln_hrz_posn hrz;

	if (update_position)
		getPosition (position);

	if (getObsId () > 0)		 // we already observe that target
		return OBS_ALREADY_STARTED;

	if (isnan (position->ra) || isnan (position->dec))
	{
		d_obs_ra = d_obs_dec = d_obs_alt = d_obs_az = rts2_nan ("f");
		d_pos_ind = -1;
	}
	else
	{
		d_obs_ra = position->ra;
		d_obs_dec = position->dec;
		ln_get_hrz_from_equ (position, observer, ln_get_julian_from_sys (), &hrz);
		d_obs_alt = hrz.alt;
		d_obs_az = hrz.az;
		d_pos_ind = 0;
	}

	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "select nextval ( 'obs_id' )", ECPGt_EOIT, 
	ECPGt_int,&(d_obs_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EORT);}
#line 672 "target.ec"

	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "insert into observations ( tar_id , obs_id , obs_slew , obs_ra , obs_dec , obs_alt , obs_az ) values ( $1  , $2  , now ( ) , $3  , $4  , $5  , $6  )", 
	ECPGt_int,&(d_tar_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(d_obs_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(d_obs_ra),(long)1,(long)1,sizeof(double), 
	ECPGt_int,&(d_pos_ind),(long)1,(long)1,sizeof(int), 
	ECPGt_double,&(d_obs_dec),(long)1,(long)1,sizeof(double), 
	ECPGt_int,&(d_pos_ind),(long)1,(long)1,sizeof(int), 
	ECPGt_float,&(d_obs_alt),(long)1,(long)1,sizeof(float), 
	ECPGt_int,&(d_pos_ind),(long)1,(long)1,sizeof(int), 
	ECPGt_float,&(d_obs_az),(long)1,(long)1,sizeof(float), 
	ECPGt_int,&(d_pos_ind),(long)1,(long)1,sizeof(int), ECPGt_EOIT, ECPGt_EORT);}
#line 694 "target.ec"

	if (sqlca.sqlcode != 0)
	{
		logMsgDb ("cannot insert observation slew start to db", MESSAGE_ERROR);
		{ ECPGtrans(__LINE__, NULL, "rollback");}
#line 698 "target.ec"

		return OBS_MOVE_FAILED;
	}
	{ ECPGtrans(__LINE__, NULL, "commit");}
#line 701 "target.ec"

	setObsId (d_obs_id);
	observation = new Observation (d_obs_id);
	return afterSlewProcessed ();
}

int Target::newObsSlew (struct ln_equ_posn *position)
{
	endObservation (-1);
	delete observation;
	observation = NULL;
	nullImgId ();
	return updateSlew (position);
}

int Target::updateSlew (struct ln_equ_posn *position)
{
	if (observation == NULL)
		return startSlew (position, false) == OBS_MOVE_FAILED ? -1 : 0;

	struct ln_hrz_posn hrz;
	ln_get_hrz_from_equ (position, observer, ln_get_julian_from_sys (), &hrz);
	try
	{
		observation->startSlew (position, &hrz);
	}
	catch (SqlError &er)
	{
		logStream (MESSAGE_ERROR) << er << sendLog;
		return -1;
	}
	return 0;
}

moveType Target::afterSlewProcessed ()
{
	return OBS_MOVE;
}

int Target::startObservation ()
{
	if (observationStarted ())
		return 0;
	time (&observationStart);
	if (observation && getObsId () > 0)
	{
		observation->startObservation ();
		obsStarted ();
	}
	return 0;
}

int Target::endObservation (int in_next_id, rts2core::Block *master)
{
	int old_obs_id = getObsId ();

	int ret = endObservation (in_next_id);

	// check if that was the last observation..
	Observation out_observation = Observation (old_obs_id);
	out_observation.checkUnprocessedImages (master);

	return ret;
}

int Target::endObservation (int in_next_id)
{
	if (isContinues () == 1 && in_next_id == getTargetID ())
		return 1;
	if (getObsId () > 0)
	{
		observation->endObservation (getObsState ());
		setObsId (-1);
	}
	observationStart = -1;
	return 0;
}

int Target::isContinues ()
{
	return 0;
}

int Target::observationStarted ()
{
	return (observationStart > 0) ? 1 : 0;
}

int Target::secToObjectSet (double JD)
{
	struct ln_rst_time rst;
	int ret;
	ret = getRST (&rst, JD, getMinObsAlt ());
	if (ret)
		return -1;				 // don't rise, circumpolar etc..
	ret = int ((rst.set - JD) * 86400.0);
	if (ret < 0)
		// hope we get current set, we are interested in next set..
		return ret + ((int (ret/86400) + 1) * -86400);
	return ret;
}

int Target::secToObjectRise (double JD)
{
	struct ln_rst_time rst;
	int ret;
	ret = getRST (&rst, JD, getMinObsAlt ());
	if (ret)
		return -1;				 // don't rise, circumpolar etc..
	ret = int ((rst.rise - JD) * 86400.0);
	if (ret < 0)
		// hope we get current set, we are interested in next set..
		return ret + ((int (ret/86400) + 1) * -86400);
	return ret;
}

int Target::secToObjectMeridianPass (double JD)
{
	struct ln_rst_time rst;
	int ret;
	ret = getRST (&rst, JD, getMinObsAlt ());
	if (ret)
		return -1;				 // don't rise, circumpolar etc..
	ret = int ((rst.transit - JD) * 86400.0);
	if (ret < 0)
		// hope we get current set, we are interested in next set..
		return ret + ((int (ret/86400) + 1) * -86400);
	return ret;
}

bool Target::isVisibleDuringNight (double jd, double horizon)
{
	struct ln_rst_time rst;
	int ret;
	Rts2Night night (jd, getObserver ());

	ret = getRST (&rst, night.getJDFrom (), horizon);
	// not visible at all
	if (ret < 0)
		return false;
	// circumpolar
	else if (ret > 0)
		return true;
	if (night.getJDFrom () < rst.set && night.getJDTo () > rst.set)
		return true;
	if (night.getJDFrom () < rst.rise && night.getJDTo () > rst.rise)
		return true;
	if (night.getJDFrom () < rst.transit && night.getJDTo () > rst.transit)
		return true;
	return false;
}

int Target::beforeMove ()
{
	startCalledNum++;
	return 0;
}

int Target::postprocess ()
{
	return 0;
}

void Target::getDBScript (const char *camera_name, std::string &script)
{
	/* exec sql begin declare section */
		    
		 
		 
		 
	
#line 867 "target.ec"
 int tar_id = getTargetID () ;
 
#line 868 "target.ec"
  struct varchar_6  { int len; char arr[ 8 ]; }  d_camera_name ;
 
#line 869 "target.ec"
  struct varchar_7  { int len; char arr[ 2000 ]; }  sc_script ;
 
#line 870 "target.ec"
 int sc_indicator ;
/* exec sql end declare section */
#line 871 "target.ec"


	d_camera_name.len = strlen (camera_name);
	strncpy (d_camera_name.arr, camera_name, d_camera_name.len);

	ECPGset_var( 0, &( d_camera_name ), __LINE__);\
 ECPGset_var( 1, &( tar_id ), __LINE__);\
 /* declare find_script cursor for select script from scripts where tar_id = $1  and camera_name = $2  */
#line 882 "target.ec"

	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "declare find_script cursor for select script from scripts where tar_id = $1  and camera_name = $2 ", 
	ECPGt_int,&(tar_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_varchar,&(d_camera_name),(long)8,(long)1,sizeof(struct varchar_6), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, ECPGt_EORT);}
#line 883 "target.ec"

	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "fetch next from find_script", ECPGt_EOIT, 
	ECPGt_varchar,&(sc_script),(long)2000,(long)1,sizeof(struct varchar_7), 
	ECPGt_int,&(sc_indicator),(long)1,(long)1,sizeof(int), ECPGt_EORT);}
#line 885 "target.ec"


	if (sqlca.sqlcode == ECPG_NOT_FOUND)
	{
		{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "close find_script", ECPGt_EOIT, ECPGt_EORT);}
#line 889 "target.ec"

		{ ECPGtrans(__LINE__, NULL, "rollback");}
#line 890 "target.ec"

		throw SqlError ();
	}

	if (sqlca.sqlcode || sc_indicator < 0)
	{
		logStream (MESSAGE_ERROR) << "while loading script for device " << camera_name << " and target " << tar_id << " : " << sqlca.sqlerrm.sqlerrmc << sendLog;
		{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "close find_script", ECPGt_EOIT, ECPGt_EORT);}
#line 897 "target.ec"

		{ ECPGtrans(__LINE__, NULL, "rollback");}
#line 898 "target.ec"

		throw SqlError ();
	}

	sc_script.arr[sc_script.len] = '\0';
	script = std::string (sc_script.arr);

	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "close find_script", ECPGt_EOIT, ECPGt_EORT);}
#line 905 "target.ec"

	{ ECPGtrans(__LINE__, NULL, "rollback");}
#line 906 "target.ec"

}

bool Target::getScript (const char *device_name, std::string &buf)
{
	int ret;
	rts2core::Configuration *config;
	config = rts2core::Configuration::instance ();

	try
	{
		getDBScript (device_name, buf);
		return false;
	}
	catch (rts2core::Error &er)
	{
	}

	ret = config->getString (device_name, "script", buf);
	if (!ret)
		return false;
	throw DeviceMissingExcetion (device_name);
}

void Target::setScript (const char *device_name, const char *buf)
{
	/* exec sql begin declare section */
	 
	 
	    
	
#line 933 "target.ec"
  struct varchar_8  { int len; char arr[ 8 ]; }  d_camera_name ;
 
#line 934 "target.ec"
  struct varchar_9  { int len; char arr[ 2000 ]; }  d_script ;
 
#line 935 "target.ec"
 int d_tar_id = getTargetID () ;
/* exec sql end declare section */
#line 936 "target.ec"


	d_camera_name.len = strlen (device_name);
	if (d_camera_name.len > 8)
		d_camera_name.len = 8;
	strncpy (d_camera_name.arr, device_name, 8);

	d_script.len = strlen (buf);
	if (d_script.len > 2000)
		d_script.len = 2000;
	strncpy (d_script.arr, buf, d_script.len);

	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "insert into scripts ( camera_name , tar_id , script ) values ( $1  , $2  , $3  )", 
	ECPGt_varchar,&(d_camera_name),(long)8,(long)1,sizeof(struct varchar_8), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(d_tar_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_varchar,&(d_script),(long)2000,(long)1,sizeof(struct varchar_9), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, ECPGt_EORT);}
#line 960 "target.ec"

	// insert failed - try update
	if (sqlca.sqlcode)
	{
		{ ECPGtrans(__LINE__, NULL, "rollback");}
#line 964 "target.ec"

		{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "update scripts set script = $1  where camera_name = $2  and tar_id = $3 ", 
	ECPGt_varchar,&(d_script),(long)2000,(long)1,sizeof(struct varchar_9), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_varchar,&(d_camera_name),(long)8,(long)1,sizeof(struct varchar_8), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(d_tar_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, ECPGt_EORT);}
#line 971 "target.ec"

		if (sqlca.sqlcode)
		{
			if (sqlca.sqlcode == ECPG_NOT_FOUND)
				throw CameraMissingExcetion (device_name);
			{ ECPGtrans(__LINE__, NULL, "rollback");}
#line 976 "target.ec"

			throw SqlError ();
		}
	}
	{ ECPGtrans(__LINE__, NULL, "commit");}
#line 980 "target.ec"

}

std::string Target::getPIName ()
{
	return labels.getTargetLabels (getTargetID (), LABEL_PI).getString ("not set", ",");
}

void Target::setPIName (const char *name)
{
	deleteLabels (LABEL_PI);
 	addLabel (name, LABEL_PI, true);
}

std::string Target::getProgramName ()
{
	return labels.getTargetLabels (getTargetID (), LABEL_PROGRAM).getString ("not set", ",");
}

void Target::setProgramName (const char *program)
{
	deleteLabels (LABEL_PROGRAM);
 	addLabel (program, LABEL_PROGRAM, true);
}

void Target::setConstraints (Constraints &cons)
{
	int ret = mkpath (getConstraintFile (), 0777);
	if (ret)
		throw rts2core::Error ((std::string ("cannot create directory for ") + getConstraintFile () + " : " + strerror (errno)).c_str ());

	std::ofstream ofs;
	
	ofs.exceptions ( std::ofstream::eofbit | std::ofstream::failbit | std::ofstream::badbit );
	try
	{
		ofs.open (getConstraintFile ());

		ofs << "<?xml version=\"1.0\"?>" << std::endl << std::endl;
		cons.printXML (ofs);
		ofs.close ();
	}
	catch (std::ofstream::failure f)
	{
		throw rts2core::Error ((std::string ("cannot write constraint file ") + getConstraintFile () + " : " + strerror (errno)).c_str ());
	}
}

void Target::appendConstraints (Constraints &cons)
{
  	Constraints tarc (cons);
	try
	{
		tarc.load (getConstraintFile (), false);
	}
	catch (XmlError er)
	{
		logStream (MESSAGE_WARNING) << er << sendLog;
	}
	setConstraints (tarc);
}

void Target::getAltAz (struct ln_hrz_posn *hrz, double JD, struct ln_lnlat_posn *obs)
{
	struct ln_equ_posn object;

	getPosition (&object, JD);

	if (isnan (object.ra) || isnan (object.dec))
	{
		hrz->alt = hrz->az = rts2_nan ("f");
	}
	else
	{
		ln_get_hrz_from_equ (&object, obs, JD, hrz);
	}
}

void Target::getMinMaxAlt (double _start, double _end, double &_min, double &_max)
{
	struct ln_equ_posn mid;
	double midJD = (_start + _end) / 2.0;

  	getPosition (&mid, midJD);

	double absLat = fabs (observer->lat);
	// change sign if we are on south hemisphere
	if (observer->lat < 0)
		mid.dec *= -1;

	// difference of JD is larger then 1, for sure we will get a max and min alt..
	if ((_end - _start) >= 1.0)
	{
		// use midJD to calculate DEC at maximum / minimum
		// this is not a correct use
		_min = absLat - 90 + mid.dec;
		_max = 90 - absLat + mid.dec;
		return;
	}

	struct ln_hrz_posn startHrz, endHrz;

	getAltAz (&startHrz, _start);
	getAltAz (&endHrz, _end);

	_min = (startHrz.alt < endHrz.alt) ? startHrz.alt : endHrz.alt;
	_max = (startHrz.alt > endHrz.alt) ? startHrz.alt : endHrz.alt;

	// now compute if object is in low or high transit during given interval
	double midRa = mid.ra;
	double startLst = ln_range_degrees (ln_get_mean_sidereal_time (_start) * 15.0 + observer->lng);
	double endLst = ln_range_degrees (ln_get_mean_sidereal_time (_end) * 15.0 + observer->lng);

	if (startLst > endLst)
	{
		// arange times so they are ordered
		endLst += 360.0;
		if (startLst > midRa)
			midRa += 360.0;
	}

	// it culmitaes during given period
	if (midRa > startLst && midRa < endLst)
		_max = 90 - absLat + mid.dec;
	midRa = ln_range_degrees (midRa + 180);
	if (midRa > endLst + 360)
		midRa = endLst - 360;
	// it pass low point during given period
	if (midRa > startLst && midRa < endLst)
		_min = absLat - 90 + mid.dec;
}

void Target::getGalLng (struct ln_gal_posn *gal, double JD)
{
	struct ln_equ_posn curr;
	getPosition (&curr, JD);
	ln_get_gal_from_equ (&curr, gal);
}

double Target::getGalCenterDist (double JD)
{
	static struct ln_equ_posn cntr = { 265.610844, -28.916790 };
	struct ln_equ_posn curr;
	getPosition (&curr, JD);
	return ln_get_angular_separation (&curr, &cntr);
}

double Target::getAirmass (double JD)
{
	struct ln_hrz_posn hrz;
	getAltAz (&hrz, JD);
	return ln_get_airmass (hrz.alt, airmassScale);
}

double Target::getZenitDistance (double JD)
{
	struct ln_hrz_posn hrz;
	getAltAz (&hrz, JD);
	return 90.0 - hrz.alt;
}

double Target::getHourAngle (double JD, struct ln_lnlat_posn *obs)
{
	double lst;
	double ha;
	struct ln_equ_posn pos;
	lst = ln_get_mean_sidereal_time (JD) * 15.0 + obs->lng;
	getPosition (&pos);
	ha = ln_range_degrees (lst - pos.ra);
	if (ha > 180)
		ha -= 360;
	return ha;
}

double Target::getDistance (struct ln_equ_posn *in_pos, double JD)
{
	struct ln_equ_posn object;
	getPosition (&object, JD);
	return ln_get_angular_separation (&object, in_pos);
}

double Target::getRaDistance (struct ln_equ_posn *in_pos, double JD)
{
	struct ln_equ_posn object;
	getPosition (&object, JD);
	return ln_range_degrees (object.ra - in_pos->ra);
}

double Target::getSolarDistance (double JD)
{
	struct ln_equ_posn eq_sun;
	ln_get_solar_equ_coords (JD, &eq_sun);
	return getDistance (&eq_sun, JD);
}

double Target::getSolarRaDistance (double JD)
{
	struct ln_equ_posn eq_sun;
	ln_get_solar_equ_coords (JD, &eq_sun);
	return getRaDistance (&eq_sun, JD);
}

double Target::getLunarDistance (double JD)
{
	struct ln_equ_posn moon;
	ln_get_lunar_equ_coords (JD, &moon);
	return getDistance (&moon, JD);
}

double Target::getLunarRaDistance (double JD)
{
	struct ln_equ_posn moon;
	ln_get_lunar_equ_coords (JD, &moon);
	return getRaDistance (&moon, JD);
}

/**
 * This method is called to check that target which was selected as good is
 * still the best.
 *
 * It should reload from DB values, which are important for selection process,
 * and if they indicate that target should not be observed, it should return -1.
 */
int Target::selectedAsGood ()
{
	/* exec sql begin declare section */
		 
		    
		 
		 
		 
		 
		 
		 
	
#line 1206 "target.ec"
 bool d_tar_enabled ;
 
#line 1207 "target.ec"
 int d_tar_id = getTargetID () ;
 
#line 1208 "target.ec"
 float d_tar_priority ;
 
#line 1209 "target.ec"
 int d_tar_priority_ind ;
 
#line 1210 "target.ec"
 float d_tar_bonus ;
 
#line 1211 "target.ec"
 int d_tar_bonus_ind ;
 
#line 1212 "target.ec"
 long d_tar_next_observable ;
 
#line 1213 "target.ec"
 int d_tar_next_observable_ind ;
/* exec sql end declare section */
#line 1214 "target.ec"


	// check if we are still enabled..
	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "select tar_enabled , tar_priority , tar_bonus , extract ( EPOCH from tar_next_observable ) from targets where tar_id = $1 ", 
	ECPGt_int,&(d_tar_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, 
	ECPGt_bool,&(d_tar_enabled),(long)1,(long)1,sizeof(bool), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_float,&(d_tar_priority),(long)1,(long)1,sizeof(float), 
	ECPGt_int,&(d_tar_priority_ind),(long)1,(long)1,sizeof(int), 
	ECPGt_float,&(d_tar_bonus),(long)1,(long)1,sizeof(float), 
	ECPGt_int,&(d_tar_bonus_ind),(long)1,(long)1,sizeof(int), 
	ECPGt_long,&(d_tar_next_observable),(long)1,(long)1,sizeof(long), 
	ECPGt_int,&(d_tar_next_observable_ind),(long)1,(long)1,sizeof(int), ECPGt_EORT);}
#line 1231 "target.ec"

	if (sqlca.sqlcode)
	{
		logMsgDb ("Target::selectedAsGood", MESSAGE_ERROR);
		return -1;
	}
	setTargetEnabled (d_tar_enabled, false);
	if (d_tar_priority_ind >= 0)
		tar_priority = d_tar_priority;
	else
		tar_priority = 0;
	if (d_tar_bonus_ind >= 0)
		tar_bonus = d_tar_bonus;
	else
		tar_bonus = 0;

	if (getTargetEnabled () && tar_priority + tar_bonus >= 0)
	{
		if (d_tar_next_observable_ind >= 0)
		{
			time_t now;
			time (&now);
			if (now < d_tar_next_observable)
				return -1;
		}
		return 0;
	}
	return -1;
}

/**
 * return 0 if we cannot observe that target, 1 if it's above horizon.
 */
bool Target::isGood (double lst, double JD, struct ln_equ_posn * pos)
{
	struct ln_hrz_posn hrz;
	getAltAz (&hrz, JD);
	return isAboveHorizon (&hrz);
}

bool Target::isGood (double JD)
{
	struct ln_equ_posn pos;
	getPosition (&pos, JD);
	return isGood (ln_get_mean_sidereal_time (JD) + observer->lng / 15.0, JD, &pos);
}

bool Target::isAboveHorizon (struct ln_hrz_posn *hrz)
{
	// assume that undefined objects are always above horizon
	if (isnan (hrz->alt))
		return true;
	if (hrz->alt < getMinObsAlt ())
		return false;
	return rts2core::Configuration::instance ()->getObjectChecker ()->is_good (hrz);
}

int Target::considerForObserving (double JD)
{
	// horizon constrain..
	struct ln_equ_posn curr_position;
	double lst = ln_get_mean_sidereal_time (JD) + observer->lng / 15.0;
	int ret;
	getPosition (&curr_position, JD);

	ret = isGood (lst, JD, &curr_position);
	if (!ret)
	{
		struct ln_rst_time rst;
		ret = getRST (&rst, JD, getMinObsAlt ());
		if (ret == -1)
		{
			// object doesn't rise, let's hope tomorrow it will rise
			logStream (MESSAGE_DEBUG)
				<< "Target::considerForObserving tar " << getTargetID ()
				<< " obstarid " << getObsTargetID ()
				<< " don't rise"
				<< sendLog;
			setNextObservable (JD + 1);
			return -1;
		}
		// handle circumpolar objects..
		if (ret == 1)
		{
			logStream (MESSAGE_DEBUG) << "Target::considerForObserving tar "
				<< getTargetID ()
				<< " obstarid " << getObsTargetID ()
				<< " is circumpolar, but is not good, scheduling after 10 minutes"
				<< sendLog;
			setNextObservable (JD + 10.0 / (24.0 * 60.0));
			return -1;
		}
		// object is above horizon, but checker reject it..let's see what
		// will hapens in 12 minutes
		if (rst.transit < rst.set && rst.set < rst.rise)
		{
			// object rose, but is not above horizon, let's hope in 12 minutes it will get above horizon
			logStream (MESSAGE_DEBUG) << "Target::considerForObserving "
				<< getTargetID ()
				<< " obstarid " << getObsTargetID ()
				<< " will rise tommorow: " << LibnovaDate (rst.rise)
				<< " JD:" << JD
				<< sendLog;
			setNextObservable (JD + 12*(1.0/1440.0));
			return -1;
		}
		// object is setting, let's target it for next rise..
		logStream (MESSAGE_DEBUG)
			<< "Target::considerForObserving " << getTargetID ()
			<< " obstarid " << getObsTargetID ()
			<< " will rise at: " << LibnovaDate (rst.rise)
			<< sendLog;
		setNextObservable (rst.rise);
		return -1;
	}
	// target was selected for observation
	return selectedAsGood ();
}

int Target::dropBonus ()
{
	/* exec sql begin declare section */
		 
	
#line 1353 "target.ec"
 int db_tar_id ;
/* exec sql end declare section */
#line 1354 "target.ec"


	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "update targets set tar_bonus = null , tar_bonus_time = null where tar_id = $1 ", 
	ECPGt_int,&(db_tar_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, ECPGt_EORT);}
#line 1362 "target.ec"

	if (sqlca.sqlcode)
	{
		logMsgDb ("Target::dropBonus", MESSAGE_ERROR);
		{ ECPGtrans(__LINE__, NULL, "rollback");}
#line 1366 "target.ec"

		return -1;
	}
	{ ECPGtrans(__LINE__, NULL, "commit");}
#line 1369 "target.ec"

	return 0;
}

float Target::getBonus (double JD)
{
	return tar_priority + tar_bonus;
}

int Target::changePriority (int pri_change, time_t *time_ch)
{
	/* exec sql begin declare section */
		    
		   
		    
	
#line 1381 "target.ec"
 int db_tar_id = getObsTargetID () ;
 
#line 1382 "target.ec"
 int db_priority_change = pri_change ;
 
#line 1383 "target.ec"
 int db_next_t = ( int ) * time_ch ;
/* exec sql end declare section */
#line 1384 "target.ec"


	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "update targets set tar_bonus = tar_bonus + $1  , tar_bonus_time = to_timestamp ( $2  ) where tar_id = $3 ", 
	ECPGt_int,&(db_priority_change),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(db_next_t),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(db_tar_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, ECPGt_EORT);}
#line 1392 "target.ec"

	if (sqlca.sqlcode)
	{
		logMsgDb ("Target::changePriority", MESSAGE_ERROR);
		{ ECPGtrans(__LINE__, NULL, "rollback");}
#line 1396 "target.ec"

		return -1;
	}
	{ ECPGtrans(__LINE__, NULL, "commit");}
#line 1399 "target.ec"

	return 0;
}

int Target::changePriority (int pri_change, double validJD)
{
	time_t next;
	ln_get_timet_from_julian (validJD, &next);
	return changePriority (pri_change, &next);
}

int Target::setNextObservable (time_t *time_ch)
{
	/* exec sql begin declare section */
	    
	 
	 
	
#line 1413 "target.ec"
 int db_tar_id = getObsTargetID () ;
 
#line 1414 "target.ec"
 int db_next_observable ;
 
#line 1415 "target.ec"
 int db_next_observable_ind ;
/* exec sql end declare section */
#line 1416 "target.ec"


	if (time_ch)
	{
		db_next_observable = (int) *time_ch;
		db_next_observable_ind = 0;
		tar_next_observable = *time_ch;
	}
	else
	{
		db_next_observable = 0;
		db_next_observable_ind = -1;
		tar_next_observable = 0;
	}

	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "update targets set tar_next_observable = to_timestamp ( $1  ) where tar_id = $2 ", 
	ECPGt_int,&(db_next_observable),(long)1,(long)1,sizeof(int), 
	ECPGt_int,&(db_next_observable_ind),(long)1,(long)1,sizeof(int), 
	ECPGt_int,&(db_tar_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, ECPGt_EORT);}
#line 1436 "target.ec"

	if (sqlca.sqlcode)
	{
		logMsgDb ("Target::setNextObservable", MESSAGE_ERROR);
		{ ECPGtrans(__LINE__, NULL, "rollback");}
#line 1440 "target.ec"

		return -1;
	}
	{ ECPGtrans(__LINE__, NULL, "commit");}
#line 1443 "target.ec"

	return 0;
}

int Target::setNextObservable (double validJD)
{
	time_t next;
	ln_get_timet_from_julian (validJD, &next);
	return setNextObservable (&next);
}

int Target::getNumObs (time_t *start_time, time_t *end_time)
{
	/* exec sql begin declare section */
		    
		    
		 
		    
	
#line 1457 "target.ec"
 int d_start_time = ( int ) * start_time ;
 
#line 1458 "target.ec"
 int d_end_time = ( int ) * end_time ;
 
#line 1459 "target.ec"
 int d_count ;
 
#line 1460 "target.ec"
 int d_tar_id = getTargetID () ;
/* exec sql end declare section */
#line 1461 "target.ec"


	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "select count ( * ) from observations where tar_id = $1  and obs_slew >= to_timestamp ( $2  ) and ( obs_end is null or obs_end <= to_timestamp ( $3  ) )", 
	ECPGt_int,&(d_tar_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(d_start_time),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(d_end_time),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, 
	ECPGt_int,&(d_count),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EORT);}
#line 1475 "target.ec"

	// runnign observations counts as well - hence obs_end is null

	{ ECPGtrans(__LINE__, NULL, "commit");}
#line 1478 "target.ec"


	return d_count;
}

int Target::getTotalNumberOfObservations ()
{
	/* exec sql begin declare section */
	 
	    
	
#line 1486 "target.ec"
 int d_count ;
 
#line 1487 "target.ec"
 int d_tar_id = getTargetID () ;
/* exec sql end declare section */
#line 1488 "target.ec"


	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "select count ( * ) from observations where tar_id = $1 ", 
	ECPGt_int,&(d_tar_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, 
	ECPGt_int,&(d_count),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EORT);}
#line 1498 "target.ec"

	// runnign observations counts as well - hence obs_end is null

	{ ECPGtrans(__LINE__, NULL, "commit");}
#line 1501 "target.ec"


	return d_count;
}

double Target::getLastObsTime ()
{
	/* exec sql begin declare section */
		    
		 
	
#line 1509 "target.ec"
 int d_tar_id = getTargetID () ;
 
#line 1510 "target.ec"
 double d_time_diff ;
/* exec sql end declare section */
#line 1511 "target.ec"


	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "select min ( extract ( EPOCH from ( now ( ) - obs_slew ) ) ) from observations where tar_id = $1  group by tar_id", 
	ECPGt_int,&(d_tar_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, 
	ECPGt_double,&(d_time_diff),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EORT);}
#line 1523 "target.ec"


	if (sqlca.sqlcode)
	{
		if (sqlca.sqlcode == ECPG_NOT_FOUND)
		{
			// 1 year was the last observation..
			return rts2_nan ("f");
		}
		else
			logMsgDb ("Target::getLastObsTime", MESSAGE_ERROR);
	}
	
	{ ECPGtrans(__LINE__, NULL, "commit");}
#line 1536 "target.ec"


	return d_time_diff;
}

int Target::getTotalNumberOfImages ()
{
	/* exec sql begin declare section */
	   
	    
	
#line 1544 "target.ec"
 int d_count = 0 ;
 
#line 1545 "target.ec"
 int d_tar_id = getTargetID () ;
/* exec sql end declare section */
#line 1546 "target.ec"


	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "select count ( * ) from observations , images where observations . tar_id = $1  and images . obs_id = observations . obs_id", 
	ECPGt_int,&(d_tar_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, 
	ECPGt_int,&(d_count),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EORT);}
#line 1556 "target.ec"

	// runnign observations counts as well - hence obs_end is null

	{ ECPGtrans(__LINE__, NULL, "commit");}
#line 1559 "target.ec"


	return d_count;
}

double Target::getTotalOpenTime ()
{
	/* exec sql begin declare section */
	 
	    
	
#line 1567 "target.ec"
 double total ;
 
#line 1568 "target.ec"
 int d_tar_id = getTargetID () ;
/* exec sql end declare section */
#line 1569 "target.ec"


	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "select sum ( images . img_exposure ) from observations , images where observations . tar_id = $1  and images . obs_id = observations . obs_id", 
	ECPGt_int,&(d_tar_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, 
	ECPGt_double,&(total),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EORT);}
#line 1579 "target.ec"

	// runnign observations counts as well - hence obs_end is null

	{ ECPGtrans(__LINE__, NULL, "commit");}
#line 1582 "target.ec"


	return total;
}

double Target::getFirstObs ()
{
	/* exec sql begin declare section */
		    
		 
	
#line 1590 "target.ec"
 int db_tar_id = getTargetID () ;
 
#line 1591 "target.ec"
 double ret ;
/* exec sql end declare section */
#line 1592 "target.ec"

	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "select MIN ( extract ( EPOCH from obs_start ) ) from observations where tar_id = $1 ", 
	ECPGt_int,&(db_tar_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, 
	ECPGt_double,&(ret),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EORT);}
#line 1601 "target.ec"

	if (sqlca.sqlcode)
	{
		{ ECPGtrans(__LINE__, NULL, "rollback");}
#line 1604 "target.ec"

		return rts2_nan("f");
	}
	{ ECPGtrans(__LINE__, NULL, "rollback");}
#line 1607 "target.ec"

	return ret;
}

double Target::getLastObs ()
{
	/* exec sql begin declare section */
	    
	 
	
#line 1614 "target.ec"
 int db_tar_id = getTargetID () ;
 
#line 1615 "target.ec"
 double ret ;
/* exec sql end declare section */
#line 1616 "target.ec"

	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "select MAX ( extract ( EPOCH from obs_start ) ) from observations where tar_id = $1 ", 
	ECPGt_int,&(db_tar_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, 
	ECPGt_double,&(ret),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EORT);}
#line 1625 "target.ec"

	if (sqlca.sqlcode)
	{
		{ ECPGtrans(__LINE__, NULL, "rollback");}
#line 1628 "target.ec"

		return rts2_nan("f");
	}
	{ ECPGtrans(__LINE__, NULL, "rollback");}
#line 1631 "target.ec"

	return ret;
}

void Target::printExtra (Rts2InfoValStream &_os, double JD)
{
	if (getTargetEnabled ())
	{
		_os << "Target is enabled" << std::endl;
	}
	else
	{
		_os << "Target is disabled" << std::endl;
	}
	_os
		<< std::endl
		<< InfoVal<const char*> ("IS VISIBLE TONIGHT", isVisibleDuringNight (JD, getMinObsAlt ()) ? "yes" : "no")
		<< std::endl
		<< InfoVal<double> ("TARGET PRIORITY", tar_priority)
		<< InfoVal<double> ("TARGET BONUS", tar_bonus)
		<< InfoVal<Timestamp> ("TARGET BONUS TIME", Timestamp(tar_bonus_time))
		<< InfoVal<Timestamp> ("TARGET NEXT OBS.", Timestamp(tar_next_observable))
		<< std::endl;
}

void Target::printShortInfo (std::ostream & _os, double JD)
{
	struct ln_equ_posn pos;
	struct ln_hrz_posn hrz;
	const char * name = getTargetName ();
	int old_prec = _os.precision (2);
	getPosition (&pos, JD);
	getAltAz (&hrz, JD);
	LibnovaRaDec raDec (&pos);
	LibnovaHrz hrzP (&hrz);
	double h = getHourAngle (JD);
	LibnovaHA ha (h);
	_os
		<< std::setw (5) << getTargetID () << SEP
		<< getTargetType () << SEP
		<< raDec << SEP << ha << SEP;
	writeAirmass (_os, JD);
	_os << SEP << hrzP << SEP;
	if (h < -30)
		_os << "rising    ";
	else if (h > 30)
		_os << "setting   ";
	else
		_os << "transiting";

	_os << SEP;
	_os << std::left << std::setw (25) << (name ? name :  "null") << std::right << SEP;
	_os.precision (old_prec);
}

void Target::printDS9Reg (std::ostream & _os, double JD)
{
	struct ln_equ_posn pos;
	getPosition (&pos, JD);
	_os << "circle(" << pos.ra << "," << pos.dec << "," << getSDiam (JD) << ")" << std::endl;
}

void Target::printShortBonusInfo (std::ostream & _os, double JD)
{
	struct ln_equ_posn pos;
	struct ln_hrz_posn hrz;
	const char * name = getTargetName ();
	int old_prec = _os.precision (2);
	getPosition (&pos, JD);
	getAltAz (&hrz, JD);
	LibnovaRaDec raDec (&pos);
	LibnovaHrz hrzP (&hrz);
	_os
		<< std::setw (5) << getTargetID () << SEP
		<< std::setw (7) << getBonus (JD) << SEP
		<< getTargetType () << SEP
		<< std::left << std::setw (40) << (name ? name :  "null") << std::right << SEP
		<< raDec << SEP;
	writeAirmass (_os, JD);
	_os
		<< SEP << hrzP;
	_os.precision (old_prec);
}

int Target::printObservations (double radius, double JD, std::ostream &_os)
{
	struct ln_equ_posn tar_pos;
	getPosition (&tar_pos, JD);

	ObservationSet obsset = ObservationSet ();
	obsset.loadRadius (&tar_pos, radius);
	_os << obsset;

	return obsset.size ();
}

TargetSet Target::getTargets (double radius)
{
	return getTargets (radius, ln_get_julian_from_sys ());
}

TargetSet Target::getTargets (double radius, double JD)
{
	struct ln_equ_posn tar_pos;
	getPosition (&tar_pos, JD);

	TargetSet tarset = TargetSet (&tar_pos, radius);
	tarset.load ();
	return tarset;
}

int Target::printTargets (double radius, double JD, std::ostream &_os)
{
	TargetSet tarset = getTargets (radius, JD);
	_os << tarset;

	return tarset.size ();
}

int Target::printImages (double JD, std::ostream &_os, int flags)
{
	struct ln_equ_posn tar_pos;
	int ret;

	getPosition (&tar_pos, JD);

	ImageSetPosition img_set = ImageSetPosition (&tar_pos);
	ret = img_set.load ();
	if (ret)
		return ret;

	img_set.print (std::cout, flags);

	return img_set.size ();
}

Target *createTarget (int _tar_id, struct ln_lnlat_posn *_obs, rts2core::ConnNotify *watchConn)
{
	/* exec sql begin declare section */
	   
	 
	
#line 1770 "target.ec"
 int db_tar_id = _tar_id ;
 
#line 1771 "target.ec"
 char db_type_id ;
/* exec sql end declare section */
#line 1772 "target.ec"


	Target *retTarget;

	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "select type_id from targets where tar_id = $1 ", 
	ECPGt_int,&(db_tar_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, 
	ECPGt_char,&(db_type_id),(long)1,(long)1,(1)*sizeof(char), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EORT);}
#line 1784 "target.ec"


	if (sqlca.sqlcode)
	{
	  	std::ostringstream err;
		err << "target with ID " << db_tar_id << " does not exists";
	  	throw SqlError (err.str ().c_str ());
	}

	// get more informations about target..
	switch (db_type_id)
	{
		// calibration targets..
		case TYPE_DARK:
			retTarget = new DarkTarget (_tar_id, _obs);
			break;
		case TYPE_FLAT:
			retTarget = new FlatTarget (_tar_id, _obs);
			break;
		case TYPE_CALIBRATION:
			retTarget = new CalibrationTarget (_tar_id, _obs);
			break;
		case TYPE_MODEL:
			retTarget = new ModelTarget (_tar_id, _obs);
			break;
		case TYPE_OPORTUNITY:
			retTarget = new OportunityTarget (_tar_id, _obs);
			break;
		case TYPE_ELLIPTICAL:
			retTarget = new EllTarget (_tar_id, _obs);
			break;
		case TYPE_GRB:
			retTarget = new TargetGRB (_tar_id, _obs, 3600, 86400, 5 * 86400);
			break;
		case TYPE_SWIFT_FOV:
			retTarget = new TargetSwiftFOV (_tar_id, _obs);
			break;
		case TYPE_INTEGRAL_FOV:
			retTarget = new TargetIntegralFOV (_tar_id, _obs);
			break;
		case TYPE_GPS:
			retTarget = new TargetGps (_tar_id, _obs);
			break;
		case TYPE_SKY_SURVEY:
			retTarget = new TargetSkySurvey (_tar_id, _obs);
			break;
		case TYPE_TERESTIAL:
			retTarget = new TargetTerestial (_tar_id, _obs);
			break;
		case TYPE_PLAN:
			retTarget = new TargetPlan (_tar_id, _obs);
			break;
		case TYPE_AUGER:
			retTarget = new TargetAuger (_tar_id, _obs, 1800);
			break;
		case TYPE_PLANET:
			retTarget = new TargetPlanet (_tar_id, _obs);
			break;
		default:
			retTarget = new ConstTarget (_tar_id, _obs);
			break;
	}

	if (watchConn)
		retTarget->setWatchConnection (watchConn);

	retTarget->setTargetType (db_type_id);
	retTarget->load ();
	{ ECPGtrans(__LINE__, NULL, "commit");}
#line 1852 "target.ec"

	return retTarget;
}

Target *createTargetByName (const char *tar_name, struct ln_lnlat_posn * obs)
{
	TargetSet ts (obs);
	ts.loadByName (tar_name);
	if (ts.size () == 1)
	{
		Target *ret = ts.begin ()->second;
		ts.clear ();
		return ret;
	}
	return NULL;
}

void Target::sendPositionInfo (Rts2InfoValStream &_os, double JD)
{
	struct ln_hrz_posn hrz;
	struct ln_gal_posn gal;
	struct ln_rst_time rst;
	double hourangle;
	time_t now;
	int ret;

	ln_get_timet_from_julian (JD, &now);

	getAltAz (&hrz, JD);
	hourangle = getHourAngle (JD);
	_os
		<< InfoVal<LibnovaDeg90> ("ALTITUDE", LibnovaDeg90 (hrz.alt))
		<< InfoVal<LibnovaDeg90> ("ZENITH DISTANCE", LibnovaDeg90 (getZenitDistance (JD)))
		<< InfoVal<LibnovaDeg360> ("AZIMUTH", LibnovaDeg360 (hrz.az))
		<< InfoVal<LibnovaRa> ("HOUR ANGLE", LibnovaRa (hourangle))
		<< InfoVal<double> ("AIRMASS", getAirmass (JD))
		<< std::endl;

	ret = getRST (&rst, JD, LN_STAR_STANDART_HORIZON);
	switch (ret)
	{
		case 1:
			_os << " - CIRCUMPOLAR - " << std::endl;
			rst.transit = JD + ((360 - hourangle) / 15.0 / 24.0);
			_os << InfoVal<TimeJDDiff> ("TRANSIT", TimeJDDiff (rst.transit, now));
			break;
		case -1:
			_os << " - DON'T RISE - " << std::endl;
			break;
		default:
			if (rst.set < rst.rise && rst.rise < rst.transit)
			{
				_os
					<< InfoVal<TimeJDDiff> ("SET", TimeJDDiff (rst.set, now))
					<< InfoVal<TimeJDDiff> ("RISE", TimeJDDiff (rst.rise, now))
					<< InfoVal<TimeJDDiff> ("TRANSIT", TimeJDDiff (rst.transit, now));
			}
			else if (rst.transit < rst.set && rst.set < rst.rise)
			{
				_os
					<< InfoVal<TimeJDDiff> ("TRANSIT", TimeJDDiff (rst.transit, now))
					<< InfoVal<TimeJDDiff> ("SET", TimeJDDiff (rst.set, now))
					<< InfoVal<TimeJDDiff> ("RISE", TimeJDDiff (rst.rise, now));
			}
			else
			{
				_os
					<< InfoVal<TimeJDDiff> ("RISE", TimeJDDiff (rst.rise, now))
					<< InfoVal<TimeJDDiff> ("TRANSIT", TimeJDDiff (rst.transit, now))
					<< InfoVal<TimeJDDiff> ("SET", TimeJDDiff (rst.set, now));
			}
	}

	// test min-max routines
	double maxAlt, minAlt;
	getMinMaxAlt (JD - 1, JD + 1, minAlt, maxAlt);
	_os << InfoVal<LibnovaDeg90> ("MIN ALT", LibnovaDeg90 (minAlt))
		<< InfoVal<LibnovaDeg90> ("MAX ALT", LibnovaDeg90 (maxAlt));

	getMinMaxAlt (JD - 0.2, JD + 0.2, minAlt, maxAlt);
	_os <<  std::endl
		<< InfoVal<LibnovaDeg90> ("MIN ALT", LibnovaDeg90 (minAlt))
		<< InfoVal<LibnovaDeg90> ("MAX ALT", LibnovaDeg90 (maxAlt));


	_os << std::endl
		<< InfoVal<double> ("MIN_OBS_ALT", getMinObsAlt ())
		<< std::endl
		<< "RISE, SET AND TRANSIT ABOVE MIN_ALT"
		<< std::endl
		<< std::endl;

	ret = getRST (&rst, JD, getMinObsAlt ());
	switch (ret)
	{
		case 1:
			_os << " - CIRCUMPOLAR - " << std::endl;
			rst.transit = JD + ((360 - hourangle) / 15.0 / 24.0);
			_os << InfoVal<TimeJDDiff> ("TRANSIT", TimeJDDiff (rst.transit, now));
			break;
		case -1:
			_os << " - DON'T RISE - " << std::endl;
			break;
		default:
			if (rst.set < rst.rise && rst.rise < rst.transit)
			{
				_os
					<< InfoVal<TimeJDDiff> ("SET", TimeJDDiff (rst.set, now))
					<< InfoVal<TimeJDDiff> ("RISE", TimeJDDiff (rst.rise, now))
					<< InfoVal<TimeJDDiff> ("TRANSIT", TimeJDDiff (rst.transit, now));
			}
			else if (rst.transit < rst.set && rst.set < rst.rise)
			{
				_os
					<< InfoVal<TimeJDDiff> ("TRANSIT", TimeJDDiff (rst.transit, now))
					<< InfoVal<TimeJDDiff> ("SET", TimeJDDiff (rst.set, now))
					<< InfoVal<TimeJDDiff> ("RISE", TimeJDDiff (rst.rise, now));
			}
			else
			{
				_os
					<< InfoVal<TimeJDDiff> ("RISE", TimeJDDiff (rst.rise, now))
					<< InfoVal<TimeJDDiff> ("TRANSIT", TimeJDDiff (rst.transit, now))
					<< InfoVal<TimeJDDiff> ("SET", TimeJDDiff (rst.set, now));
			}
	}

	if (_os.getStream ())
		printAltTable (*(_os.getStream ()), JD);

	ConstraintsList violated;
	getViolatedConstraints (JD, violated);

	std::ostringstream violatedNames;
	for (ConstraintsList::iterator iter = violated.begin (); iter != violated.end (); iter++)
	{
		if (iter != violated.begin ())
			violatedNames << " ";
		violatedNames << (*iter)->getName ();
	}

	ConstraintsList satisfied;
	getSatisfiedConstraints (JD, satisfied);

	std::ostringstream satisfiedNames;
	for (ConstraintsList::iterator iter = satisfied.begin (); iter != satisfied.end (); iter++)
	{
		if (iter != satisfied.begin ())
			satisfiedNames << " ";
		satisfiedNames << (*iter)->getName ();
	}


	getGalLng (&gal, JD);
	_os
		<< std::endl
		<< InfoVal<LibnovaDeg360> ("GAL. LONGITUDE", LibnovaDeg360 (gal.l))
		<< InfoVal<LibnovaDeg90> ("GAL. LATITUDE", LibnovaDeg90 (gal.b))
		<< InfoVal<LibnovaDeg180> ("GAL. CENTER DIST.", LibnovaDeg180 (getGalCenterDist (JD)))
		<< InfoVal<LibnovaDeg360> ("SOLAR DIST.", LibnovaDeg360 (getSolarDistance (JD)))
		<< InfoVal<LibnovaDeg180> ("SOLAR RA DIST.", LibnovaDeg180 (getSolarRaDistance (JD)))
		<< InfoVal<LibnovaDeg360> ("LUNAR DIST.", LibnovaDeg360 (getLunarDistance (JD)))
		<< InfoVal<LibnovaDeg180> ("LUNAR RA DIST.", LibnovaDeg180 (getLunarRaDistance (JD)))
		<< InfoVal<LibnovaDeg360> ("LUNAR PHASE", LibnovaDeg360 (ln_get_lunar_phase (JD)))
		<< std::endl
		<< InfoVal<const char*> ("SYSTEM CONSTRAINTS", rts2core::Configuration::instance ()->getMasterConstraintFile ())
		<< InfoVal<const char*> ("SYSTEM CONSTRAINTS", (constraintsLoaded & CONSTRAINTS_SYSTEM) ? "used" : "empy/not used")
		<< InfoVal<const char*> ("GROUP CONSTRAINTS", getGroupConstraintFile ())
		<< InfoVal<const char*> ("GROUP CONSTRAINTS", (constraintsLoaded & CONSTRAINTS_GROUP) ? "used" : "empty/not used")
		<< InfoVal<const char*> ("TARGET CONSTRAINTS", getConstraintFile ())
		<< InfoVal<const char*> ("TARGET CONSTRAINTS", (constraintsLoaded & CONSTRAINTS_TARGET) ? "used" : "empty/not used")
		<< InfoVal<const char*> ("CONSTRAINTS", checkConstraints (JD) ? "satisfied" : "not met")
		<< InfoVal<std::string> ("VIOLATED", violatedNames.str ())
		<< InfoVal<std::string> ("SATISFIED", satisfiedNames.str ())
		<< std::endl;
}

void Target::sendInfo (Rts2InfoValStream & _os, double JD)
{
	struct ln_equ_posn pos;
	double gst;
	double lst;
	time_t now, last;

	const char *name = getTargetName ();
	char tar_type[2];
	tar_type[0] = getTargetType ();
	tar_type[1] = '\0';

	ln_get_timet_from_julian (JD, &now);

	getPosition (&pos, JD);

	_os
		<< InfoVal<int> ("ID", getTargetID ())
		<< InfoVal<int> ("SEL_ID", getObsTargetID ())
		<< InfoVal<const char *> ("NAME", (name ? name : "null name"))
		<< InfoVal<const char *> ("TYPE", tar_type)
		<< InfoVal<std::string> ("PI", getPIName ())
		<< InfoVal<std::string> ("PROGRAM", getProgramName ())
		<< InfoVal<LibnovaRaJ2000> ("RA", LibnovaRaJ2000 (pos.ra))
		<< InfoVal<LibnovaDecJ2000> ("DEC", LibnovaDecJ2000 (pos.dec))
		<< std::endl;

	sendPositionInfo (_os, JD);

	last = now - 86400;
	_os
		<< InfoVal<int> ("24 HOURS OBS", getNumObs (&last, &now));
	last = now - 7 * 86400;
	_os
		<< InfoVal<int> ("7 DAYS OBS", getNumObs (&last, &now))
		<< InfoVal<double> ("BONUS", getBonus (JD));

	// is above horizon?
	gst = ln_get_mean_sidereal_time (JD);
	lst = gst + getObserver()->lng / 15.0;
	_os << (isGood (lst, JD, & pos)
		? "Target is above local horizon."
		: "Target is below local horizon, it's not possible to observe it.")
		<< std::endl;
	printExtra (_os, JD);
}

void Target::sendConstraints (Rts2InfoValStream & _os, double JD)
{
	_os << "Constraints" << std::endl;
	constraints->printXML (*(_os.getStream ()));
	_os << std::endl;
}

size_t Target::getViolatedConstraints (double JD, ConstraintsList &violated)
{
	return getConstraints ()->getViolated (this, JD, violated);
}

ConstraintsList Target::getViolatedConstraints (double JD)
{
	ConstraintsList ret;
	getViolatedConstraints (JD, ret);
	return ret;
}

bool Target::isViolated (Constraint *cons, double JD)
{
	return !(cons->satisfy (this, JD));
}

size_t Target::getSatisfiedConstraints (double JD, ConstraintsList &satisfied)
{
	return getConstraints ()->getSatisfied (this, JD, satisfied);
}

ConstraintsList Target::getSatisfiedConstraints (double JD)
{
	ConstraintsList ret;
	getSatisfiedConstraints (JD, ret);
	return ret;
}

size_t Target::getAltitudeConstraints (std::map <std::string, std::vector <rts2db::ConstraintDoubleInterval> > &ac)
{
	return getConstraints ()->getAltitudeConstraints (ac);
}

size_t Target::getAltitudeViolatedConstraints (std::map <std::string, std::vector <rts2db::ConstraintDoubleInterval> > &ac)
{
	return getConstraints ()->getAltitudeViolatedConstraints (ac);
}

size_t Target::getTimeConstraints (std::map <std::string, ConstraintPtr> &cons)
{
	return getConstraints ()->getTimeConstraints (cons);
}

void Target::getSatisfiedIntervals (time_t from, time_t to, int length, int step, interval_arr_t &satisfiedIntervals)
{
	getConstraints ()->getSatisfiedIntervals (this, from, to, length, step, satisfiedIntervals);
}

double Target::getSatisfiedDuration (double from, double to, double length, double step)
{
	if (!isnan (satisfiedFrom) && !isnan (satisfiedProbedUntil))
	{
		if (from >= satisfiedFrom && from <= satisfiedProbedUntil)
		{
			// did not find end of interval..
			if (!isinf (satisfiedTo) || isnan (satisfiedTo) || to <= satisfiedProbedUntil)
				return satisfiedTo;
			// don't recalculate full interval
			from = satisfiedProbedUntil;
		}
	}
	satisfiedTo = getConstraints ()->getSatisfiedDuration (this, from, to, length, step);
	// update boundaries where we probed..
	if (isnan (satisfiedFrom) || from < satisfiedFrom)
		satisfiedFrom = from;
	// visible during full interval
	if (isinf (satisfiedTo))
	{
		satisfiedProbedUntil = to;
		// visible during full interval
		return INFINITY;
	}
	// not visible at all..
	else if (isnan (satisfiedTo))
	{
		satisfiedProbedUntil = from;
		return NAN;
	}
	satisfiedProbedUntil = satisfiedTo;
	return satisfiedTo;
}

void Target::getViolatedIntervals (time_t from, time_t to, int length, int step, interval_arr_t &satisfiedIntervals)
{
	getConstraints ()->getViolatedIntervals (this, from, to, length, step, satisfiedIntervals);
}

TargetSet * Target::getCalTargets (double JD, double minaird)
{
	TargetSetCalibration *ret = new TargetSetCalibration (this, JD, minaird);
	ret->load ();
	return ret;
}

void Target::writeToImage (rts2image::Image * image, double JD)
{
	// write lunar distance
	struct ln_equ_posn moon;
	struct ln_hrz_posn hmoon;
	ln_get_lunar_equ_coords (JD, &moon);
	ln_get_hrz_from_equ (&moon, observer, JD, &hmoon);
	image->setValue ("MOONDIST", getDistance (&moon, JD), "angular distance to between observation and the moon");
	image->setValue ("MOONRA", moon.ra, "lunar RA");
	image->setValue ("MOONDEC", moon.dec, "lunar DEC");
	image->setValue ("MOONPHA", ln_get_lunar_phase (JD) / 1.8, "moon phase");
	image->setValue ("MOONALT", hmoon.alt, "lunar altitude");
	image->setValue ("MOONAZ", hmoon.az, "lunar azimuth");
}

const char *Target::getConstraintFile ()
{
	if (constraintFile)
		return constraintFile;

	std::ostringstream os;
	os << rts2core::Configuration::instance ()->getTargetDir () << "/" << getTargetID () << "/constraints.xml";
	constraintFile = new char[os.str ().length () + 1];
	strcpy (constraintFile, os.str ().c_str ());
	return constraintFile;
}

const char *Target::getGroupConstraintFile ()
{
	if (groupConstraintFile)
		return groupConstraintFile;
	
	std::ostringstream os;
	os << rts2core::Configuration::instance ()->getTargetDir () << "/groups/" << getTargetType () << ".xml";
	groupConstraintFile = new char[os.str ().length () + 1];
	strcpy (groupConstraintFile, os.str ().c_str ());
	return groupConstraintFile;
}

Constraints * Target::getConstraints ()
{
	// if constraints were modified, reload them..
	if (constraints)
		return constraints;

	constraintsLoaded = CONSTRAINTS_NONE;
	watchIDs.clear ();

	struct stat fs;
	// check for system level constraints
	try
	{
		constraints = new Constraints (MasterConstraints::getConstraint ());
		constraintsLoaded |= CONSTRAINTS_SYSTEM;
		addWatch (rts2core::Configuration::instance ()->getMasterConstraintFile ());
	}
	catch (XmlError er)
	{
		logStream (MESSAGE_WARNING) << "cannot load master constraint file:" << er << sendLogNoEndl;
		constraints = new Constraints ();
	}
	if (stat (getGroupConstraintFile (), &fs))
	{
		logStream (errno == ENOENT ? MESSAGE_DEBUG : MESSAGE_WARNING) << "cannot open " << getGroupConstraintFile () << ":" << strerror(errno) << sendLog;
	}
	else
	{
		// check for group constraint
		try
		{
			constraints->load (getGroupConstraintFile ());
			constraintsLoaded |= CONSTRAINTS_GROUP;
			addWatch (getGroupConstraintFile ());
		}
		catch (XmlError er)
		{
			logStream (MESSAGE_WARNING) << "cannot load group constraint file " << getGroupConstraintFile () << ":" << er << sendLogNoEndl;
		}
	}
	// load target constrainst
	try
	{
		constraints->load (getConstraintFile ());
		constraintsLoaded |= CONSTRAINTS_TARGET;
		addWatch (getConstraintFile ());
	}
	catch (XmlError er)
	{
		logStream (MESSAGE_WARNING) << "cannot load target constraint file " << getConstraintFile () << ":" << er << sendLogNoEndl;
	}
	return constraints;
}

bool Target::checkConstraints (double JD)
{
	return getConstraints ()->satisfy (this, JD);
}

void Target::revalidateConstraints (int watchID)
{
	for (std::vector <int>::iterator iter = watchIDs.begin (); iter != watchIDs.end (); iter++)
	{
		if (*iter == watchID)
		{
			delete constraints;
			constraints = NULL;
			satisfiedFrom = satisfiedTo = rts2_nan ("f");
			return;
		}
	}
}

std::ostream & operator << (std::ostream &_os, Target &target)
{
	Rts2InfoValOStream _ivos = Rts2InfoValOStream (&_os);
	target.sendInfo (_ivos);
	return _os;
}

Rts2InfoValStream & operator << (Rts2InfoValStream &_os, Target &target)
{
	target.sendInfo (_os);
	return _os;
}
