/* Processed by ecpg (4.7.0) */
/* These include files are added by the preprocessor */
#include <ecpglib.h>
#include <ecpgerrno.h>
#include <sqlca.h>
/* End of automatic include section */

#line 1 "target_auger.ec"
/* 
 * Auger cosmic rays showers follow-up target.
 * Copyright (C) 2005-2010 Petr Kubanek <petr@kubanek.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "rts2db/sqlerror.h"
#include "rts2db/target_auger.h"

#include "timestamp.h"
#include "infoval.h"
#include "rts2fits/image.h"

using namespace rts2db;

/*****************************************************************/
//Utility functions

/* vector cross product */
void crossProduct(struct vec * a, struct vec * b, struct vec * c)
{
	c->x = a->y * b->z - a->z * b->y;
	c->y = a->z * b->x - a->x * b->z;
	c->z = a->x * b->y - a->y * b->x;
}

/* horizontal coordinates to vector transformation */
void hrz_to_vec (struct ln_hrz_posn * dirh, struct vec * dirv)
{
	dirv->x = cos (dirh->alt) * sin (dirh->az) * (-1);
	dirv->y = cos (dirh->alt) * cos (dirh->az) * (-1);
	dirv->z = sin (dirh->alt);
}

/* vector to horizontal coordinates transformation */
void vec_to_hrz (struct vec * dirv, struct ln_hrz_posn * dirh)
{
	dirh->alt = asin (dirv->z);
	if (dirv->y > 0.)
		dirh->az = atan (dirv->x / dirv->y) + M_PI;
	if (dirv->y < 0.)
	{
		if (dirv->x < 0.)
			dirh->az = atan (dirv->x / dirv->y);
		else
			dirh->az = atan (dirv->x / dirv->y) + 2 * M_PI;
	}
	if (dirv->y == 0.)
	{
		if (dirv->x < 0.)
			dirh->az = 0.5 * M_PI;
		else
			dirh->az = 1.5 * M_PI;
	}
}

/* transformation to unit length vector */
void vec_unit (struct vec * in, struct vec * unit)
{
	double length = sqrt (in->x*in->x + in->y*in->y + in->z*in->z);
	unit->x = in->x / length;
	unit->y = in->y / length;
	unit->z = in->z / length;
}

/* rotation of vector "b" around "axis" by "angle" */
vec rotateVector(struct vec * axis, struct vec * b, double angle)
{
	struct vec ret;
	struct vec cross;
	crossProduct(axis, b, &cross);
	ret.x = b->x * cos(angle) + cross.x * sin(angle);
	ret.y = b->y * cos(angle) + cross.y * sin(angle);
	ret.z = b->z * cos(angle) + cross.z * sin(angle);
	return ret;
}

void dirToEqu (vec *dir, struct ln_lnlat_posn *observer, double JD, struct ln_equ_posn *pos)
{
	struct ln_hrz_posn hrz;
	vec_to_hrz (dir, &hrz);
	hrz.alt = ln_rad_to_deg (hrz.alt);
	hrz.az = ln_rad_to_deg (hrz.az);
	ln_get_equ_from_hrz (&hrz, observer, JD, pos);
}

/*****************************************************************/

TargetAuger::TargetAuger (int in_tar_id, struct ln_lnlat_posn * _obs, int in_augerPriorityTimeout):ConstTarget (in_tar_id, _obs)
{
	augerPriorityTimeout = in_augerPriorityTimeout;
	cor.x = rts2_nan ("f");
	cor.y = rts2_nan ("f");
	cor.z = rts2_nan ("f");
}

TargetAuger::TargetAuger (int _auger_t3id, double _auger_date, double _auger_ra, double _auger_dec, struct ln_lnlat_posn *_obs):ConstTarget (-1, _obs)
{
	augerPriorityTimeout = -1;

	t3id = _auger_t3id;
	auger_date = _auger_date;
	setPosition (_auger_ra, _auger_dec);
}

TargetAuger::~TargetAuger (void)
{
}

void TargetAuger::load ()
{
	/* exec sql begin declare section */
	 
	 
	 
	 

	
#line 126 "target_auger.ec"
 int d_auger_t3id ;
 
#line 127 "target_auger.ec"
 double d_auger_date ;
 
#line 128 "target_auger.ec"
 double d_auger_ra ;
 
#line 129 "target_auger.ec"
 double d_auger_dec ;
/* exec sql end declare section */
#line 131 "target_auger.ec"


	struct ln_equ_posn pos;
	struct ln_hrz_posn hrz;
	double JD = ln_get_julian_from_sys ();

	/* declare cur_auger cursor for select auger_t3id , extract ( EPOCH from auger_date ) , ra , dec from auger order by auger_date desc */
#line 146 "target_auger.ec"


	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "declare cur_auger cursor for select auger_t3id , extract ( EPOCH from auger_date ) , ra , dec from auger order by auger_date desc", ECPGt_EOIT, ECPGt_EORT);}
#line 148 "target_auger.ec"

	while (1)
	{
		{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "fetch next from cur_auger", ECPGt_EOIT, 
	ECPGt_int,&(d_auger_t3id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(d_auger_date),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(d_auger_ra),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(d_auger_dec),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EORT);}
#line 155 "target_auger.ec"

		if (sqlca.sqlcode)
			break;
		pos.ra = d_auger_ra;
		pos.dec = d_auger_dec;
		ln_get_hrz_from_equ (&pos, observer, JD, &hrz);
		if (hrz.alt > 10)
		{
			{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "close cur_auger", ECPGt_EOIT, ECPGt_EORT);}
#line 163 "target_auger.ec"

			{ ECPGtrans(__LINE__, NULL, "commit");}
#line 164 "target_auger.ec"

			load (d_auger_t3id);
			return;
		}
	}
	auger_date = 0;
	if (sqlca.sqlcode == ECPG_NOT_FOUND)
	{
		pos.ra = pos.dec = 0;
		{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "close cur_auger", ECPGt_EOIT, ECPGt_EORT);}
#line 173 "target_auger.ec"

		{ ECPGtrans(__LINE__, NULL, "rollback");}
#line 174 "target_auger.ec"

		return;
	}
	logStream (MESSAGE_ERROR) << "cannot load Auger target: " << sqlca.sqlerrm.sqlerrmc << sendLog;
	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "close cur_target", ECPGt_EOIT, ECPGt_EORT);}
#line 178 "target_auger.ec"

	{ ECPGtrans(__LINE__, NULL, "rollback");}
#line 179 "target_auger.ec"

}

void TargetAuger::load (int auger_id)
{
	/* exec sql begin declare section */
	   
	 
	 
	 

	                /// FD eye Id
	                /// FD run number
	              /// FD event number
	    /// Event Auger Id after SD/FD merger
	             /// GPS second (SD)
	            /// GPS nano second (SD)
	               /// SD Event Id

	               /// Num. pixels with a pulse after FdPulseFinder

	        /// Zenith angle of SDP normal vector (deg)
	     /// Uncertainty of SDPtheta
	          /// Azimuth angle of SDP normal vector (deg)
	       /// Uncertainty of SDPphi
	         /// Chi^2 of SDP db_it
	             /// Degrees of db_reedom of SDP db_it

	              /// Shower impact parameter Rp (m)
	           /// Uncertainty of Rp (m)
	            /// Angle of shower in the SDP (deg)
	         /// Uncertainty of Chi0 (deg)
	              /// FD time db_it T_0 (ns)
	           /// Uncertainty of T_0 (ns)
	        /// Full Chi^2 of axis db_it
	      /// Chi^2 of axis db_it (FD only)
	            /// Degrees of db_reedom of axis db_it

	         /// Core position in easting coordinate (m)
	        /// Core position in northing coordinate (m)
	        /// Core position altitude (m)
	     /// Uncertainty of northing coordinate (m)
	      /// Uncertainty of easting coordinate (m)
	           /// Shower zenith angle in core coords. (deg)
	        /// Uncertainty of zenith angle (deg)
	             /// Shower azimuth angle in core coords. (deg)
	          /// Uncertainty of azimuth angle (deg)

	         /// Energy deposit at shower max (GeV/(g/cm^2))
	      /// Uncertainty of Nmax (GeV/(g/cm^2))
	            /// Slant depth of shower maximum (g/cm^2)
	         /// Uncertainty of Xmax (g/cm^2)
	              /// X0 Gaisser-Hillas db_it (g/cm^2)
	           /// Uncertainty of X0 (g/cm^2)
	          /// Lambda of Gaisser-Hillas db_it (g/cm^2)
	       /// Uncertainty of Lambda (g/cm^2)
	          /// Chi^2 of Gaisser-Hillas db_it
	              /// Degrees of db_reedom of GH db_it
	     /// Chi^2 of linear db_it to profile

	        /// Calorimetric energy db_rom GH db_it (EeV)
	     /// Uncertainty of Eem (EeV)
	          /// Total energy db_rom GH db_it (EeV)
	       /// Uncertainty of Etot (EeV)

	        /// Minimum viewing angle (deg)
	        /// Maximum viewing angle (deg)
	       /// Mean viewing angle (deg)

	              /// Number of stations in hybrid db_it
	        /// Station used in hybrid-geometry reco
	        /// Shower axis distance to hottest station (m)
	         /// SDP distance to hottest station (m)

	          /// SD/FD time offset after the minimization (ns)
	     /// Distance to shower maximum (m)
	       /// First recorded slant depth of track (g/cm^2)
	       /// Last recorded slant depth of track (g/cm^2)
	         /// First slant depth inside FOV (g/cm^2)
	         /// Last slant depth inside FOV (g/cm^2)
	       /// Observed track length depth (g/cm^2)
	     /// Observed track length angle (deg)
	       /// Observed track length time (100 ns)

	   
	
#line 185 "target_auger.ec"
 int d_auger_t3id = auger_id ;
 
#line 186 "target_auger.ec"
 double d_auger_date ;
 
#line 187 "target_auger.ec"
 double d_auger_ra ;
 
#line 188 "target_auger.ec"
 double d_auger_dec ;
 
#line 190 "target_auger.ec"
 int db_Eye ;
 
#line 191 "target_auger.ec"
 int db_Run ;
 
#line 192 "target_auger.ec"
 int db_Event ;
 
#line 193 "target_auger.ec"
  struct varchar_1  { int len; char arr[ 50 ]; }  db_AugerId ;
 
#line 194 "target_auger.ec"
 int db_GPSSec ;
 
#line 195 "target_auger.ec"
 int db_GPSNSec ;
 
#line 196 "target_auger.ec"
 int db_SDId ;
 
#line 198 "target_auger.ec"
 int db_NPix ;
 
#line 200 "target_auger.ec"
 double db_SDPTheta ;
 
#line 201 "target_auger.ec"
 double db_SDPThetaErr ;
 
#line 202 "target_auger.ec"
 double db_SDPPhi ;
 
#line 203 "target_auger.ec"
 double db_SDPPhiErr ;
 
#line 204 "target_auger.ec"
 double db_SDPChi2 ;
 
#line 205 "target_auger.ec"
 int db_SDPNdf ;
 
#line 207 "target_auger.ec"
 double db_Rp ;
 
#line 208 "target_auger.ec"
 double db_RpErr ;
 
#line 209 "target_auger.ec"
 double db_Chi0 ;
 
#line 210 "target_auger.ec"
 double db_Chi0Err ;
 
#line 211 "target_auger.ec"
 double db_T0 ;
 
#line 212 "target_auger.ec"
 double db_T0Err ;
 
#line 213 "target_auger.ec"
 double db_TimeChi2 ;
 
#line 214 "target_auger.ec"
 double db_TimeChi2FD ;
 
#line 215 "target_auger.ec"
 int db_TimeNdf ;
 
#line 217 "target_auger.ec"
 double db_Easting ;
 
#line 218 "target_auger.ec"
 double db_Northing ;
 
#line 219 "target_auger.ec"
 double db_Altitude ;
 
#line 220 "target_auger.ec"
 double db_NorthingErr ;
 
#line 221 "target_auger.ec"
 double db_EastingErr ;
 
#line 222 "target_auger.ec"
 double db_Theta ;
 
#line 223 "target_auger.ec"
 double db_ThetaErr ;
 
#line 224 "target_auger.ec"
 double db_Phi ;
 
#line 225 "target_auger.ec"
 double db_PhiErr ;
 
#line 227 "target_auger.ec"
 double db_dEdXmax ;
 
#line 228 "target_auger.ec"
 double db_dEdXmaxErr ;
 
#line 229 "target_auger.ec"
 double db_Xmax ;
 
#line 230 "target_auger.ec"
 double db_XmaxErr ;
 
#line 231 "target_auger.ec"
 double db_X0 ;
 
#line 232 "target_auger.ec"
 double db_X0Err ;
 
#line 233 "target_auger.ec"
 double db_Lambda ;
 
#line 234 "target_auger.ec"
 double db_LambdaErr ;
 
#line 235 "target_auger.ec"
 double db_GHChi2 ;
 
#line 236 "target_auger.ec"
 int db_GHNdf ;
 
#line 237 "target_auger.ec"
 double db_LineFitChi2 ;
 
#line 239 "target_auger.ec"
 double db_EmEnergy ;
 
#line 240 "target_auger.ec"
 double db_EmEnergyErr ;
 
#line 241 "target_auger.ec"
 double db_Energy ;
 
#line 242 "target_auger.ec"
 double db_EnergyErr ;
 
#line 244 "target_auger.ec"
 double db_MinAngle ;
 
#line 245 "target_auger.ec"
 double db_MaxAngle ;
 
#line 246 "target_auger.ec"
 double db_MeanAngle ;
 
#line 248 "target_auger.ec"
 int db_NTank ;
 
#line 249 "target_auger.ec"
 int db_HottestTank ;
 
#line 250 "target_auger.ec"
 double db_AxisDist ;
 
#line 251 "target_auger.ec"
 double db_SDPDist ;
 
#line 253 "target_auger.ec"
 double db_SDFDdT ;
 
#line 254 "target_auger.ec"
 double db_XmaxEyeDist ;
 
#line 255 "target_auger.ec"
 double db_XTrackMin ;
 
#line 256 "target_auger.ec"
 double db_XTrackMax ;
 
#line 257 "target_auger.ec"
 double db_XFOVMin ;
 
#line 258 "target_auger.ec"
 double db_XFOVMax ;
 
#line 259 "target_auger.ec"
 double db_XTrackObs ;
 
#line 260 "target_auger.ec"
 double db_DegTrackObs ;
 
#line 261 "target_auger.ec"
 double db_TTrackObs ;
 
#line 263 "target_auger.ec"
 int db_cut = 0 ;
/* exec sql end declare section */
#line 264 "target_auger.ec"


	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "select auger_t3id , extract ( EPOCH from auger_date ) , ra , dec , cut , eye , run , event , augerid , GPSSec , GPSNSec , SDId , NPix , SDPTheta , SDPThetaErr , SDPPhi , SDPPhiErr , SDPChi2 , SDPNdf , Rp , RpErr , Chi0 , Chi0Err , T0 , T0Err , TimeChi2 , TimeChi2FD , TimeNdf , Easting , Northing , Altitude , NorthingErr , EastingErr , Theta , ThetaErr , Phi , PhiErr , dEdXmax , dEdXmaxErr , X_max , X_maxErr , X0 , X0Err , Lambda , LambdaErr , GHChi2 , GHNdf , LineFitChi2 , EmEnergy , EmEnergyErr , Energy , EnergyErr , MinAngle , MaxAngle , MeanAngle , NTank , HottestTank , AxisDist , SDPDist , SDFDdT , XmaxEyeDist , XTrackMin , XTrackMax , XFOVMin , XFOVMax , XTrackObs , DegTrackObs , TTrackObs from auger where auger_t3id = $1 ", 
	ECPGt_int,&(d_auger_t3id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, 
	ECPGt_int,&(d_auger_t3id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(d_auger_date),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(d_auger_ra),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(d_auger_dec),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(db_cut),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(db_Eye),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(db_Run),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(db_Event),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_varchar,&(db_AugerId),(long)50,(long)1,sizeof(struct varchar_1), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(db_GPSSec),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(db_GPSNSec),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(db_SDId),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(db_NPix),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_SDPTheta),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_SDPThetaErr),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_SDPPhi),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_SDPPhiErr),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_SDPChi2),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(db_SDPNdf),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_Rp),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_RpErr),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_Chi0),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_Chi0Err),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_T0),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_T0Err),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_TimeChi2),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_TimeChi2FD),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(db_TimeNdf),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_Easting),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_Northing),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_Altitude),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_NorthingErr),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_EastingErr),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_Theta),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_ThetaErr),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_Phi),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_PhiErr),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_dEdXmax),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_dEdXmaxErr),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_Xmax),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_XmaxErr),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_X0),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_X0Err),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_Lambda),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_LambdaErr),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_GHChi2),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(db_GHNdf),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_LineFitChi2),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_EmEnergy),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_EmEnergyErr),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_Energy),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_EnergyErr),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_MinAngle),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_MaxAngle),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_MeanAngle),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(db_NTank),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(db_HottestTank),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_AxisDist),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_SDPDist),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_SDFDdT),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_XmaxEyeDist),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_XTrackMin),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_XTrackMax),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_XFOVMin),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_XFOVMax),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_XTrackObs),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_DegTrackObs),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_TTrackObs),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EORT);}
#line 407 "target_auger.ec"


	if (sqlca.sqlcode)
	{
		logMsgDb ("TargetAuger::load", MESSAGE_ERROR);
		{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "close cur_auger", ECPGt_EOIT, ECPGt_EORT);}
#line 412 "target_auger.ec"

		{ ECPGtrans(__LINE__, NULL, "rollback");}
#line 413 "target_auger.ec"

		auger_date = 0;
		std::ostringstream err;
		err << "cannot load auger event for event " << d_auger_t3id;
		throw SqlError (err.str ().c_str ());
	}

	t3id = d_auger_t3id;
	auger_date = d_auger_date;
	setPosition (d_auger_ra, d_auger_dec);

	cut = db_cut;
	Eye = db_Eye;
	Run = db_Run;
	Event = db_Event;
	AugerId = db_AugerId.arr; 
	GPSSec = db_GPSSec; 
	GPSNSec = db_GPSNSec; 
	SDId = db_SDId; 
	NPix = db_NPix;
	SDPTheta = db_SDPTheta;
	SDPThetaErr = db_SDPThetaErr;
	SDPPhi = db_SDPPhi;
	SDPPhiErr = db_SDPPhiErr;
	SDPChi2 = db_SDPChi2;
	SDPNdf = db_SDPNdf; 
	Rp = db_Rp;
	RpErr = db_RpErr;
	Chi0 = db_Chi0;
	Chi0Err = db_Chi0Err;
	T0 = db_T0;
	T0Err = db_T0Err;
	TimeChi2 = db_TimeChi2;
	TimeChi2FD = db_TimeChi2FD;
	TimeNdf = db_TimeNdf;	
	Easting = db_Easting;
	Northing = db_Northing;
	Altitude = db_Altitude;
	NorthingErr = db_NorthingErr;
	EastingErr = db_EastingErr;
	Theta = db_Theta;
	ThetaErr = db_ThetaErr;
	Phi = db_Phi;
	PhiErr = db_PhiErr;
	dEdXmax = db_dEdXmax;
	dEdXmaxErr = db_dEdXmaxErr;
	Xmax = db_Xmax;
	XmaxErr = db_XmaxErr;
	X0 = db_X0;
	X0Err = db_X0Err;
	Lambda = db_Lambda;
	LambdaErr = db_LambdaErr;
	GHChi2 = db_GHChi2;
	GHNdf = db_GHNdf;
	LineFitChi2 = db_LineFitChi2;
	EmEnergy = db_EmEnergy;
	EmEnergyErr = db_EmEnergyErr;
	Energy = db_Energy;
	EnergyErr = db_EnergyErr;
	MinAngle = db_MinAngle;
	MaxAngle = db_MaxAngle;
	MeanAngle = db_MeanAngle;
	NTank = db_NTank;
	HottestTank = db_HottestTank;
	AxisDist = db_AxisDist;
	SDPDist = db_SDPDist;
	SDFDdT = db_SDFDdT;
	XmaxEyeDist = db_XmaxEyeDist;
	XTrackMin = db_XTrackMin;
	XTrackMax = db_XTrackMax;
	XFOVMin = db_XFOVMin;
	XFOVMax = db_XFOVMax;
	XTrackObs = db_XTrackObs;
	DegTrackObs = db_DegTrackObs;
	TTrackObs = db_TTrackObs;

	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "close cur_auger", ECPGt_EOIT, ECPGt_EORT);}
#line 489 "target_auger.ec"

	{ ECPGtrans(__LINE__, NULL, "commit");}
#line 490 "target_auger.ec"

	Target::load ();
}

void TargetAuger::updateShowerFields ()
{
	time_t aug_date = auger_date;
	double JD = ln_get_julian_from_timet (&aug_date);

	struct ln_equ_posn pos;
	struct ln_hrz_posn hrz;

	cor.x = Easting - 459201;
	cor.y = Northing - 6071873;
	cor.z = Altitude - 1422;

	getPosition (&pos, JD);
	ln_get_hrz_from_equ (&pos, observer, JD, &hrz);

	hrz.az = ln_deg_to_rad (hrz.az);
	hrz.alt = ln_deg_to_rad (hrz.alt);

	struct vec cortmp, dir, axis, unit;

	// shower direction vector
	hrz_to_vec (&hrz, &dir);

	// core position vector
	vec_unit (&cor, &cortmp);

	// vector perpendicular to SDP - axis of rotation
	crossProduct (&dir, &cortmp, &axis);
	vec_unit (&axis, &unit);
	axis = unit;

	// current and previous equatorial positions
	struct ln_equ_posn equ,prev_equ;

	dirToEqu (&dir, observer, JD, &equ);
	equ.ra -= pos.ra;
	equ.dec -= pos.dec;
	addShowerOffset (equ);

	vec prev_dir = dir;
	prev_equ = equ;

	double angle = 160 * M_PI / (180 * 60);

	dir = rotateVector (&axis, &dir, angle);

	while (dir.z > 0.)
	{
		dirToEqu (&dir, observer, JD, &equ);
		equ.ra -= pos.ra;
		equ.dec -= pos.dec;

		if (dir.z <= 0.5 && prev_dir.z > 0.5 && prev_dir.z < 0.520016128)
			addShowerOffset (prev_equ);
        	if (dir.z <= 0.5 && dir.z >= 0.034899497)
			addShowerOffset (equ);
        	if (prev_dir.z >= 0.034899497 && dir.z > 0.011635266 && dir.z < 0.034899497)
			addShowerOffset (equ);
		
		prev_dir = dir;
		prev_equ = equ;

		dir = rotateVector (&axis, &dir, angle);
	}
}

void TargetAuger::addShowerOffset (struct ln_equ_posn &pos)
{
	if (fabs (pos.ra) < 1 / 3600.0)
		pos.ra = 0;
	if (fabs (pos.dec) < 1 / 3600.0)
		pos.dec = 0;
	showerOffsets.push_back (pos);
}

bool TargetAuger::getScript (const char *device_name, std::string &buf)
{
	if (showerOffsets.size () == 0)
		updateShowerFields ();

	if (!strcmp (device_name, "WF"))
	{
		std::ostringstream _os;
	 	_os << "filter=B ";
		for (std::vector <struct ln_equ_posn>::iterator iter = showerOffsets.begin (); iter != showerOffsets.end (); iter++)
		{
			if (iter->ra != 0 || iter->dec != 0)
				_os << "PARA.WOFFS=(" << (iter->ra) << "," << (iter->dec) << ") ";
			_os << "E 10 ";
		}
		buf = _os.str ();
		return false;
	}
	buf = std::string ("E 5");
	return false;
}

float TargetAuger::getBonus (double JD)
{
	time_t jd_date;
	ln_get_timet_from_julian (JD, &jd_date);
	// shower too old - observe for 30 minutes..
	if (jd_date > auger_date + augerPriorityTimeout)
		return 1;
	// else return value from DB
	return ConstTarget::getBonus (JD);
}

moveType TargetAuger::afterSlewProcessed ()
{
	/* exec sql begin declare section */
		 
		   
	
#line 605 "target_auger.ec"
 int d_obs_id ;
 
#line 606 "target_auger.ec"
 int d_auger_t3id = t3id ;
/* exec sql end declare section */
#line 607 "target_auger.ec"


	d_obs_id = getObsId ();
	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "insert into auger_observation ( auger_t3id , obs_id ) values ( $1  , $2  )", 
	ECPGt_int,&(d_auger_t3id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(d_obs_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, ECPGt_EORT);}
#line 619 "target_auger.ec"

	if (sqlca.sqlcode)
	{
		logMsgDb ("TargetAuger::startSlew SQL error", MESSAGE_ERROR);
		{ ECPGtrans(__LINE__, NULL, "rollback");}
#line 623 "target_auger.ec"

		return OBS_MOVE_FAILED;
	}
	{ ECPGtrans(__LINE__, NULL, "commit");}
#line 626 "target_auger.ec"

	return OBS_MOVE;
}

int TargetAuger::considerForObserving (double JD)
{
	load ();
	return ConstTarget::considerForObserving (JD);
}

void TargetAuger::printExtra (Rts2InfoValStream & _os, double JD)
{
	std::vector <struct ln_equ_posn> positions;
	getEquPositions (positions);

	int i = 1;

	for (std::vector <struct ln_equ_posn>::iterator iter = positions.begin (); iter != positions.end (); iter++)
	{
		std::ostringstream _osra;
		_osra << "RA " << i;

		std::ostringstream _osdec;
		_osdec << "RA " << i;

		_os
			<< std::endl
			<< InfoVal<LibnovaRaJ2000> (_osra.str ().c_str (), LibnovaRaJ2000 (iter->ra))
			<< InfoVal<LibnovaRaJ2000> (_osdec.str ().c_str (), LibnovaRaJ2000 (iter->dec));

		i++;
	}

	_os
		<< std::endl
		<< InfoVal<int> ("T3ID", t3id)
		<< InfoVal<Timestamp> ("DATE", Timestamp(auger_date))
		<< InfoVal<int> ("NPIXELS", NPix)
		<< InfoVal<double> ("CORE_X", cor.x)
		<< InfoVal<double> ("CORE_Y", cor.y)
		<< InfoVal<double> ("CORE_Z", cor.z)

		<< InfoVal<int> ("Eye", Eye)
		<< InfoVal<int> ("Run", Run)
		<< InfoVal<int> ("Event", Event)
		<< InfoVal<const char*> ("AugerId", AugerId.c_str ())
		<< InfoVal<int> ("GPSSec", GPSSec)
		<< InfoVal<int> ("GPSNSec", GPSNSec)
		<< InfoVal<int> ("SDId", SDId)
	
		<< InfoVal<int> ("NPix", NPix)
	
		<< InfoVal<double> ("SDPTheta", SDPTheta)
		<< InfoVal<double> ("SDPThetaErr", SDPThetaErr)
		<< InfoVal<double> ("SDPPhi", SDPPhi)
		<< InfoVal<double> ("SDPPhiErr", SDPPhiErr)
		<< InfoVal<double> ("SDPChi2", SDPChi2)
		<< InfoVal<int> ("SDPNdf", SDPNdf)
	
		<< InfoVal<double> ("Rp", Rp)
		<< InfoVal<double> ("RpErr", RpErr)
		<< InfoVal<double> ("Chi0", Chi0)
		<< InfoVal<double> ("Chi0Err", Chi0Err)
		<< InfoVal<double> ("T0", T0)
		<< InfoVal<double> ("T0Err", T0Err)
		<< InfoVal<double> ("TimeChi2", TimeChi2)
		<< InfoVal<double> ("TimeChi2FD", TimeChi2FD)
		<< InfoVal<int> ("TimeNdf", TimeNdf)
	
		<< InfoVal<double> ("Easting", Easting)
		<< InfoVal<double> ("Northing", Northing)
		<< InfoVal<double> ("Altitude", Altitude)
		<< InfoVal<double> ("NorthingErr", NorthingErr)
		<< InfoVal<double> ("EastingErr", EastingErr)
		<< InfoVal<double> ("Theta", Theta)
		<< InfoVal<double> ("ThetaErr", ThetaErr)
		<< InfoVal<double> ("Phi", Phi)
		<< InfoVal<double> ("PhiErr", PhiErr)
	
		<< InfoVal<double> ("dEdXmax", dEdXmax)
		<< InfoVal<double> ("dEdXmaxErr", dEdXmaxErr)
		<< InfoVal<double> ("Xmax", Xmax)
		<< InfoVal<double> ("XmaxErr", XmaxErr)
		<< InfoVal<double> ("X0", X0)
		<< InfoVal<double> ("X0Err", X0Err)
		<< InfoVal<double> ("Lambda", Lambda)
		<< InfoVal<double> ("LambdaErr", LambdaErr)
		<< InfoVal<double> ("GHChi2", GHChi2)
		<< InfoVal<int> ("GHNdf", GHNdf)
		<< InfoVal<double> ("LineFitChi2", LineFitChi2)
	
		<< InfoVal<double> ("EmEnergy", EmEnergy)
		<< InfoVal<double> ("EmEnergyErr", EmEnergyErr)
		<< InfoVal<double> ("Energy", Energy)
		<< InfoVal<double> ("EnergyErr", EnergyErr)
	
		<< InfoVal<double> ("MinAngle", MinAngle)
		<< InfoVal<double> ("MaxAngle", MaxAngle)
		<< InfoVal<double> ("MeanAngle", MeanAngle)
	
		<< InfoVal<int> ("NTank", NTank)
		<< InfoVal<int> ("HottestTank", HottestTank)
		<< InfoVal<double> ("AxisDist", AxisDist)
		<< InfoVal<double> ("SDPDist", SDPDist)
	
		<< InfoVal<double> ("SDFDdT", SDFDdT)
		<< InfoVal<double> ("XmaxEyeDist", XmaxEyeDist)
		<< InfoVal<double> ("XTrackMin", XTrackMin)
		<< InfoVal<double> ("XTrackMax", XTrackMax)
		<< InfoVal<double> ("XFOVMin", XFOVMin)
		<< InfoVal<double> ("XFOVMax", XFOVMax)
		<< InfoVal<double> ("XTrackObs", XTrackObs)
		<< InfoVal<double> ("DegTrackObs", DegTrackObs)
		<< InfoVal<double> ("TTrackObs", TTrackObs)

		<< InfoVal<int> ("cut", cut);

	ConstTarget::printExtra (_os, JD);
}

void TargetAuger::printHTMLRow (std::ostringstream &_os, double JD)
{
	struct ln_equ_posn pos;
	struct ln_hrz_posn hrz;

	getPosition (&pos, JD);
	getAltAz (&hrz, JD);

	_os
		<< "<tr><td><a href='" << t3id << "'>" << t3id << "</a></td><td>"
		<< LibnovaRaDec (pos.ra, pos.dec) << "</td><td>"
		<< LibnovaHrz (hrz.alt, hrz.az) << "</td><td>"
		<< Timestamp (auger_date) << "</td><td>"
		<< cor.x << "</td><td>"
		<< cor.y << "</td><td>"
		<< cor.z << "</td><td>"
		<< cor.x + 459201 << "</td><td>"
		<< cor.y + 6071873 << "</td><td>"
		<< cor.z + 1422 << "</td></tr>";
}

void TargetAuger::writeToImage (rts2image::Image * image, double JD)
{
	ConstTarget::writeToImage (image, JD);
	image->setValue ("AGR_T3ID", t3id, "Auger target id");

	image->setValue ("AGR_Eye", Eye, "FD eye Id");
	image->setValue ("AGR_Run", Run, "FD run number");
	image->setValue ("AGR_Event", Event, "FD event number");
	image->setValue ("AGR_AugerId", AugerId.c_str (), "Event Auger Id after SD/FD merger");
	image->setValue ("AGR_GPSSec", GPSSec, "GPS second (SD)");
	image->setValue ("AGR_GPSNSec", GPSNSec, "GPS nano second (SD)");
	image->setValue ("AGR_SDId", SDId, "SD Event Id");
	
	image->setValue ("AGR_NPix", NPix, "Num. pixels with a pulse after FdPulseFinder");
	
	image->setValue ("AGR_SDPTheta", SDPTheta, "Zenith angle of SDP normal vector (deg)");
	image->setValue ("AGR_SDPThetaErr", SDPThetaErr, "Uncertainty of SDPtheta");
	image->setValue ("AGR_SDPPhi", SDPPhi, "Azimuth angle of SDP normal vector (deg)");
	image->setValue ("AGR_SDPPhiErr", SDPPhiErr, "Uncertainty of SDPphi");
	image->setValue ("AGR_SDPChi2", SDPChi2, "Chi^2 of SDP db_it");
	image->setValue ("AGR_SDPNdf", SDPNdf, "Degrees of db_reedom of SDP db_it");
	
	image->setValue ("AGR_Rp", Rp, "Shower impact parameter Rp (m)");
	image->setValue ("AGR_RpErr", RpErr, "Uncertainty of Rp (m)");
	image->setValue ("AGR_Chi0", Chi0, "Angle of shower in the SDP (deg)");
	image->setValue ("AGR_Chi0Err", Chi0Err, "Uncertainty of Chi0 (deg)");
	image->setValue ("AGR_T0", T0, "FD time db_it T_0 (ns)");
	image->setValue ("AGR_T0Err", T0Err, "Uncertainty of T_0 (ns)");
	image->setValue ("AGR_TimeChi2", TimeChi2, "Full Chi^2 of axis db_it");
	image->setValue ("AGR_TimeChi2FD", TimeChi2FD, "Chi^2 of axis db_it (FD only)");
	image->setValue ("AGR_TimeNdf", TimeNdf, "Degrees of db_reedom of axis db_it");
	
	image->setValue ("AGR_Easting", Easting, "Core position in easting coordinate (m)");
	image->setValue ("AGR_Northing", Northing, "Core position in northing coordinate (m)");
	image->setValue ("AGR_Altitude", Altitude, "Core position altitude (m)");
	image->setValue ("AGR_NorthingErr", NorthingErr, "Uncertainty of northing coordinate (m)");
	image->setValue ("AGR_EastingErr", EastingErr, "Uncertainty of easting coordinate (m)");
	image->setValue ("AGR_Theta", Theta, "Shower zenith angle in core coords. (deg)");
	image->setValue ("AGR_ThetaErr", ThetaErr, "Uncertainty of zenith angle (deg)");
	image->setValue ("AGR_Phi", Phi, "Shower azimuth angle in core coords. (deg)");
	image->setValue ("AGR_PhiErr", PhiErr, "Uncertainty of azimuth angle (deg)");
	
	image->setValue ("AGR_dEdXmax", dEdXmax, "Energy deposit at shower max (GeV/(g/cm^2))");
	image->setValue ("AGR_dEdXmaxErr", dEdXmaxErr, "Uncertainty of Nmax (GeV/(g/cm^2))");
	image->setValue ("AGR_Xmax", Xmax, "Slant depth of shower maximum (g/cm^2)");
	image->setValue ("AGR_XmaxErr", XmaxErr, "Uncertainty of Xmax (g/cm^2)");
	image->setValue ("AGR_X0", X0, "X0 Gaisser-Hillas db_it (g/cm^2)");
	image->setValue ("AGR_X0Err", X0Err, "Uncertainty of X0 (g/cm^2)");
	image->setValue ("AGR_Lambda", Lambda, "Lambda of Gaisser-Hillas db_it (g/cm^2)");
	image->setValue ("AGR_LambdaErr", LambdaErr, "Uncertainty of Lambda (g/cm^2)");
	image->setValue ("AGR_GHChi2", GHChi2, "Chi^2 of Gaisser-Hillas db_it");
	image->setValue ("AGR_GHNdf", GHNdf, "Degrees of db_reedom of GH db_it");
	image->setValue ("AGR_LineFitChi2", LineFitChi2, "Chi^2 of linear db_it to profile");
	
	image->setValue ("AGR_EmEnergy", EmEnergy, "Calorimetric energy db_rom GH db_it (EeV)");
	image->setValue ("AGR_EmEnergyErr", EmEnergyErr, "Uncertainty of Eem (EeV)");
	image->setValue ("AGR_Energy", Energy, "Total energy db_rom GH db_it (EeV)");
	image->setValue ("AGR_EnergyErr", EnergyErr, "Uncertainty of Etot (EeV)");
	
	image->setValue ("AGR_MinAngle", MinAngle, "Minimum viewing angle (deg)");
	image->setValue ("AGR_MaxAngle", MaxAngle, "Maximum viewing angle (deg)");
	image->setValue ("AGR_MeanAngle", MeanAngle, "Mean viewing angle (deg)");
	
	image->setValue ("AGR_NTank", NTank, "Number of stations in hybrid db_it");
	image->setValue ("AGR_HottestTank", HottestTank, "Station used in hybrid-geometry reco");
	image->setValue ("AGR_AxisDist", AxisDist, "Shower axis distance to hottest station (m)");
	image->setValue ("AGR_SDPDist", SDPDist, "SDP distance to hottest station (m)");
	
	image->setValue ("AGR_SDFDdT", SDFDdT, "SD/FD time offset after the minimization (ns)");
	image->setValue ("AGR_XmaxEyeDist", XmaxEyeDist, "Distance to shower maximum (m)");
	image->setValue ("AGR_XTrackMin", XTrackMin, "First recorded slant depth of track (g/cm^2)");
	image->setValue ("AGR_XTrackMax", XTrackMax, "Last recorded slant depth of track (g/cm^2)");
	image->setValue ("AGR_XFOVMin", XFOVMin, "First slant depth inside FOV (g/cm^2)");
	image->setValue ("AGR_XFOVMax", XFOVMax, "Last slant depth inside FOV (g/cm^2)");
	image->setValue ("AGR_XTrackObs", XTrackObs, "Observed track length depth (g/cm^2)");
	image->setValue ("AGR_DegTrackObs", DegTrackObs, "Observed track length angle (deg)");
	image->setValue ("AGR_TTrackObs", TTrackObs, "Observed track length time (100 ns)");

	image->setValue ("AGR_cut", cut, "Cuts pased by shower");
}

void TargetAuger::getEquPositions (std::vector <struct ln_equ_posn> &positions)
{
	if (showerOffsets.size () == 0)
		updateShowerFields ();
	struct ln_equ_posn pos;
	getPosition (&pos, ln_get_julian_from_sys ());
	// calculate offsets
	for (std::vector <struct ln_equ_posn>::iterator iter = showerOffsets.begin (); iter != showerOffsets.end (); iter++)
	{
		struct ln_equ_posn sp;
		sp.ra = pos.ra + iter->ra;
		sp.dec = pos.dec + iter->dec;
		positions.push_back (sp);
	}
}
