/* Processed by ecpg (4.7.0) */
/* These include files are added by the preprocessor */
#include <ecpglib.h>
#include <ecpgerrno.h>
#include <sqlca.h>
/* End of automatic include section */

#line 1 "targetgrb.ec"
/*
 * Class for GRB target.
 * Copyright (C) 2005-2007 Petr Kubanek <petr@kubanek.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "targetgrb.h"

#include "configuration.h"
#include "libnova_cpp.h"
#include "infoval.h"

#include "rts2db/sqlerror.h"
#include "rts2fits/image.h"

using namespace rts2db;

TargetGRB::TargetGRB (int in_tar_id, struct ln_lnlat_posn *in_obs, int in_maxBonusTimeout, int in_dayBonusTimeout, int in_fiveBonusTimeout):ConstTarget (in_tar_id, in_obs)
{
	shouldUpdate = 1;
	grb_is_grb = true;
	gcnPacketType = -1;
	maxBonusTimeout = in_maxBonusTimeout;
	dayBonusTimeout = in_dayBonusTimeout;
	fiveBonusTimeout = in_fiveBonusTimeout;

	grb.ra = rts2_nan("f");
	grb.dec = rts2_nan("f");
	errorbox = rts2_nan("f");
	autodisabled = false;
}

void TargetGRB::load ()
{
	/* exec sql begin declare section */
	  
	  
	 
	 
	 
	    
	 
	 
	 
	 
	 
	
#line 49 "targetgrb.ec"
 double db_grb_date ;
 
#line 50 "targetgrb.ec"
 double db_grb_last_update ;
 
#line 51 "targetgrb.ec"
 int db_grb_type ;
 
#line 52 "targetgrb.ec"
 int db_grb_id ;
 
#line 53 "targetgrb.ec"
 bool db_grb_is_grb ;
 
#line 54 "targetgrb.ec"
 int db_tar_id = getTargetID () ;
 
#line 55 "targetgrb.ec"
 double db_grb_ra ;
 
#line 56 "targetgrb.ec"
 double db_grb_dec ;
 
#line 57 "targetgrb.ec"
 double db_grb_errorbox ;
 
#line 58 "targetgrb.ec"
 int db_grb_errorbox_ind ;
 
#line 59 "targetgrb.ec"
 bool db_grb_autodisabled ;
/* exec sql end declare section */
#line 60 "targetgrb.ec"


	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "select extract ( EPOCH from grb_date ) , extract ( EPOCH from grb_last_update ) , grb_type , grb_id , grb_is_grb , grb_ra , grb_dec , grb_errorbox , grb_autodisabled from grb where tar_id = $1 ", 
	ECPGt_int,&(db_tar_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, 
	ECPGt_double,&(db_grb_date),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_grb_last_update),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(db_grb_type),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(db_grb_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_bool,&(db_grb_is_grb),(long)1,(long)1,sizeof(bool), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_grb_ra),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_grb_dec),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_grb_errorbox),(long)1,(long)1,sizeof(double), 
	ECPGt_int,&(db_grb_errorbox_ind),(long)1,(long)1,sizeof(int), 
	ECPGt_bool,&(db_grb_autodisabled),(long)1,(long)1,sizeof(bool), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EORT);}
#line 86 "targetgrb.ec"

	if (sqlca.sqlcode)
	{
	  	std::ostringstream err;
		err << "cannot load GRB data for target ID " << db_tar_id;
	  	throw SqlError (err.str ().c_str ());
	}
	grbDate = db_grb_date;
	// we don't expect grbDate to change much during observation,
	// so we will not update that in beforeMove (or somewhere else)
	lastUpdate = db_grb_last_update;
	gcnPacketType = db_grb_type;
	// switch of packet type - packet class
	if (gcnPacketType >= 40 && gcnPacketType <= 45)
	{
		// HETE burst
		gcnPacketMin = 40;
		gcnPacketMax = 45;
	}
	else if (gcnPacketType >= 50 && gcnPacketType <= 55)
	{
		// INTEGRAL burst
		gcnPacketMin = 50;
		gcnPacketMax = 55;
	}
	else if (gcnPacketType >= 60 && gcnPacketType <= 90)
	{
		// SWIFT burst
		gcnPacketMin = 60;
		gcnPacketMax = 90;
	}
	else if (gcnPacketType >= 100 && gcnPacketType < 110)
	{
		// AGILE
		gcnPacketMin = 100;
		gcnPacketMax = 109;
	}
	else if (gcnPacketType >= 110 && gcnPacketType <= 150)
	{
		// GLAST
		gcnPacketMin = 110;
		gcnPacketMax = 150;
	}
	else
	{
		// UNKNOW
		gcnPacketMin = 0;
		gcnPacketMax = 1000;
	}

	gcnGrbId = db_grb_id;
	grb_is_grb = db_grb_is_grb;
	grb.ra = db_grb_ra;
	grb.dec = db_grb_dec;
	if (db_grb_errorbox_ind)
		errorbox = rts2_nan ("f");
	else
		errorbox = db_grb_errorbox;
	shouldUpdate = 0;
	autodisabled = db_grb_autodisabled;

	// check if we are still valid target
	checkValidity ();

	ConstTarget::load ();
}

void TargetGRB::getPosition (struct ln_equ_posn *pos, double JD)
{
	time_t now;
	ln_get_timet_from_julian (JD, &now);
	// it's SWIFT ALERT, ra&dec are S/C ra&dec
	if (gcnPacketType == 60 && (now - lastUpdate < 3600))
	{
		struct ln_hrz_posn hrz;
		getAltAz (&hrz, JD);
		if (hrz.alt > -20 && hrz.alt < 35)
		{
			hrz.alt = 35;
			ln_get_equ_from_hrz (&hrz, observer, JD, pos);
			return;
		}
	}
	ConstTarget::getPosition (pos, JD);
}

int TargetGRB::compareWithTarget (Target *in_target, double in_sep_limit)
{
	// when it's SWIFT GRB and we are currently observing Swift burst, don't interrupt us
	// that's good only for WF instruments which observe Swift FOV
	if (in_target->getTargetID () == TARGET_SWIFT_FOV && gcnPacketType >= 60 && gcnPacketType <= 90)
	{
		return ConstTarget::compareWithTarget (in_target, 4 * in_sep_limit);
	}
	return ConstTarget::compareWithTarget (in_target, in_sep_limit);
}

double TargetGRB::getPostSec ()
{
	time_t now;
	time (&now);
	return now - (time_t) getGrbDate ();
}

void TargetGRB::checkValidity ()
{
	/* exec sql begin declare section */
	    
	
#line 193 "targetgrb.ec"
 int db_tar_id = getTargetID () ;
/* exec sql end declare section */
#line 194 "targetgrb.ec"

	// do not auto disable already disabled targets
	if (autodisabled == true)
		return;

	bool autodisable = false;
	if (isGrb () == false && rts2core::Configuration::instance()->grbdFollowTransients () == false)
	{
		if (getTargetEnabled () == true)
		{
 			logStream (MESSAGE_INFO) << "disabling GRB target " << getTargetName () << " (#" << getObsTargetID () << ") as it is fake GRB." << sendLog;
			autodisable = true;
		}
	}
	if (rts2core::Configuration::instance()->grbdValidity () > 0 && getPostSec () > rts2core::Configuration::instance()->grbdValidity ())
	{
		if (getTargetEnabled () == true)
		{
			logStream (MESSAGE_INFO) << "disabling GRB target " << getTargetName () << " (#" << getObsTargetID () << ") because it is too late after GRB." << sendLog;
			autodisable = true;
		}
	}

	if (autodisable)
	{
		setTargetEnabled (false);
		{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "update targets set tar_enabled = false where tar_id = $1 ", 
	ECPGt_int,&(db_tar_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, ECPGt_EORT);}
#line 220 "targetgrb.ec"

		{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "update grb set grb_autodisabled = true where tar_id = $1 ", 
	ECPGt_int,&(db_tar_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, ECPGt_EORT);}
#line 221 "targetgrb.ec"

		if (sqlca.sqlcode)
			throw SqlError ();
	}
}

void TargetGRB::getDBScript (const char *camera_name, std::string &script)
{
	/* exec sql begin declare section */
	 
	 
	 
	 
	
#line 230 "targetgrb.ec"
 int d_post_sec ;
 
#line 231 "targetgrb.ec"
 int d_pos_ind ;
 
#line 232 "targetgrb.ec"
  struct varchar_1  { int len; char arr[ 2000 ]; }  sc_script ;
 
#line 233 "targetgrb.ec"
  struct varchar_2  { int len; char arr[ 8 ]; }  d_camera_name ;
/* exec sql end declare section */
#line 234 "targetgrb.ec"


	// use target script, if such exist..
	try
	{
		ConstTarget::getDBScript (camera_name, script);
		return;
	}
	catch (rts2core::Error &er)
	{
	}

	d_camera_name.len = strlen (camera_name);
	strncpy (d_camera_name.arr, camera_name, d_camera_name.len);

	// grb_script_script is NOT NULL

	ECPGset_var( 0, &( d_camera_name ), __LINE__);\
 /* declare find_grb_script cursor for select grb_script_end , grb_script_script from grb_script where camera_name = $1  order by grb_script_end asc */
#line 260 "targetgrb.ec"

	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "declare find_grb_script cursor for select grb_script_end , grb_script_script from grb_script where camera_name = $1  order by grb_script_end asc", 
	ECPGt_varchar,&(d_camera_name),(long)8,(long)1,sizeof(struct varchar_2), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, ECPGt_EORT);}
#line 261 "targetgrb.ec"

	while (1)
	{
		{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "fetch next from find_grb_script", ECPGt_EOIT, 
	ECPGt_int,&(d_post_sec),(long)1,(long)1,sizeof(int), 
	ECPGt_int,&(d_pos_ind),(long)1,(long)1,sizeof(int), 
	ECPGt_varchar,&(sc_script),(long)2000,(long)1,sizeof(struct varchar_1), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EORT);}
#line 266 "targetgrb.ec"

		if (sqlca.sqlcode)
			break;
		if (getPostSec () < d_post_sec || d_pos_ind < 0)
		{
			sc_script.arr[sc_script.len] = '\0';
			script = std::string (sc_script.arr);
			break;
		}
	}
	if (sqlca.sqlcode)
	{
		{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "close find_grb_script", ECPGt_EOIT, ECPGt_EORT);}
#line 278 "targetgrb.ec"

		{ ECPGtrans(__LINE__, NULL, "rollback");}
#line 279 "targetgrb.ec"

		throw rts2db::SqlError ();
	}
	sc_script.arr[sc_script.len] = '\0';
	script  = std::string (sc_script.arr);
	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "close find_grb_script", ECPGt_EOIT, ECPGt_EORT);}
#line 284 "targetgrb.ec"

	{ ECPGtrans(__LINE__, NULL, "commit");}
#line 285 "targetgrb.ec"

}

bool TargetGRB::getScript (const char *deviceName, std::string &buf)
{
	time_t now;

	// try to find values first in DB..
	try
	{
		getDBScript (deviceName, buf);
		return false;
	}
	catch (rts2core::Error &er)
	{
	}

	time (&now);

	// switch based on time after burst..
	// that one has to be hard-coded, there is no way how to
	// specify it from config (there can be some, but it will be rather
	// horible way)

	if (now - (time_t) grbDate < 1000)
	{
		buf = std::string ("for 4 { E 10 } ");
	}
	else
	{
		buf = std::string ("for 4 { E 20 } ");
	}
	return false;
}

int TargetGRB::beforeMove ()
{
	// update our position..
	if (shouldUpdate)
		endObservation (-1);
	load ();
	return 0;
}

float TargetGRB::getBonus (double JD)
{
	// time from GRB
	time_t now;
	float retBonus;
	ln_get_timet_from_julian (JD, &now);
	// special SWIFT handling..
	// last packet we have is SWIFT_ALERT..
	switch (gcnPacketType)
	{
		case 60:
			// we don't get ACK | position within 1 hour..drop our priority back to some mimimum value
			if (now - (time_t) grbDate > 3600)
				return 1;
			break;
		case 62:
			if (now - lastUpdate > 1800)
				return -1;
			break;
	}
	if (now - (time_t) grbDate < maxBonusTimeout)
	{
		// < 1 hour fixed boost to be sure to not miss it
		retBonus = ConstTarget::getBonus (JD) + 2000;
	}
	else if (now - (time_t) grbDate < dayBonusTimeout)
	{
		// < 24 hour post burst
		retBonus = ConstTarget::getBonus (JD)
			+ 1000.0 * cos ((double)((double)(now - (time_t) grbDate) / (2.0*86400.0)))
			+ 10.0 * cos ((double)((double)(now - lastUpdate) / (2.0*86400.0)));
	}
	else if (now - (time_t) grbDate < fiveBonusTimeout)
	{
		// < 5 days post burst - add some time after last observation
		retBonus = ConstTarget::getBonus (JD)
			+ 10.0 * sin (getLastObsTime () / 3600.0);
	}
	else
	{
		// default
		retBonus = ConstTarget::getBonus (JD);
	}
	// if it was not GRB..
	if (grb_is_grb == false)
		retBonus /= 2;
	return retBonus;
}

int TargetGRB::isContinues ()
{
	/* exec sql begin declare section */
	    
	 
	 
	
#line 381 "targetgrb.ec"
 int db_tar_id = getTargetID () ;
 
#line 382 "targetgrb.ec"
 double db_tar_ra ;
 
#line 383 "targetgrb.ec"
 double db_tar_dec ;
/* exec sql end declare section */
#line 384 "targetgrb.ec"


	struct ln_equ_posn pos;

	// once we detect need to update, we need to update - stop the observation and start new
	if (shouldUpdate)
		return 0;

	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "select tar_ra , tar_dec from grb , targets where targets . tar_id = $1  and targets . tar_id = grb . tar_id", 
	ECPGt_int,&(db_tar_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, 
	ECPGt_double,&(db_tar_ra),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(db_tar_dec),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EORT);}
#line 403 "targetgrb.ec"


	if (sqlca.sqlcode)
	{
		logMsgDb ("TargetGRB::isContinues", MESSAGE_ERROR);
		return -1;
	}
	getPosition (&pos, ln_get_julian_from_sys ());
	if (pos.ra == db_tar_ra && pos.dec == db_tar_dec)
		return 1;
	// we get big update, lets synchronize again..
	shouldUpdate = 1;
	return 0;
}

double TargetGRB::getFirstPacket ()
{
	/* exec sql begin declare section */
		   
		   
		   
		 
		 
	
#line 421 "targetgrb.ec"
 int db_grb_id = gcnGrbId ;
 
#line 422 "targetgrb.ec"
 int db_grb_type_min = gcnPacketMin ;
 
#line 423 "targetgrb.ec"
 int db_grb_type_max = gcnPacketMax ;
 
#line 424 "targetgrb.ec"
 long db_grb_update ;
 
#line 425 "targetgrb.ec"
 long db_grb_update_usec ;
/* exec sql end declare section */
#line 426 "targetgrb.ec"

	ECPGset_var( 1, &( db_grb_type_max ), __LINE__);\
 ECPGset_var( 2, &( db_grb_type_min ), __LINE__);\
 ECPGset_var( 3, &( db_grb_id ), __LINE__);\
 /* declare cur_grb_first_packet cursor for select extract ( EPOCH from grb_update ) , grb_update_usec from grb_gcn where grb_id = $1  and grb_type between $2  and $3  order by grb_update asc , grb_update_usec asc */
#line 438 "targetgrb.ec"

	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "declare cur_grb_first_packet cursor for select extract ( EPOCH from grb_update ) , grb_update_usec from grb_gcn where grb_id = $1  and grb_type between $2  and $3  order by grb_update asc , grb_update_usec asc", 
	ECPGt_int,&(db_grb_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(db_grb_type_min),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(db_grb_type_max),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, ECPGt_EORT);}
#line 439 "targetgrb.ec"

	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "fetch next from cur_grb_first_packet", ECPGt_EOIT, 
	ECPGt_long,&(db_grb_update),(long)1,(long)1,sizeof(long), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_long,&(db_grb_update_usec),(long)1,(long)1,sizeof(long), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EORT);}
#line 442 "targetgrb.ec"

	if (sqlca.sqlcode)
	{
		{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "close cur_grb_first_packet", ECPGt_EOIT, ECPGt_EORT);}
#line 445 "targetgrb.ec"

		{ ECPGtrans(__LINE__, NULL, "rollback");}
#line 446 "targetgrb.ec"

		return rts2_nan ("f");
	}
	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "close cur_grb_first_packet", ECPGt_EOIT, ECPGt_EORT);}
#line 449 "targetgrb.ec"

	{ ECPGtrans(__LINE__, NULL, "rollback");}
#line 450 "targetgrb.ec"

	return db_grb_update + (double) db_grb_update_usec / USEC_SEC;
}

const char * TargetGRB::getSatelite ()
{
	// get satelite
	if (gcnPacketMin == 40)
		return "HETE BURST ";
	else if (gcnPacketMin == 50)
		return "INTEGRAL BURST ";
	else if (gcnPacketMin == 60)
	{
		if (gcnPacketType >= 81)
			return "SWIFT BURST (UVOT)";
		if (gcnPacketType >= 67)
			return "SWIFT BURST (XRT)";
		return "SWIFT BURST (BAT)";
	}
	else if (gcnPacketMin == 100)
	{
		return "AGILE";
	}
	else if (gcnPacketMin == 110)
	{
		if (gcnPacketType >= 120)
			return "FERMI (LAT)";
		return "FERMI (GBM)";
	}
	return "Unknow type";
}

void TargetGRB::printExtra (Rts2InfoValStream &_os, double JD)
{
	double firstPacket = getFirstPacket ();
	double firstObs = getFirstObs ();
	ConstTarget::printExtra (_os, JD);
	_os
		<< getSatelite () << std::endl
		<< InfoVal<int> ("TYPE", gcnPacketType)
		<< (isGrb () ? "IS GRB flag is set" : "not GRB - is grb flag is not set") << std::endl
		<< InfoVal<Timestamp> ("GRB DATE", Timestamp (grbDate))
		<< InfoVal<Timestamp> ("GCN FIRST PACKET", Timestamp (firstPacket))
		<< InfoVal<TimeDiff> ("GRB->GCN", TimeDiff (grbDate, firstPacket))
		<< InfoVal<Timestamp> ("GCN LAST UPDATE", Timestamp (lastUpdate))
		<< InfoVal<LibnovaRaJ2000> ("GRB RA", LibnovaRaJ2000 (grb.ra))
		<< InfoVal<LibnovaDecJ2000> ("GRB DEC", LibnovaDecJ2000 (grb.dec))
		<< InfoVal<LibnovaDegDist> ("GRB ERR", LibnovaDegDist (errorbox))
		<< InfoVal<bool> ("AUTODISABLED", autodisabled)
		<< std::endl;
	// get information about obsering time..
	if (isnan (firstObs))
	{
		_os << "GRB without observation";
	}
	else
	{
		_os
			<< InfoVal<Timestamp> ("FIRST OBSERVATION", Timestamp (firstObs))
			<< InfoVal<TimeDiff> ("GRB delta", TimeDiff (grbDate, firstObs))
			<< InfoVal<TimeDiff> ("GCN delta", TimeDiff (firstPacket, firstObs));
	}
	_os << std::endl;
}

void TargetGRB::printDS9Reg (std::ostream & _os, double JD)
{
	struct ln_equ_posn pos;
	getPosition (&pos, JD);
	_os << "circle(" << pos.ra << "," << pos.dec << "," << errorbox << ")" << std::endl;
}

void TargetGRB::printGrbList (std::ostream & _os)
{
	struct ln_equ_posn pos;
	double firstObs = getFirstObs ();
	getPosition (&pos, ln_get_julian_from_sys ());
	_os
		<< std::setw(5) << getTargetID () << SEP
		<< LibnovaRa (pos.ra) << SEP
		<< LibnovaDec (pos.dec) << SEP
		<< Timestamp (grbDate) << SEP
		<< std::setw(15) << TimeDiff (grbDate, firstObs) << SEP
		<< std::setw(15) << TimeDiff (getFirstPacket (), firstObs) << SEP
		<< Timestamp (getLastObs ())
		<< std::endl;
}

void TargetGRB::writeToImage (rts2image::Image * image, double JD)
{
	ConstTarget::writeToImage (image, JD);
	image->setValue ("GRB_RA", grb.ra, "GRB RA know at time of exposure");
	image->setValue ("GRB_DEC", grb.dec, "GRB DEC know at time of exposure");
	image->setValue ("GRB_ERR", errorbox, "GRB errorbox diameter");
}
