/* Processed by ecpg (4.7.0) */
/* These include files are added by the preprocessor */
#include <ecpglib.h>
#include <ecpgerrno.h>
#include <sqlca.h>
/* End of automatic include section */

#line 1 "conngrb.ec"
/* 
 * GCN socket connection.
 * Copyright (C) 2003-2008 Petr Kubanek <petr@kubanek.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "conngrb.h"
#include "libnova_cpp.h"

#include "connection/fork.h"
#include "rts2db/sqlerror.h"

#include <arpa/inet.h>
#include <errno.h>
#include <netdb.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <fcntl.h>
#include <unistd.h>

#include <libnova/libnova.h>


#line 1 "/usr/include/sqlca.h"
#ifndef POSTGRES_SQLCA_H
#define POSTGRES_SQLCA_H

#ifndef PGDLLIMPORT
#if  defined(WIN32) || defined(__CYGWIN__)
#define PGDLLIMPORT __declspec (dllimport)
#else
#define PGDLLIMPORT
#endif   /* __CYGWIN__ */
#endif   /* PGDLLIMPORT */

#define SQLERRMC_LEN	150

#ifdef __cplusplus
extern		"C"
{
#endif

struct sqlca_t
{
	char		sqlcaid[8];
	long		sqlabc;
	long		sqlcode;
	struct
	{
		int			sqlerrml;
		char		sqlerrmc[SQLERRMC_LEN];
	}			sqlerrm;
	char		sqlerrp[8];
	long		sqlerrd[6];
	/* Element 0: empty						*/
	/* 1: OID of processed tuple if applicable			*/
	/* 2: number of rows processed				*/
	/* after an INSERT, UPDATE or				*/
	/* DELETE statement					*/
	/* 3: empty						*/
	/* 4: empty						*/
	/* 5: empty						*/
	char		sqlwarn[8];
	/* Element 0: set to 'W' if at least one other is 'W'	*/
	/* 1: if 'W' at least one character string		*/
	/* value was truncated when it was			*/
	/* stored into a host variable.				*/

	/*
	 * 2: if 'W' a (hopefully) non-fatal notice occurred
	 */	/* 3: empty */
	/* 4: empty						*/
	/* 5: empty						*/
	/* 6: empty						*/
	/* 7: empty						*/

	char		sqlstate[5];
};

struct sqlca_t *ECPGget_sqlca(void);

#ifndef POSTGRES_ECPG_INTERNAL
#define sqlca (*ECPGget_sqlca())
#endif

#ifdef __cplusplus
}
#endif

#endif

#line 37 "conngrb.ec"


using namespace rts2grbd;

double ConnGrb::getPktSod ()
{
	return lbuf[PKT_SOD]/100.0;
}

void ConnGrb::getTimeTfromTJD (long TJD, double SOD, time_t * in_time, long * usec)
{
	double JD = getJDfromTJD (TJD, SOD);
	ln_get_timet_from_julian (JD, in_time);
	if (usec)
		*usec = (int)((SOD - int (SOD)) * USEC_SEC);
}

int ConnGrb::pr_test ()
{
	logStream (MESSAGE_INFO) << "Test GCN notice Trig#: " << (lbuf[BURST_TRIG])
		<< " TJD: " << (lbuf[BURST_TJD])
		<< " RA: " << (lbuf[BURST_RA] / 10000.0)
		<< " DEC: " << (lbuf[BURST_DEC] / 10000.0)
		<< " Error: " << (lbuf[BURST_ERROR])
		<< sendLog;
	return 0;
}

int ConnGrb::pr_imalive ()
{
	deltaValue = here_sod - getPktSod ();
	logStream (MESSAGE_DEBUG) << "ConnGrb::pr_imalive last packet SN="
		<< getPktSod ()
		<< " delta=" << deltaValue
		<< " last_delta=" << (getPktSod () - last_imalive_sod) << sendLog;
	last_imalive_sod = getPktSod ();
	return 0;
}

int ConnGrb::pr_swift_point ()
{
	double roll;
	char *obs_name;
	float obstime;
	float merit;
	swiftLastRa  = lbuf[7]/10000.0;
	swiftLastDec = lbuf[8]/10000.0;
	roll = lbuf[9]/10000.0;
	getTimeTfromTJD (lbuf[5], lbuf[6]/100.0, &swiftLastPoint);
	obs_name = (char*) (&lbuf[BURST_URL]);
	obstime = lbuf[14]/100.0;
	merit = lbuf[15]/100.0;
	return addSwiftPoint (roll, obs_name, obstime, merit);
}

int ConnGrb::pr_integral_point ()
{
	struct ln_equ_posn pos_int, pos_j2000;
	time_t t;
	pos_int.ra = lbuf[14]/10000.0;
	pos_int.dec = lbuf[15]/10000.0;
	// precess to J2000
	ln_get_equ_prec2 (&pos_int, ln_get_julian_from_sys (), JD2000, &pos_j2000);
	getTimeTfromTJD (lbuf[5], lbuf[6]/100.0, &t);
	return addIntegralPoint (pos_j2000.ra, pos_j2000.dec, &t);
}

int ConnGrb::pr_agile_point ()
{
}

int ConnGrb::pr_fermi_point ()
{
}

int ConnGrb::pr_hete ()
{
	int grb_id;
	int grb_seqn;
	int grb_type;
	struct ln_equ_posn pos_int, pos_j2000;
	int grb_is_grb = 1;
	time_t grb_date;
	long grb_date_usec;
	float grb_errorbox;

	grb_id = ((lbuf[BURST_TRIG] & H_TRIGNUM_MASK) >> H_TRIGNUM_SHIFT);
	grb_seqn = ((lbuf[BURST_TRIG] & H_SEQNUM_MASK) >> H_SEQNUM_SHIFT);
	grb_type = lbuf[PKT_TYPE];

	pos_int.ra = lbuf[BURST_RA] / 10000.0;
	pos_int.dec = lbuf[BURST_DEC] / 10000.0;

	getTimeTfromTJD (lbuf[BURST_TJD], lbuf[BURST_SOD]/100.0, &grb_date, &grb_date_usec);

	grb_errorbox = (lbuf[H_WXM_DIM_NSIG] >> 16) / 3600.0;

	if (!do_hete_test
		&& (grb_type == TYPE_HETE_TEST
		|| (lbuf[H_TRIG_FLAGS] & H_ART_TRIG)
		)
		)
	{
		logStream (MESSAGE_DEBUG) << "ConnGrb::pr_hete test packet" << sendLog;
		return 0;
	}

	// convert to J2000 only when it's true GRB
	// HETE non GRB notices (GRB retraction notices) have ra and dec -999.99, and we
	// need to pass that value futher to addGcnPoint, so it will not update RA & DEC in DB.
	if (pos_int.ra > -300 && pos_int.dec > -300)
	{
		ln_get_equ_prec2 (&pos_int, ln_get_julian_from_timet (&grb_date), JD2000, &pos_j2000);
	}
	else
	{
		pos_j2000.ra = pos_int.ra;
		pos_j2000.dec = pos_int.dec;
	}

	if ((lbuf[H_TRIG_FLAGS] & H_DEF_NOT_GRB)
		|| (lbuf[H_TRIG_FLAGS] & H_DEF_SGR)
		|| (lbuf[H_TRIG_FLAGS] & H_DEF_XRB))
		grb_is_grb = 0;

	return addGcnPoint (grb_id, grb_seqn, grb_type, pos_j2000.ra, pos_j2000.dec, grb_is_grb, &grb_date, grb_date_usec, grb_errorbox, false, true);
}

int ConnGrb::pr_integral ()
{
	int grb_id;
	int grb_seqn;
	int grb_type;
	struct ln_equ_posn pos_int, pos_j2000;
	int grb_is_grb = 1;
	time_t grb_date;
	long grb_date_usec;
	float grb_errorbox;

	if (!do_hete_test
		&& ((lbuf[12] & (1 << 31))
		)
		)
	{
		logStream (MESSAGE_DEBUG) << "ConnGrb::pr_integral test packet (" << lbuf[12] << ")" << sendLog;
		return 0;
	}

	grb_id = (lbuf[BURST_TRIG] & I_TRIGNUM_MASK) >> I_TRIGNUM_SHIFT;
	grb_seqn = (lbuf[BURST_TRIG] & I_SEQNUM_MASK) >> I_SEQNUM_SHIFT;
	grb_type = lbuf[PKT_TYPE];

	pos_int.ra = lbuf[BURST_RA]/10000.0;
	pos_int.dec = lbuf[BURST_DEC]/10000.0;

	getTimeTfromTJD (lbuf[BURST_TJD], lbuf[BURST_SOD]/100.0, &grb_date, &grb_date_usec);

	ln_get_equ_prec2 (&pos_int, ln_get_julian_from_timet (&grb_date), JD2000, &pos_j2000);

	grb_errorbox = (float) lbuf[BURST_ERROR] / 3600.0;

	if (grb_errorbox < 0 && grb_type == TYPE_INTEGRAL_OFFLINE_SRC)
	{
		grb_is_grb = 0;
		grb_errorbox *= -1;
	}

	return addGcnPoint (grb_id, grb_seqn, grb_type, pos_j2000.ra, pos_j2000.dec, grb_is_grb, &grb_date, grb_date_usec, grb_errorbox, false, true);
}

int ConnGrb::pr_integral_spicas ()
{
	logStream (MESSAGE_INFO) << "INTEGRAL SPIACS" << sendLog;
	return 0;
}

int ConnGrb::pr_swift_with_radec ()
{
	int grb_id;
	int grb_seqn;
	int grb_type;
	double grb_ra;
	double grb_dec;
	int grb_is_grb = 1;
	time_t grb_date;
	long grb_date_usec;
	float grb_errorbox;

	grb_type = (int) (lbuf[PKT_TYPE]);
	grb_id = (lbuf[BURST_TRIG] >> S_TRIGNUM_SHIFT) & S_TRIGNUM_MASK;
	grb_seqn = (lbuf[BURST_TRIG] >> S_SEGNUM_SHIFT) & S_SEGNUM_MASK;
	grb_ra = lbuf[BURST_RA] / 10000.0;
	grb_dec = lbuf[BURST_DEC] / 10000.0;

	// we will set grb_is_grb to true in some special cases..
	switch (grb_type)
	{
		case TYPE_SWIFT_BAT_GRB_POS_ACK_SRC:
		case TYPE_SWIFT_BAT_GRB_LC_SRC:
		case TYPE_SWIFT_SCALEDMAP_SRC:
			if ((lbuf[TRIGGER_ID] & 0x00000002)
				&& ((lbuf[TRIGGER_ID] & 0x00000020) == 0)
				&& !(lbuf[TRIGGER_ID] & 0x00000100))
			{
				grb_is_grb = 1;
			}
			else
			{
				grb_is_grb = 0;
			}
			break;
		case TYPE_SWIFT_XRT_POSITION_SRC:
			// if it's not a grb, or if its in ground cat, ignore it..
			if ((lbuf[TRIGGER_ID] & 0x00000020)
				|| (lbuf[TRIGGER_ID] & 0x00000100))
			{
				grb_is_grb = 0;
			}
			break;
	}

	getTimeTfromTJD (lbuf[BURST_TJD], lbuf[BURST_SOD]/100.0, &grb_date, &grb_date_usec);
	switch (grb_type)
	{
		case TYPE_SWIFT_BAT_GRB_POS_ACK_SRC:
		case TYPE_SWIFT_XRT_POSITION_SRC:
		case TYPE_SWIFT_UVOT_POS_SRC:
			grb_errorbox = (float) lbuf[BURST_ERROR] / 10000.0;
			break;
		default:
			grb_errorbox = getInstrumentErrorBox (grb_type);
	}

	// ignore wrong packets.. (2^10 or 2^11 set)
	if (lbuf[MISC] & 0xC00)
	{
		logStream (MESSAGE_INFO) << "setting error box to 360 degrees, as GCN packet, as it's MISC field suggest it is not correct: " << std::hex << lbuf[MISC] << " at RA " << std::dec << grb_ra << " DEC " << grb_dec << sendLog;
		grb_errorbox = 360;
	}

	return addGcnPoint (grb_id, grb_seqn, grb_type, grb_ra, grb_dec, grb_is_grb, &grb_date, grb_date_usec, grb_errorbox, false, true);
}

int ConnGrb::pr_swift_without_radec ()
{
	// those messages have only sence, when they set grb_is_grb flag to false
	/* exec sql begin declare section */
		 
		 
		 
		 
		 
	
#line 284 "conngrb.ec"
 int d_grb_id ;
 
#line 285 "conngrb.ec"
 int d_grb_seqn ;
 
#line 286 "conngrb.ec"
 int d_grb_type ;
 
#line 287 "conngrb.ec"
 int d_grb_type_start ;
 
#line 288 "conngrb.ec"
 int d_grb_type_end ;
/* exec sql end declare section */
#line 289 "conngrb.ec"


	time_t grb_date;
	long grb_date_usec;

	double grb_ra;
	double grb_dec;

	int ret;

	d_grb_type = (int)(lbuf[PKT_TYPE]);
	d_grb_id = (lbuf[BURST_TRIG] >> S_TRIGNUM_SHIFT) & S_TRIGNUM_MASK;
	d_grb_seqn = (lbuf[BURST_TRIG] >> S_SEGNUM_SHIFT) & S_SEGNUM_MASK;

	getTimeTfromTJD (lbuf[BURST_TJD], lbuf[BURST_SOD]/100.0, &grb_date, &grb_date_usec);

	switch (d_grb_type)
	{
		case TYPE_SWIFT_BAT_GRB_ALERT_SRC:
			// get S/C coordinates to slew on
			// that's special in big errror-box
			// but as we specify last know ra/dec, we will slew to best location we know about burst
			// assume that swift will never spend more then three hours on one location, due to orbit parameters
			// as burst can happen during slew, we have to put in fabs - otherwise we will not respond to burst
			// catched during/before slew, but after pointdir notice was send
			if (fabs (grb_date - swiftLastPoint) < 3 * 3600)
				addGcnPoint (d_grb_id, d_grb_seqn, d_grb_type, swiftLastRa, swiftLastDec, 1, &grb_date, grb_date_usec, getInstrumentErrorBox (d_grb_type), false, true);
			break;
		case TYPE_SWIFT_BAT_GRB_POS_NACK_SRC:
			// update if not grb..
			getGrbBound (d_grb_type, d_grb_type_start, d_grb_type_end);
			{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "update grb set grb_type = $1  , grb_seqn = $2  , grb_is_grb = false where grb_id = $3  and grb_type >= $4  and grb_type <= $5 ", 
	ECPGt_int,&(d_grb_type),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(d_grb_seqn),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(d_grb_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(d_grb_type_start),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(d_grb_type_end),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, ECPGt_EORT);}
#line 330 "conngrb.ec"

			if (sqlca.sqlcode)
			{
				throw rts2db::SqlError ("cannot update Swift GRB with POS_NACK_SRC");
			}
			else
			{
				logStream (MESSAGE_INFO) << "ConnGrb::pr_swift_without_radec grb_is_grb = false grb_id " << d_grb_id << sendLog;
				{ ECPGtrans(__LINE__, NULL, "commit");}
#line 338 "conngrb.ec"

			}
			break;
		case TYPE_SWIFT_SCALEDMAP_SRC:
		case TYPE_SWIFT_XRT_CENTROID_SRC:
		case TYPE_SWIFT_UVOT_IMAGE_SRC:
		case TYPE_SWIFT_UVOT_IMAGE_PROC_SRC:
		case TYPE_SWIFT_UVOT_SLIST_SRC:
		case TYPE_SWIFT_UVOT_SLIST_PROC_SRC:
			grb_ra = lbuf[BURST_RA] / 10000.0;
			grb_dec = lbuf[BURST_DEC] / 10000.0;
			ret = addGcnPoint (d_grb_id, d_grb_seqn, d_grb_type, grb_ra, grb_dec, 1, &grb_date, grb_date_usec, getInstrumentErrorBox(d_grb_type), true, true);
			// when it was sucessfullt added
			// we don't have to add raw, as it was added in GcnPoint
			if (!ret)
				return ret;
			break;
	}

	return addGcnRaw (d_grb_id, d_grb_seqn, d_grb_type);
}

int ConnGrb::pr_agile ()
{
	int grb_id;
	int grb_type;
	double grb_ra;
	double grb_dec;

	int grb_is_grb = 1;
	time_t grb_date;
	long grb_date_usec;
	float grb_errorbox;

	grb_type = (int) (lbuf[PKT_TYPE]);
	grb_id = lbuf[BURST_TRIG];
	grb_ra = lbuf[BURST_RA] / 10000.0;
	grb_dec = lbuf[BURST_DEC] / 10000.0;

	getTimeTfromTJD (lbuf[BURST_TJD], lbuf[BURST_SOD]/100.0, &grb_date, &grb_date_usec);

	grb_errorbox = (float) lbuf[BURST_ERROR] / 10000.0;

	grb_is_grb = lbuf[TRIGGER_ID] & 0x0022;

	return addGcnPoint (grb_id, 1, grb_type, grb_ra, grb_dec, grb_is_grb, &grb_date, grb_date_usec, grb_errorbox, false, true);
}

int ConnGrb::pr_fermi_gbm ()
{
	time_t grb_date;
	long grb_date_usec;

	getTimeTfromTJD (lbuf[BURST_TJD], lbuf[BURST_SOD]/100.0, &grb_date, &grb_date_usec);

	double _error = lbuf[BURST_ERROR] / 10000.0;

	bool enabled = false;
	if (gbm_error < 0 || _error <= gbm_error || gbm_enable_above)
		enabled = true;

	if (gbm_record_above || gbm_error < 0 || _error <= gbm_error)
		return addGcnPoint (lbuf[BURST_TRIG], lbuf[PKT_SERNUM], (int) lbuf[PKT_TYPE], lbuf[BURST_RA] / 10000.0, lbuf[BURST_DEC] / 10000.0, true, &grb_date, grb_date_usec, _error, false, enabled);
	logStream (MESSAGE_INFO) << "ignoring GBM above error limit - " << gbm_error << " > " << _error << sendLog;
	return 0;
}

int ConnGrb::pr_fermi_lat ()
{
	logStream (MESSAGE_INFO) << "LAT messagye, type " << lbuf[PKT_TYPE] << sendLog;
	return 0;
}

int ConnGrb::pr_fermi_sc ()
{

}

int ConnGrb::addSwiftPoint (double roll, char * obs_name, float obstime, float merit)
{
	/* exec sql begin declare section */
		   
		   
		   
		    
		         
		   
		 
		   
	
#line 419 "conngrb.ec"
 double d_swift_ra = swiftLastRa ;
 
#line 420 "conngrb.ec"
 double d_swift_dec = swiftLastDec ;
 
#line 421 "conngrb.ec"
 double d_swift_roll = roll ;
 
#line 422 "conngrb.ec"
 double d_swift_time = ( long ) swiftLastPoint ;
 
#line 423 "conngrb.ec"
 double d_swift_received = ( long ) last_packet . tv_sec + ( double ) last_packet . tv_usec / USEC_SEC ;
 
#line 424 "conngrb.ec"
 float d_swift_obstime = obstime ;
 
#line 425 "conngrb.ec"
  struct varchar_1  { int len; char arr[ 70 ]; }  d_swift_name ;
 
#line 426 "conngrb.ec"
 float d_swift_merit = merit ;
/* exec sql end declare section */
#line 427 "conngrb.ec"


	strcpy (d_swift_name.arr, obs_name);
	d_swift_name.len = strlen (obs_name);

	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "insert into swift ( swift_id , swift_ra , swift_dec , swift_roll , swift_time , swift_received , swift_name , swift_obstime , swift_merit ) values ( nextval ( 'point_id' ) , $1  , $2  , $3  , to_timestamp ( $4  ) , to_timestamp ( $5  ) , $6  , $7  , $8  )", 
	ECPGt_double,&(d_swift_ra),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(d_swift_dec),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(d_swift_roll),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(d_swift_time),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(d_swift_received),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_varchar,&(d_swift_name),(long)70,(long)1,sizeof(struct varchar_1), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_float,&(d_swift_obstime),(long)1,(long)1,sizeof(float), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_float,&(d_swift_merit),(long)1,(long)1,sizeof(float), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, ECPGt_EORT);}
#line 455 "conngrb.ec"

	if (sqlca.sqlcode != 0)
	{
		throw rts2db::SqlError ("cannot insert swift trigger");
	}
	{ ECPGtrans(__LINE__, NULL, "commit");}
#line 460 "conngrb.ec"

	master->updateSwift (swiftLastPoint, swiftLastRa, swiftLastDec);
	return 0;
}

int ConnGrb::addIntegralPoint (double ra, double dec, const time_t *t)
{
	/* exec sql begin declare section */
		   
		   
		    
		         
	
#line 468 "conngrb.ec"
 double d_integral_ra = ra ;
 
#line 469 "conngrb.ec"
 double d_integral_dec = dec ;
 
#line 470 "conngrb.ec"
 double d_integral_time = ( int ) * t ;
 
#line 471 "conngrb.ec"
 double d_integral_received = ( long ) last_packet . tv_sec + ( double ) last_packet . tv_usec / USEC_SEC ;
/* exec sql end declare section */
#line 472 "conngrb.ec"


	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "insert into integral ( integral_id , integral_ra , integral_dec , integral_time , integral_received ) values ( nextval ( 'point_id' ) , $1  , $2  , to_timestamp ( $3  ) , to_timestamp ( $4  ) )", 
	ECPGt_double,&(d_integral_ra),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(d_integral_dec),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(d_integral_time),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(d_integral_received),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, ECPGt_EORT);}
#line 489 "conngrb.ec"

	if (sqlca.sqlcode != 0)
	{
		throw rts2db::SqlError ("cannot insert INTEGRAL pointing");
	}
	{ ECPGtrans(__LINE__, NULL, "commit");}
#line 494 "conngrb.ec"

	master->updateIntegral (d_integral_time, ra, dec);
	return 0;
}

void ConnGrb::getGrbBound (int grb_type, int &grb_start, int &grb_end)
{
	if (grb_type <= TYPE_FERMI_POINTDIR)
	{
		if (grb_type >= TYPE_FERMI_GBM_ALERT)
		{
			grb_start = TYPE_FERMI_GBM_ALERT;
			grb_end = TYPE_FERMI_POINTDIR;
		}
		else if (grb_type >= TYPE_AGILE_GRB_WAKEUP)
		{
			grb_start = TYPE_AGILE_GRB_WAKEUP;
			grb_end = TYPE_AGILE_GRB_POS_TEST;
		}
		else if (grb_type >= TYPE_SWIFT_BAT_GRB_ALERT_SRC)
		{
			grb_start = TYPE_SWIFT_BAT_GRB_ALERT_SRC;
			grb_end = TYPE_SWIFT_BAT_SLEW_POS_SRC;
		}
		else if (grb_type >= TYPE_MILAGRO_POS_SRC)
		{
			grb_start = TYPE_MILAGRO_POS_SRC;
			grb_end = TYPE_KONUS_LC_SRC;
		}
		else if (grb_type >= TYPE_INTEGRAL_POINTDIR_SRC)
		{
			grb_start = TYPE_INTEGRAL_POINTDIR_SRC;
			grb_end = TYPE_INTEGRAL_OFFLINE_SRC;
		}
		else if (grb_type >= TYPE_HETE_ALERT_SRC)
		{
			grb_start = TYPE_HETE_ALERT_SRC;
			grb_end = TYPE_GRB_CNTRPART_SRC;
		}
		else if (grb_type == TYPE_IPN_POS_SRC)
		{
			grb_start = TYPE_IPN_POS_SRC;
			grb_end = TYPE_IPN_POS_SRC;
		}
		else
		{
			// all fields counts - unknow type
			logStream (MESSAGE_ERROR) << "ConnGrb::getGrbBound cannot get type for grb_type " << grb_type << sendLog;
			grb_start = 0;
			grb_end = 5000;
		}
	}
	else
	{
		// all fields counts - unknow type
		logStream (MESSAGE_ERROR) << "ConnGrb::getGrbBound cannot get type for grb_type " << grb_type << sendLog;
		grb_start = 0;
		grb_end = 5000;
	}
}

bool ConnGrb::gcnContainsGrbPos (int grb_type)
{
	switch (grb_type)
	{
		case TYPE_INTEGRAL_POINTDIR_SRC:
		case TYPE_INTEGRAL_SPIACS_SRC:
		case TYPE_SWIFT_SCALEDMAP_SRC:
		case TYPE_SWIFT_XRT_CENTROID_SRC:
		case TYPE_SWIFT_UVOT_IMAGE_SRC:
		case TYPE_SWIFT_UVOT_SLIST_SRC:
		case TYPE_SWIFT_UVOT_SLIST_PROC_SRC:
		case TYPE_SWIFT_POINTDIR_SRC:
		case TYPE_SWIFT_UVOT_NACK_POSITION:
		case TYPE_AGILE_POINTDIR:
		case TYPE_FERMI_POINTDIR:
			return false;
		default:
			return true;
	}
}

float ConnGrb::getInstrumentErrorBox (int grb_type)
{
	// rules:
	//  if it's only detection, return FOV of instrument
	//  if it's position, return some conservative instrument everage position error
	switch (grb_type)
	{
		// INTEGRAL FOV
		case TYPE_INTEGRAL_POINTDIR_SRC:
			return 30.0;
			// Swift FOV
		case TYPE_SWIFT_POINTDIR_SRC:
			return 60.0;
			// HETE instrumental error..
		case TYPE_HETE_ALERT_SRC:
		case TYPE_HETE_UPDATE_SRC:
		case TYPE_HETE_FINAL_SRC:
		case TYPE_HETE_GNDANA_SRC:
		case TYPE_HETE_TEST:
		case TYPE_GRB_CNTRPART_SRC:
			// .. is 4 arcmin
			return 4.0 / 60.0;
		case TYPE_INTEGRAL_WAKEUP_SRC:
		case TYPE_INTEGRAL_REFINED_SRC:
		case TYPE_INTEGRAL_OFFLINE_SRC:
			//INTEGRAL instrument error is 4 armin
			return 4.0 / 60.0;
		case TYPE_INTEGRAL_SPIACS_SRC:
			// SPIACS is in fact all sky detector
			return 180.0;
		case TYPE_SWIFT_BAT_GRB_ALERT_SRC:
			// BAT have 60 deg
			return 60.0;
		case TYPE_SWIFT_BAT_GRB_POS_ACK_SRC:
		case TYPE_SWIFT_BAT_GRB_LC_SRC:
		case TYPE_SWIFT_FOM_2OBSAT_SRC:
		case TYPE_SWIFT_FOSC_2OBSAT_SRC:
		case TYPE_SWIFT_BAT_GRB_LC_PROC_SRC:
		case TYPE_SWIFT_BAT_TRANS:
		case TYPE_SWIFT_BAT_GRB_POS_NACK_SRC:
		case TYPE_SWIFT_SCALEDMAP_SRC:
			// BAT have 4 arcmin
			return 4.0 / 60.0;
		case TYPE_SWIFT_XRT_POSITION_SRC:
		case TYPE_SWIFT_XRT_SPECTRUM_SRC:
		case TYPE_SWIFT_XRT_IMAGE_SRC:
		case TYPE_SWIFT_XRT_LC_SRC:
		case TYPE_SWIFT_XRT_SPECTRUM_PROC_SRC:
		case TYPE_SWIFT_XRT_IMAGE_PROC_SRC:
			// conservative estimate for XRT is 20 arcsec, including uncertanities
			return 20.0 / 3600.0;
		case TYPE_SWIFT_XRT_CENTROID_SRC:
			// XRT FOV
			return 15.0 / 60.0;
		case TYPE_SWIFT_UVOT_SLIST_PROC_SRC:
		case TYPE_SWIFT_UVOT_POS_SRC:
			// that's VERY conservative estimate, we might refine it
			return 19.0 / 3600.0;
		case TYPE_SWIFT_UVOT_SLIST_SRC:
		case TYPE_SWIFT_UVOT_IMAGE_SRC:
		case TYPE_SWIFT_UVOT_IMAGE_PROC_SRC:
			// UVOT FOV
			return 15.0 / 60.0;
		case TYPE_AGILE_GRB_WAKEUP:
		case TYPE_AGILE_GRB_PROMPT:
		case TYPE_AGILE_GRB_REFINED:
		case TYPE_AGILE_TRANS:
			return 3;
		case TYPE_AGILE_POINTDIR:
			return 60;
		case TYPE_FERMI_GBM_ALERT:
		case TYPE_FERMI_GBM_FLT_POS:
		case TYPE_FERMI_GBM_GND_POS:
		case TYPE_FERMI_GBM_LC:
		case TYPE_FERMI_GBM_TRANS:
		case TYPE_FERMI_GBM_POS_TEST:
		case TYPE_FERMI_LAT_POS_INI:
		case TYPE_FERMI_LAT_POS_UPD:
		case TYPE_FERMI_LAT_POS_DIAG:
		case TYPE_FERMI_LAT_TRANS:
		case TYPE_FERMI_OBS_REQ:
		case TYPE_FERMI_SC_SLEW:
			// TODO fill that with FERMI S/C specifications, when it will be
			// on launch pad
			return 3.0;
	}
	logStream (MESSAGE_WARNING) << "ConnGrb::getInstrumentErrorBox unknow type: " << grb_type
		<< ", returning 180.0" << sendLog;
	return 180.0;
}

int ConnGrb::addGcnPoint (int grb_id, int grb_seqn, int grb_type, double grb_ra, double grb_dec, bool grb_is_grb, time_t *grb_date, long grb_date_usec, float grb_errorbox, bool insertOnly, bool enabled)
{
	/* exec sql begin declare section */
	 
	   
	   
	   
	   
	   
	   
	   
	 
	        
	        
	   
	 
	// used to find correct grb - based on type
	 
	 
	// target stuff
	 
	 
	   
	
#line 670 "conngrb.ec"
 int d_tar_id ;
 
#line 671 "conngrb.ec"
 int d_grb_id = grb_id ;
 
#line 672 "conngrb.ec"
 int d_grb_seqn = grb_seqn ;
 
#line 673 "conngrb.ec"
 int d_grb_type = grb_type ;
 
#line 674 "conngrb.ec"
 int d_curr_grb_type = - 1 ;
 
#line 675 "conngrb.ec"
 double d_grb_ra = grb_ra ;
 
#line 676 "conngrb.ec"
 double d_grb_dec = grb_dec ;
 
#line 677 "conngrb.ec"
 bool d_grb_is_grb = grb_is_grb ;
 
#line 678 "conngrb.ec"
 bool db_was_grb ;
 
#line 679 "conngrb.ec"
 double d_grb_date = * grb_date + ( double ) grb_date_usec / USEC_SEC ;
 
#line 680 "conngrb.ec"
 double d_grb_update = last_packet . tv_sec + ( double ) last_packet . tv_usec / USEC_SEC ;
 
#line 681 "conngrb.ec"
 float d_grb_errorbox = grb_errorbox ;
 
#line 682 "conngrb.ec"
 int d_grb_errorbox_ind ;
 
#line 684 "conngrb.ec"
 int d_grb_type_start ;
 
#line 685 "conngrb.ec"
 int d_grb_type_end ;
 
#line 687 "conngrb.ec"
  struct varchar_2  { int len; char arr[ 150 ]; }  d_tar_name ;
 
#line 688 "conngrb.ec"
  struct varchar_3  { int len; char arr[ 2000 ]; }  d_tar_comment ;
 
#line 689 "conngrb.ec"
 bool d_tar_enabled = enabled ;
/* exec sql end declare section */
#line 690 "conngrb.ec"


	int grb_isnew = 0;

	if ((master->getRecordNotVisible () == false) && 
		((master->observer->lat > 0 && grb_dec < (master->observer->lat - 90 ))
		 || (master->observer->lat < 0 && grb_dec > (master->observer->lat + 90 ))
	        )
	)	
	{
		logStream (MESSAGE_INFO) << "not recording GRB at " << LibnovaRaDec (grb_ra, grb_dec) << ", as it is never visible from current location (latitude " << master->observer->lat << ")" << sendLog;
		return 0;
	}

	if (master->getRecordOnlyVisibleTonight () == true)
	{
		// check if target is visible tonight at all..
		struct ln_rst_time rst;
		double JD = ln_get_julian_from_sys ();
		Rts2Night night (JD, master->observer);

		struct ln_equ_posn pos;
		pos.ra = grb_ra;
		pos.dec = grb_dec;

		ln_get_object_next_rst_horizon (night.getJDFrom (), master->observer, &pos, master->getMinGrbAltitute (), &rst);

		if (!((night.getJDFrom () < rst.set && night.getJDTo () > rst.set) || 
			(night.getJDFrom () < rst.rise && night.getJDTo () > rst.rise) || 
			(night.getJDFrom () < rst.transit && night.getJDTo () > rst.transit)))
		{
			logStream (MESSAGE_INFO) << "not recording GRB at " << LibnovaRaDec (grb_ra, grb_dec) << " as it is not durring current night above minimal altitude " << LibnovaDeg90 (master->getMinGrbAltitute ()) << sendLog;
			return 0;
		}
	}

	struct tm grb_broken_time;

	int ret = 0;

	gmtime_r (grb_date, &grb_broken_time);
	d_tar_name.len = snprintf (d_tar_name.arr, 150, "GRB %02d%02d%06.3f GCN #%i",
		grb_broken_time.tm_year % 100, grb_broken_time.tm_mon + 1, grb_broken_time.tm_mday +
		(grb_broken_time.tm_hour * 3600 + grb_broken_time.tm_min * 60 + grb_broken_time.tm_sec) / 86400.0,
		d_grb_id);

	getGrbBound (grb_type, d_grb_type_start, d_grb_type_end);

	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "select tar_id , grb_type , grb_errorbox , grb_is_grb from grb where grb_id = $1  and grb_type >= $2  and grb_type <= $3 ", 
	ECPGt_int,&(d_grb_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(d_grb_type_start),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(d_grb_type_end),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, 
	ECPGt_int,&(d_tar_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(d_curr_grb_type),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_float,&(d_grb_errorbox),(long)1,(long)1,sizeof(float), 
	ECPGt_int,&(d_grb_errorbox_ind),(long)1,(long)1,sizeof(int), 
	ECPGt_bool,&(db_was_grb),(long)1,(long)1,sizeof(bool), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EORT);}
#line 754 "conngrb.ec"


	if (sqlca.sqlcode == ECPG_NOT_FOUND)
	{
		// create new GCN entry..
		d_grb_errorbox = grb_errorbox;
		if (isnan (d_grb_errorbox))
		{
			d_grb_errorbox_ind = -1;
			d_grb_errorbox = 0;
		}
		else
		{
			d_grb_errorbox_ind = 0;
		}
		// do not insert if it's know source and follow transient is false
		if (grb_is_grb == false && rts2core::Configuration::instance()->grbdFollowTransients () == false)
		{
			logStream (MESSAGE_INFO) << "Ignoring know source target creation" << sendLog;
			return 0;
		}
		// insert part..we do care about HETE burst without coordinates
		if (d_grb_ra < -300 && d_grb_dec < -300)
		{
			
			logStream (MESSAGE_DEBUG) << "ConnGrb::addGcnPoint HETE GRB without coords? ra="
				<< d_grb_ra << " dec=" << d_grb_dec << sendLog;
			{ ECPGtrans(__LINE__, NULL, "rollback");}
#line 781 "conngrb.ec"

			return -1;
		}
		// generate new GRB details
		d_tar_comment.len = sprintf (d_tar_comment.arr, "Generated by GRBD for event %d-%02d-%02dT%02d:%02d:%02d, GCN #%i, type %i",
			grb_broken_time.tm_year + 1900, grb_broken_time.tm_mon + 1, grb_broken_time.tm_mday,
			grb_broken_time.tm_hour, grb_broken_time.tm_min, grb_broken_time.tm_sec, d_grb_id, d_grb_type);

		{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "select nextval ( 'grb_tar_id' )", ECPGt_EOIT, 
	ECPGt_int,&(d_tar_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EORT);}
#line 793 "conngrb.ec"


		if (sqlca.sqlcode)
		{
		  	throw rts2db::SqlError ("cannot retrieve next value from grb_tar_id sequence");
		}

		// check and honest create_disabled value
		if (master->getCreateDisabled ())
			d_tar_enabled = false;

		// insert new target
		{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "insert into targets ( tar_id , type_id , tar_name , tar_ra , tar_dec , tar_enabled , tar_comment , tar_priority , tar_bonus , tar_bonus_time ) values ( $1  , 'G' , $2  , $3  , $4  , $5  , $6  , 100 , 100 , null )", 
	ECPGt_int,&(d_tar_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_varchar,&(d_tar_name),(long)150,(long)1,sizeof(struct varchar_2), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(d_grb_ra),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(d_grb_dec),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_bool,&(d_tar_enabled),(long)1,(long)1,sizeof(bool), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_varchar,&(d_tar_comment),(long)2000,(long)1,sizeof(struct varchar_3), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, ECPGt_EORT);}
#line 832 "conngrb.ec"

		if (sqlca.sqlcode)
		{
			throw rts2db::SqlError ("cannot add GCN coordinates");
		}
		grb_isnew = 1;
		// insert new grb packet
		{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "insert into grb ( tar_id , grb_id , grb_seqn , grb_type , grb_ra , grb_dec , grb_is_grb , grb_date , grb_last_update , grb_errorbox , grb_autodisabled ) values ( $1  , $2  , $3  , $4  , $5  , $6  , $7  , to_timestamp ( $8  ) , to_timestamp ( $9  ) , $10  , false )", 
	ECPGt_int,&(d_tar_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(d_grb_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(d_grb_seqn),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(d_grb_type),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(d_grb_ra),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(d_grb_dec),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_bool,&(d_grb_is_grb),(long)1,(long)1,sizeof(bool), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(d_grb_date),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(d_grb_update),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_float,&(d_grb_errorbox),(long)1,(long)1,sizeof(float), 
	ECPGt_int,&(d_grb_errorbox_ind),(long)1,(long)1,sizeof(int), ECPGt_EOIT, ECPGt_EORT);}
#line 868 "conngrb.ec"

		if (sqlca.sqlcode)
		{
			throw rts2db::SqlError ("cannot insert new GCN coordinates");
		}
		else
		{
			logStream (MESSAGE_INFO) << "ConnGrb::addGcnPoint grb created: tar_id: "
				<< d_tar_id
				<< " grb_id: " << d_grb_id
				<< " grb_seqn: " << d_grb_seqn
				<< " ra dec: " << LibnovaRaDec (d_grb_ra, d_grb_dec) << sendLog;
			{ ECPGtrans(__LINE__, NULL, "commit");}
#line 880 "conngrb.ec"

		}
	}
	else if (sqlca.sqlcode)
	{
		throw rts2db::SqlError ("cannot check for GRB coordinates in the database");
	}
	else
	{
		// update know event
		if (insertOnly)
		{
			{ ECPGtrans(__LINE__, NULL, "rollback");}
#line 892 "conngrb.ec"

			return 1;
		}
		// HETE burst have values -999 in some retraction notices..
		// do updates only when new position is better than old one
		if (
			(d_grb_errorbox_ind < 0
			|| isnan (grb_errorbox)
			|| grb_errorbox <= d_grb_errorbox
			)
			&& d_grb_ra > -300
			&& d_grb_dec > -300)
		{
			// update target informations..
			{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "update targets set tar_ra = $1  , tar_dec = $2  , tar_enabled = $3  where tar_id = $4 ", 
	ECPGt_double,&(d_grb_ra),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(d_grb_dec),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_bool,&(d_tar_enabled),(long)1,(long)1,sizeof(bool), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(d_tar_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, ECPGt_EORT);}
#line 914 "conngrb.ec"

			if (sqlca.sqlcode)
			{
			  	throw rts2db::SqlError ("cannot update GRB target coordinates");
			}
			logStream (MESSAGE_INFO) << "Update target #" << d_tar_id << " RA DEC: " << LibnovaRaDec (d_grb_ra, d_grb_dec) << ", set state to " << (d_tar_enabled ? "enabled" : "disabled") << sendLog;

			// update grb informations..
			// do updates only when new position is better then old one
			if (gcnContainsGrbPos (d_grb_type)
				&& !isnan(grb_errorbox)
				&& (d_grb_errorbox_ind < 0
				|| grb_errorbox <= d_grb_errorbox)
				)
			{
				{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "update grb set grb_seqn = $1  , grb_type = $2  , grb_ra = $3  , grb_dec = $4  , grb_is_grb = $5  , grb_last_update = to_timestamp ( $6  ) where tar_id = $7 ", 
	ECPGt_int,&(d_grb_seqn),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(d_grb_type),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(d_grb_ra),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(d_grb_dec),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_bool,&(d_grb_is_grb),(long)1,(long)1,sizeof(bool), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_double,&(d_grb_update),(long)1,(long)1,sizeof(double), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(d_tar_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, ECPGt_EORT);}
#line 940 "conngrb.ec"


				if (sqlca.sqlcode)
				{
					throw rts2db::SqlError ("cannot update GRB GCN entry");
				}
				else
				{
					{ ECPGtrans(__LINE__, NULL, "commit");}
#line 948 "conngrb.ec"

				}

				d_grb_errorbox = grb_errorbox;
				{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "update grb set grb_errorbox = $1  where tar_id = $2 ", 
	ECPGt_float,&(d_grb_errorbox),(long)1,(long)1,sizeof(float), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(d_tar_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, ECPGt_EORT);}
#line 958 "conngrb.ec"

				if (sqlca.sqlcode)
				{
					throw rts2db::SqlError ("cannot update GCN error box");
				}
				else
				{
					logStream (MESSAGE_INFO) << "ConnGrb::addGcnPoint grb updated: tar_id: "
						<< d_tar_id << " grb_id: " << d_grb_id << " grb_seqn: " << d_grb_seqn << sendLog;
					{ ECPGtrans(__LINE__, NULL, "commit");}
#line 967 "conngrb.ec"

				}
			}
		}
		else
		{
			logStream (MESSAGE_INFO) << "ConnGrb::addGcnPoint grb update ignored: grb_errorbox "
				<< grb_errorbox
				<< " d_grb_errorbox " << d_grb_errorbox
				<< " d_grb_errorbox_ind " << d_grb_errorbox_ind << sendLog;
			// update grb_is_grb, if that has changed
			if (d_grb_is_grb != db_was_grb)
			{
				{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "update grb set grb_is_grb = $1  where tar_id = $2 ", 
	ECPGt_bool,&(d_grb_is_grb),(long)1,(long)1,sizeof(bool), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(d_tar_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, ECPGt_EORT);}
#line 985 "conngrb.ec"

				if (sqlca.sqlcode)
				{
					throw rts2db::SqlError ("cannot update grb_is_grb");
				}
				else
				{
					{ ECPGtrans(__LINE__, NULL, "commit");}
#line 992 "conngrb.ec"

				}
			}
			// do not update
			d_grb_errorbox_ind = -1;
		}
	}

	addGcnRaw (grb_id, grb_seqn, grb_type);

	// do not follow if it's know transient and FollowTransients is false
	if (grb_is_grb == false && rts2core::Configuration::instance ()->grbdFollowTransients () == false)
	{
		logStream (MESSAGE_INFO) << "Disabling know source." << sendLog;
		{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "update targets set tar_enabled = false where tar_id = $1 ", 
	ECPGt_int,&(d_tar_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, ECPGt_EORT);}
#line 1012 "conngrb.ec"

		if (sqlca.sqlcode)
		{
			throw rts2db::SqlError ("cannot update tar_enabled");
		}
		{ ECPGtrans(__LINE__, NULL, "commit");}
#line 1017 "conngrb.ec"

		return 0;
	}

	if (d_grb_type_start == TYPE_FERMI_GBM_ALERT && gbm_error > 0 && d_grb_errorbox > gbm_error)
	{
		logStream (MESSAGE_INFO) << "only recorded GBM GRB with id " << d_grb_id << ", as it is above gbm_error_limit" << sendLog;
		return ret;
	}

	// test if that's only follow-up
	if (!execFollowups)
	{
		// swift burst
		if (d_grb_type_start == TYPE_SWIFT_BAT_GRB_ALERT_SRC
			&& grb_id < 100000
			&& d_grb_errorbox_ind == -1)
		{
			// and it's only follow-up slew notice without errorbox..don't do anything
			return ret;
		}
	}

	ret = master->newGcnGrb (d_tar_id);

	// last thing is to call some external exe..
	if (addExe)
	{
		int execRet;
		rts2core::ConnFork *execConn = new rts2core::ConnFork (master, addExe, false, false, 100);

		execConn->addArg (d_tar_id);
		execConn->addArg (grb_id);
		execConn->addArg (grb_seqn);
		execConn->addArg (grb_type);
		execConn->addArg (grb_ra);
		execConn->addArg (grb_dec);
		execConn->addArg (grb_is_grb);
		execConn->addArg (*grb_date);
		execConn->addArg (grb_errorbox);
		execConn->addArg (grb_isnew);

		execRet = execConn->init ();
		if (execRet < 0)
		{
			delete execConn;
		}
		else if (execRet == 0)
		{
			master->addConnection (execConn);
		}
	}

	delete[] last_target;
	last_target = new char[d_tar_name.len + 1];
	strcpy (last_target, d_tar_name.arr);
	last_target_time = d_grb_update;

	last_ra = d_grb_ra;
	last_dec = d_grb_dec;

	return ret;
}

int ConnGrb::addGcnRaw (int grb_id, int grb_seqn, int grb_type)
{
	/* exec sql begin declare section */
		   
		   
		   
		     
		    

		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
		 
	
#line 1084 "conngrb.ec"
 int d_grb_id = grb_id ;
 
#line 1085 "conngrb.ec"
 int d_grb_seqn = grb_seqn ;
 
#line 1086 "conngrb.ec"
 int d_grb_type = grb_type ;
 
#line 1087 "conngrb.ec"
 long d_grb_update = ( int ) last_packet . tv_sec ;
 
#line 1088 "conngrb.ec"
 int d_grb_update_usec = ( int ) last_packet . tv_usec ;
 
#line 1090 "conngrb.ec"
 long d_packet0 ;
 
#line 1091 "conngrb.ec"
 long d_packet1 ;
 
#line 1092 "conngrb.ec"
 long d_packet2 ;
 
#line 1093 "conngrb.ec"
 long d_packet3 ;
 
#line 1094 "conngrb.ec"
 long d_packet4 ;
 
#line 1095 "conngrb.ec"
 long d_packet5 ;
 
#line 1096 "conngrb.ec"
 long d_packet6 ;
 
#line 1097 "conngrb.ec"
 long d_packet7 ;
 
#line 1098 "conngrb.ec"
 long d_packet8 ;
 
#line 1099 "conngrb.ec"
 long d_packet9 ;
 
#line 1100 "conngrb.ec"
 long d_packet10 ;
 
#line 1101 "conngrb.ec"
 long d_packet11 ;
 
#line 1102 "conngrb.ec"
 long d_packet12 ;
 
#line 1103 "conngrb.ec"
 long d_packet13 ;
 
#line 1104 "conngrb.ec"
 long d_packet14 ;
 
#line 1105 "conngrb.ec"
 long d_packet15 ;
 
#line 1106 "conngrb.ec"
 long d_packet16 ;
 
#line 1107 "conngrb.ec"
 long d_packet17 ;
 
#line 1108 "conngrb.ec"
 long d_packet18 ;
 
#line 1109 "conngrb.ec"
 long d_packet19 ;
 
#line 1110 "conngrb.ec"
 long d_packet20 ;
 
#line 1111 "conngrb.ec"
 long d_packet21 ;
 
#line 1112 "conngrb.ec"
 long d_packet22 ;
 
#line 1113 "conngrb.ec"
 long d_packet23 ;
 
#line 1114 "conngrb.ec"
 long d_packet24 ;
 
#line 1115 "conngrb.ec"
 long d_packet25 ;
 
#line 1116 "conngrb.ec"
 long d_packet26 ;
 
#line 1117 "conngrb.ec"
 long d_packet27 ;
 
#line 1118 "conngrb.ec"
 long d_packet28 ;
 
#line 1119 "conngrb.ec"
 long d_packet29 ;
 
#line 1120 "conngrb.ec"
 long d_packet30 ;
 
#line 1121 "conngrb.ec"
 long d_packet31 ;
 
#line 1122 "conngrb.ec"
 long d_packet32 ;
 
#line 1123 "conngrb.ec"
 long d_packet33 ;
 
#line 1124 "conngrb.ec"
 long d_packet34 ;
 
#line 1125 "conngrb.ec"
 long d_packet35 ;
 
#line 1126 "conngrb.ec"
 long d_packet36 ;
 
#line 1127 "conngrb.ec"
 long d_packet37 ;
 
#line 1128 "conngrb.ec"
 long d_packet38 ;
 
#line 1129 "conngrb.ec"
 long d_packet39 ;
/* exec sql end declare section */
#line 1130 "conngrb.ec"



	d_packet0 = lbuf[0];
	d_packet1 = lbuf[1];
	d_packet2 = lbuf[2];
	d_packet3 = lbuf[3];
	d_packet4 = lbuf[4];
	d_packet5 = lbuf[5];
	d_packet6 = lbuf[6];
	d_packet7 = lbuf[7];
	d_packet8 = lbuf[8];
	d_packet9 = lbuf[9];

	d_packet10 = lbuf[10];
	d_packet11 = lbuf[11];
	d_packet12 = lbuf[12];
	d_packet13 = lbuf[13];
	d_packet14 = lbuf[14];
	d_packet15 = lbuf[15];
	d_packet16 = lbuf[16];
	d_packet17 = lbuf[17];
	d_packet18 = lbuf[18];
	d_packet19 = lbuf[19];

	d_packet20 = lbuf[20];
	d_packet21 = lbuf[21];
	d_packet22 = lbuf[22];
	d_packet23 = lbuf[23];
	d_packet24 = lbuf[24];
	d_packet25 = lbuf[25];
	d_packet26 = lbuf[26];
	d_packet27 = lbuf[27];
	d_packet28 = lbuf[28];
	d_packet29 = lbuf[29];

	d_packet30 = lbuf[30];
	d_packet31 = lbuf[31];
	d_packet32 = lbuf[32];
	d_packet33 = lbuf[33];
	d_packet34 = lbuf[34];
	d_packet35 = lbuf[35];
	d_packet36 = lbuf[36];
	d_packet37 = lbuf[37];
	d_packet38 = lbuf[38];
	d_packet39 = lbuf[39];

	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "insert into grb_gcn ( grb_id , grb_seqn , grb_type , grb_update , grb_update_usec , packet ) values ( $1  , $2  , $3  , to_timestamp ( $4  ) , $5  , array [ $6  , $7  , $8  , $9  , $10  , $11  , $12  , $13  , $14  , $15  , $16  , $17  , $18  , $19  , $20  , $21  , $22  , $23  , $24  , $25  , $26  , $27  , $28  , $29  , $30  , $31  , $32  , $33  , $34  , $35  , $36  , $37  , $38  , $39  , $40  , $41  , $42  , $43  , $44  , $45  ] :: bigint [] )", 
	ECPGt_int,&(d_grb_id),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(d_grb_seqn),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(d_grb_type),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_long,&(d_grb_update),(long)1,(long)1,sizeof(long), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_int,&(d_grb_update_usec),(long)1,(long)1,sizeof(int), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_long,&(d_packet0),(long)1,(long)1,sizeof(long), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_long,&(d_packet1),(long)1,(long)1,sizeof(long), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_long,&(d_packet2),(long)1,(long)1,sizeof(long), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_long,&(d_packet3),(long)1,(long)1,sizeof(long), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_long,&(d_packet4),(long)1,(long)1,sizeof(long), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_long,&(d_packet5),(long)1,(long)1,sizeof(long), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_long,&(d_packet6),(long)1,(long)1,sizeof(long), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_long,&(d_packet7),(long)1,(long)1,sizeof(long), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_long,&(d_packet8),(long)1,(long)1,sizeof(long), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_long,&(d_packet9),(long)1,(long)1,sizeof(long), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_long,&(d_packet10),(long)1,(long)1,sizeof(long), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_long,&(d_packet11),(long)1,(long)1,sizeof(long), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_long,&(d_packet12),(long)1,(long)1,sizeof(long), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_long,&(d_packet13),(long)1,(long)1,sizeof(long), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_long,&(d_packet14),(long)1,(long)1,sizeof(long), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_long,&(d_packet15),(long)1,(long)1,sizeof(long), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_long,&(d_packet16),(long)1,(long)1,sizeof(long), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_long,&(d_packet17),(long)1,(long)1,sizeof(long), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_long,&(d_packet18),(long)1,(long)1,sizeof(long), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_long,&(d_packet19),(long)1,(long)1,sizeof(long), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_long,&(d_packet20),(long)1,(long)1,sizeof(long), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_long,&(d_packet21),(long)1,(long)1,sizeof(long), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_long,&(d_packet22),(long)1,(long)1,sizeof(long), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_long,&(d_packet23),(long)1,(long)1,sizeof(long), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_long,&(d_packet24),(long)1,(long)1,sizeof(long), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_long,&(d_packet25),(long)1,(long)1,sizeof(long), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_long,&(d_packet26),(long)1,(long)1,sizeof(long), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_long,&(d_packet27),(long)1,(long)1,sizeof(long), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_long,&(d_packet28),(long)1,(long)1,sizeof(long), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_long,&(d_packet29),(long)1,(long)1,sizeof(long), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_long,&(d_packet30),(long)1,(long)1,sizeof(long), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_long,&(d_packet31),(long)1,(long)1,sizeof(long), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_long,&(d_packet32),(long)1,(long)1,sizeof(long), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_long,&(d_packet33),(long)1,(long)1,sizeof(long), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_long,&(d_packet34),(long)1,(long)1,sizeof(long), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_long,&(d_packet35),(long)1,(long)1,sizeof(long), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_long,&(d_packet36),(long)1,(long)1,sizeof(long), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_long,&(d_packet37),(long)1,(long)1,sizeof(long), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_long,&(d_packet38),(long)1,(long)1,sizeof(long), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
	ECPGt_long,&(d_packet39),(long)1,(long)1,sizeof(long), 
	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, ECPGt_EORT);}
#line 1199 "conngrb.ec"

	if (sqlca.sqlcode)
	{
		throw rts2db::SqlError ("cannot insert raw GCN packet");
	}
	else
	{
		{ ECPGtrans(__LINE__, NULL, "commit");}
#line 1206 "conngrb.ec"

		return 0;
	}
}

ConnGrb::ConnGrb (char *in_gcn_hostname, int in_gcn_port, int in_do_hete_test, char *in_addExe, int in_execFollowups, Grbd *in_master):rts2core::ConnNoSend (in_master)
{
	master = in_master;
	gcn_hostname = new char[strlen (in_gcn_hostname) + 1];
	strcpy (gcn_hostname, in_gcn_hostname);
	gcn_port = in_gcn_port;
	gcn_listen_sock = -1;

	time (&last_packet.tv_sec);
	last_packet.tv_sec -= 600;
	last_packet.tv_usec = 0;
	last_imalive_sod = -1;

	deltaValue = 0;
	last_target = NULL;
	last_target_time = -1;

	last_ra = rts2_nan ("f");
	last_dec = rts2_nan ("f");

	do_hete_test = in_do_hete_test;

	setConnTimeout (90);
	time (&nextTime);
	nextTime += getConnTimeout ();

	swiftLastPoint = 0;
	swiftLastRa = rts2_nan ("f");
	swiftLastDec = rts2_nan ("f");

	addExe = in_addExe;
	execFollowups = in_execFollowups;

	gcnReceivedBytes = 0;

	gbm_error = 0.25;
	gbm_record_above = true;
	gbm_enable_above = false;
}

ConnGrb::~ConnGrb (void)
{
	delete[] gcn_hostname;
	delete[] last_target;
	if (gcn_listen_sock >= 0)
		close (gcn_listen_sock);
}

int ConnGrb::idle ()
{
	int ret;
	int err;
	socklen_t len = sizeof (err);

	time_t now;
	time (&now);

	switch (getConnState ())
	{
		case CONN_CONNECTING:
			ret = getsockopt (sock, SOL_SOCKET, SO_ERROR, &err, &len);
			if (ret)
			{
				logStream (MESSAGE_ERROR) << "ConnGrb::idle getsockopt " << strerror (errno) << sendLog;
				connectionError (-1);
			}
			else if (err)
			{
				logStream (MESSAGE_ERROR) << "ConnGrb::idle getsockopt " << strerror (err) << sendLog;
				connectionError (-1);
			}
			else
			{
				setConnState (CONN_CONNECTED);
			}
			break;
			// kill us when we were in conn_connecting state for to long
		case CONN_BROKEN:
			if (nextTime < now)
			{
				ret = init ();
				if (ret)
				{
					time (&nextTime);
					nextTime += getConnTimeout ();
				}
			}
			break;
		case CONN_CONNECTED:
			if (last_packet.tv_sec + getConnTimeout () < now
				&& nextTime < now)
				connectionError (-1);
			break;
		default:
			break;
	}
	// we don't like to get called upper code with timeouting stuff..
	return 0;
}

int ConnGrb::init_call ()
{
	struct addrinfo hints;
	struct addrinfo *info;
	int ret;

	hints.ai_flags = 0;
	hints.ai_family = PF_INET;
	hints.ai_socktype = SOCK_STREAM;
	hints.ai_protocol = 0;
	std::ostringstream _os;
	_os << gcn_port;
	ret = getaddrinfo (gcn_hostname, _os.str ().c_str (), &hints, &info);
	if (ret)
	{
		logStream (MESSAGE_ERROR) << "NetworkAddress::getAddress getaddrinfor: " << gai_strerror (ret) << sendLog;
		freeaddrinfo (info);
		return -1;
	}
	sock = socket (info->ai_family, info->ai_socktype, info->ai_protocol);
	if (sock == -1)
	{
		freeaddrinfo (info);
		return -1;
	}
	ret = connect (sock, info->ai_addr, info->ai_addrlen);
	freeaddrinfo (info);
	time (&nextTime);
	nextTime += getConnTimeout ();
	if (ret == -1)
	{
		if (errno == EINPROGRESS)
		{
			setConnState (CONN_CONNECTING);
			return 0;
		}
		return -1;
	}
	setConnState (CONN_CONNECTED);
	return 0;
}

int ConnGrb::init_listen ()
{
	int ret;

	if (gcn_listen_sock >= 0)
	{
		close (gcn_listen_sock);
		gcn_listen_sock = -1;
	}

	connectionError (-1);

	gcn_listen_sock = socket (PF_INET, SOCK_STREAM, 0);
	if (gcn_listen_sock == -1)
	{
		logStream (MESSAGE_ERROR) << "ConnGrb::init_listen socket " << strerror (errno) << sendLog;
		return -1;
	}
	const int so_reuseaddr = 1;
	setsockopt (gcn_listen_sock, SOL_SOCKET, SO_REUSEADDR, &so_reuseaddr, sizeof (so_reuseaddr));
	struct sockaddr_in server;
	server.sin_family = AF_INET;
	server.sin_port = htons (gcn_port);
	server.sin_addr.s_addr = htonl (INADDR_ANY);
	ret = bind (gcn_listen_sock, (struct sockaddr *) &server, sizeof (server));
	if (ret)
	{
		logStream (MESSAGE_ERROR) << "ConnGrb::init_listen bind: " << strerror (errno) << sendLog;
		return -1;
	}
	ret = listen (gcn_listen_sock, 1);
	if (ret)
	{
		logStream (MESSAGE_ERROR) << "ConnGrb::init_listen listen: " << strerror (errno) << sendLog;
		return -1;
	}
	setConnState (CONN_CONNECTED);
	time (&nextTime);
	nextTime += 2 * getConnTimeout ();
	return 0;
}

int ConnGrb::init ()
{
	if (!strcmp (gcn_hostname, "-"))
		return init_listen ();
	else
		return init_call ();
}

int ConnGrb::add (fd_set * readset, fd_set * writeset, fd_set * expset)
{
	if (gcn_listen_sock >= 0)
	{
		FD_SET (gcn_listen_sock, readset);
		return 0;
	}
	return rts2core::Connection::add (readset, writeset, expset);
}

void ConnGrb::connectionError (int last_data_size)
{
	logStream (MESSAGE_DEBUG) << "ConnGrb::connectionError" << sendLog;
	if (sock > 0)
	{
		close (sock);
		sock = -1;
	}
	if (!isConnState (CONN_BROKEN))
	{
		time (&nextTime);
		sock = -1;
		setConnState (CONN_BROKEN);
	}
	gcnReceivedBytes = 0;
}

int ConnGrb::receive (fd_set *set)
{
	int ret = 0;
	struct tm *t;
	if (gcn_listen_sock >= 0 && FD_ISSET (gcn_listen_sock, set))
	{
		// try to accept connection..
		close (sock);			 // close previous connections..we support only one GCN connection
		sock = -1;
		struct sockaddr_in other_side;
		socklen_t addr_size = sizeof (struct sockaddr_in);
		sock = accept (gcn_listen_sock, (struct sockaddr *) &other_side, &addr_size);
		if (sock == -1)
		{
			// bad accept - strange
			logStream (MESSAGE_ERROR) << "ConnGrb::receive accept on gcn_listen_sock: " << strerror (errno) << sendLog;
			connectionError (-1);
		}
		// close listening socket..when we get connection
		close (gcn_listen_sock);
		gcn_listen_sock = -1;
		setConnState (CONN_CONNECTED);
		logStream (MESSAGE_INFO) << "ConnGrb::receive accept gcn_listen_sock from "
			<< inet_ntoa (other_side.sin_addr) << " port " << ntohs (other_side.sin_port) << sendLog;
	}
	else if (sock >= 0 && FD_ISSET (sock, set))
	{
		try
		{
			ret = read (sock, ((char*) nbuf) + gcnReceivedBytes, sizeof (nbuf) - gcnReceivedBytes);
			if (ret == 0 && isConnState (CONN_CONNECTING))
			{
				setConnState (CONN_CONNECTED);
			}
			else if (ret < 0)
			{
				connectionError (ret);
				return -1;
			}
			gcnReceivedBytes += ret;
			// we don't receive full packet..
			if (gcnReceivedBytes < (int) (SIZ_PKT * sizeof(nbuf[0])))
				return ret;
			gcnReceivedBytes = 0;
			successfullRead ();
			gettimeofday (&last_packet, NULL);
			// swap bytes..
			for (int i=0; i < SIZ_PKT; i++)
			{
				lbuf[i] = ntohl (nbuf[i]);
			}
	
			/* Immediately echo back the packet so GCN can monitor:
			 * (1) the actual receipt by the site, and
			 * (2) the roundtrip travel times.
			 * Everything except KILL's get echo-ed back.            */
			if(lbuf[PKT_TYPE] != TYPE_KILL_SOCKET)
			{
				write (sock, (char *)nbuf, sizeof(nbuf));
				successfullSend ();
			}
			t = gmtime (&last_packet.tv_sec);
			here_sod = t->tm_hour*3600 + t->tm_min*60 + t->tm_sec + last_packet.tv_usec / USEC_SEC;
	
			// switch based on packet content
			switch (lbuf[PKT_TYPE])
			{
				case TYPE_TEST_COORDS:
					pr_test ();
					break;
				case TYPE_IM_ALIVE:
					pr_imalive ();
					break;
					// pondtirs messages with history..
				case TYPE_INTEGRAL_POINTDIR_SRC:
					pr_integral_point ();
					break;
									 // 83  // Swift Pointing Direction
				case TYPE_SWIFT_POINTDIR_SRC:
					pr_swift_point ();
					break;
				case TYPE_AGILE_POINTDIR:
					pr_agile_point ();
					break;
				case TYPE_FERMI_POINTDIR:
					pr_fermi_point ();
					break;
				// hete, integral & swift GRB observations
				case TYPE_HETE_ALERT_SRC:
				case TYPE_HETE_UPDATE_SRC:
				case TYPE_HETE_FINAL_SRC:
				case TYPE_HETE_GNDANA_SRC:
				case TYPE_HETE_TEST:
				case TYPE_GRB_CNTRPART_SRC:
					pr_hete ();
					break;
				case TYPE_INTEGRAL_WAKEUP_SRC:
				case TYPE_INTEGRAL_REFINED_SRC:
				case TYPE_INTEGRAL_OFFLINE_SRC:
					pr_integral ();
					break;
					// integral spiacs
				case TYPE_INTEGRAL_SPIACS_SRC:
					pr_integral_spicas ();
					break;
				case TYPE_SWIFT_BAT_GRB_POS_ACK_SRC:
				case TYPE_SWIFT_BAT_GRB_LC_SRC:
				case TYPE_SWIFT_FOM_2OBSAT_SRC:
				case TYPE_SWIFT_FOSC_2OBSAT_SRC:
				case TYPE_SWIFT_XRT_POSITION_SRC:
				case TYPE_SWIFT_XRT_SPECTRUM_SRC:
				case TYPE_SWIFT_XRT_IMAGE_SRC:
				case TYPE_SWIFT_XRT_LC_SRC:
				case TYPE_SWIFT_UVOT_SLIST_SRC:
					// processed messages
				case TYPE_SWIFT_BAT_GRB_LC_PROC_SRC:
				case TYPE_SWIFT_XRT_SPECTRUM_PROC_SRC:
				case TYPE_SWIFT_XRT_IMAGE_PROC_SRC:
				case TYPE_SWIFT_UVOT_SLIST_PROC_SRC:
				case TYPE_SWIFT_UVOT_POS_SRC:
					// transient
				case TYPE_SWIFT_BAT_TRANS:
					pr_swift_with_radec ();
					break;
				case TYPE_SWIFT_BAT_GRB_ALERT_SRC:
				case TYPE_SWIFT_BAT_GRB_POS_NACK_SRC:
				case TYPE_SWIFT_SCALEDMAP_SRC:
				case TYPE_SWIFT_XRT_CENTROID_SRC:
				case TYPE_SWIFT_UVOT_IMAGE_SRC:
					// processed messages
				case TYPE_SWIFT_UVOT_IMAGE_PROC_SRC:
				case TYPE_SWIFT_UVOT_NACK_POSITION:
					pr_swift_without_radec ();
					break;
				case TYPE_AGILE_GRB_WAKEUP:
				case TYPE_AGILE_GRB_PROMPT:
				case TYPE_AGILE_GRB_REFINED:
				case TYPE_AGILE_TRANS:
				case TYPE_AGILE_GRB_POS_TEST:
					pr_agile ();
					break;
				case TYPE_FERMI_GBM_ALERT:
					break;
				case TYPE_FERMI_GBM_FLT_POS:
				case TYPE_FERMI_GBM_GND_POS:
				case TYPE_FERMI_GBM_LC:
				case TYPE_FERMI_GBM_TRANS:
					pr_fermi_gbm ();
					break;
				case TYPE_FERMI_LAT_POS_INI:
				case TYPE_FERMI_LAT_POS_UPD:
				case TYPE_FERMI_LAT_POS_DIAG:
				case TYPE_FERMI_LAT_TRANS:
					pr_fermi_lat ();
					break;
				case TYPE_FERMI_OBS_REQ:
				case TYPE_FERMI_SC_SLEW:
					pr_fermi_sc ();
					break;
				case TYPE_KILL_SOCKET:
					connectionError (-1);
					break;
				default:
					logStream (MESSAGE_ERROR) << "ConnGrb::receive unknow packet type: " << lbuf[PKT_TYPE] << sendLog;
					break;
			}
			// enable others to catch-up (FW connections will forward packet to their sockets)
			getMaster ()->postEvent (new rts2core::Event (RTS2_EVENT_GRB_PACKET, nbuf));
		}
		catch (rts2core::Error er)
		{
			logStream (MESSAGE_ERROR) << er << sendLog;
		}
	}
	return ret;
}

int ConnGrb::lastPacket ()
{
	return last_packet.tv_sec;
}

double ConnGrb::delta ()
{
	return deltaValue;
}

char* ConnGrb::lastTarget ()
{
	return last_target;
}
