/*!
* This Source Code Form is subject to the terms of the Mozilla Public
* License, v. 2.0. If a copy of the MPL was not distributed with this file,
* You can obtain one at http://mozilla.org/MPL/2.0/.
*
* Copyright(c) 2012 Apogee Instruments, Inc.
* \brief This namespace contains the camera pattern data
*   created on Wed Apr 18 11:42:13 2012
*
*/
#include <stdint.h>
#include "patterns.h"

bool patterns::GetVPatternFx(const std::string & name, patterns::getVPatternPtr & out )
{
    std::map<std::string, patterns::getVPatternPtr> fxMap;
    patterns::getStrVPatternFxMap( fxMap );
    std::map<std::string, patterns::getVPatternPtr>::iterator iter = fxMap.find(name);
    if( iter != fxMap.end() )
    {
         out = (*iter).second;
         return true;
     }
     return false;
}

void patterns::getStrVPatternFxMap(std::map<std::string, patterns::getVPatternPtr> & out )
{
    out.clear();

    out["AV1"] = patterns::getPatternAV1;
    out["AV2"] = patterns::getPatternAV2;
    out["AV2S"] = patterns::getPatternAV2S;
    out["AV2V"] = patterns::getPatternAV2V;
    out["AV2Vb"] = patterns::getPatternAV2Vb;
    out["AV4"] = patterns::getPatternAV4;
    out["AV4V"] = patterns::getPatternAV4V;
    out["AV4Vb"] = patterns::getPatternAV4Vb;
    out["AV5"] = patterns::getPatternAV5;
    out["AV5V"] = patterns::getPatternAV5V;
    out["AV6"] = patterns::getPatternAV6;
    out["AV7"] = patterns::getPatternAV7;
    out["FEV1"] = patterns::getPatternFEV1;
    out["FKV1"] = patterns::getPatternFKV1;
    out["FKV2"] = patterns::getPatternFKV2;
    out["FKV3"] = patterns::getPatternFKV3;
    out["FKV4"] = patterns::getPatternFKV4;
    out["FKV5"] = patterns::getPatternFKV5;
    out["FKV6"] = patterns::getPatternFKV6;
    out["FV1"] = patterns::getPatternFV1;
    out["FV1B"] = patterns::getPatternFV1B;
    out["HV1"] = patterns::getPatternHV1;
    out["HV1a"] = patterns::getPatternHV1a;
    out["KV1"] = patterns::getPatternKV1;
    out["KV2"] = patterns::getPatternKV2;
    out["KV2B"] = patterns::getPatternKV2B;
    out["KV3"] = patterns::getPatternKV3;
    out["KV3B"] = patterns::getPatternKV3B;
    out["KV4"] = patterns::getPatternKV4;
    out["KV4B"] = patterns::getPatternKV4B;
    out["KV5"] = patterns::getPatternKV5;
    out["KV5ns"] = patterns::getPatternKV5ns;
    out["KV6"] = patterns::getPatternKV6;
    out["KV6B"] = patterns::getPatternKV6B;
    out["KV7"] = patterns::getPatternKV7;
    out["KV7B"] = patterns::getPatternKV7B;
    out["KV8"] = patterns::getPatternKV8;
    out["KV8B"] = patterns::getPatternKV8B;
    out["KV9"] = patterns::getPatternKV9;
    out["MV1"] = patterns::getPatternMV1;
    out["MV1B"] = patterns::getPatternMV1B;
    out["MV1i"] = patterns::getPatternMV1i;
    out["MV1nimo"] = patterns::getPatternMV1nimo;
    out["MV1p2high"] = patterns::getPatternMV1p2high;
    out["MV2"] = patterns::getPatternMV2;
    out["MV230"] = patterns::getPatternMV230;
    out["MV2B"] = patterns::getPatternMV2B;
    out["MV2C"] = patterns::getPatternMV2C;
    out["MV2CN"] = patterns::getPatternMV2CN;
    out["MV2i"] = patterns::getPatternMV2i;
    out["MV3"] = patterns::getPatternMV3;
    out["MV4"] = patterns::getPatternMV4;
    out["MV4B"] = patterns::getPatternMV4B;
    out["MV7"] = patterns::getPatternMV7;
    out["MV7i"] = patterns::getPatternMV7i;
    out["MV8"] = patterns::getPatternMV8;
    out["NoOpV"] = patterns::getPatternNoOpV;
    out["XAV1"] = patterns::getPatternXAV1;
}

bool patterns::GetHPatternFx(const std::string & name, patterns::getHPatternPtr & out )
{
    std::map<std::string, patterns::getHPatternPtr> fxMap;
    patterns::getStrHPatternFxMap( fxMap );
    std::map<std::string, patterns::getHPatternPtr>::iterator iter = fxMap.find(name);;
    if( iter != fxMap.end() )
    {
         out = (*iter).second;
         return true;
     }
     return false;
}

void patterns::getStrHPatternFxMap(std::map<std::string, patterns::getHPatternPtr> & out )
{
    out.clear();

    out["A12HC"] = patterns::getPatternA12HC;
    out["A12HR"] = patterns::getPatternA12HR;
    out["A12HS"] = patterns::getPatternA12HS;
    out["A16HC"] = patterns::getPatternA16HC;
    out["A16HR"] = patterns::getPatternA16HR;
    out["A16HS"] = patterns::getPatternA16HS;
    out["AR2"] = patterns::getPatternAR2;
    out["AR2F"] = patterns::getPatternAR2F;
    out["AR2Fa"] = patterns::getPatternAR2Fa;
    out["AR2S"] = patterns::getPatternAR2S;
    out["AR2V"] = patterns::getPatternAR2V;
    out["AR2Vb"] = patterns::getPatternAR2Vb;
    out["AR2a"] = patterns::getPatternAR2a;
    out["AR4"] = patterns::getPatternAR4;
    out["AR4F"] = patterns::getPatternAR4F;
    out["AR4Fb"] = patterns::getPatternAR4Fb;
    out["AR4V"] = patterns::getPatternAR4V;
    out["AR4Vb"] = patterns::getPatternAR4Vb;
    out["AR5"] = patterns::getPatternAR5;
    out["AR5F"] = patterns::getPatternAR5F;
    out["AR5V"] = patterns::getPatternAR5V;
    out["AS2"] = patterns::getPatternAS2;
    out["AS2S"] = patterns::getPatternAS2S;
    out["AS4"] = patterns::getPatternAS4;
    out["AS5"] = patterns::getPatternAS5;
    out["F16HR1"] = patterns::getPatternF16HR1;
    out["F16HR1B"] = patterns::getPatternF16HR1B;
    out["F16HS1"] = patterns::getPatternF16HS1;
    out["F16HS1B"] = patterns::getPatternF16HS1B;
    out["FEHR1A"] = patterns::getPatternFEHR1A;
    out["FEHR1AQ"] = patterns::getPatternFEHR1AQ;
    out["FEHR1Ah2_3"] = patterns::getPatternFEHR1Ah2_3;
    out["FEHR1B"] = patterns::getPatternFEHR1B;
    out["FEHR1BQ"] = patterns::getPatternFEHR1BQ;
    out["FEHS1"] = patterns::getPatternFEHS1;
    out["FEHS1A"] = patterns::getPatternFEHS1A;
    out["FEHS1Ah2_3"] = patterns::getPatternFEHS1Ah2_3;
    out["FEHS1B"] = patterns::getPatternFEHS1B;
    out["FKHR1"] = patterns::getPatternFKHR1;
    out["FKHR1A"] = patterns::getPatternFKHR1A;
    out["FKHR1B"] = patterns::getPatternFKHR1B;
    out["FKHR1_3_22MHz"] = patterns::getPatternFKHR1_3_22MHz;
    out["FKHR1_6_25MHz"] = patterns::getPatternFKHR1_6_25MHz;
    out["FKHR2A"] = patterns::getPatternFKHR2A;
    out["FKHR2B"] = patterns::getPatternFKHR2B;
    out["FKHR3A"] = patterns::getPatternFKHR3A;
    out["FKHR3B"] = patterns::getPatternFKHR3B;
    out["FKHR4A"] = patterns::getPatternFKHR4A;
    out["FKHR4AQ"] = patterns::getPatternFKHR4AQ;
    out["FKHR4B"] = patterns::getPatternFKHR4B;
    out["FKHR4BQ"] = patterns::getPatternFKHR4BQ;
    out["FKHR5A"] = patterns::getPatternFKHR5A;
    out["FKHR5B"] = patterns::getPatternFKHR5B;
    out["FKHR6A"] = patterns::getPatternFKHR6A;
    out["FKHR6B"] = patterns::getPatternFKHR6B;
    out["FKHS1"] = patterns::getPatternFKHS1;
    out["FKHS1A"] = patterns::getPatternFKHS1A;
    out["FKHS1B"] = patterns::getPatternFKHS1B;
    out["FKHS2"] = patterns::getPatternFKHS2;
    out["FKHS3"] = patterns::getPatternFKHS3;
    out["FKHS4"] = patterns::getPatternFKHS4;
    out["FKHS5"] = patterns::getPatternFKHS5;
    out["FKHS6"] = patterns::getPatternFKHS6;
    out["FM12HC"] = patterns::getPatternFM12HC;
    out["FM12HR"] = patterns::getPatternFM12HR;
    out["FM12HS"] = patterns::getPatternFM12HS;
    out["FM16HR3"] = patterns::getPatternFM16HR3;
    out["FM16HS1"] = patterns::getPatternFM16HS1;
    out["FR2"] = patterns::getPatternFR2;
    out["FR2F"] = patterns::getPatternFR2F;
    out["H12HC"] = patterns::getPatternH12HC;
    out["H12HCi"] = patterns::getPatternH12HCi;
    out["H12HR"] = patterns::getPatternH12HR;
    out["H12HRi"] = patterns::getPatternH12HRi;
    out["H12HS"] = patterns::getPatternH12HS;
    out["H12HSi"] = patterns::getPatternH12HSi;
    out["H16HR"] = patterns::getPatternH16HR;
    out["H16HRa"] = patterns::getPatternH16HRa;
    out["H16HRi"] = patterns::getPatternH16HRi;
    out["H16HS"] = patterns::getPatternH16HS;
    out["H16HSa"] = patterns::getPatternH16HSa;
    out["H16HSi"] = patterns::getPatternH16HSi;
    out["K12HC2"] = patterns::getPatternK12HC2;
    out["K12HC2B"] = patterns::getPatternK12HC2B;
    out["K12HC3"] = patterns::getPatternK12HC3;
    out["K12HC3i"] = patterns::getPatternK12HC3i;
    out["K12HC4"] = patterns::getPatternK12HC4;
    out["K12HC5"] = patterns::getPatternK12HC5;
    out["K12HC5B"] = patterns::getPatternK12HC5B;
    out["K12HC6"] = patterns::getPatternK12HC6;
    out["K12HC6B"] = patterns::getPatternK12HC6B;
    out["K12HC7"] = patterns::getPatternK12HC7;
    out["K12HC8"] = patterns::getPatternK12HC8;
    out["K12HC9"] = patterns::getPatternK12HC9;
    out["K12HR10B"] = patterns::getPatternK12HR10B;
    out["K12HR2"] = patterns::getPatternK12HR2;
    out["K12HR2B"] = patterns::getPatternK12HR2B;
    out["K12HR3"] = patterns::getPatternK12HR3;
    out["K12HR3i"] = patterns::getPatternK12HR3i;
    out["K12HR4"] = patterns::getPatternK12HR4;
    out["K12HR5"] = patterns::getPatternK12HR5;
    out["K12HR5B"] = patterns::getPatternK12HR5B;
    out["K12HR6"] = patterns::getPatternK12HR6;
    out["K12HR6B"] = patterns::getPatternK12HR6B;
    out["K12HR7"] = patterns::getPatternK12HR7;
    out["K12HR8"] = patterns::getPatternK12HR8;
    out["K12HR9"] = patterns::getPatternK12HR9;
    out["K12HS10B"] = patterns::getPatternK12HS10B;
    out["K12HS2"] = patterns::getPatternK12HS2;
    out["K12HS2B"] = patterns::getPatternK12HS2B;
    out["K12HS3"] = patterns::getPatternK12HS3;
    out["K12HS3i"] = patterns::getPatternK12HS3i;
    out["K12HS4"] = patterns::getPatternK12HS4;
    out["K12HS5"] = patterns::getPatternK12HS5;
    out["K12HS5B"] = patterns::getPatternK12HS5B;
    out["K12HS6"] = patterns::getPatternK12HS6;
    out["K12HS6B"] = patterns::getPatternK12HS6B;
    out["K12HS7"] = patterns::getPatternK12HS7;
    out["K12HS8"] = patterns::getPatternK12HS8;
    out["K12HS9"] = patterns::getPatternK12HS9;
    out["K16HC1"] = patterns::getPatternK16HC1;
    out["K16HC2"] = patterns::getPatternK16HC2;
    out["K16HC2B"] = patterns::getPatternK16HC2B;
    out["K16HR1"] = patterns::getPatternK16HR1;
    out["K16HR10B"] = patterns::getPatternK16HR10B;
    out["K16HR2"] = patterns::getPatternK16HR2;
    out["K16HR2B"] = patterns::getPatternK16HR2B;
    out["K16HR3"] = patterns::getPatternK16HR3;
    out["K16HR31"] = patterns::getPatternK16HR31;
    out["K16HR3i"] = patterns::getPatternK16HR3i;
    out["K16HR3ns"] = patterns::getPatternK16HR3ns;
    out["K16HR5"] = patterns::getPatternK16HR5;
    out["K16HR5B"] = patterns::getPatternK16HR5B;
    out["K16HR6"] = patterns::getPatternK16HR6;
    out["K16HR6B"] = patterns::getPatternK16HR6B;
    out["K16HR7"] = patterns::getPatternK16HR7;
    out["K16HR8"] = patterns::getPatternK16HR8;
    out["K16HR8a"] = patterns::getPatternK16HR8a;
    out["K16HR9"] = patterns::getPatternK16HR9;
    out["K16HS1"] = patterns::getPatternK16HS1;
    out["K16HS10B"] = patterns::getPatternK16HS10B;
    out["K16HS2"] = patterns::getPatternK16HS2;
    out["K16HS2B"] = patterns::getPatternK16HS2B;
    out["K16HS3"] = patterns::getPatternK16HS3;
    out["K16HS3i"] = patterns::getPatternK16HS3i;
    out["K16HS5"] = patterns::getPatternK16HS5;
    out["K16HS5B"] = patterns::getPatternK16HS5B;
    out["K16HS6"] = patterns::getPatternK16HS6;
    out["K16HS6B"] = patterns::getPatternK16HS6B;
    out["K16HS7"] = patterns::getPatternK16HS7;
    out["K16HS8"] = patterns::getPatternK16HS8;
    out["K16HS9"] = patterns::getPatternK16HS9;
    out["M12HC"] = patterns::getPatternM12HC;
    out["M12HC7"] = patterns::getPatternM12HC7;
    out["M12HC7i"] = patterns::getPatternM12HC7i;
    out["M12HCB"] = patterns::getPatternM12HCB;
    out["M12HCC"] = patterns::getPatternM12HCC;
    out["M12HCi"] = patterns::getPatternM12HCi;
    out["M12HR"] = patterns::getPatternM12HR;
    out["M12HR2"] = patterns::getPatternM12HR2;
    out["M12HR7"] = patterns::getPatternM12HR7;
    out["M12HR7i"] = patterns::getPatternM12HR7i;
    out["M12HRB"] = patterns::getPatternM12HRB;
    out["M12HRC"] = patterns::getPatternM12HRC;
    out["M12HRi"] = patterns::getPatternM12HRi;
    out["M12HS"] = patterns::getPatternM12HS;
    out["M12HS2"] = patterns::getPatternM12HS2;
    out["M12HS7"] = patterns::getPatternM12HS7;
    out["M12HS7i"] = patterns::getPatternM12HS7i;
    out["M12HSB"] = patterns::getPatternM12HSB;
    out["M12HSC"] = patterns::getPatternM12HSC;
    out["M12HSi"] = patterns::getPatternM12HSi;
    out["M16HC1"] = patterns::getPatternM16HC1;
    out["M16HR1"] = patterns::getPatternM16HR1;
    out["M16HR2"] = patterns::getPatternM16HR2;
    out["M16HR230"] = patterns::getPatternM16HR230;
    out["M16HR3"] = patterns::getPatternM16HR3;
    out["M16HR3B"] = patterns::getPatternM16HR3B;
    out["M16HR3C"] = patterns::getPatternM16HR3C;
    out["M16HR3i"] = patterns::getPatternM16HR3i;
    out["M16HR4"] = patterns::getPatternM16HR4;
    out["M16HR5"] = patterns::getPatternM16HR5;
    out["M16HR5B"] = patterns::getPatternM16HR5B;
    out["M16HR5i"] = patterns::getPatternM16HR5i;
    out["M16HR7"] = patterns::getPatternM16HR7;
    out["M16HR7i"] = patterns::getPatternM16HR7i;
    out["M16HS1"] = patterns::getPatternM16HS1;
    out["M16HS1B"] = patterns::getPatternM16HS1B;
    out["M16HS1C"] = patterns::getPatternM16HS1C;
    out["M16HS1i"] = patterns::getPatternM16HS1i;
    out["M16HS2"] = patterns::getPatternM16HS2;
    out["M16HS230"] = patterns::getPatternM16HS230;
    out["M16HS7"] = patterns::getPatternM16HS7;
    out["M16HS7i"] = patterns::getPatternM16HS7i;
    out["NoOpH"] = patterns::getPatternNoOpH;
}


void patterns::getPatternAV1(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 30;

    uint16_t temp[] = {0x0002,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0202,0x0202,0x0202,0x0202,0x0202,0x0202,0x0202,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0003,0x0002};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternAV2(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 30;

    uint16_t temp[] = {0x0002,0x0002,0x0004,0x0004,0x0004,0x0004,0x0002,0x0002,0x0202,0x0202,0x0202,0x0202,0x0202,0x0002,0x0002,0x0003,0x0002};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternAV2S(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 6;

    uint16_t temp[] = {0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0002,0x0002,0x0002,0x0202,0x0202,0x0202,0x0202,0x0202,0x0202,0x0202,0x0002,0x0002,0x0003,0x0002};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternAV2V(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 6;

    uint16_t temp[] = {0x0002,0x0002,0x0004,0x0004,0x0202,0x0202,0x0002,0x0004,0x0004,0x0202,0x0203,0x0002};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternAV2Vb(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 6;

    uint16_t temp[] = {0x0002,0x0002,0x0004,0x0004,0x0202,0x0202,0x0002,0x0004,0x0004,0x0202,0x0202,0x0002,0x0004,0x0004,0x0202,0x0202,0x0002,0x0004,0x0004,0x0202,0x0203,0x0002};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternAV4(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 30;

    uint16_t temp[] = {0x000a,0x0006,0x0014,0x0014,0x0018,0x000a,0x020a,0x020a,0x020a,0x000a,0x000a,0x000b,0x000a};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternAV4V(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 30;

    uint16_t temp[] = {0x000a,0x000a,0x0006,0x0014,0x0018,0x020a,0x020a,0x000a,0x0006,0x0014,0x0018,0x020a,0x020b,0x000a};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternAV4Vb(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 30;

    uint16_t temp[] = {0x000a,0x000a,0x0006,0x0014,0x0018,0x020a,0x020a,0x000a,0x0006,0x0014,0x0018,0x020a,0x020a,0x000a,0x0006,0x0014,0x0018,0x020a,0x020a,0x000a,0x0006,0x0014,0x0018,0x020a,0x020b,0x000a};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternAV5(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 30;

    uint16_t temp[] = {0x000c,0x001c,0x001c,0x0018,0x0018,0x001a,0x001a,0x021a,0x0212,0x0212,0x0216,0x0216,0x0216,0x0206,0x0006,0x000e,0x000e,0x000d,0x000c};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternAV5V(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 30;

    uint16_t temp[] = {0x000c,0x001c,0x0018,0x001a,0x0212,0x0216,0x0206,0x000e,0x000c,0x001c,0x0018,0x001a,0x0212,0x0216,0x0206,0x000f,0x000c};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternAV6(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 30;

    uint16_t temp[] = {0x0004,0x0006,0x0006,0x0002,0x0002,0x000a,0x000a,0x0208,0x0208,0x020c,0x020c,0x0005,0x0004};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternAV7(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 30;

    uint16_t temp[] = {0x000a,0x0006,0x0006,0x0006,0x0006,0x0014,0x0014,0x0014,0x0014,0x0014,0x0014,0x0014,0x0014,0x0014,0x0014,0x0018,0x0018,0x0018,0x0018,0x000a,0x000a,0x020a,0x020a,0x020a,0x000a,0x000a,0x000b,0x000a};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternFEV1(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 30;

    uint16_t temp[] = {0x000c,0x001c,0x001c,0x001c,0x001c,0x001c,0x001c,0x001c,0x001c,0x001c,0x001c,0x001c,0x001c,0x001c,0x001c,0x001c,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x001a,0x001a,0x001a,0x001a,0x001a,0x001a,0x001a,0x001a,0x001a,0x001a,0x001a,0x001a,0x001a,0x001a,0x001a,0x0012,0x0012,0x0012,0x0012,0x0012,0x0012,0x0012,0x0012,0x0012,0x0012,0x0012,0x0012,0x0012,0x0012,0x0012,0x0016,0x0016,0x0016,0x0016,0x0016,0x0016,0x0016,0x0016,0x0016,0x0016,0x0016,0x0016,0x0016,0x0016,0x0016,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x000e,0x000e,0x000e,0x000e,0x000e,0x000e,0x000e,0x000e,0x000e,0x000e,0x000e,0x000e,0x000e,0x000e,0x000e,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000d,0x000c};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternFKV1(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 6;

    uint16_t temp[] = {000000,000000,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,000000,000000,000000,000000,000000,000000,000000,0x0001,000000};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternFKV2(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 30;

    uint16_t temp[] = {000000,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,0x0001,000000};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternFKV3(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 6;

    uint16_t temp[] = {000000,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,000000,000000,0x0001,000000};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternFKV4(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 30;

    uint16_t temp[] = {000000,000000,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,0x0001,000000};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternFKV5(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 30;

    uint16_t temp[] = {000000,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,000000,000000,000000,0x0001,000000};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternFKV6(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 30;

    uint16_t temp[] = {000000,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,000000,0x0001,000000};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternFV1(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 0;

    uint16_t temp[] = {000000,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x001c,0x001c,0x001c,0x001c,0x001c,0x001c,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x001a,0x001a,0x001a,0x001a,0x001a,0x001a,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,000000,000000,000000,000000,000000,000000,000000,000000,000000,0x0001,000000};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternFV1B(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 30;

    uint16_t temp[] = {000000,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x001c,0x001c,0x001c,0x001c,0x001c,0x001c,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x001a,0x001a,0x001a,0x001a,0x001a,0x001a,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,000000,000000,000000,000000,000000,000000,000000,000000,000000,0x0001,000000};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternHV1(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 6;

    uint16_t temp[] = {0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0003,0x0002};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternHV1a(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 6;

    uint16_t temp[] = {0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0003,0x0002};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternKV1(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 0;

    uint16_t temp[] = {000000,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,000000,000000,000000,000000,000000,000000,000000,000000,0x0001,000000};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternKV2(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 0;

    uint16_t temp[] = {000000,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0204,0x0204,0x0204,0x0204,0x0204,0x0204,0x0204,0x0204,0x0204,0x0204,000000,000000,000000,000000,000000,000000,0x0001,000000};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternKV2B(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 6;

    uint16_t temp[] = {000000,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0204,0x0204,0x0204,0x0204,0x0204,0x0204,0x0204,0x0204,0x0204,0x0204,000000,000000,000000,000000,000000,000000,0x0001,000000};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternKV3(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 262;

    uint16_t temp[] = {0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0302,0x0302,0x0302,0x0302,0x0302,0x0302,0x0302,0x0302,0x0302,0x0302,0x0002,0x0002,0x0002,0x0003,0x0002};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternKV3B(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 22;

    uint16_t temp[] = {0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0302,0x0302,0x0302,0x0302,0x0302,0x0302,0x0302,0x0302,0x0302,0x0302,0x0002,0x0002,0x0002,0x0003,0x0002};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternKV4(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 0;

    uint16_t temp[] = {000000,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0204,0x0204,0x0204,0x0204,0x0204,0x0204,0x0204,0x0204,0x0204,0x0204,0x0004,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,0x0001,000000};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternKV4B(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 6;

    uint16_t temp[] = {000000,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0204,0x0204,0x0204,0x0204,0x0204,0x0204,0x0204,0x0204,0x0204,0x0204,0x0004,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,0x0001,000000};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternKV5(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 6;

    uint16_t temp[] = {000000,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,000000,000000,000000,000000,000000,000000,000000,000000,0x0001,000000};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternKV5ns(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 6;

    uint16_t temp[] = {000000,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,000000,000000,000000,000000,000000,000000,000000,000000,0x0001,000000};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternKV6(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 262;

    uint16_t temp[] = {0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0302,0x0302,0x0302,0x0302,0x0302,0x0302,0x0302,0x0302,0x0302,0x0302,0x0002,0x0002,0x0002,0x0003,0x0002};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternKV6B(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 278;

    uint16_t temp[] = {0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0302,0x0302,0x0302,0x0302,0x0302,0x0302,0x0302,0x0302,0x0302,0x0302,0x0002,0x0002,0x0002,0x0003,0x0002};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternKV7(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 0;

    uint16_t temp[] = {000000,000000,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0202,0x0202,0x0202,0x0202,0x0202,0x0202,0x0202,0x0202,0x0202,0x0202,0x0002,0x0002,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,0x0001,000000};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternKV7B(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 6;

    uint16_t temp[] = {000000,000000,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0202,0x0202,0x0202,0x0202,0x0202,0x0202,0x0202,0x0202,0x0202,0x0202,0x0002,0x0002,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,000000,0x0001,000000};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternKV8(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 0;

    uint16_t temp[] = {000000,000000,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,000000,000000,000000,000000,000000,000000,000000,0x0001,000000};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternKV8B(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 278;

    uint16_t temp[] = {0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0302,0x0302,0x0302,0x0302,0x0302,0x0302,0x0302,0x0302,0x0302,0x0302,0x0002,0x0002,0x0002,0x0003,0x0002};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternKV9(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 6;

    uint16_t temp[] = {000000,000000,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,000000,000000,000000,000000,000000,000000,000000,0x0001,000000};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternMV1(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 0;

    uint16_t temp[] = {000000,000000,000000,000000,000000,000000,000000,000000,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,000000,000000,000000,000000,000000,000000,000000,0x0001,000000};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternMV1B(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 14;

    uint16_t temp[] = {000000,000000,000000,000000,000000,000000,000000,000000,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,000000,000000,000000,000000,000000,000000,000000,0x0001,000000};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternMV1i(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 30;

    uint16_t temp[] = {000000,000000,000000,000000,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,000000,000000,000000,000000,000000,000000,000000,0x0001,000000};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternMV1nimo(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 2;

    uint16_t temp[] = {000000,0x0004,0x0004,0x0006,0x000e,0x000e,0x000a,0x0008,0x0008,000000,000000,0x0001,000000};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternMV1p2high(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 0;

    uint16_t temp[] = {0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0005,0x0004};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternMV2(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 0;

    uint16_t temp[] = {000000,000000,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,000000,000000,000000,0x0001,000000};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternMV230(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 30;

    uint16_t temp[] = {000000,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0010,0x0010,0x0010,0x0012,0x0012,0x0012,0x0012,0x0012,0x0012,0x0012,0x0016,0x0016,0x0016,0x0016,0x0016,0x0016,0x0016,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,000000,000000,000000,0x0001,000000};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternMV2B(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 14;

    uint16_t temp[] = {000000,000000,000000,000000,000000,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,000000,000000,000000,000000,000000,000000,000000,0x0001,000000};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternMV2C(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 14;

    uint16_t temp[] = {000000,000000,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,000000,000000,000000,0x0001,000000};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternMV2CN(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 14;

    uint16_t temp[] = {0x0004,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0004,0x0004,0x0004,0x0005,0x0004};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternMV2i(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 30;

    uint16_t temp[] = {000000,000000,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,000000,000000,000000,0x0001,000000};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternMV3(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 0;

    uint16_t temp[] = {000000,000000,000000,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x0008,0x0008,0x0008,0x000a,0x000a,0x000a,0x000a,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,000000,000000,000000,000000,000000,000000,000000,0x0001,000000};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternMV4(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 0;

    uint16_t temp[] = {000000,000000,000000,000000,000000,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,000000,000000,000000,000000,000000,0x0001,000000};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternMV4B(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 14;

    uint16_t temp[] = {000000,000000,000000,000000,000000,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,000000,000000,000000,000000,000000,0x0001,000000};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternMV7(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 0;

    uint16_t temp[] = {000000,000000,000000,000000,000000,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x0008,0x0008,0x0008,0x0008,0x000c,0x000c,0x000c,0x000c,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,000000,000000,000000,0x0001,000000};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternMV7i(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 30;

    uint16_t temp[] = {000000,000000,000000,000000,000000,000000,000000,000000,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,000000,000000,000000,000000,000000,000000,000000,0x0001,000000};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternMV8(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 0;

    uint16_t temp[] = {0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0006,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x000c,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0002,0x0003,0x0002};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternNoOpV(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 0;

    //no pattern data 
    out.PatternData.clear();
}

void patterns::getPatternXAV1(CamCfg::APN_VPATTERN_FILE & out)
{
    out.Mask = 0;

    uint16_t temp[] = {000000,0x0018,0x0018,0x001a,0x001a,0x0012,0x0012,0x0016,0x0016,0x0006,0x0006,0x000e,0x000e,0x000c,0x000c,000000,000000,000000,000000,000000,000000,0x0200,0x0201,0x0200};
    std::vector<uint16_t> vect( temp, temp + sizeof(temp) / sizeof(uint16_t ) );

    out.PatternData = vect;
}

void patterns::getPatternA12HC(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26654;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x080a,0x081a,0x0818,0x0818,0x0818,0x0818,0x0814,0x0804,0x0804,0x0804,0x0805,0x0814};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternA12HR(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26654;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x000a,0x021a,0x0018,0x0018,0x0018,0x0014,0x2004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0014,0x021a,0x0018,0x0018,0x0018,0x2014,0x0014,0x0014,0x0014,0x0018,0x0018,0x0018,0x0018,0x0014,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

    uint16_t tempBin8[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin8( tempBin8, tempBin8 + sizeof(tempBin8) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin8 );

    uint16_t tempBin9[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin9( tempBin9, tempBin9 + sizeof(tempBin9) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin9 );

}

void patterns::getPatternA12HS(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26654;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x000a,0x021a,0x0018,0x0018,0x0018,0x2018,0x0014,0x0004,0x0004,0x0005,0x4014};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternA16HC(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 30;

    uint16_t aRef[] = {0x0074,0x007a,0x007a,0x0078,0x0078,0x1078,0x1078,0x1078,0x0078,0x0058,0x00d8,0x00d8,0x00d8,0x00d8,0x00d8,0x00d8,0x00d8,0x00d8,0x00d8,0x00d8,0x00d8,0x00d8,0x00d8,0x00d8,0x00d8};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0004,0x0004,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0004,0x0404,0x0004,0x0005,0x0074};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0058,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternA16HR(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 30;

    uint16_t aRef[] = {0x006a,0x007a,0x007a,0x007a,0x1078,0x1078,0x1078,0x0078,0x00f8,0x00d8,0x00d8,0x00d8,0x00d8,0x00d8,0x00d8,0x00d8,0x00d8,0x00d8,0x00d8,0x00d8,0x00d8,0x00d8,0x00d8,0x00d8,0x00d8};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0144,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x8004,0x8015,0x0414};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0058,0x0054};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

    uint16_t tempBin8[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin8( tempBin8, tempBin8 + sizeof(tempBin8) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin8 );

    uint16_t tempBin9[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin9( tempBin9, tempBin9 + sizeof(tempBin9) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin9 );

}

void patterns::getPatternA16HS(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 30;

    uint16_t aRef[] = {0x006a,0x007a,0x007a,0x0078,0x0078,0x1078,0x1078,0x1078,0x0078,0x0058,0x00d8,0x00d8,0x00d8,0x00d8,0x00d8,0x00d8,0x00d8,0x00d8,0x00d8,0x00d8,0x00d8,0x00d8,0x00d8,0x00d8,0x00d8};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0004,0x0004,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0004,0x0404,0x0004,0x0005,0x0074};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0058,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternAR2(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 60;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x2004,0x204a,0x204a,0x2048,0xa048,0x2088,0x2088,0x2008,0x2008,0x2008,0x2008,0x2008,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x2004,0x204a,0x204a,0x2048,0xa048,0x2088,0x2088,0x2088,0x2008,0x2008,0x2008,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0xa004,0x2104,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x2004,0x204a,0x204a,0x2048,0xa048,0x2088,0x2088,0x2088,0x2008,0x2008,0x2008,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0xa004,0x2104,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x2004,0x204a,0x204a,0x2048,0xa048,0x2088,0x2088,0x2088,0x2008,0x2008,0x2008,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0xa004,0x2104,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x2004,0x204a,0x204a,0x2048,0xa048,0x2088,0x2088,0x2088,0x2008,0x2008,0x2008,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0xa004,0x2104,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x2004,0x204a,0x204a,0x2048,0xa048,0x2088,0x2088,0x2088,0x2008,0x2008,0x2008,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0xa004,0x2104,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x2004,0x204a,0x204a,0x2048,0xa048,0x2088,0x2088,0x2088,0x2008,0x2008,0x2008,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0xa004,0x2104,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x2004,0x204a,0x204a,0x2048,0xa048,0x2088,0x2088,0x2088,0x2008,0x2008,0x2008,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0xa004,0x2104,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

}

void patterns::getPatternAR2F(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 12;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x2004,0x204a,0x204a,0xa048,0x2088,0x2088,0x2008,0x2004,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x2004,0x204a,0x204a,0xa048,0x2088,0x2088,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x2004,0x204a,0x204a,0xa048,0x2088,0x2088,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x2004,0x204a,0x204a,0xa048,0x2088,0x2088,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x2004,0x204a,0x204a,0xa048,0x2088,0x2088,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x2004,0x204a,0x204a,0xa048,0x2088,0x2088,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x2004,0x204a,0x204a,0xa048,0x2088,0x2088,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x2004,0x204a,0x204a,0xa048,0x2088,0x2088,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

}

void patterns::getPatternAR2Fa(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 12;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x2004,0x204a,0xa04a,0x2088,0x2088,0x2008,0x2004,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x2004,0x204a,0x204a,0xa048,0x2088,0x2088,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x2004,0x204a,0x204a,0xa048,0x2088,0x2088,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x2004,0x204a,0x204a,0xa048,0x2088,0x2088,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x2004,0x204a,0x204a,0xa048,0x2088,0x2088,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x2004,0x204a,0x204a,0xa048,0x2088,0x2088,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x2004,0x204a,0x204a,0xa048,0x2088,0x2088,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x2004,0x204a,0x204a,0xa048,0x2088,0x2088,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

}

void patterns::getPatternAR2S(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 12;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0004,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x8048,0x0088,0x0088,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0208,0x0208,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x8004,0x0104,0x0104,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0404,0x0404,0x4005,0x4004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0004,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x8048,0x0088,0x0088,0x0208,0x0208,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x8004,0x0104,0x0104,0x0404,0x0404,0x4005,0x4004};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0004,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x8048,0x0088,0x0088,0x0208,0x0208,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x8004,0x0104,0x0104,0x0404,0x0404,0x4005,0x4004};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0004,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x8048,0x0088,0x0088,0x0208,0x0208,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x8004,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0404,0x0404,0x0404,0x0404,0x0404,0x0404,0x4005,0x4004};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0004,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x8048,0x0088,0x0088,0x0208,0x0208,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x8004,0x0104,0x0104,0x0404,0x0404,0x4005,0x4004};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0004,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x8048,0x0088,0x0088,0x0208,0x0208,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x8004,0x0104,0x0104,0x0404,0x0404,0x4005,0x4004};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x0004,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x8048,0x0088,0x0088,0x0208,0x0208,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x8004,0x0104,0x0104,0x0404,0x0404,0x4005,0x4004};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x0004,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x8048,0x0088,0x0088,0x0208,0x0208,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x8004,0x0104,0x0104,0x0404,0x0404,0x4005,0x4004};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

}

void patterns::getPatternAR2V(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 12;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0004,0x004a,0x8048,0x0288,0x0004,0x8004,0x0504,0x400a,0x0008,0x0004,0x000b,0x0008};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternAR2Vb(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 12;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0004,0x004a,0x8048,0x0288,0x0004,0x8004,0x0504,0x400a,0x0008,0x0004,0x000a,0x0008,0x0004,0x000a,0x0008,0x0004,0x000b,0x0008};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternAR2a(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 12;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x2004,0x200a,0x200a,0x2048,0x2048,0xa048,0x2088,0x2088,0x2008,0x2008,0x2008,0x2008,0x2008,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x2004,0x200a,0x200a,0x2048,0x2048,0xa048,0x2088,0x2088,0x2008,0x2008,0x2008,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x2004,0x200a,0x200a,0x2048,0x2048,0xa048,0x2088,0x2088,0x2008,0x2008,0x2008,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x2004,0x200a,0x200a,0x2048,0x2048,0xa048,0x2088,0x2088,0x2008,0x2008,0x2008,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x2004,0x200a,0x200a,0x2048,0x2048,0xa048,0x2088,0x2088,0x2008,0x2008,0x2008,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x2004,0x200a,0x200a,0x2048,0x2048,0xa048,0x2088,0x2088,0x2008,0x2008,0x2008,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x2004,0x200a,0x200a,0x2048,0x2048,0xa048,0x2088,0x2088,0x2008,0x2008,0x2008,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x2004,0x200a,0x200a,0x2048,0x2048,0xa048,0x2088,0x2088,0x2008,0x2008,0x2008,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

}

void patterns::getPatternAR4(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 12;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x2004,0x204a,0x204a,0x2048,0xa048,0x2088,0x2088,0x2008,0x2008,0x2008,0x2008,0x2008,0x2008,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x2004,0x204a,0x204a,0x2048,0xa048,0x2088,0x2088,0x2008,0x2008,0x2008,0x2008,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x2004,0x204a,0x204a,0x2048,0xa048,0x2088,0x2088,0x2008,0x2008,0x2008,0x2008,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x2004,0x204a,0x204a,0x2048,0xa048,0x2088,0x2088,0x2008,0x2008,0x2008,0x2008,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x2004,0x204a,0x204a,0x2048,0xa048,0x2088,0x2088,0x2008,0x2008,0x2008,0x2008,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x2004,0x204a,0x204a,0x2048,0xa048,0x2088,0x2088,0x2008,0x2008,0x2008,0x2008,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x2004,0x204a,0x204a,0x2048,0xa048,0x2088,0x2088,0x2008,0x2008,0x2008,0x2008,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x2004,0x204a,0x204a,0x2048,0xa048,0x2088,0x2088,0x2008,0x2008,0x2008,0x2008,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

}

void patterns::getPatternAR4F(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 12;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x2004,0x204a,0x204a,0xa048,0x2088,0x2088,0x2008,0x2004,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x2004,0x204a,0x204a,0xa048,0x2088,0x2088,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x2004,0x204a,0x204a,0xa048,0x2088,0x2088,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x2004,0x204a,0x204a,0xa048,0x2088,0x2088,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x2004,0x204a,0x204a,0xa048,0x2088,0x2088,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x2004,0x204a,0x204a,0xa048,0x2088,0x2088,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x2004,0x204a,0x204a,0xa048,0x2088,0x2088,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x2004,0x204a,0x204a,0xa048,0x2088,0x2088,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

}

void patterns::getPatternAR4Fb(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 12;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x2004,0x204a,0x204a,0xa048,0x2088,0x2088,0x2008,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x2004,0x204a,0x204a,0xa048,0x2088,0x2088,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x2004,0x204a,0x204a,0xa048,0x2088,0x2088,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x2004,0x204a,0x204a,0xa048,0x2088,0x2088,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x2004,0x204a,0x204a,0xa048,0x2088,0x2088,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x2004,0x204a,0x204a,0xa048,0x2088,0x2088,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x2004,0x204a,0x204a,0xa048,0x2088,0x2088,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x2004,0x204a,0x204a,0xa048,0x2088,0x2088,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0x2004,0x2008,0x2008,0x2004,0xa004,0x2104,0x2104,0x3005,0x3004};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

}

void patterns::getPatternAR4V(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 12;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0004,0x004a,0x8048,0x0288,0x0004,0x8004,0x0504,0x400a,0x0008,0x0004,0x000b,0x0008};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternAR4Vb(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 12;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0004,0x004a,0x8048,0x0288,0x0004,0x8004,0x0504,0x400a,0x0008,0x0004,0x000b,0x0008,0x0004,0x000b,0x0008,0x0004,0x000b,0x0008};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternAR5(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 12;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0008,0x0046,0x0046,0x0044,0x8044,0x0084,0x0084,0x0084,0x0204,0x0204,0x0204,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0108,0x0108,0x0108,0x0408,0x0408,0x0409,0x4008};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0008,0x0046,0x0046,0x0044,0x8044,0x0084,0x0084,0x0084,0x0204,0x0204,0x0204,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0108,0x0108,0x0108,0x0408,0x0408,0x0409,0x4008};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0008,0x0046,0x0046,0x0044,0x8044,0x0084,0x0084,0x0084,0x0204,0x0204,0x0204,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0108,0x0108,0x0108,0x0408,0x0408,0x0409,0x4008};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0008,0x0046,0x0046,0x0044,0x8044,0x0084,0x0084,0x0084,0x0204,0x0204,0x0204,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0108,0x0108,0x0108,0x0408,0x0408,0x0409,0x4008};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0008,0x0046,0x0046,0x0044,0x8044,0x0084,0x0084,0x0084,0x0204,0x0204,0x0204,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0108,0x0108,0x0108,0x0408,0x0408,0x0409,0x4008};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0008,0x0046,0x0046,0x0044,0x8044,0x0084,0x0084,0x0084,0x0204,0x0204,0x0204,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0108,0x0108,0x0108,0x0408,0x0408,0x0409,0x4008};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x0008,0x0046,0x0046,0x0044,0x8044,0x0084,0x0084,0x0084,0x0204,0x0204,0x0204,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0108,0x0108,0x0108,0x0408,0x0408,0x0409,0x4008};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x0008,0x0046,0x0046,0x0044,0x8044,0x0084,0x0084,0x0084,0x0204,0x0204,0x0204,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0108,0x0108,0x0108,0x0408,0x0408,0x0409,0x4008};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

    uint16_t tempBin8[] = {0x0008,0x0046,0x0046,0x0044,0x8044,0x0084,0x0084,0x0084,0x0204,0x0204,0x0204,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0108,0x0108,0x0108,0x0408,0x0408,0x0409,0x4008};
    std::vector<uint16_t> vectBin8( tempBin8, tempBin8 + sizeof(tempBin8) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin8 );

    uint16_t tempBin9[] = {0x0008,0x0046,0x0046,0x0044,0x8044,0x0084,0x0084,0x0084,0x0204,0x0204,0x0204,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0108,0x0108,0x0108,0x0408,0x0408,0x0409,0x4008};
    std::vector<uint16_t> vectBin9( tempBin9, tempBin9 + sizeof(tempBin9) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin9 );

    uint16_t tempBin10[] = {0x0008,0x0046,0x0046,0x0044,0x8044,0x0084,0x0084,0x0084,0x0204,0x0204,0x0204,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0108,0x0108,0x0108,0x0408,0x0408,0x0409,0x4008};
    std::vector<uint16_t> vectBin10( tempBin10, tempBin10 + sizeof(tempBin10) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin10 );

    uint16_t tempBin11[] = {0x0008,0x0046,0x0046,0x0044,0x8044,0x0084,0x0084,0x0084,0x0204,0x0204,0x0204,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0108,0x0108,0x0108,0x0408,0x0408,0x0409,0x4008};
    std::vector<uint16_t> vectBin11( tempBin11, tempBin11 + sizeof(tempBin11) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin11 );

    uint16_t tempBin12[] = {0x0008,0x0046,0x0046,0x0044,0x8044,0x0084,0x0084,0x0084,0x0204,0x0204,0x0204,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0108,0x0108,0x0108,0x0408,0x0408,0x0409,0x4008};
    std::vector<uint16_t> vectBin12( tempBin12, tempBin12 + sizeof(tempBin12) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin12 );

    uint16_t tempBin13[] = {0x0008,0x0046,0x0046,0x0044,0x8044,0x0084,0x0084,0x0084,0x0204,0x0204,0x0204,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0108,0x0108,0x0108,0x0408,0x0408,0x0409,0x4008};
    std::vector<uint16_t> vectBin13( tempBin13, tempBin13 + sizeof(tempBin13) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin13 );

    uint16_t tempBin14[] = {0x0008,0x0046,0x0046,0x0044,0x8044,0x0084,0x0084,0x0084,0x0204,0x0204,0x0204,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0108,0x0108,0x0108,0x0408,0x0408,0x0409,0x4008};
    std::vector<uint16_t> vectBin14( tempBin14, tempBin14 + sizeof(tempBin14) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin14 );

    uint16_t tempBin15[] = {0x0008,0x0046,0x0046,0x0044,0x8044,0x0084,0x0084,0x0084,0x0204,0x0204,0x0204,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0108,0x0108,0x0108,0x0408,0x0408,0x0409,0x4008};
    std::vector<uint16_t> vectBin15( tempBin15, tempBin15 + sizeof(tempBin15) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin15 );

}

void patterns::getPatternAR5F(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 12;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0008,0x0046,0x0046,0x8044,0x0084,0x0284,0x0204,0x0004,0x0008,0x8008,0x0108,0x0508,0x0409,0x4008};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0008,0x0046,0x0046,0x8044,0x0284,0x0284,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x8008,0x0508,0x0509,0x4008};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0008,0x0046,0x0046,0x8044,0x0284,0x0284,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x8008,0x0508,0x0509,0x4008};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0008,0x0046,0x0046,0x8044,0x0284,0x0284,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x8008,0x0508,0x0509,0x4008};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0008,0x0046,0x0046,0x8044,0x0284,0x0284,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x8008,0x0508,0x0509,0x4008};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0008,0x0046,0x0046,0x8044,0x0284,0x0284,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x8008,0x0508,0x0509,0x4008};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x0008,0x0046,0x0046,0x8044,0x0284,0x0284,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x8008,0x0508,0x0509,0x4008};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x0008,0x0046,0x0046,0x8044,0x0284,0x0284,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x8008,0x0508,0x0509,0x4008};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

    uint16_t tempBin8[] = {0x0008,0x0046,0x0046,0x8044,0x0284,0x0284,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x8008,0x0508,0x0509,0x4008};
    std::vector<uint16_t> vectBin8( tempBin8, tempBin8 + sizeof(tempBin8) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin8 );

    uint16_t tempBin9[] = {0x0008,0x0046,0x0046,0x8044,0x0284,0x0284,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x8008,0x0508,0x0509,0x4008};
    std::vector<uint16_t> vectBin9( tempBin9, tempBin9 + sizeof(tempBin9) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin9 );

    uint16_t tempBin10[] = {0x0008,0x0046,0x0046,0x8044,0x0284,0x0284,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x8008,0x0508,0x0509,0x4008};
    std::vector<uint16_t> vectBin10( tempBin10, tempBin10 + sizeof(tempBin10) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin10 );

    uint16_t tempBin11[] = {0x0008,0x0046,0x0046,0x8044,0x0284,0x0284,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x8008,0x0508,0x0509,0x4008};
    std::vector<uint16_t> vectBin11( tempBin11, tempBin11 + sizeof(tempBin11) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin11 );

    uint16_t tempBin12[] = {0x0008,0x0046,0x0046,0x8044,0x0284,0x0284,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x8008,0x0508,0x0509,0x4008};
    std::vector<uint16_t> vectBin12( tempBin12, tempBin12 + sizeof(tempBin12) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin12 );

    uint16_t tempBin13[] = {0x0008,0x0046,0x0046,0x8044,0x0284,0x0284,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x8008,0x0508,0x0509,0x4008};
    std::vector<uint16_t> vectBin13( tempBin13, tempBin13 + sizeof(tempBin13) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin13 );

    uint16_t tempBin14[] = {0x0008,0x0046,0x0046,0x8044,0x0284,0x0284,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x8008,0x0508,0x0509,0x4008};
    std::vector<uint16_t> vectBin14( tempBin14, tempBin14 + sizeof(tempBin14) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin14 );

    uint16_t tempBin15[] = {0x0008,0x0046,0x0046,0x8044,0x0284,0x0284,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x8008,0x0508,0x0509,0x4008};
    std::vector<uint16_t> vectBin15( tempBin15, tempBin15 + sizeof(tempBin15) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin15 );

}

void patterns::getPatternAR5V(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 12;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0008,0x0046,0x8044,0x0284,0x0008,0x8008,0x0508,0x4006,0x0004,0x0008,0x0007,0x0004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternAS2(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 60;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0004,0x000a,0x000a,0x0008,0x0008,0x0008,0x0004,0x0004,0x0005,0x0004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternAS2S(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 12;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0004,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0005,0x0004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternAS4(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 12;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0004,0x000a,0x000a,0x0008,0x0008,0x0008,0x0004,0x0004,0x0005,0x0004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternAS5(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 12;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0008,0x0006,0x0006,0x0004,0x0004,0x0004,0x0008,0x0008,0x0009,0x0008};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternF16HR1(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 542;

    uint16_t aRef[] = {0x006c,0x0276,0x0276,0x0276,0x0276,0x0276,0x0276,0x0276,0x0276,0x0270,0x0270,0x0270,0x0270,0x0270,0x0270,0x0270,0x1270,0x1270,0x02f0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d8,0x02d8,0x02d8,0x02d8,0x02d8};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0148,0x0108,0x0108,0x0108,0x0108,0x0108,0x0108,0x0108,0x0108,0x0108,0x0108,0x0108,0x0108,0x010c,0x010c,0x010c,0x010c,0x010c,0x010c,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x8104,0x8004,0x0004,0x0004,0x0004,0x0405,0x0404};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x02d8,0x0048};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0048,0x0048,0x004c,0x004c,0x004c,0x004c,0x004c,0x004c,0x004c,0x004c,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0050,0x0050,0x0050,0x0050,0x0050,0x0050,0x0058,0x0058,0x0058,0x0058,0x0058,0x0058,0x0058};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

}

void patterns::getPatternF16HR1B(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 542;

    uint16_t aRef[] = {0x006c,0x0276,0x0276,0x0276,0x0276,0x0276,0x0276,0x0276,0x0276,0x0270,0x0270,0x0270,0x0270,0x0270,0x0270,0x0270,0x1270,0x1270,0x02f0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d8,0x02d8,0x02d8,0x02d8,0x02d8};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0148,0x0108,0x0108,0x0108,0x0108,0x0108,0x0108,0x0108,0x0108,0x0108,0x0108,0x0108,0x0108,0x010c,0x010c,0x010c,0x010c,0x010c,0x010c,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0004,0x0004,0x0004,0x0004,0x8405,0x8404};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x02d8,0x0048};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0048,0x0048,0x0048,0x004c,0x004c,0x004c,0x0044,0x0044,0x0044,0x0254,0x0254,0x0254,0x0250,0x0250,0x0250,0x0258,0x0258,0x0258};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0048,0x0048,0x0048,0x004c,0x004c,0x004c,0x0044,0x0044,0x0044,0x0254,0x0254,0x0254,0x0250,0x0250,0x0250,0x0258,0x0258,0x0258,0x0048,0x0048,0x0048,0x004c,0x004c,0x004c,0x0044,0x0044,0x0044,0x0254,0x0254,0x0254,0x0250,0x0250,0x0250,0x0258,0x0258,0x0258};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0048,0x0048,0x0048,0x004c,0x004c,0x004c,0x0044,0x0044,0x0044,0x0254,0x0254,0x0254,0x0250,0x0250,0x0250,0x0258,0x0258,0x0258,0x0048,0x0048,0x0048,0x004c,0x004c,0x004c,0x0044,0x0044,0x0044,0x0254,0x0254,0x0254,0x0250,0x0250,0x0250,0x0258,0x0258,0x0258,0x0048,0x0048,0x0048,0x004c,0x004c,0x004c,0x0044,0x0044,0x0044,0x0254,0x0254,0x0254,0x0250,0x0250,0x0250,0x0258,0x0258,0x0258};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0048,0x0048,0x0048,0x004c,0x004c,0x004c,0x0044,0x0044,0x0044,0x0254,0x0254,0x0254,0x0250,0x0250,0x0250,0x0258,0x0258,0x0258,0x0048,0x0048,0x0048,0x004c,0x004c,0x004c,0x0044,0x0044,0x0044,0x0254,0x0254,0x0254,0x0250,0x0250,0x0250,0x0258,0x0258,0x0258,0x0048,0x0048,0x0048,0x004c,0x004c,0x004c,0x0044,0x0044,0x0044,0x0254,0x0254,0x0254,0x0250,0x0250,0x0250,0x0258,0x0258,0x0258,0x0048,0x0048,0x0048,0x004c,0x004c,0x004c,0x0044,0x0044,0x0044,0x0254,0x0254,0x0254,0x0250,0x0250,0x0250,0x0258,0x0258,0x0258};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

}

void patterns::getPatternF16HS1(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 542;

    uint16_t aRef[] = {0x006c,0x0276,0x0276,0x0276,0x0276,0x0276,0x0276,0x0276,0x0276,0x0270,0x0270,0x0270,0x0270,0x0270,0x0270,0x0270,0x0270,0x0270,0x02f0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d8,0x02d8,0x02d8,0x02d8,0x02d8};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0148,0x0108,0x0108,0x0108,0x0108,0x0108,0x0108,0x0108,0x0108,0x0108,0x0108,0x0108,0x0108,0x010c,0x010c,0x010c,0x010c,0x010c,0x010c,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0004,0x0004,0x0004,0x0004,0x0004,0x0405,0x0404};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x02d8,0x0048};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

}

void patterns::getPatternF16HS1B(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 542;

    uint16_t aRef[] = {0x006c,0x0276,0x0276,0x0276,0x0270,0x1270,0x1270,0x02f0,0x02d8,0x02d8,0x02d8};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0048,0x0148,0x010c,0x010c,0x0104,0x0004,0x0405,0x0404};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0048,0x0048};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternFEHR1A(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 30;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x201a,0x205a,0x2058,0xa058,0x2018,0x2018,0x2018,0x2018,0x2018,0x2018,0x2090,0x2090,0x2010,0x2010,0x2014,0x2014,0x2014,0x2014,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0xa004,0x2104,0x2104,0x2004,0x2004,0x2004,0x200c,0x200c,0x200e,0x200e,0x300b,0x300a};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x201a,0x205a,0x2058,0xa058,0x2018,0x2018,0x2018,0x2018,0x2018,0x2018,0x2090,0x2090,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x200c,0x200c,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2014,0x2014,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0xa004,0x2104,0x2104,0x2004,0x2004,0x2004,0x200c,0x200c,0x200e,0x200e,0x300b,0x300a};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x201a,0x205a,0x2058,0xa058,0x2018,0x2018,0x2018,0x2018,0x2018,0x2018,0x2090,0x2090,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2014,0x2014,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0xa004,0x2104,0x2104,0x2004,0x2004,0x2004,0x200c,0x200c,0x200e,0x200e,0x300b,0x300a};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x201a,0x205a,0x2058,0xa058,0x2018,0x2018,0x2018,0x2018,0x2018,0x2018,0x2090,0x2090,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2014,0x2014,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0xa004,0x2104,0x2104,0x2004,0x2004,0x2004,0x200c,0x200c,0x200e,0x200e,0x300b,0x300a};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x201a,0x205a,0x2058,0xa058,0x2018,0x2018,0x2018,0x2018,0x2018,0x2018,0x2090,0x2090,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2014,0x2014,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0xa004,0x2104,0x2104,0x2004,0x2004,0x2004,0x200c,0x200c,0x200e,0x200e,0x300b,0x300a};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x201a,0x205a,0x2058,0xa058,0x2018,0x2018,0x2018,0x2018,0x2018,0x2018,0x2090,0x2090,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2014,0x2014,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0xa004,0x2104,0x2104,0x2004,0x2004,0x2004,0x200c,0x200c,0x200e,0x200e,0x300b,0x300a};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x201a,0x205a,0x2058,0xa058,0x2018,0x2018,0x2018,0x2018,0x2018,0x2018,0x2090,0x2090,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2014,0x2014,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0xa004,0x2104,0x2104,0x2004,0x2004,0x2004,0x200c,0x200c,0x200e,0x200e,0x300b,0x300a};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x201a,0x205a,0x2058,0xa058,0x2018,0x2018,0x2018,0x2018,0x2018,0x2018,0x2090,0x2090,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2014,0x2014,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0xa004,0x2104,0x2104,0x2004,0x2004,0x2004,0x200c,0x200c,0x200e,0x200e,0x300b,0x300a};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

}

void patterns::getPatternFEHR1AQ(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 30;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x001a,0x0010,0x0010,0x0010,0x0050,0x0050,0x8050,0x0050,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x0050,0x0050,0x0010,0x0010,0x8010,0x0004,0x0004,0x0004,0x0004,0x0504,0x0504,0x0d44,0x0d44,0x8d44,0x0d44,0x0d44,0x0d44,0x0d44,0x0d44,0x0d44,0x0844,0x0844,0x4804,0x0804,0x8804,0x0804,0x0804,0x0804,0x080a,0x080a,0x080a,0x080b,0x080a};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternFEHR1Ah2_3(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 30;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x201a,0x2058,0x2048,0x2048,0x2048,0x2048,0x2048,0x2048,0x2048,0xa048,0x2088,0x2088,0x2008,0x2008,0x2008,0x2008,0x2008,0x2008,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0xa004,0x2104,0x2104,0x2004,0x2004,0x2004,0x2004,0x2010,0x2012,0x2012,0x2012,0x3013,0x3012};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternFEHR1B(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 30;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x201a,0x205a,0x2058,0xa058,0x2018,0x2010,0x2010,0x2010,0x2010,0x2010,0x2010,0x2090,0x2090,0x2010,0x2010,0x2010,0x2010,0x2014,0x2014,0x2014,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0xa004,0x2104,0x2104,0x2004,0x2004,0x2004,0x200c,0x200c,0x200c,0x300b,0x300a};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x201a,0x205a,0x2058,0xa058,0x2018,0x2010,0x2010,0x2010,0x2010,0x2010,0x2010,0x2090,0x2090,0x2010,0x2010,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2014,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0xa004,0x2104,0x2104,0x2004,0x2004,0x2004,0x200c,0x200c,0x200c,0x300b,0x300a};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x201a,0x205a,0x2058,0xa058,0x2018,0x2010,0x2010,0x2010,0x2010,0x2010,0x2010,0x2090,0x2090,0x2010,0x2010,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2014,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0xa004,0x2104,0x2104,0x2004,0x2004,0x2004,0x200c,0x200c,0x200c,0x300b,0x300a};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x201a,0x205a,0x2058,0xa058,0x2018,0x2010,0x2010,0x2010,0x2010,0x2010,0x2010,0x2090,0x2090,0x2010,0x2010,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2014,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0xa004,0x2104,0x2104,0x2004,0x2004,0x2004,0x200c,0x200c,0x200c,0x300b,0x300a};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x201a,0x205a,0x2058,0xa058,0x2018,0x2010,0x2010,0x2010,0x2010,0x2010,0x2010,0x2090,0x2090,0x2010,0x2010,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2014,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0xa004,0x2104,0x2104,0x2004,0x2004,0x2004,0x200c,0x200c,0x200c,0x300b,0x300a};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x201a,0x205a,0x2058,0xa058,0x2018,0x2010,0x2010,0x2010,0x2010,0x2010,0x2010,0x2090,0x2090,0x2010,0x2010,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2014,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0xa004,0x2104,0x2104,0x2004,0x2004,0x2004,0x200c,0x200c,0x200c,0x300b,0x300a};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x201a,0x205a,0x2058,0xa058,0x2018,0x2010,0x2010,0x2010,0x2010,0x2010,0x2010,0x2090,0x2090,0x2010,0x2010,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2014,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0xa004,0x2104,0x2104,0x2004,0x2004,0x2004,0x200c,0x200c,0x200c,0x300b,0x300a};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x201a,0x205a,0x2058,0xa058,0x2018,0x2010,0x2010,0x2010,0x2010,0x2010,0x2010,0x2090,0x2090,0x2010,0x2010,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2004,0x2004,0x200c,0x200c,0x2008,0x2008,0x2018,0x2018,0x2010,0x2010,0x2014,0x2014,0x2014,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0x2004,0xa004,0x2104,0x2104,0x2004,0x2004,0x2004,0x200c,0x200c,0x200c,0x300b,0x300a};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

}

void patterns::getPatternFEHR1BQ(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 30;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x001a,0x001a,0x0018,0x0018,0x0018,0x0050,0x0050,0x8050,0x0050,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x0050,0x0050,0x0010,0x0014,0x8014,0x0014,0x0004,0x0004,0x0004,0x0504,0x0504,0x0d44,0x0d44,0x8d44,0x0d44,0x0d44,0x0d44,0x0d44,0x0d44,0x0d44,0x0d44,0x0844,0x0844,0x4804,0x0804,0x8804,0x0804,0x080c,0x080c,0x080c,0x080b,0x080a};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternFEHS1(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 62;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0004,0x000c,0x000c,0x000e,0x000e,0x000a,0x000a,0x0008,0x0008,0x0038,0x0038,0x0038,0x0038,0x0038,0x0038,0x0038,0x0038,0x0030,0x0030,0x0030,0x0030,0x0034,0x0034,0x0034,0x0034,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0005,0x0004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternFEHS1A(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 30;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x001a,0x0010,0x0010,0x0010,0x0010,0x0010,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x000a,0x000b,0x000a};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternFEHS1Ah2_3(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 30;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x001a,0x0018,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0010,0x0012,0x0013,0x0012};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternFEHS1B(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 30;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x001a,0x001a,0x0018,0x0018,0x0010,0x0010,0x0014,0x0014,0x0004,0x0004,0x000c,0x000c,0x000b,0x000a};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternFKHR1(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 14;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0048,0x004a,0x004a,0x0048,0x8048,0x0048,0x0008,0x0008,0x0008,0x0208,0x0208,0x0088,0x0088,0x0008,0x0008,0x0004,0x0004,0x0004,0x8004,0x0004,0x0404,0x0404,0x0004,0x0104,0x0104,0x4005,0x0004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternFKHR1A(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 14;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0004,0x0046,0x0046,0x0046,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x8044,0x0204,0x0284,0x0084,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0408,0x0508,0x0109,0x4008};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0004,0x0046,0x0046,0x0046,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x8044,0x0204,0x0284,0x0084,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0408,0x0508,0x0109,0x4008};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0004,0x0046,0x0046,0x0046,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x8044,0x0204,0x0284,0x0084,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0408,0x0508,0x0109,0x4008};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0004,0x0046,0x0046,0x0046,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x8044,0x0204,0x0284,0x0084,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0408,0x0508,0x0109,0x4008};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0004,0x0046,0x0046,0x0046,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x8044,0x0204,0x0284,0x0084,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0408,0x0508,0x0109,0x4008};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0004,0x0046,0x0046,0x0046,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x8044,0x0204,0x0284,0x0084,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0408,0x0508,0x0109,0x4008};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x0004,0x0046,0x0046,0x0046,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x8044,0x0204,0x0284,0x0084,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0408,0x0508,0x0109,0x4008};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x0004,0x0046,0x0046,0x0046,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x8044,0x0204,0x0284,0x0084,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0408,0x0508,0x0109,0x4008};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

}

void patterns::getPatternFKHR1B(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 62;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0004,0x0046,0x0046,0x0044,0x0044,0x8044,0x0204,0x0284,0x0084,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0408,0x0508,0x0109,0x4008};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0004,0x0046,0x0046,0x0044,0x0044,0x8044,0x0204,0x0284,0x0084,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0408,0x0508,0x0109,0x4008};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0004,0x0046,0x0046,0x0044,0x0044,0x8044,0x0204,0x0284,0x0084,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0408,0x0508,0x0109,0x4008};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0004,0x0046,0x0046,0x0044,0x0044,0x8044,0x0204,0x0284,0x0084,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0408,0x0508,0x0109,0x4008};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0004,0x0046,0x0046,0x0044,0x0044,0x8044,0x0204,0x0284,0x0084,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0408,0x0508,0x0109,0x4008};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0004,0x0046,0x0046,0x0044,0x0044,0x8044,0x0204,0x0284,0x0084,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0408,0x0508,0x0109,0x4008};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x0004,0x0046,0x0046,0x0044,0x0044,0x8044,0x0204,0x0284,0x0084,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0408,0x0508,0x0109,0x4008};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x0004,0x0046,0x0046,0x0044,0x0044,0x8044,0x0204,0x0284,0x0084,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0408,0x0508,0x0109,0x4008};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

    uint16_t tempBin8[] = {0x0004,0x0046,0x0046,0x0044,0x0044,0x8044,0x0204,0x0284,0x0084,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0408,0x0508,0x0109,0x4008};
    std::vector<uint16_t> vectBin8( tempBin8, tempBin8 + sizeof(tempBin8) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin8 );

    uint16_t tempBin9[] = {0x0004,0x0046,0x0046,0x0044,0x0044,0x8044,0x0204,0x0284,0x0084,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0408,0x0508,0x0109,0x4008};
    std::vector<uint16_t> vectBin9( tempBin9, tempBin9 + sizeof(tempBin9) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin9 );

}

void patterns::getPatternFKHR1_3_22MHz(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 14;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0004,0x0046,0x8046,0x0084,0x0084,0x0204,0x0204,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0108,0x0108,0x0408,0x0409,0x4008};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0004,0x0046,0x8046,0x0084,0x0084,0x0204,0x0204,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x8008,0x0108,0x0108,0x0408,0x0409,0x4008};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0004,0x0046,0x8046,0x0084,0x0084,0x0204,0x0204,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0108,0x0108,0x0408,0x0409,0x4008};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0004,0x0046,0x8046,0x0084,0x0084,0x0204,0x0204,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0108,0x0108,0x0408,0x0409,0x4008};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

}

void patterns::getPatternFKHR1_6_25MHz(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 14;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0004,0x0046,0x8046,0x0204,0x0204,0x0084,0x0084,0x0008,0x0008,0x0008,0x0008,0x8008,0x0408,0x0408,0x0108,0x0109,0x4008};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0004,0x0046,0x8046,0x0084,0x0084,0x0204,0x0204,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x8008,0x0108,0x0108,0x0408,0x0409,0x4008};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0004,0x0046,0x8046,0x0084,0x0084,0x0204,0x0204,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0108,0x0108,0x0408,0x0409,0x4008};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0004,0x0046,0x8046,0x0084,0x0084,0x0204,0x0204,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0108,0x0108,0x0408,0x0409,0x4008};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

}

void patterns::getPatternFKHR2A(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 14;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0004,0x004a,0x004a,0x004a,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x8048,0x0088,0x0288,0x0208,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x8004,0x0404,0x0504,0x0104,0x4005,0x4004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0004,0x004a,0x004a,0x004a,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x8048,0x0088,0x0288,0x0208,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x8004,0x0404,0x0504,0x0104,0x4005,0x4004};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0004,0x004a,0x004a,0x004a,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x8048,0x0088,0x0288,0x0208,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x8004,0x0404,0x0504,0x0104,0x4005,0x4004};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0004,0x004a,0x004a,0x004a,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x8048,0x0088,0x0288,0x0208,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x8004,0x0404,0x0504,0x0104,0x4005,0x4004};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0004,0x004a,0x004a,0x004a,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x8048,0x0088,0x0288,0x0208,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x8004,0x0404,0x0504,0x0104,0x4005,0x4004};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0004,0x004a,0x004a,0x004a,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x8048,0x0088,0x0288,0x0208,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x8004,0x0404,0x0504,0x0104,0x4005,0x4004};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x0004,0x004a,0x004a,0x004a,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x8048,0x0088,0x0288,0x0208,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x8004,0x0404,0x0504,0x0104,0x4005,0x4004};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x0004,0x004a,0x004a,0x004a,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x8048,0x0088,0x0288,0x0208,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x8004,0x0404,0x0504,0x0104,0x4005,0x4004};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

}

void patterns::getPatternFKHR2B(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 14;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0044,0x004a,0x004a,0x004a,0x0048,0x0048,0x8048,0x0088,0x0288,0x0208,0x0004,0x0004,0x0004,0x0004,0x8004,0x0404,0x0504,0x0104,0x4005,0x4004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0044,0x004a,0x004a,0x004a,0x0048,0x0048,0x8048,0x0088,0x0288,0x0208,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x8004,0x0404,0x0504,0x0104,0x4005,0x4004};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0044,0x004a,0x004a,0x004a,0x0048,0x0048,0x8048,0x0088,0x0288,0x0208,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x8004,0x0404,0x0504,0x0104,0x4005,0x4004};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0044,0x004a,0x004a,0x004a,0x0048,0x0048,0x8048,0x0088,0x0288,0x0208,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x8004,0x0404,0x0504,0x0104,0x4005,0x4004};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0044,0x004a,0x004a,0x004a,0x0048,0x0048,0x8048,0x0088,0x0288,0x0208,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x8004,0x0404,0x0504,0x0104,0x4005,0x4004};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0044,0x004a,0x004a,0x004a,0x0048,0x0048,0x8048,0x0088,0x0288,0x0208,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x8004,0x0404,0x0504,0x0104,0x4005,0x4004};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x0044,0x004a,0x004a,0x004a,0x0048,0x0048,0x8048,0x0088,0x0288,0x0208,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x8004,0x0404,0x0504,0x0104,0x4005,0x4004};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x0044,0x004a,0x004a,0x004a,0x0048,0x0048,0x8048,0x0088,0x0288,0x0208,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x8004,0x0404,0x0504,0x0104,0x4005,0x4004};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

}

void patterns::getPatternFKHR3A(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 14;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0008,0x0046,0x0046,0x8046,0x0006,0x0004,0x0004,0x0004,0x0004,0x0004,0x0084,0x0284,0x0204,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0408,0x0508,0x0108,0x4009,0x4008};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0008,0x0046,0x0046,0x8046,0x0006,0x0004,0x0004,0x0004,0x0004,0x0004,0x0204,0x0284,0x0084,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0108,0x0508,0x0408,0x0008,0x4009,0x4008};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0008,0x0046,0x0046,0x8046,0x0006,0x0004,0x0004,0x0004,0x0004,0x0004,0x0204,0x0284,0x0084,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0108,0x0508,0x0408,0x0008,0x4009,0x4008};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0008,0x0046,0x0046,0x8046,0x0006,0x0004,0x0004,0x0004,0x0004,0x0004,0x0204,0x0284,0x0084,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0108,0x0508,0x0408,0x0008,0x4009,0x4008};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0008,0x0046,0x0046,0x8046,0x0006,0x0004,0x0004,0x0004,0x0004,0x0004,0x0204,0x0284,0x0084,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0108,0x0508,0x0408,0x0008,0x4009,0x4008};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0008,0x0046,0x0046,0x8046,0x0006,0x0004,0x0004,0x0004,0x0004,0x0004,0x0204,0x0284,0x0084,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0108,0x0508,0x0408,0x0008,0x4009,0x4008};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x0008,0x0046,0x0046,0x8046,0x0006,0x0004,0x0004,0x0004,0x0004,0x0004,0x0204,0x0284,0x0084,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0108,0x0508,0x0408,0x0008,0x4009,0x4008};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x0008,0x0046,0x0046,0x8046,0x0006,0x0004,0x0004,0x0004,0x0004,0x0004,0x0204,0x0284,0x0084,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0108,0x0508,0x0408,0x0008,0x4009,0x4008};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

}

void patterns::getPatternFKHR3B(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 14;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0008,0x0046,0x0046,0x8044,0x0004,0x0004,0x0084,0x0284,0x0204,0x0008,0x0008,0x8008,0x0408,0x0508,0x0108,0x4009,0x4008};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0008,0x0046,0x0046,0x8046,0x0004,0x0004,0x0284,0x0284,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x8008,0x0508,0x0508,0x0508,0x4009,0x4008};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0008,0x0046,0x0046,0x8046,0x0004,0x0004,0x0284,0x0284,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x8008,0x0508,0x0508,0x0508,0x4009,0x4008};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0008,0x0046,0x0046,0x8046,0x0004,0x0004,0x0284,0x0284,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x8008,0x0508,0x0508,0x0508,0x4009,0x4008};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0008,0x0046,0x0046,0x8046,0x0004,0x0004,0x0284,0x0284,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x8008,0x0508,0x0508,0x0508,0x4009,0x4008};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0008,0x0046,0x0046,0x8046,0x0004,0x0004,0x0284,0x0284,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x8008,0x0508,0x0508,0x0508,0x4009,0x4008};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x0008,0x0046,0x0046,0x8046,0x0004,0x0004,0x0284,0x0284,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x8008,0x0508,0x0508,0x0508,0x4009,0x4008};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x0008,0x0046,0x0046,0x8046,0x0004,0x0004,0x0284,0x0284,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x8008,0x0508,0x0508,0x0508,0x4009,0x4008};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

}

void patterns::getPatternFKHR4A(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 14;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x2004,0x2046,0x2046,0x2044,0x2044,0x2044,0x2044,0x2044,0x2044,0xa044,0x2084,0x2084,0x2084,0x2084,0x2084,0x2084,0x2084,0x2084,0x2084,0x2084,0x2084,0x2084,0x2084,0x2084,0x2084,0x2084,0x2084,0x2084,0x2084,0x2004,0x2004,0x2004,0x2008,0x2008,0x2008,0x2008,0x2008,0x2008,0x2008,0x2008,0x2008,0x2008,0x2008,0x2008,0x2008,0xa008,0x2108,0x2108,0x2108,0x2108,0x2108,0x2108,0x2108,0x2108,0x2108,0x2108,0x2108,0x2108,0x2108,0x2108,0x2108,0x2108,0x2108,0x2108,0x3109,0x3008};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternFKHR4AQ(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 62;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0026,0x0026,0x0026,0x0026,0x0024,0x0024,0x0024,0x0024,0x0024,0x0024,0x0024,0x0024,0x0024,0x0024,0x0024,0x0024,0x0024,0x0024,0x0024,0x0024,0x0024,0x0064,0x0064,0x8064,0x0064,0x02e4,0x02e4,0x02e4,0x02e4,0x02e4,0x02e4,0x02e4,0x0064,0x0064,0x0008,0x0008,0x8008,0x0008,0x0008,0x0008,0x0508,0x0508,0x0d48,0x0d48,0x8d48,0x0d48,0x0d48,0x0d48,0x0d48,0x0d48,0x0d48,0x0d48,0x0848,0x4848,0x0848,0x0808,0x0808,0x8808,0x0808,0x0808,0x0808,0x0808,0x0808,0x0808,0x0808,0x0808,0x0809,0x0808};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternFKHR4B(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 62;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x2004,0x2046,0x2046,0x2044,0x2044,0x2044,0x2044,0x2044,0x2044,0x2044,0x2044,0x2044,0xa044,0x2084,0x2084,0x2004,0x2008,0x2008,0x2008,0x2008,0x2008,0x2008,0x2008,0x2008,0x2008,0x2008,0x2008,0x2008,0xa008,0x2008,0x2108,0x3109,0x3008};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternFKHR4BQ(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 46;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0026,0x0026,0x0026,0x0026,0x0024,0x0024,0x0064,0x0064,0x82e4,0x02e4,0x02e4,0x0064,0x0064,0x0008,0x0008,0x8008,0x0508,0x0508,0x0508,0x0d48,0x0d48,0x8d48,0x0d48,0x0d48,0x4848,0x0848,0x0808,0x0808,0x8808,0x0808,0x0808,0x0809,0x0808};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternFKHR5A(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 14;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0008,0x0046,0x0046,0x0046,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x8044,0x0204,0x0284,0x0084,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0408,0x0508,0x0109,0x4008};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0008,0x0046,0x0046,0x0046,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x8044,0x0204,0x0284,0x0084,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0408,0x0508,0x0109,0x4008};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0008,0x0046,0x0046,0x0046,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x8044,0x0204,0x0284,0x0084,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0408,0x0508,0x0109,0x4008};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0008,0x0046,0x0046,0x0046,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x8044,0x0204,0x0284,0x0084,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0408,0x0508,0x0109,0x4008};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0008,0x0046,0x0046,0x0046,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x8044,0x0204,0x0284,0x0084,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0408,0x0508,0x0109,0x4008};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0008,0x0046,0x0046,0x0046,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x8044,0x0204,0x0284,0x0084,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0408,0x0508,0x0109,0x4008};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x0008,0x0046,0x0046,0x0046,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x8044,0x0204,0x0284,0x0084,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0408,0x0508,0x0109,0x4008};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x0008,0x0046,0x0046,0x0046,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x8044,0x0204,0x0284,0x0084,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0408,0x0508,0x0109,0x4008};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

}

void patterns::getPatternFKHR5B(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 62;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0008,0x0046,0x0046,0x0046,0x0044,0x8044,0x0204,0x0284,0x0084,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0408,0x0508,0x0109,0x4008};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0008,0x0046,0x0046,0x0046,0x0044,0x8044,0x0204,0x0284,0x0084,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0408,0x0508,0x0109,0x4008};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0008,0x0046,0x0046,0x0046,0x0044,0x8044,0x0204,0x0284,0x0084,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0408,0x0508,0x0109,0x4008};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0008,0x0046,0x0046,0x0046,0x0044,0x8044,0x0204,0x0284,0x0084,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0408,0x0508,0x0109,0x4008};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0008,0x0046,0x0046,0x0046,0x0044,0x8044,0x0204,0x0284,0x0084,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0408,0x0508,0x0109,0x4008};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0008,0x0046,0x0046,0x0046,0x0044,0x8044,0x0204,0x0284,0x0084,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0408,0x0508,0x0109,0x4008};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x0008,0x0046,0x0046,0x0044,0x0044,0x8044,0x0204,0x0284,0x0084,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0408,0x0508,0x0109,0x4008};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x0008,0x0046,0x0046,0x0046,0x0044,0x8044,0x0204,0x0284,0x0084,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x8008,0x0408,0x0508,0x0109,0x4008};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

}

void patterns::getPatternFKHR6A(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 62;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0044,0x004a,0x004a,0x004a,0x004a,0x004a,0x804a,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0288,0x0288,0x0288,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x8004,0x0004,0x0004,0x0004,0x0504,0x0504,0x0504,0x0504,0x0504,0x0504,0x0544,0x0544,0x0044,0x4045,0x4044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0044,0x004a,0x004a,0x004a,0x004a,0x004a,0x804a,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0288,0x0288,0x0288,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x8004,0x0004,0x0004,0x0004,0x0504,0x0504,0x0504,0x0504,0x0504,0x0504,0x0544,0x0544,0x0044,0x4045,0x4044};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0044,0x004a,0x004a,0x004a,0x004a,0x004a,0x804a,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0288,0x0288,0x0288,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x8004,0x0004,0x0004,0x0004,0x0504,0x0504,0x0504,0x0504,0x0504,0x0504,0x0544,0x0544,0x0044,0x4045,0x4044};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

}

void patterns::getPatternFKHR6B(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 62;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0044,0x004a,0x004a,0x004a,0x004a,0x004a,0x804a,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0288,0x0288,0x0288,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x8004,0x0004,0x0004,0x0004,0x0504,0x0504,0x0504,0x0504,0x0504,0x0504,0x0544,0x0544,0x0044,0x4045,0x4044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0044,0x004a,0x004a,0x004a,0x004a,0x004a,0x804a,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0288,0x0288,0x0288,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x8004,0x0004,0x0004,0x0004,0x0504,0x0504,0x0504,0x0504,0x0504,0x0504,0x0544,0x0544,0x0044,0x4045,0x4044};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0044,0x004a,0x004a,0x004a,0x004a,0x004a,0x804a,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0288,0x0288,0x0288,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x8004,0x0004,0x0004,0x0004,0x0504,0x0504,0x0504,0x0504,0x0504,0x0504,0x0544,0x0544,0x0044,0x4045,0x4044};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

}

void patterns::getPatternFKHS1(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 62;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0004,0x0046,0x8044,0x0284,0x0284,0x0008,0x8008,0x0508,0x0509,0x0008};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternFKHS1A(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 62;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0004,0x0006,0x0006,0x0006,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0009,0x0008};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternFKHS1B(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 62;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0004,0x0006,0x0006,0x0006,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0009,0x0008};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternFKHS2(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 14;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0004,0x000a,0x000a,0x000a,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0005,0x0004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternFKHS3(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 14;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0008,0x0006,0x0006,0x0006,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0009,0x0008};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternFKHS4(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 46;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0026,0x0026,0x0026,0x0026,0x0024,0x0024,0x0024,0x0024,0x0024,0x0024,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0009,0x0008};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternFKHS5(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 14;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0008,0x0006,0x0006,0x0006,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0009,0x0008};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternFKHS6(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 62;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0004,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0005,0x0004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternFM12HC(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26624;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x000c,0x000a,0x0008,0x0018,0x0018,0x2810,0x2810,0x0014,0x0014,0x4805,0x4a04};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x000c,0x000a,0x0008,0x0018,0x0018,0x2810,0x2810,0x0014,0x0004,0x000c,0x0008,0x0018,0x0010,0x0014,0x4805,0x4a04};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x000c,0x000a,0x0008,0x0018,0x0018,0x2810,0x2810,0x0014,0x0004,0x000c,0x0008,0x0018,0x0010,0x0014,0x0004,0x000c,0x0008,0x0018,0x0010,0x0014,0x4805,0x4a04};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x000c,0x000a,0x0008,0x0018,0x0018,0x2810,0x2810,0x0014,0x0004,0x000c,0x0008,0x0018,0x0010,0x0014,0x0004,0x000c,0x0008,0x0018,0x0010,0x0014,0x0004,0x000c,0x0008,0x0018,0x0010,0x0014,0x4805,0x4a04};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x000c,0x000a,0x0008,0x0018,0x0018,0x2810,0x2810,0x0014,0x0004,0x000c,0x0008,0x0018,0x0010,0x0014,0x0004,0x000c,0x0008,0x0018,0x0010,0x0014,0x0004,0x000c,0x0008,0x0018,0x0010,0x0014,0x0004,0x000c,0x0008,0x0018,0x0010,0x0014,0x4805,0x4a04};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x000c,0x000a,0x0008,0x0018,0x0018,0x2810,0x2810,0x0014,0x0004,0x000c,0x0008,0x0018,0x0010,0x0014,0x0004,0x000c,0x0008,0x0018,0x0010,0x0014,0x0004,0x000c,0x0008,0x0008,0x0018,0x0010,0x0014,0x0004,0x000c,0x0008,0x0018,0x0010,0x0014,0x0004,0x000c,0x0008,0x0018,0x0010,0x0014,0x0014,0x4805,0x4a04};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

}

void patterns::getPatternFM12HR(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26624;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x000c,0x000a,0x0008,0x0018,0x0018,0x2810,0x2810,0x0014,0x0014,0xc805,0xca04};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x000c,0x000a,0x0008,0x0018,0x0018,0x2810,0x2810,0x0014,0x0004,0x000c,0x0008,0x0018,0x0010,0x0014,0xc805,0xca04};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x000c,0x000a,0x0008,0x0018,0x0018,0x2810,0x2810,0x0014,0x0004,0x000c,0x0008,0x0018,0x0010,0x0014,0x0004,0x000c,0x0008,0x0018,0x0010,0x0014,0xc805,0xca04};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x000c,0x000a,0x0008,0x0018,0x0018,0x2810,0x2810,0x0014,0x0004,0x000c,0x0008,0x0018,0x0010,0x0014,0x0004,0x000c,0x0008,0x0018,0x0010,0x0014,0x0004,0x000c,0x0008,0x0018,0x0010,0x0014,0xc805,0xca04};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x000c,0x000a,0x0008,0x0018,0x0018,0x2810,0x2810,0x0014,0x0004,0x000c,0x0008,0x0018,0x0010,0x0014,0x0004,0x000c,0x0008,0x0018,0x0010,0x0014,0x0004,0x000c,0x0008,0x0018,0x0010,0x0014,0x0004,0x000c,0x0008,0x0018,0x0010,0x0014,0xc805,0xca04};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x000c,0x000a,0x0008,0x0018,0x0018,0x2810,0x2810,0x0014,0x0004,0x000c,0x0008,0x0018,0x0010,0x0014,0x0004,0x000c,0x0008,0x0018,0x0010,0x0014,0x0004,0x000c,0x0008,0x0008,0x0018,0x0010,0x0014,0x0004,0x000c,0x0008,0x0018,0x0010,0x0014,0x0004,0x000c,0x0008,0x0018,0x0010,0x0014,0x0014,0xc805,0xca04};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

}

void patterns::getPatternFM12HS(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26624;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x000c,0x000a,0x0008,0x0018,0x0010,0x2810,0x2810,0x0014,0x0004,0x4805,0x4a04};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternFM16HR3(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 0;

    uint16_t aRef[] = {0x006c,0x006e,0x0068,0x0068,0x0068,0x1068,0x0078,0x00f0,0x00f0,0x00d0,0x00d0,0x00d0,0x00d0,0x00d0,0x00d0,0x00d0};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0044,0x0144,0x0144,0x0104,0x0104,0x0104,0x0104,0x0104,0x8505,0x8504};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0054,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0044,0x004c,0x0048,0x0048,0x0058,0x0050,0x0050,0x0054};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0044,0x0044,0x004c,0x0048,0x0048,0x0058,0x0050,0x0050,0x0054,0x0044,0x0044,0x004c,0x0048,0x0048,0x0058,0x0050,0x0050,0x0054};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0044,0x004c,0x0048,0x0048,0x0058,0x0050,0x0050,0x0054,0x0044,0x0044,0x004c,0x0048,0x0048,0x0058,0x0050,0x0050,0x0054,0x0044,0x0044,0x004c,0x0048,0x0048,0x0058,0x0050,0x0050,0x0054};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0044,0x0044,0x004c,0x0048,0x0048,0x0058,0x0050,0x0050,0x0054,0x0044,0x0044,0x004c,0x0048,0x0048,0x0058,0x0050,0x0050,0x0054,0x0044,0x0044,0x004c,0x0048,0x0048,0x0058,0x0050,0x0050,0x0054,0x0044,0x0044,0x004c,0x0048,0x0048,0x0058,0x0050,0x0050,0x0054};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0044,0x004c,0x0048,0x0048,0x0058,0x0050,0x0050,0x0054,0x0044,0x0044,0x004c,0x0048,0x0048,0x0058,0x0050,0x0050,0x0054,0x0044,0x0044,0x004c,0x0048,0x0048,0x0058,0x0050,0x0050,0x0054,0x0044,0x0044,0x004c,0x0048,0x0048,0x0058,0x0050,0x0050,0x0054,0x0044,0x0044,0x004c,0x0048,0x0048,0x0058,0x0050,0x0050,0x0054};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

}

void patterns::getPatternFM16HS1(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 0;

    uint16_t aRef[] = {0x006c,0x006a,0x1068,0x0078,0x0070,0x00f0,0x00d0,0x00d4};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0144,0x0105,0x0104};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0044,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternFR2(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 60;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0004,0x004a,0x004a,0x0048,0x0048,0x0048,0x8048,0x0208,0x0208,0x0288,0x0288,0x0088,0x0088,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x8004,0x0404,0x0404,0x0504,0x0504,0x0104,0x4105,0x4004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0004,0x004a,0x004a,0x0048,0x0048,0x0048,0x8048,0x0208,0x0208,0x0288,0x0288,0x0088,0x0088,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x8004,0x0404,0x0404,0x0504,0x0504,0x0104,0x4105,0x4004};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0004,0x004a,0x004a,0x0048,0x0048,0x0048,0x8048,0x0208,0x0208,0x0288,0x0288,0x0088,0x0088,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x8004,0x0404,0x0404,0x0504,0x0504,0x0104,0x4105,0x4004};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0004,0x004a,0x004a,0x0048,0x0048,0x0048,0x8048,0x0208,0x0208,0x0288,0x0288,0x0088,0x0088,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x8004,0x0404,0x0404,0x0504,0x0504,0x0104,0x4105,0x4004};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0004,0x004a,0x004a,0x0048,0x0048,0x0048,0x8048,0x0208,0x0208,0x0288,0x0288,0x0088,0x0088,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x8004,0x0404,0x0404,0x0504,0x0504,0x0104,0x4105,0x4004};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0004,0x004a,0x004a,0x0048,0x0048,0x0048,0x8048,0x0208,0x0208,0x0288,0x0288,0x0088,0x0088,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x8004,0x0404,0x0404,0x0504,0x0504,0x0104,0x4105,0x4004};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x0004,0x004a,0x004a,0x0048,0x0048,0x0048,0x8048,0x0208,0x0208,0x0288,0x0288,0x0088,0x0088,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x8004,0x0404,0x0404,0x0504,0x0504,0x0104,0x4105,0x4004};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x0004,0x004a,0x004a,0x0048,0x0048,0x0048,0x8048,0x0208,0x0208,0x0288,0x0288,0x0088,0x0088,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x8004,0x0404,0x0404,0x0504,0x0504,0x0104,0x4105,0x4004};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

}

void patterns::getPatternFR2F(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 12;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0004,0x004a,0x004a,0x0048,0x8048,0x0088,0x0288,0x0288,0x0208,0x0004,0x0004,0x8004,0x0104,0x0504,0x0504,0x0405,0x4004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0004,0x004a,0x004a,0x0048,0x8048,0x0088,0x0288,0x0288,0x0208,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x8004,0x0104,0x0504,0x0504,0x0405,0x4004};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0004,0x004a,0x004a,0x0048,0x8048,0x0088,0x0288,0x0288,0x0208,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x8004,0x0104,0x0504,0x0504,0x0405,0x4004};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0004,0x004a,0x004a,0x0048,0x8048,0x0088,0x0288,0x0288,0x0208,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x8004,0x0104,0x0504,0x0504,0x0405,0x4004};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0004,0x004a,0x004a,0x0048,0x8048,0x0088,0x0288,0x0288,0x0208,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x8004,0x0104,0x0504,0x0504,0x0405,0x4004};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0004,0x004a,0x004a,0x0048,0x8048,0x0088,0x0288,0x0288,0x0208,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x8004,0x0104,0x0504,0x0504,0x0405,0x4004};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x0004,0x004a,0x004a,0x0048,0x8048,0x0088,0x0288,0x0288,0x0208,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x8004,0x0104,0x0504,0x0504,0x0405,0x4004};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x0004,0x004a,0x004a,0x0048,0x8048,0x0088,0x0288,0x0288,0x0208,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x8004,0x0104,0x0504,0x0504,0x0405,0x4004};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

}

void patterns::getPatternH12HC(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26626;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0800,0x0a0a,0x0808,0x0808,0x0808,0x0808,0x0804,0x2804,0x0804,0x0804,0x0805,0x4804};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternH12HCi(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26638;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0800,0x0a0a,0x0808,0x0808,0x0808,0x0808,0x0804,0x2804,0x0804,0x0804,0x0805,0x4804};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternH12HR(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26626;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {000000,0x000a,0x020a,0x0208,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x2004,0x0004,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

    uint16_t tempBin8[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin8( tempBin8, tempBin8 + sizeof(tempBin8) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin8 );

    uint16_t tempBin9[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin9( tempBin9, tempBin9 + sizeof(tempBin9) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin9 );

}

void patterns::getPatternH12HRi(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26638;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {000000,0x000a,0x020a,0x0208,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x2004,0x0004,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

    uint16_t tempBin8[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin8( tempBin8, tempBin8 + sizeof(tempBin8) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin8 );

    uint16_t tempBin9[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin9( tempBin9, tempBin9 + sizeof(tempBin9) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin9 );

}

void patterns::getPatternH12HS(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26626;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {000000,0x020a,0x0008,0x0008,0x0008,0x2008,0x0004,0x0004,0x0004,0x0005,0x4004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternH12HSi(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26638;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {000000,0x020a,0x0008,0x0008,0x0008,0x2008,0x0004,0x0004,0x0004,0x0005,0x4004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternH16HR(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 2;

    uint16_t aRef[] = {0x0064,0x106a,0x106a,0x106a,0x106a,0x106a,0x106a,0x106a,0x106a,0x1068,0x1068,0x1068,0x1068,0x1068,0x1068,0x1068,0x1068,0x1068,0x1068,0x1068,0x1068,0x00e8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0144,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x8405,0x8004};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0048,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

}

void patterns::getPatternH16HRa(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 2;

    uint16_t aRef[] = {0x0064,0x006a,0x006a,0x006a,0x006a,0x006a,0x1068,0x1068,0x1068,0x1068,0x1068,0x1068,0x1068,0x1068,0x1068,0x1068,0x1068,0x1068,0x1068,0x1068,0x1068,0x1068,0x1068,0x1068,0x1068,0x1068,0x1068,0x1068,0x1068,0x1068,0x00e8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0144,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x8405,0x8004};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0044,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

}

void patterns::getPatternH16HRi(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 14;

    uint16_t aRef[] = {0x0064,0x106a,0x106a,0x106a,0x106a,0x106a,0x106a,0x106a,0x106a,0x1068,0x1068,0x1068,0x1068,0x1068,0x1068,0x1068,0x1068,0x1068,0x1068,0x1068,0x1068,0x00e8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0144,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x8405,0x8004};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0048,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

}

void patterns::getPatternH16HS(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 2;

    uint16_t aRef[] = {0x0004,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0005,0x0004};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0008,0x0004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternH16HSa(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 2;

    uint16_t aRef[] = {0x0004,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0005,0x0004};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0004,0x0004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternH16HSi(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 14;

    uint16_t aRef[] = {0x0004,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x000a,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008,0x0008};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0005,0x0004};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0008,0x0004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternK12HC2(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26626;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0804,0x0a0a,0x0808,0x0808,0x0808,0x0808,0x0804,0x2804,0x0804,0x0804,0x0805,0x4804};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternK12HC2B(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26638;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0804,0x0a0a,0x0808,0x0808,0x0808,0x0808,0x0804,0x2804,0x0804,0x0804,0x0805,0x4804};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternK12HC3(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26626;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0804,0x0a0a,0x0808,0x0808,0x0808,0x0808,0x0804,0x2804,0x0804,0x0804,0x0805,0x4804};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternK12HC3i(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26638;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0804,0x0a0a,0x0808,0x0808,0x0808,0x0808,0x0804,0x2804,0x0804,0x0804,0x0805,0x4804};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternK12HC4(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26638;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0004,0x080a,0x0808,0x0808,0x0808,0x0808,0x0808,0x0808,0x0808,0x0808,0x0808,0x0808,0x0808,0x0808,0x0808,0x0004,0x0004,0x0004,0x0004,0x0005,0x0004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternK12HC5(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26624;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x4806,0x0808,0x0a08,0x2805,0x0804};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternK12HC5B(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26626;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0806,0x080a,0x0808,0x0808,0x2808,0x0804,0x0804,0x0804,0x4805,0x0804};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternK12HC6(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26624;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0804,0x0a0a,0x0a08,0x0808,0x2808,0x0808,0x0804,0x0804,0x0805,0x4804};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternK12HC6B(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26638;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0804,0x0a0a,0x0a08,0x0808,0x2808,0x0808,0x0804,0x0804,0x0805,0x4804};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternK12HC7(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26626;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0808,0x0a0a,0x0808,0x0808,0x0808,0x0808,0x0804,0x2804,0x0804,0x0804,0x0805,0x4804};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternK12HC8(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26638;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0808,0x0a0a,0x0808,0x0808,0x0808,0x0808,0x0804,0x2804,0x0804,0x0804,0x0805,0x4804};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternK12HC9(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26638;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0808,0x0a0a,0x0808,0x0808,0x0808,0x0808,0x0804,0x2804,0x0804,0x0804,0x0805,0x4804};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternK12HR10B(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26638;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0004,0x020a,0x0208,0x0208,0x0208,0x2004,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0004,0x020a,0x0208,0x0208,0x0208,0x2004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0004,0x020a,0x0208,0x0208,0x0208,0x2004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0004,0x020a,0x0208,0x0208,0x0208,0x2004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0004,0x020a,0x0208,0x0208,0x0208,0x2004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0004,0x020a,0x0208,0x0208,0x0208,0x2004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x0004,0x020a,0x0208,0x0208,0x0208,0x2004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x0004,0x020a,0x0208,0x0208,0x0208,0x2004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

}

void patterns::getPatternK12HR2(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26626;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x4004,0x000a,0x0208,0x0208,0x2204,0x8005,0x8004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

    uint16_t tempBin8[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin8( tempBin8, tempBin8 + sizeof(tempBin8) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin8 );

    uint16_t tempBin9[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin9( tempBin9, tempBin9 + sizeof(tempBin9) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin9 );

}

void patterns::getPatternK12HR2B(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26638;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0004,0x000a,0x0008,0x0208,0x0208,0x2204,0x0204,0x8005,0xc004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

    uint16_t tempBin8[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin8( tempBin8, tempBin8 + sizeof(tempBin8) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin8 );

    uint16_t tempBin9[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin9( tempBin9, tempBin9 + sizeof(tempBin9) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin9 );

}

void patterns::getPatternK12HR3(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26626;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0004,0x000a,0x020a,0x0208,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x2004,0x0004,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

    uint16_t tempBin8[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin8( tempBin8, tempBin8 + sizeof(tempBin8) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin8 );

    uint16_t tempBin9[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin9( tempBin9, tempBin9 + sizeof(tempBin9) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin9 );

}

void patterns::getPatternK12HR3i(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26638;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0004,0x000a,0x020a,0x0208,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x2004,0x0004,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

    uint16_t tempBin8[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin8( tempBin8, tempBin8 + sizeof(tempBin8) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin8 );

    uint16_t tempBin9[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin9( tempBin9, tempBin9 + sizeof(tempBin9) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin9 );

}

void patterns::getPatternK12HR4(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26638;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0004,0x000a,0x020a,0x0008,0x0008,0x2008,0x0004,0x0004,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

    uint16_t tempBin8[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin8( tempBin8, tempBin8 + sizeof(tempBin8) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin8 );

    uint16_t tempBin9[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin9( tempBin9, tempBin9 + sizeof(tempBin9) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin9 );

}

void patterns::getPatternK12HR5(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26624;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {000000,0x020a,0x0208,0x0208,0x0208,0x2004,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x4006,0x0008,0x0208,0x2004,0x0004,0x0008,0x0008,0x8005,0x8004};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x4006,0x0008,0x0208,0x2004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x8005,0x8004};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x4006,0x0008,0x0208,0x2004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x8005,0x8004};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x4006,0x0008,0x0208,0x2004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x8005,0x8004};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x4006,0x0008,0x0208,0x2004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x8005,0x8004};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x4006,0x0008,0x0208,0x2004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x8005,0x8004};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x4006,0x0008,0x0208,0x2004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x8005,0x8004};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

}

void patterns::getPatternK12HR5B(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26626;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0004,0x020a,0x0208,0x0208,0x0208,0x2004,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0004,0x020a,0x0208,0x0208,0x0208,0x2004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0004,0x020a,0x0208,0x0208,0x0208,0x2004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0004,0x020a,0x0208,0x0208,0x0208,0x2004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0004,0x020a,0x0208,0x0208,0x0208,0x2004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0004,0x020a,0x0208,0x0208,0x0208,0x2004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x0004,0x020a,0x0208,0x0208,0x0208,0x2004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x0004,0x020a,0x0208,0x0208,0x0208,0x2004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

}

void patterns::getPatternK12HR6(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26624;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0004,0x020a,0x0208,0x0008,0x2008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0004,0x020a,0x0208,0x0008,0x2008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0004,0x020a,0x0208,0x0008,0x2008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0004,0x020a,0x0208,0x0008,0x2008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0004,0x020a,0x0208,0x0008,0x2008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0004,0x020a,0x0208,0x0008,0x2008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

}

void patterns::getPatternK12HR6B(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26638;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0004,0x020a,0x0208,0x0008,0x2008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0004,0x020a,0x0208,0x0008,0x2008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0004,0x020a,0x0208,0x0008,0x2008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0004,0x020a,0x0208,0x0008,0x2008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0004,0x020a,0x0208,0x0008,0x2008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0004,0x020a,0x0208,0x0008,0x2008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x0008,0x0008,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

}

void patterns::getPatternK12HR7(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26626;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0008,0x020a,0x0008,0x0008,0x0008,0x0008,0x0004,0x2004,0x0004,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x4008,0x000a,0x0008,0x0008,0x0208,0x0004,0x2004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x4008,0x000a,0x0008,0x0008,0x0208,0x0004,0x2004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x4008,0x000a,0x0008,0x0008,0x0208,0x0004,0x2004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x4008,0x000a,0x0008,0x0008,0x0208,0x0004,0x2004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x4008,0x000a,0x0008,0x0008,0x0208,0x0004,0x2004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x4008,0x000a,0x0008,0x0008,0x0208,0x0004,0x2004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x4008,0x000a,0x0008,0x0008,0x0208,0x0004,0x2004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

    uint16_t tempBin8[] = {0x4008,0x000a,0x0008,0x0008,0x0208,0x0004,0x2004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin8( tempBin8, tempBin8 + sizeof(tempBin8) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin8 );

    uint16_t tempBin9[] = {0x4008,0x000a,0x0008,0x0008,0x0208,0x0004,0x2004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin9( tempBin9, tempBin9 + sizeof(tempBin9) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin9 );

}

void patterns::getPatternK12HR8(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26638;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0008,0x020a,0x0008,0x0008,0x0008,0x0008,0x0004,0x2004,0x0004,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x4008,0x000a,0x0008,0x0008,0x0208,0x0004,0x2004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x4008,0x000a,0x0008,0x0008,0x0208,0x0004,0x2004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x4008,0x000a,0x0008,0x0008,0x0208,0x0004,0x2004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x4008,0x000a,0x0008,0x0008,0x0208,0x0004,0x2004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x4008,0x000a,0x0008,0x0008,0x0208,0x0004,0x2004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x4008,0x000a,0x0008,0x0008,0x0208,0x0004,0x2004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x4008,0x000a,0x0008,0x0008,0x0208,0x0004,0x2004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

    uint16_t tempBin8[] = {0x4008,0x000a,0x0008,0x0008,0x0208,0x0004,0x2004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin8( tempBin8, tempBin8 + sizeof(tempBin8) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin8 );

    uint16_t tempBin9[] = {0x4008,0x000a,0x0008,0x0008,0x0208,0x0004,0x2004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin9( tempBin9, tempBin9 + sizeof(tempBin9) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin9 );

}

void patterns::getPatternK12HR9(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26638;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0008,0x020a,0x0008,0x0008,0x0008,0x0008,0x0004,0x2004,0x0004,0x0004,0x0004,0x8005,0xc004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x4008,0x000a,0x0008,0x0008,0x0208,0x0004,0x2004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x4008,0x000a,0x0008,0x0008,0x0208,0x0004,0x2004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x4008,0x000a,0x0008,0x0008,0x0208,0x0004,0x2004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x4008,0x000a,0x0008,0x0008,0x0208,0x0004,0x2004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x4008,0x000a,0x0008,0x0008,0x0208,0x0004,0x2004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x4008,0x000a,0x0008,0x0008,0x0208,0x0004,0x2004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x4008,0x000a,0x0008,0x0008,0x0208,0x0004,0x2004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

    uint16_t tempBin8[] = {0x4008,0x000a,0x0008,0x0008,0x0208,0x0004,0x2004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin8( tempBin8, tempBin8 + sizeof(tempBin8) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin8 );

    uint16_t tempBin9[] = {0x4008,0x000a,0x0008,0x0008,0x0208,0x0004,0x2004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin9( tempBin9, tempBin9 + sizeof(tempBin9) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin9 );

}

void patterns::getPatternK12HS10B(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26638;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x4006,0x0008,0x0208,0x2005,0x0004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternK12HS2(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26626;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x4004,0x020a,0x0008,0x0008,0x0004,0x2005,0x0004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternK12HS2B(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26638;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x4004,0x020a,0x0008,0x0008,0x0004,0x2005,0x0004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternK12HS3(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26626;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2008,0x0004,0x0004,0x0004,0x0005,0x4004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternK12HS3i(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26638;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2008,0x0004,0x0004,0x0004,0x0005,0x4004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternK12HS4(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26638;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0004,0x020a,0x0008,0x0008,0x0008,0x2008,0x0004,0x0004,0x0004,0x0005,0x4004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternK12HS5(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26624;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x4006,0x0008,0x0208,0x2005,0x0004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternK12HS5B(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26626;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x4006,0x0008,0x0208,0x2005,0x0004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternK12HS6(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26624;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x4004,0x000a,0x0008,0x0008,0x0004,0x0004,0x0005,0x0004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternK12HS6B(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26638;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x4004,0x000a,0x0008,0x0008,0x0004,0x0004,0x0005,0x0004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternK12HS7(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26626;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0008,0x020a,0x0008,0x0008,0x0008,0x2008,0x0004,0x0004,0x0004,0x0005,0x4004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternK12HS8(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26638;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0008,0x020a,0x0008,0x0008,0x0008,0x2008,0x0004,0x0004,0x0004,0x0005,0x4004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternK12HS9(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26638;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0008,0x020a,0x0008,0x0008,0x0008,0x2008,0x0004,0x0004,0x0004,0x0005,0x4004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternK16HC1(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 14;

    uint16_t aRef[] = {0x0064,0x006a,0x006a,0x0068,0x0068,0x1068,0x1068,0x1068,0x1068,0x1048,0x10c8,0x10c8,0x10c8,0x10c8,0x10c8,0x10c8,0x10c8,0x10c8,0x10c8,0x10c8,0x10c8,0x10c8,0x10c8,0x10c8,0x10c8};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0004,0x0004,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0004,0x0404,0x0004,0x0005,0x0064};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0048,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternK16HC2(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 2;

    uint16_t aRef[] = {0x0064,0x006a,0x006a,0x0068,0x0068,0x1068,0x1068,0x1068,0x1068,0x1048,0x10c8,0x10c8,0x10c8,0x10c8,0x10c8,0x10c8,0x10c8,0x10c8,0x10c8,0x10c8,0x10c8,0x10c8,0x10c8,0x10c8,0x10c8};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0004,0x0004,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0004,0x0404,0x0004,0x0005,0x0064};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0048,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternK16HC2B(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 14;

    uint16_t aRef[] = {0x0064,0x006a,0x006a,0x0068,0x0068,0x1068,0x1068,0x1068,0x1068,0x1048,0x10c8,0x10c8,0x10c8,0x10c8,0x10c8,0x10c8,0x10c8,0x10c8,0x10c8,0x10c8,0x10c8,0x10c8,0x10c8,0x10c8,0x10c8};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0004,0x0004,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0004,0x0404,0x0004,0x0005,0x0064};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0048,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternK16HR1(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 14;

    uint16_t aRef[] = {0x0064,0x006a,0x006a,0x0068,0x1068,0x1068,0x1068,0x0068,0x00e8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0144,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x8004,0x8005,0x0404};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0048,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

    uint16_t tempBin8[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin8( tempBin8, tempBin8 + sizeof(tempBin8) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin8 );

    uint16_t tempBin9[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin9( tempBin9, tempBin9 + sizeof(tempBin9) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin9 );

}

void patterns::getPatternK16HR10B(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 14;

    uint16_t aRef[] = {0x0064,0x006a,0x006a,0x0068,0x1068,0x1068,0x1068,0x0068,0x00e8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0144,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x8004,0x8005,0x0404};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0048,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

    uint16_t tempBin8[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin8( tempBin8, tempBin8 + sizeof(tempBin8) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin8 );

    uint16_t tempBin9[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin9( tempBin9, tempBin9 + sizeof(tempBin9) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin9 );

}

void patterns::getPatternK16HR2(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 2;

    uint16_t aRef[] = {0x0064,0x006a,0x006a,0x0068,0x1068,0x1068,0x1068,0x0068,0x00e8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0144,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x8004,0x8005,0x0404};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0048,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

    uint16_t tempBin8[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin8( tempBin8, tempBin8 + sizeof(tempBin8) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin8 );

    uint16_t tempBin9[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin9( tempBin9, tempBin9 + sizeof(tempBin9) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin9 );

}

void patterns::getPatternK16HR2B(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 14;

    uint16_t aRef[] = {0x0064,0x006a,0x006a,0x0068,0x1068,0x1068,0x1068,0x0068,0x00e8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0144,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x8004,0x8005,0x0404};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0048,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

    uint16_t tempBin8[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin8( tempBin8, tempBin8 + sizeof(tempBin8) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin8 );

    uint16_t tempBin9[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin9( tempBin9, tempBin9 + sizeof(tempBin9) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin9 );

}

void patterns::getPatternK16HR3(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 2;

    uint16_t aRef[] = {0x0064,0x006a,0x006a,0x0068,0x1068,0x1068,0x1068,0x0068,0x00e8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0144,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x8004,0x8005,0x0404};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0048,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

}

void patterns::getPatternK16HR31(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 2;

    uint16_t aRef[] = {0x0064,0x006a,0x006a,0x0068,0x1068,0x1068,0x1068,0x0068,0x00e8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0064,0x0064,0x0064,0x0064,0x0064,0x0064,0x0064,0x0164,0x0124,0x0124,0x0124,0x0124,0x0124,0x0124,0x0124,0x0124,0x0124,0x0124,0x0124,0x0124,0x0124,0x0124,0x0124,0x0124,0x8064,0x8065,0x0464};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0068,0x0064};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

}

void patterns::getPatternK16HR3i(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 14;

    uint16_t aRef[] = {0x0064,0x006a,0x006a,0x0068,0x1068,0x1068,0x1068,0x0068,0x00e8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0144,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x8004,0x8005,0x0404};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0048,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

}

void patterns::getPatternK16HR3ns(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 2;

    uint16_t aRef[] = {0x0064,0x006a,0x006a,0x0068,0x1068,0x1068,0x1068,0x0068,0x00e8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0144,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x8004,0x8005,0x0404};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0048,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

}

void patterns::getPatternK16HR5(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 0;

    uint16_t aRef[] = {0x0064,0x006a,0x006a,0x0068,0x1068,0x1068,0x1068,0x0068,0x00e8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0144,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x8004,0x8005,0x0404};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0048,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

    uint16_t tempBin8[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin8( tempBin8, tempBin8 + sizeof(tempBin8) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin8 );

    uint16_t tempBin9[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin9( tempBin9, tempBin9 + sizeof(tempBin9) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin9 );

}

void patterns::getPatternK16HR5B(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 2;

    uint16_t aRef[] = {0x0064,0x006a,0x006a,0x0068,0x1068,0x1068,0x1068,0x0068,0x00e8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0144,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x8004,0x8005,0x0404};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0048,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

    uint16_t tempBin8[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin8( tempBin8, tempBin8 + sizeof(tempBin8) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin8 );

    uint16_t tempBin9[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin9( tempBin9, tempBin9 + sizeof(tempBin9) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin9 );

}

void patterns::getPatternK16HR6(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 0;

    uint16_t aRef[] = {0x0068,0x006a,0x006a,0x0068,0x1068,0x1068,0x0068,0x00e8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0144,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0004,0x8005,0x8404};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0048,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

    uint16_t tempBin8[] = {0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048};
    std::vector<uint16_t> vectBin8( tempBin8, tempBin8 + sizeof(tempBin8) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin8 );

    uint16_t tempBin9[] = {0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048};
    std::vector<uint16_t> vectBin9( tempBin9, tempBin9 + sizeof(tempBin9) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin9 );

}

void patterns::getPatternK16HR6B(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 14;

    uint16_t aRef[] = {0x0064,0x006a,0x006a,0x0068,0x1068,0x1068,0x0068,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0004,0x8005,0x8404};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0048,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

}

void patterns::getPatternK16HR7(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 2;

    uint16_t aRef[] = {0x0068,0x006a,0x006a,0x0068,0x1068,0x1068,0x1068,0x0068,0x0068,0x00e8,0x00e8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0144,0x0144,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0004,0x8004,0x8405,0x0004};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x00c8,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

    uint16_t tempBin8[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin8( tempBin8, tempBin8 + sizeof(tempBin8) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin8 );

    uint16_t tempBin9[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin9( tempBin9, tempBin9 + sizeof(tempBin9) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin9 );

}

void patterns::getPatternK16HR8(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 2;

    uint16_t aRef[] = {0x0068,0x106a,0x106a,0x1068,0x0068,0x00e8,0x00e8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0044,0x0144,0x0144,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x8005,0x8404};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x00c8,0x00c4};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

    uint16_t tempBin8[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin8( tempBin8, tempBin8 + sizeof(tempBin8) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin8 );

    uint16_t tempBin9[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin9( tempBin9, tempBin9 + sizeof(tempBin9) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin9 );

}

void patterns::getPatternK16HR8a(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 2;

    uint16_t aRef[] = {0x0068,0x106a,0x106a,0x1068,0x0068,0x00e8,0x00e8,0x00c8,0x00c8};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0044,0x0144,0x0144,0x0104,0x0104,0x0104,0x0104,0x8005,0x8404};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x00c8,0x00c4};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

    uint16_t tempBin8[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin8( tempBin8, tempBin8 + sizeof(tempBin8) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin8 );

    uint16_t tempBin9[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin9( tempBin9, tempBin9 + sizeof(tempBin9) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin9 );

}

void patterns::getPatternK16HR9(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 14;

    uint16_t aRef[] = {0x0068,0x006a,0x006a,0x0068,0x0068,0x1068,0x1068,0x00e8,0x00e8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8,0x00c8};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0144,0x0144,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x8104,0x8004,0x0004,0x0404,0x0005,0x0004};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x00c8,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

    uint16_t tempBin8[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin8( tempBin8, tempBin8 + sizeof(tempBin8) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin8 );

    uint16_t tempBin9[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin9( tempBin9, tempBin9 + sizeof(tempBin9) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin9 );

}

void patterns::getPatternK16HS1(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 14;

    uint16_t aRef[] = {0x0064,0x006a,0x006a,0x0068,0x1068,0x1068,0x1068,0x0068,0x00e8,0x00c8,0x00c8};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0144,0x0104,0x0004,0x0005,0x0004};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0048,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternK16HS10B(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 14;

    uint16_t aRef[] = {0x0064,0x006a,0x006a,0x0068,0x1068,0x1068,0x1068,0x0068,0x00e8,0x00c8,0x00c8};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0144,0x0104,0x0104,0x0005,0x0004};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0048,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternK16HS2(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 2;

    uint16_t aRef[] = {0x0064,0x006a,0x006a,0x0068,0x1068,0x1068,0x1068,0x0068,0x00e8,0x00c8,0x00c8};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0144,0x0104,0x0004,0x0005,0x0004};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0048,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternK16HS2B(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 14;

    uint16_t aRef[] = {0x0064,0x006a,0x006a,0x0068,0x1068,0x1068,0x1068,0x0068,0x00e8,0x00c8,0x00c8};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0144,0x0104,0x0004,0x0005,0x0004};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0048,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternK16HS3(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 2;

    uint16_t aRef[] = {0x0064,0x006a,0x006a,0x0068,0x1068,0x1068,0x1068,0x0068,0x00e8,0x00c8,0x00c8};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0144,0x0104,0x0104,0x0105,0x0004};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x00c8,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternK16HS3i(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 14;

    uint16_t aRef[] = {0x0064,0x006a,0x006a,0x0068,0x1068,0x1068,0x1068,0x0068,0x00e8,0x00c8,0x00c8};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0144,0x0104,0x0104,0x0105,0x0004};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x00c8,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternK16HS5(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 0;

    uint16_t aRef[] = {0x0064,0x006a,0x006a,0x0068,0x1068,0x1068,0x1068,0x0068,0x00e8,0x00c8,0x00c8};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0144,0x0104,0x0004,0x0005,0x0004};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0048,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternK16HS5B(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 2;

    uint16_t aRef[] = {0x0064,0x006a,0x006a,0x0068,0x1068,0x1068,0x1068,0x0068,0x00e8,0x00c8,0x00c8};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0144,0x0104,0x0004,0x0005,0x0004};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0048,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternK16HS6(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 0;

    uint16_t aRef[] = {0x0068,0x006a,0x006a,0x1068,0x1068};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0064,0x0064,0x0064,0x0065,0x0064};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x1068,0x0064};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternK16HS6B(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 14;

    uint16_t aRef[] = {0x0068,0x006a,0x006a,0x1068,0x1068};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0064,0x0064,0x0064,0x0065,0x0064};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x1068,0x0064};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternK16HS7(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 2;

    uint16_t aRef[] = {0x0068,0x006a,0x006a,0x0068,0x1068,0x1068,0x1068,0x0068,0x0068,0x0068,0x0068};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0064,0x0064,0x0064,0x0064,0x0064,0x0064,0x0064,0x0064,0x0064,0x0064,0x0065,0x0064};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0068,0x0064};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternK16HS8(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 2;

    uint16_t aRef[] = {0x0068,0x006a,0x006a,0x1068,0x1068};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0064,0x0064,0x0064,0x0065,0x0064};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x1068,0x0064};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternK16HS9(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 14;

    uint16_t aRef[] = {0x0068,0x006a,0x006a,0x0068,0x0068,0x0068,0x0068,0x0068,0x0068,0x0068,0x0068};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0064,0x0064,0x0064,0x0064,0x0064,0x0064,0x0064,0x0064,0x0064,0x0064,0x0065,0x0064};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0068,0x0064};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternM12HC(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26624;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x080c,0x0808,0x0a0a,0x080a,0x0808,0x0808,0x0818,0x0818,0x0818,0x0818,0x0810,0x0810,0x2810,0x0810,0x0814,0x0814,0x0804,0x0804,0x0804,0x0804,0x4804,0x0805,0x0804};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x000c,0x0008,0x400a,0x000a,0x0008,0x0008,0x0218,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x2014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x000c,0x0008,0x0008,0x0008,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x0014,0x0004,0x0004,0x0004,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x000c,0x0008,0x400a,0x000a,0x0008,0x0008,0x0218,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x2014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x000c,0x0008,0x0008,0x0008,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x0014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x0014,0x0004,0x0004,0x0004,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

}

void patterns::getPatternM12HC7(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26624;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x080c,0x0808,0x0a0a,0x080a,0x0808,0x0808,0x0818,0x0818,0x0818,0x0818,0x0810,0x0810,0x2810,0x0810,0x0814,0x0814,0x0804,0x0804,0x0804,0x0804,0x4804,0x0805,0x0804};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x000c,0x0008,0x400a,0x000a,0x0008,0x0008,0x0218,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x2014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x000c,0x0008,0x0008,0x0008,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x0014,0x0004,0x0004,0x0004,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x000c,0x0008,0x400a,0x000a,0x0008,0x0008,0x0218,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x2014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x000c,0x0008,0x0008,0x0008,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x0014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x0014,0x0004,0x0004,0x0004,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

}

void patterns::getPatternM12HC7i(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26654;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x080c,0x0808,0x0a0a,0x080a,0x0808,0x0808,0x0818,0x0818,0x0818,0x0818,0x0810,0x0810,0x2810,0x0810,0x0814,0x0814,0x0804,0x0804,0x0804,0x0804,0x4804,0x0805,0x0804};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x000c,0x0008,0x400a,0x000a,0x0008,0x0008,0x0218,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x2014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x000c,0x0008,0x0008,0x0008,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x0014,0x0004,0x0004,0x0004,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x000c,0x0008,0x400a,0x000a,0x0008,0x0008,0x0218,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x2014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x000c,0x0008,0x0008,0x0008,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x0014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x0014,0x0004,0x0004,0x0004,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

}

void patterns::getPatternM12HCB(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26626;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x080c,0x0808,0x0a0a,0x080a,0x0808,0x0808,0x0818,0x0818,0x0818,0x0818,0x0810,0x0810,0x2810,0x0810,0x0814,0x0814,0x0804,0x0804,0x0804,0x0804,0x4804,0x0805,0x0804};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x000c,0x0008,0x400a,0x000a,0x0008,0x0008,0x0218,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x2014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x000c,0x0008,0x0008,0x0008,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x0014,0x0004,0x0004,0x0004,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x000c,0x0008,0x400a,0x000a,0x0008,0x0008,0x0218,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x2014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x000c,0x0008,0x0008,0x0008,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x0014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x0014,0x0004,0x0004,0x0004,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

}

void patterns::getPatternM12HCC(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26654;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x080c,0x0808,0x0a0a,0x080a,0x0808,0x0808,0x0818,0x0818,0x0818,0x0818,0x0810,0x0810,0x2810,0x0810,0x0814,0x0814,0x0804,0x0804,0x0804,0x0804,0x4804,0x0805,0x0804};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x000c,0x0008,0x400a,0x000a,0x0008,0x0008,0x0218,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x2014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x000c,0x0008,0x0008,0x0008,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x0014,0x0004,0x0004,0x0004,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x000c,0x0008,0x400a,0x000a,0x0008,0x0008,0x0218,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x2014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x000c,0x0008,0x0008,0x0008,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x0014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x0014,0x0004,0x0004,0x0004,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

}

void patterns::getPatternM12HCi(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26654;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x080c,0x0808,0x0a0a,0x080a,0x0808,0x0808,0x0818,0x0818,0x0818,0x0818,0x0810,0x0810,0x2810,0x0810,0x0814,0x0814,0x0804,0x0804,0x0804,0x0804,0x4804,0x0805,0x0804};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x000c,0x0008,0x400a,0x000a,0x0008,0x0008,0x0218,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x2014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x000c,0x0008,0x0008,0x0008,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x0014,0x0004,0x0004,0x0004,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x000c,0x0008,0x400a,0x000a,0x0008,0x0008,0x0218,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x2014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x000c,0x0008,0x0008,0x0008,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x0014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x0014,0x0004,0x0004,0x0004,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

}

void patterns::getPatternM12HR(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26624;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x000c,0x0008,0x020a,0x000a,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x2010,0x2010,0x0014,0x0014,0x0004,0x0004,0x0004,0x0004,0x0004,0xc005,0xc004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x000c,0x0008,0x400a,0x000a,0x0008,0x0008,0x0218,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x2014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x000c,0x0008,0x0008,0x0008,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x0014,0x0004,0x0004,0x0004,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x000c,0x0008,0x400a,0x000a,0x0008,0x0008,0x0218,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x2014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x000c,0x0008,0x0008,0x0008,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x0014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x0014,0x0004,0x0004,0x0004,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

}

void patterns::getPatternM12HR2(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26636;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x000c,0x0008,0x400a,0x000a,0x0008,0x0008,0x0218,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x2014,0x0004,0x0004,0x0004,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x000c,0x0008,0x400a,0x000a,0x0008,0x0008,0x0218,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x2014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x000c,0x0008,0x0008,0x0008,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x0014,0x0004,0x0004,0x0004,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x000c,0x0008,0x400a,0x000a,0x0008,0x0008,0x0218,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x2014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x000c,0x0008,0x0008,0x0008,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x0014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x0014,0x0004,0x0004,0x0004,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

}

void patterns::getPatternM12HR7(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26624;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x000c,0x0008,0x020a,0x000a,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x2010,0x2010,0x0014,0x0014,0x0004,0x0004,0x0004,0x0004,0x0004,0xc005,0xc004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x000c,0x0008,0x400a,0x000a,0x0008,0x0008,0x0218,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x2014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x000c,0x0008,0x0008,0x0008,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x0014,0x0004,0x0004,0x0004,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x000c,0x0008,0x400a,0x000a,0x0008,0x0008,0x0218,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x2014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x000c,0x0008,0x0008,0x0008,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x0014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x0014,0x0004,0x0004,0x0004,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

}

void patterns::getPatternM12HR7i(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26654;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x000c,0x0008,0x020a,0x000a,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x2010,0x2010,0x0014,0x0014,0x0004,0x0004,0x0004,0x0004,0x0004,0xc005,0xc004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x000c,0x0008,0x400a,0x000a,0x0008,0x0008,0x0218,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x2014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x000c,0x0008,0x0008,0x0008,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x0014,0x0004,0x0004,0x0004,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x000c,0x0008,0x400a,0x000a,0x0008,0x0008,0x0218,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x2014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x000c,0x0008,0x0008,0x0008,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x0014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x0014,0x0004,0x0004,0x0004,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

}

void patterns::getPatternM12HRB(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26626;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x000c,0x0008,0x020a,0x000a,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x2010,0x2010,0x0014,0x0014,0x0004,0x0004,0x0004,0x0004,0x0004,0xc005,0xc004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x000c,0x0008,0x400a,0x000a,0x0008,0x0008,0x0218,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x2014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x000c,0x0008,0x0008,0x0008,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x0014,0x0004,0x0004,0x0004,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x000c,0x0008,0x400a,0x000a,0x0008,0x0008,0x0218,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x2014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x000c,0x0008,0x0008,0x0008,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x0014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x0014,0x0004,0x0004,0x0004,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

}

void patterns::getPatternM12HRC(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26654;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x000c,0x0008,0x020a,0x000a,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x2010,0x2010,0x0014,0x0014,0x0004,0x0004,0x0004,0x0004,0x0004,0xc005,0xc004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x000c,0x0008,0x400a,0x000a,0x0008,0x0008,0x0218,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x2014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x000c,0x0008,0x0008,0x0008,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x0014,0x0004,0x0004,0x0004,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x000c,0x0008,0x400a,0x000a,0x0008,0x0008,0x0218,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x2014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x000c,0x0008,0x0008,0x0008,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x0014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x0014,0x0004,0x0004,0x0004,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

}

void patterns::getPatternM12HRi(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26654;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x000c,0x0008,0x020a,0x000a,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x2010,0x2010,0x0014,0x0014,0x0004,0x0004,0x0004,0x0004,0x0004,0xc005,0xc004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x000c,0x0008,0x400a,0x000a,0x0008,0x0008,0x0218,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x2014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x000c,0x0008,0x0008,0x0008,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x0014,0x0004,0x0004,0x0004,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x000c,0x0008,0x400a,0x000a,0x0008,0x0008,0x0218,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x2014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x000c,0x0008,0x0008,0x0008,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x0014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0004,0x0008,0x0008,0x0008,0x0008,0x0008,0x0018,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x0014,0x0004,0x0004,0x0004,0x0004,0x0004,0x8005,0x8004};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

}

void patterns::getPatternM12HS(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26624;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x000c,0x0008,0x400a,0x000a,0x0008,0x0008,0x0218,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x2014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0005,0x0004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternM12HS2(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26636;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x000c,0x0008,0x400a,0x000a,0x0008,0x0008,0x0218,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x2014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0005,0x0004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternM12HS7(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26624;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x000c,0x0008,0x400a,0x000a,0x0008,0x0008,0x0218,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x2014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0005,0x0004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternM12HS7i(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26654;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x000c,0x0008,0x400a,0x000a,0x0008,0x0008,0x0218,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x2014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0005,0x0004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternM12HSB(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26626;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x000c,0x0008,0x400a,0x000a,0x0008,0x0008,0x0218,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x2014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0005,0x0004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternM12HSC(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26654;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x000c,0x0008,0x400a,0x000a,0x0008,0x0008,0x0218,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x2014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0005,0x0004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternM12HSi(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 26654;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x000c,0x0008,0x400a,0x000a,0x0008,0x0008,0x0218,0x0018,0x0018,0x0018,0x0010,0x0010,0x0010,0x0010,0x0014,0x2014,0x0004,0x0004,0x0004,0x0004,0x0004,0x0005,0x0004};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternM16HC1(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 0;

    uint16_t aRef[] = {0x006c,0x0068,0x006a,0x006a,0x1068,0x1068,0x1068,0x0068,0x00e8,0x00c8,0x00c8,0x00c8,0x00c8,0x00d8,0x00d8,0x00d8,0x00d8,0x00d0,0x00d0,0x00d0,0x00d0,0x00d4,0x00d4,0x00d4,0x00d4};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0144,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0004,0x0005,0x0404};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0054,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternM16HR1(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 0;

    uint16_t aRef[] = {0x006c,0x0068,0x006a,0x006a,0x1068,0x1068,0x1068,0x0068,0x00e8,0x00c8,0x00c8,0x00c8,0x00c8,0x00d8,0x00d8,0x00d8,0x00d8,0x00d0,0x00d0,0x00d0,0x00d0,0x00d4,0x00d4,0x00d4,0x00d4};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0144,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x8004,0x8005,0x0404};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0054,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

    uint16_t tempBin8[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin8( tempBin8, tempBin8 + sizeof(tempBin8) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin8 );

    uint16_t tempBin9[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin9( tempBin9, tempBin9 + sizeof(tempBin9) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin9 );

}

void patterns::getPatternM16HR2(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 0;

    uint16_t aRef[] = {0x006c,0x0068,0x006a,0x006a,0x1068,0x1068,0x1068,0x0068,0x00e8,0x00c8,0x00c8,0x00c8,0x00c8,0x00d8,0x00d8,0x00d8,0x00d8,0x00d0,0x00d0,0x00d0,0x00d0,0x00d0,0x00d0,0x00d0,0x00d0,0x00d0,0x00d0,0x00d0,0x00d0,0x00d0,0x00d0,0x00d0,0x00d0,0x00d0,0x00d0,0x00d0,0x00d0,0x00d0,0x00d4,0x00d4,0x00d4,0x00d4};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0144,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x8004,0x8005,0x0404};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0054,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

    uint16_t tempBin8[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin8( tempBin8, tempBin8 + sizeof(tempBin8) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin8 );

    uint16_t tempBin9[] = {0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0044};
    std::vector<uint16_t> vectBin9( tempBin9, tempBin9 + sizeof(tempBin9) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin9 );

}

void patterns::getPatternM16HR230(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 542;

    uint16_t aRef[] = {0x006e,0x006a,0x006a,0x006a,0x0068,0x0068,0x0068,0x0068,0x0068,0x0270,0x1270,0x1270,0x0270,0x02f0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0144,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x8104,0x8504,0x0405,0x0004};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0250,0x0250};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0250,0x0250,0x0250,0x0250,0x0250,0x0250,0x0250};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0250,0x0250,0x0250,0x0250,0x0250,0x0250,0x0250,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0250,0x0250,0x0250,0x0250,0x0250,0x0250,0x0250};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0250,0x0250,0x0250,0x0250,0x0250,0x0250,0x0250,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0250,0x0250,0x0250,0x0250,0x0250,0x0250,0x0250,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0250,0x0250,0x0250,0x0250,0x0250,0x0250,0x0250};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0250,0x0250,0x0250,0x0250,0x0250,0x0250,0x0250,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0250,0x0250,0x0250,0x0250,0x0250,0x0250,0x0250,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0250,0x0250,0x0250,0x0250,0x0250,0x0250,0x0250,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0048,0x0048,0x0250,0x0250,0x0250,0x0250,0x0250,0x0250,0x0250};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

}

void patterns::getPatternM16HR3(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 0;

    uint16_t aRef[] = {0x006c,0x006c,0x0068,0x0068,0x0068,0x0068,0x0068,0x0078,0x0078,0x0078,0x0078,0x0078,0x0070,0x0070,0x0074,0x0074,0x0076,0x0076,0x0076,0x0074,0x0074,0x0074,0x0074,0x0074,0x0074,0x1074,0x1074,0x0074,0x0074,0x0074,0x0074,0x0074,0x00f4,0x00f4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00c4};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0144,0x0144,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x8104,0x8104,0x0004,0x0004,0x0405,0x0404};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0044,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

}

void patterns::getPatternM16HR3B(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 2;

    uint16_t aRef[] = {0x006c,0x006c,0x0068,0x0068,0x0068,0x0068,0x0068,0x0078,0x0078,0x0078,0x0078,0x0078,0x0070,0x0070,0x0074,0x0074,0x0076,0x0076,0x0076,0x0074,0x0074,0x0074,0x0074,0x0074,0x0074,0x1074,0x1074,0x0074,0x0074,0x0074,0x0074,0x0074,0x00f4,0x00f4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0044,0x0144,0x0144,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x8104,0x8104,0x0004,0x0004,0x0405,0x0404};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x00c4,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

}

void patterns::getPatternM16HR3C(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 30;

    uint16_t aRef[] = {0x006c,0x006c,0x0068,0x0068,0x0068,0x0068,0x0068,0x0078,0x0078,0x0078,0x0078,0x0078,0x0070,0x0070,0x0074,0x0074,0x0076,0x0076,0x0076,0x0074,0x0074,0x0074,0x0074,0x0074,0x0074,0x1074,0x1074,0x0074,0x0074,0x0074,0x0074,0x0074,0x00f4,0x00f4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0044,0x0144,0x0144,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x8104,0x8104,0x0004,0x0004,0x0405,0x0404};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x00c4,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

}

void patterns::getPatternM16HR3i(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 30;

    uint16_t aRef[] = {0x006c,0x006c,0x0068,0x0068,0x0068,0x0068,0x0068,0x0078,0x0078,0x0078,0x0078,0x0078,0x0070,0x0070,0x0074,0x0074,0x0076,0x0076,0x0076,0x0074,0x0074,0x0074,0x0074,0x0074,0x0074,0x1074,0x1074,0x0074,0x0074,0x0074,0x0074,0x0074,0x00f4,0x00f4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0044,0x0144,0x0144,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x8104,0x8104,0x0004,0x0004,0x0405,0x0404};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x00c4,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

}

void patterns::getPatternM16HR4(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 12;

    uint16_t aRef[] = {0x006c,0x006c,0x0068,0x0068,0x0068,0x0068,0x0068,0x0078,0x0078,0x0078,0x0078,0x0078,0x0070,0x0070,0x0074,0x0074,0x0076,0x0076,0x0076,0x0074,0x0074,0x0074,0x0074,0x0074,0x0074,0x1074,0x1074,0x0074,0x0074,0x0074,0x00f4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x0054};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0044,0x0044,0x0144,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x8104,0x8104,0x0004,0x0004,0x0004,0x0004,0x0404,0x0404,0x0004,0x0004,0x0005,0x0004};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0044,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

}

void patterns::getPatternM16HR5(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 0;

    uint16_t aRef[] = {0x006c,0x006c,0x0068,0x0068,0x0068,0x0068,0x0068,0x0078,0x0078,0x0078,0x0078,0x0078,0x0070,0x0070,0x0074,0x0074,0x0076,0x0076,0x0076,0x0074,0x0074,0x0074,0x0074,0x0074,0x0074,0x1074,0x1074,0x0074,0x0074,0x0074,0x0074,0x0074,0x00f4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0044,0x0144,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x8104,0x8104,0x0004,0x0004,0x0004,0x0405,0x0404};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x00c4,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

}

void patterns::getPatternM16HR5B(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 2;

    uint16_t aRef[] = {0x006c,0x006c,0x0068,0x0068,0x0068,0x0068,0x0068,0x0078,0x0078,0x0078,0x0078,0x0078,0x0070,0x0070,0x0074,0x0074,0x0076,0x0076,0x0076,0x0074,0x0074,0x0074,0x0074,0x0074,0x0074,0x1074,0x1074,0x0074,0x0074,0x0074,0x0074,0x0074,0x00f4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0044,0x0144,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x8104,0x8104,0x0004,0x0004,0x0004,0x0405,0x0404};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x00c4,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

}

void patterns::getPatternM16HR5i(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 30;

    uint16_t aRef[] = {0x006c,0x006c,0x0068,0x0068,0x0068,0x0068,0x0068,0x0078,0x0078,0x0078,0x0078,0x0078,0x0070,0x0070,0x0074,0x0074,0x0076,0x0076,0x0076,0x0074,0x0074,0x0074,0x0074,0x0074,0x0074,0x1074,0x1074,0x0074,0x0074,0x0074,0x0074,0x0074,0x00f4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0044,0x0144,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x8104,0x8104,0x0004,0x0004,0x0004,0x0405,0x0404};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x00c4,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

}

void patterns::getPatternM16HR7(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 0;

    uint16_t aRef[] = {0x006c,0x006c,0x0068,0x0068,0x0068,0x0068,0x0068,0x0078,0x0078,0x0078,0x0078,0x0078,0x0070,0x0070,0x0074,0x0074,0x0076,0x0076,0x0076,0x0074,0x0074,0x0074,0x0074,0x0074,0x0074,0x1074,0x1074,0x0074,0x0074,0x0074,0x0074,0x00f4,0x00f4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0044,0x0144,0x0144,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x8104,0x8104,0x0004,0x0004,0x0405,0x0404};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0044,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

    uint16_t tempBin6[] = {0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin6( tempBin6, tempBin6 + sizeof(tempBin6) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin6 );

    uint16_t tempBin7[] = {0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin7( tempBin7, tempBin7 + sizeof(tempBin7) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin7 );

}

void patterns::getPatternM16HR7i(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 30;

    uint16_t aRef[] = {0x006c,0x006c,0x0068,0x0068,0x0068,0x0068,0x0068,0x0078,0x0078,0x0078,0x0078,0x0078,0x0070,0x0070,0x0074,0x0074,0x0076,0x0076,0x0076,0x0074,0x0074,0x0074,0x0074,0x0074,0x0074,0x1074,0x1074,0x0074,0x0074,0x0074,0x0074,0x0074,0x00f4,0x00f4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4,0x00d4};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0044,0x0144,0x0144,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x8104,0x8104,0x0004,0x0004,0x0405,0x0404};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x00c4,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

    uint16_t tempBin1[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin1( tempBin1, tempBin1 + sizeof(tempBin1) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin1 );

    uint16_t tempBin2[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin2( tempBin2, tempBin2 + sizeof(tempBin2) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin2 );

    uint16_t tempBin3[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin3( tempBin3, tempBin3 + sizeof(tempBin3) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin3 );

    uint16_t tempBin4[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin4( tempBin4, tempBin4 + sizeof(tempBin4) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin4 );

    uint16_t tempBin5[] = {0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x0044,0x004c,0x004c,0x004c,0x004c,0x004c,0x0048,0x0048,0x0048,0x0048,0x0048,0x0048,0x0058,0x0058,0x0058,0x0058,0x0050,0x0050,0x0050,0x0050,0x0050,0x0054,0x0054,0x0054,0x0054,0x0054,0x0054,0x0044,0x0044,0x0044,0x0044,0x0044};
    std::vector<uint16_t> vectBin5( tempBin5, tempBin5 + sizeof(tempBin5) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin5 );

}

void patterns::getPatternM16HS1(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 0;

    uint16_t aRef[] = {0x006c,0x0068,0x006a,0x006a,0x1068,0x1068,0x1068,0x0068,0x00e8,0x00c8,0x00d8,0x00d8,0x00d8,0x00d8,0x00d0,0x00d0,0x00d0,0x00d0,0x00d4,0x00d4,0x00d4,0x00d4};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0144,0x0104,0x0104,0x0104,0x0104,0x0004,0x0005,0x0004};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0054,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternM16HS1B(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 2;

    uint16_t aRef[] = {0x006c,0x0068,0x006a,0x006a,0x1068,0x1068,0x1068,0x0068,0x00e8,0x00c8,0x00d8,0x00d8,0x00d8,0x00d8,0x00d0,0x00d0,0x00d0,0x00d0,0x00d4,0x00d4,0x00d4,0x00d4};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0144,0x0104,0x0104,0x0104,0x0104,0x0004,0x0005,0x0004};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0054,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternM16HS1C(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 30;

    uint16_t aRef[] = {0x006c,0x0068,0x006a,0x006a,0x1068,0x1068,0x1068,0x0068,0x00e8,0x00c8,0x00d8,0x00d8,0x00d8,0x00d8,0x00d0,0x00d0,0x00d0,0x00d0,0x00d4,0x00d4,0x00d4,0x00d4};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0144,0x0104,0x0104,0x0104,0x0104,0x0004,0x0005,0x0004};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0054,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternM16HS1i(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 30;

    uint16_t aRef[] = {0x006c,0x0068,0x006a,0x006a,0x1068,0x1068,0x1068,0x0068,0x00e8,0x00c8,0x00d8,0x00d8,0x00d8,0x00d8,0x00d0,0x00d0,0x00d0,0x00d0,0x00d4,0x00d4,0x00d4,0x00d4};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0144,0x0104,0x0104,0x0104,0x0104,0x0004,0x0005,0x0004};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0054,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternM16HS2(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 12;

    uint16_t aRef[] = {0x006c,0x0068,0x006a,0x006a,0x1068,0x1068,0x1068,0x0068,0x00e8,0x00c8,0x00d8,0x00d8,0x00d8,0x00d8,0x00d0,0x00d0,0x00d0,0x00d0,0x00d4,0x00d4,0x00d4,0x00d4};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0144,0x0104,0x0104,0x0104,0x0104,0x0004,0x0005,0x0004};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0054,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternM16HS230(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 542;

    uint16_t aRef[] = {0x006c,0x006a,0x006a,0x006a,0x0068,0x0068,0x0270,0x1270,0x1270,0x02f0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0,0x02d0};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0044,0x0044,0x0144,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0104,0x0004,0x0004,0x0005,0x0004};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0250,0x0250};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternM16HS7(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 0;

    uint16_t aRef[] = {0x006c,0x0068,0x006a,0x006a,0x1068,0x1068,0x1068,0x0068,0x00e8,0x00c8,0x00d8,0x00d8,0x00d8,0x00d8,0x00d0,0x00d0,0x00d0,0x00d0,0x00d4,0x00d4,0x00d4,0x00d4};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0144,0x0104,0x0104,0x0104,0x0104,0x0004,0x0005,0x0004};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0054,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternM16HS7i(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 30;

    uint16_t aRef[] = {0x006c,0x0068,0x006a,0x006a,0x1068,0x1068,0x1068,0x0068,0x00e8,0x00c8,0x00d8,0x00d8,0x00d8,0x00d8,0x00d0,0x00d0,0x00d0,0x00d0,0x00d4,0x00d4,0x00d4,0x00d4};
    std::vector<uint16_t> vectRef( aRef, aRef + sizeof(aRef) / sizeof(uint16_t ) );

    out.RefPatternData = vectRef;

    uint16_t aSig[] = {0x0144,0x0104,0x0104,0x0104,0x0104,0x0004,0x0005,0x0004};
    std::vector<uint16_t> vectSig( aSig, aSig + sizeof(aSig) / sizeof(uint16_t ) );

    out.SigPatternData = vectSig;

    //clear out old bin pattern 
    out.BinPatternData.clear(); 
    //fill in new bin pattern 
    uint16_t tempBin0[] = {0x0054,0x0044};
    std::vector<uint16_t> vectBin0( tempBin0, tempBin0 + sizeof(tempBin0) / sizeof(uint16_t ) );

    out.BinPatternData.push_back( vectBin0 );

}

void patterns::getPatternNoOpH(CamCfg::APN_HPATTERN_FILE & out)
{
    out.Mask = 0;

    //no reference data 
    out.RefPatternData.clear(); 

    //no signal data 
    out.SigPatternData.clear(); 

    //no pattern data 
    out.BinPatternData.clear(); 
}
